@@ -8,90 +8,113 @@ const bench = common.createBenchmark(main, {
     'arraybuffer',
     'arraybuffer-middle',
     'buffer',
-    'uint8array',
     'string',
     'string-utf8',
     'string-base64',
     'object',
+    'uint8array',
+    'uint16array',
   ],
-  len: [10, 2048],
-  n: [2048]
+  len: [100, 2048],
+  n: [8e5],
 });
 
 function main({ len, n, source }) {
-  const array = new Array(len).fill(42);
-  const arrayBuf = new ArrayBuffer(len);
-  const str = 'a'.repeat(len);
-  const buffer = Buffer.allocUnsafe(len);
-  const uint8array = new Uint8Array(len);
-  const obj = { length: null }; // Results in a new, empty Buffer
+  let i = 0;
 
   switch (source) {
-    case 'array':
+    case 'array': {
+      const array = new Array(len).fill(42);
       bench.start();
-      for (let i = 0; i < n * 1024; i++) {
+      for (i = 0; i < n; i++) {
         Buffer.from(array);
       }
       bench.end(n);
       break;
-    case 'arraybuffer':
+    }
+    case 'arraybuffer': {
+      const arrayBuf = new ArrayBuffer(len);
       bench.start();
-      for (let i = 0; i < n * 1024; i++) {
+      for (i = 0; i < n; i++) {
         Buffer.from(arrayBuf);
       }
       bench.end(n);
       break;
-    case 'arraybuffer-middle':
+    }
+    case 'arraybuffer-middle': {
+      const arrayBuf = new ArrayBuffer(len);
       const offset = ~~(len / 4);
       const length = ~~(len / 2);
       bench.start();
-      for (let i = 0; i < n * 1024; i++) {
+      for (i = 0; i < n; i++) {
         Buffer.from(arrayBuf, offset, length);
       }
       bench.end(n);
       break;
-    case 'buffer':
+    }
+    case 'buffer': {
+      const buffer = Buffer.allocUnsafe(len);
       bench.start();
-      for (let i = 0; i < n * 1024; i++) {
+      for (i = 0; i < n; i++) {
         Buffer.from(buffer);
       }
       bench.end(n);
       break;
-    case 'uint8array':
+    }
+    case 'uint8array': {
+      const uint8array = new Uint8Array(len);
       bench.start();
-      for (let i = 0; i < n * 1024; i++) {
+      for (i = 0; i < n; i++) {
         Buffer.from(uint8array);
       }
       bench.end(n);
       break;
-    case 'string':
+    }
+    case 'uint16array': {
+      const uint16array = new Uint16Array(len);
       bench.start();
-      for (let i = 0; i < n * 1024; i++) {
+      for (i = 0; i < n; i++) {
+        Buffer.from(uint16array);
+      }
+      bench.end(n);
+      break;
+    }
+    case 'string': {
+      const str = 'a'.repeat(len);
+      bench.start();
+      for (i = 0; i < n; i++) {
         Buffer.from(str);
       }
       bench.end(n);
       break;
-    case 'string-utf8':
+    }
+    case 'string-utf8': {
+      const str = 'a'.repeat(len);
       bench.start();
-      for (let i = 0; i < n * 1024; i++) {
+      for (i = 0; i < n; i++) {
         Buffer.from(str, 'utf8');
       }
       bench.end(n);
       break;
-    case 'string-base64':
+    }
+    case 'string-base64': {
+      const str = 'a'.repeat(len);
       bench.start();
-      for (let i = 0; i < n * 1024; i++) {
+      for (i = 0; i < n; i++) {
         Buffer.from(str, 'base64');
       }
       bench.end(n);
       break;
-    case 'object':
+    }
+    case 'object': {
+      const obj = { length: null }; // Results in a new, empty Buffer
       bench.start();
-      for (let i = 0; i < n * 1024; i++) {
+      for (i = 0; i < n; i++) {
         Buffer.from(obj);
       }
       bench.end(n);
       break;
+    }
     default:
       assert.fail('Should not get here');
   }
