@@ -6,7 +6,7 @@ const bench = common.createBenchmark(main, {
   encoding: ['', 'utf8', 'ascii', 'latin1', 'hex', 'UCS-2'],
   args: [0, 1, 3],
   len: [1, 64, 1024],
-  n: [1e6]
+  n: [1e6],
 }, {
   combinationFilter: (p) => {
     return (p.args === 0 && p.encoding === '') ||
