@@ -5,8 +5,7 @@ const common = require('../common.js');
 const { strictEqual } = require('assert');
 
 const tmpdir = require('../../test/common/tmpdir');
-const benchmarkDirectory =
-  path.resolve(tmpdir.path, 'benchmark-esm-parse');
+const benchmarkDirectory = tmpdir.resolve('benchmark-esm-parse');
 
 const bench = common.createBenchmark(main, {
   n: [1e2],
