@@ -7,13 +7,14 @@ const fs = require('fs');
 const file = path.join(path.resolve(__dirname, '../fixtures'), 'alice.html');
 
 const bench = common.createBenchmark(main, {
-  requests: [100, 1000, 10000, 100000],
-  streams: [100, 200, 1000],
-  clients: [1, 2],
-  benchmarker: ['h2load']
+  requests: [100, 1000, 5000],
+  streams: [1, 10, 20, 40, 100, 200],
+  clients: [2],
+  benchmarker: ['test-double-http2'],
+  duration: 5,
 }, { flags: ['--no-warnings'] });
 
-function main({ requests, streams, clients }) {
+function main({ requests, streams, clients, duration }) {
   fs.open(file, 'r', (err, fd) => {
     if (err)
       throw err;
@@ -24,13 +25,15 @@ function main({ requests, streams, clients }) {
       stream.respondWithFD(fd);
       stream.on('error', (err) => {});
     });
-    server.listen(common.PORT, () => {
+    server.listen(0, () => {
       bench.http({
         path: '/',
         requests,
+        port: server.address().port,
         maxConcurrentStreams: streams,
         clients,
-        threads: clients
+        duration,
+        threads: clients,
       }, () => server.close());
     });
 
