@@ -4,19 +4,19 @@ const common = require('../common');
 const util = require('util');
 
 const bench = common.createBenchmark(main, {
-  n: [5e2],
+  n: [5e3],
   len: [1e2, 1e5],
   type: [
     'denseArray',
     'sparseArray',
     'mixedArray',
     'denseArray_showHidden',
-  ]
+  ],
 });
 
 function main({ n, len, type }) {
-  var arr = Array(len);
-  var i, opts;
+  let arr = Array(len);
+  let opts;
 
   switch (type) {
     case 'denseArray_showHidden':
@@ -29,14 +29,14 @@ function main({ n, len, type }) {
     case 'sparseArray':
       break;
     case 'mixedArray':
-      for (i = 0; i < n; i += 2)
+      for (let i = 0; i < n; i += 2)
         arr[i] = i;
       break;
     default:
       throw new Error(`Unsupported type ${type}`);
   }
   bench.start();
-  for (i = 0; i < n; i++) {
+  for (let i = 0; i < n; i++) {
     util.inspect(arr, opts);
   }
   bench.end(n);
