@@ -12,7 +12,8 @@
 namespace v8 {
 namespace base {
 
-void PosixInitializeCommon(bool hard_abort, const char* const gc_fake_mmap);
+void PosixInitializeCommon(AbortMode abort_mode,
+                           const char* const gc_fake_mmap);
 
 class PosixTimezoneCache : public TimezoneCache {
  public:
