@@ -12,6 +12,7 @@
 #ifndef V8_BASE_SAFE_CONVERSIONS_IMPL_H_
 #define V8_BASE_SAFE_CONVERSIONS_IMPL_H_
 
+#include <stddef.h>
 #include <stdint.h>
 
 #include <limits>
@@ -195,7 +196,7 @@ class RangeCheck {
  public:
   constexpr RangeCheck(bool is_in_lower_bound, bool is_in_upper_bound)
       : is_underflow_(!is_in_lower_bound), is_overflow_(!is_in_upper_bound) {}
-  constexpr RangeCheck() : is_underflow_(0), is_overflow_(0) {}
+  constexpr RangeCheck() : is_underflow_(false), is_overflow_(false) {}
   constexpr bool IsValid() const { return !is_overflow_ && !is_underflow_; }
   constexpr bool IsInvalid() const { return is_overflow_ && is_underflow_; }
   constexpr bool IsOverflow() const { return is_overflow_ && !is_underflow_; }
