@@ -2,9 +2,9 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "src/builtins/builtins-inl.h"
 #include "src/builtins/builtins-utils-gen.h"
-#include "src/builtins/builtins.h"
-#include "src/codegen/code-stub-assembler.h"
+#include "src/codegen/code-stub-assembler-inl.h"
 #include "src/objects/dictionary.h"
 
 namespace v8 {
@@ -218,18 +218,18 @@ TF_BUILTIN(DatePrototypeToPrimitive, CodeStubAssembler) {
   // Use the OrdinaryToPrimitive builtin to convert to a Number.
   BIND(&hint_is_number);
   {
-    Callable callable = CodeFactory::OrdinaryToPrimitive(
-        isolate(), OrdinaryToPrimitiveHint::kNumber);
-    TNode<Object> result = CallStub(callable, context, receiver);
+    Builtin builtin =
+        Builtins::OrdinaryToPrimitive(OrdinaryToPrimitiveHint::kNumber);
+    TNode<Object> result = CallBuiltin(builtin, context, receiver);
     Return(result);
   }
 
   // Use the OrdinaryToPrimitive builtin to convert to a String.
   BIND(&hint_is_string);
   {
-    Callable callable = CodeFactory::OrdinaryToPrimitive(
-        isolate(), OrdinaryToPrimitiveHint::kString);
-    TNode<Object> result = CallStub(callable, context, receiver);
+    Builtin builtin =
+        Builtins::OrdinaryToPrimitive(OrdinaryToPrimitiveHint::kString);
+    TNode<Object> result = CallBuiltin(builtin, context, receiver);
     Return(result);
   }
 
