@@ -59,14 +59,15 @@ BUILTIN(ObjectDefineProperty) {
 namespace {
 
 template <AccessorComponent which_accessor>
-Object ObjectDefineAccessor(Isolate* isolate, Handle<Object> object,
-                            Handle<Object> name, Handle<Object> accessor) {
+Tagged<Object> ObjectDefineAccessor(Isolate* isolate, Handle<Object> object,
+                                    Handle<Object> name,
+                                    Handle<Object> accessor) {
   // 1. Let O be ? ToObject(this value).
   Handle<JSReceiver> receiver;
   ASSIGN_RETURN_FAILURE_ON_EXCEPTION(isolate, receiver,
                                      Object::ToObject(isolate, object));
   // 2. If IsCallable(getter) is false, throw a TypeError exception.
-  if (!accessor->IsCallable()) {
+  if (!IsCallable(*accessor)) {
     MessageTemplate message =
         which_accessor == ACCESSOR_GETTER
             ? MessageTemplate::kObjectGetterExpectingFunction
@@ -100,8 +101,9 @@ Object ObjectDefineAccessor(Isolate* isolate, Handle<Object> object,
   return ReadOnlyRoots(isolate).undefined_value();
 }
 
-Object ObjectLookupAccessor(Isolate* isolate, Handle<Object> object,
-                            Handle<Object> key, AccessorComponent component) {
+Tagged<Object> ObjectLookupAccessor(Isolate* isolate, Handle<Object> object,
+                                    Handle<Object> key,
+                                    AccessorComponent component) {
   ASSIGN_RETURN_FAILURE_ON_EXCEPTION(isolate, object,
                                      Object::ToObject(isolate, object));
   // TODO(jkummerow/verwaest): PropertyKey(..., bool*) performs a
@@ -113,18 +115,17 @@ Object ObjectLookupAccessor(Isolate* isolate, Handle<Object> object,
   LookupIterator it(isolate, object, lookup_key,
                     LookupIterator::PROTOTYPE_CHAIN_SKIP_INTERCEPTOR);
 
-  for (; it.IsFound(); it.Next()) {
+  for (;; it.Next()) {
     switch (it.state()) {
       case LookupIterator::INTERCEPTOR:
-      case LookupIterator::NOT_FOUND:
       case LookupIterator::TRANSITION:
         UNREACHABLE();
 
       case LookupIterator::ACCESS_CHECK:
         if (it.HasAccess()) continue;
-        isolate->ReportFailedAccessCheck(it.GetHolder<JSObject>());
-        RETURN_FAILURE_IF_SCHEDULED_EXCEPTION(isolate);
-        return ReadOnlyRoots(isolate).undefined_value();
+        RETURN_FAILURE_ON_EXCEPTION(isolate, isolate->ReportFailedAccessCheck(
+                                                 it.GetHolder<JSObject>()));
+        UNREACHABLE();
 
       case LookupIterator::JSPROXY: {
         PropertyDescriptor desc;
@@ -143,35 +144,32 @@ Object ObjectLookupAccessor(Isolate* isolate, Handle<Object> object,
         Handle<Object> prototype;
         ASSIGN_RETURN_FAILURE_ON_EXCEPTION(
             isolate, prototype, JSProxy::GetPrototype(it.GetHolder<JSProxy>()));
-        if (prototype->IsNull(isolate)) {
+        if (IsNull(*prototype, isolate)) {
           return ReadOnlyRoots(isolate).undefined_value();
         }
         return ObjectLookupAccessor(isolate, prototype, key, component);
       }
-
       case LookupIterator::WASM_OBJECT:
-        THROW_NEW_ERROR_RETURN_FAILURE(
-            isolate, NewTypeError(MessageTemplate::kWasmObjectsAreOpaque));
-
-      case LookupIterator::INTEGER_INDEXED_EXOTIC:
+      case LookupIterator::TYPED_ARRAY_INDEX_NOT_FOUND:
       case LookupIterator::DATA:
+      case LookupIterator::NOT_FOUND:
         return ReadOnlyRoots(isolate).undefined_value();
 
       case LookupIterator::ACCESSOR: {
         Handle<Object> maybe_pair = it.GetAccessors();
-        if (maybe_pair->IsAccessorPair()) {
-          Handle<NativeContext> native_context = it.GetHolder<JSReceiver>()
-                                                     ->GetCreationContext()
-                                                     .ToHandleChecked();
+        if (IsAccessorPair(*maybe_pair)) {
+          Handle<NativeContext> holder_realm(
+              it.GetHolder<JSReceiver>()->GetCreationContext().value(),
+              isolate);
           return *AccessorPair::GetComponent(
-              isolate, native_context, Handle<AccessorPair>::cast(maybe_pair),
+              isolate, holder_realm, Handle<AccessorPair>::cast(maybe_pair),
               component);
         }
+        continue;
       }
     }
+    UNREACHABLE();
   }
-
-  return ReadOnlyRoots(isolate).undefined_value();
 }
 
 }  // namespace
@@ -218,10 +216,11 @@ BUILTIN(ObjectLookupSetter) {
 BUILTIN(ObjectFreeze) {
   HandleScope scope(isolate);
   Handle<Object> object = args.atOrUndefined(isolate, 1);
-  if (object->IsJSReceiver()) {
-    MAYBE_RETURN(JSReceiver::SetIntegrityLevel(Handle<JSReceiver>::cast(object),
-                                               FROZEN, kThrowOnError),
-                 ReadOnlyRoots(isolate).exception());
+  if (IsJSReceiver(*object)) {
+    MAYBE_RETURN(
+        JSReceiver::SetIntegrityLevel(isolate, Handle<JSReceiver>::cast(object),
+                                      FROZEN, kThrowOnError),
+        ReadOnlyRoots(isolate).exception());
   }
   return *object;
 }
@@ -244,7 +243,7 @@ BUILTIN(ObjectPrototypeSetProto) {
   HandleScope scope(isolate);
   // 1. Let O be ? RequireObjectCoercible(this value).
   Handle<Object> object = args.receiver();
-  if (object->IsNullOrUndefined(isolate)) {
+  if (IsNullOrUndefined(*object, isolate)) {
     THROW_NEW_ERROR_RETURN_FAILURE(
         isolate, NewTypeError(MessageTemplate::kCalledOnNullOrUndefined,
                               isolate->factory()->NewStringFromAsciiChecked(
@@ -253,12 +252,12 @@ BUILTIN(ObjectPrototypeSetProto) {
 
   // 2. If Type(proto) is neither Object nor Null, return undefined.
   Handle<Object> proto = args.at(1);
-  if (!proto->IsNull(isolate) && !proto->IsJSReceiver()) {
+  if (!IsNull(*proto, isolate) && !IsJSReceiver(*proto)) {
     return ReadOnlyRoots(isolate).undefined_value();
   }
 
   // 3. If Type(O) is not Object, return undefined.
-  if (!object->IsJSReceiver()) return ReadOnlyRoots(isolate).undefined_value();
+  if (!IsJSReceiver(*object)) return ReadOnlyRoots(isolate).undefined_value();
   Handle<JSReceiver> receiver = Handle<JSReceiver>::cast(object);
 
   // 4. Let status be ? O.[[SetPrototypeOf]](proto).
@@ -273,8 +272,8 @@ BUILTIN(ObjectPrototypeSetProto) {
 
 namespace {
 
-Object GetOwnPropertyKeys(Isolate* isolate, BuiltinArguments args,
-                          PropertyFilter filter) {
+Tagged<Object> GetOwnPropertyKeys(Isolate* isolate, BuiltinArguments args,
+                                  PropertyFilter filter) {
   HandleScope scope(isolate);
   Handle<Object> object = args.atOrUndefined(isolate, 1);
   Handle<JSReceiver> receiver;
@@ -299,10 +298,11 @@ BUILTIN(ObjectGetOwnPropertySymbols) {
 BUILTIN(ObjectIsFrozen) {
   HandleScope scope(isolate);
   Handle<Object> object = args.atOrUndefined(isolate, 1);
-  Maybe<bool> result = object->IsJSReceiver()
-                           ? JSReceiver::TestIntegrityLevel(
-                                 Handle<JSReceiver>::cast(object), FROZEN)
-                           : Just(true);
+  Maybe<bool> result =
+      IsJSReceiver(*object)
+          ? JSReceiver::TestIntegrityLevel(
+                isolate, Handle<JSReceiver>::cast(object), FROZEN)
+          : Just(true);
   MAYBE_RETURN(result, ReadOnlyRoots(isolate).exception());
   return isolate->heap()->ToBoolean(result.FromJust());
 }
@@ -311,10 +311,11 @@ BUILTIN(ObjectIsFrozen) {
 BUILTIN(ObjectIsSealed) {
   HandleScope scope(isolate);
   Handle<Object> object = args.atOrUndefined(isolate, 1);
-  Maybe<bool> result = object->IsJSReceiver()
-                           ? JSReceiver::TestIntegrityLevel(
-                                 Handle<JSReceiver>::cast(object), SEALED)
-                           : Just(true);
+  Maybe<bool> result =
+      IsJSReceiver(*object)
+          ? JSReceiver::TestIntegrityLevel(
+                isolate, Handle<JSReceiver>::cast(object), SEALED)
+          : Just(true);
   MAYBE_RETURN(result, ReadOnlyRoots(isolate).exception());
   return isolate->heap()->ToBoolean(result.FromJust());
 }
@@ -338,7 +339,7 @@ BUILTIN(ObjectGetOwnPropertyDescriptors) {
       isolate->factory()->NewJSObject(isolate->object_function());
 
   for (int i = 0; i < keys->length(); ++i) {
-    Handle<Name> key = Handle<Name>::cast(FixedArray::get(*keys, i, isolate));
+    Handle<Name> key(Name::cast(keys->get(i)), isolate);
     PropertyDescriptor descriptor;
     Maybe<bool> did_get_descriptor = JSReceiver::GetOwnPropertyDescriptor(
         isolate, receiver, key, &descriptor);
@@ -359,10 +360,11 @@ BUILTIN(ObjectGetOwnPropertyDescriptors) {
 BUILTIN(ObjectSeal) {
   HandleScope scope(isolate);
   Handle<Object> object = args.atOrUndefined(isolate, 1);
-  if (object->IsJSReceiver()) {
-    MAYBE_RETURN(JSReceiver::SetIntegrityLevel(Handle<JSReceiver>::cast(object),
-                                               SEALED, kThrowOnError),
-                 ReadOnlyRoots(isolate).exception());
+  if (IsJSReceiver(*object)) {
+    MAYBE_RETURN(
+        JSReceiver::SetIntegrityLevel(isolate, Handle<JSReceiver>::cast(object),
+                                      SEALED, kThrowOnError),
+        ReadOnlyRoots(isolate).exception());
   }
   return *object;
 }
