@@ -2,12 +2,15 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "src/builtins/builtins.h"
+#include <fstream>
+
+#include "src/builtins/builtins-inl.h"
 #include "src/builtins/profile-data-reader.h"
 #include "src/codegen/assembler-inl.h"
 #include "src/codegen/interface-descriptors.h"
 #include "src/codegen/macro-assembler-inl.h"
 #include "src/codegen/macro-assembler.h"
+#include "src/codegen/reloc-info-inl.h"
 #include "src/compiler/code-assembler.h"
 #include "src/execution/isolate.h"
 #include "src/handles/handles-inl.h"
@@ -20,6 +23,10 @@
 #include "src/objects/shared-function-info.h"
 #include "src/objects/smi.h"
 
+#if V8_ENABLE_WEBASSEMBLY
+#include "src/wasm/wasm-builtin-list.h"
+#endif
+
 namespace v8 {
 namespace internal {
 
@@ -38,6 +45,14 @@ AssemblerOptions BuiltinAssemblerOptions(Isolate* isolate, Builtin builtin) {
   CHECK(!options.isolate_independent_code);
   CHECK(!options.collect_win64_unwind_info);
 
+#if V8_ENABLE_WEBASSEMBLY
+  if (wasm::BuiltinLookup::IsWasmBuiltinId(builtin) ||
+      builtin == Builtin::kJSToWasmWrapper ||
+      builtin == Builtin::kJSToWasmHandleReturns ||
+      builtin == Builtin::kWasmToJsWrapperCSA) {
+    options.is_wasm = true;
+  }
+#endif
   if (!isolate->IsGeneratingEmbeddedBuiltins()) {
     return options;
   }
@@ -52,8 +67,9 @@ AssemblerOptions BuiltinAssemblerOptions(Isolate* isolate, Builtin builtin) {
   // PC-relative call/jump instructions can be used for builtin to builtin
   // calls/tail calls. The embedded builtins blob generator also ensures that.
   // However, there are serializer tests, where we force isolate creation at
-  // runtime and at this point, Code space isn't restricted to a size s.t.
-  // PC-relative calls may be used. So, we fall back to an indirect mode.
+  // runtime and at this point, Code space isn't restricted to a
+  // size s.t. PC-relative calls may be used. So, we fall back to an indirect
+  // mode.
   options.use_pc_relative_calls_and_jumps_for_mksnapshot =
       pc_relative_calls_fit_in_code_range;
 
@@ -77,7 +93,7 @@ using CodeAssemblerGenerator = void (*)(compiler::CodeAssemblerState*);
 
 Handle<Code> BuildPlaceholder(Isolate* isolate, Builtin builtin) {
   HandleScope scope(isolate);
-  byte buffer[kBufferSize];
+  uint8_t buffer[kBufferSize];
   MacroAssembler masm(isolate, CodeObjectRequired::kYes,
                       ExternalAssemblerBuffer(buffer, kBufferSize));
   DCHECK(!masm.has_frame());
@@ -97,14 +113,11 @@ Handle<Code> BuildPlaceholder(Isolate* isolate, Builtin builtin) {
   return scope.CloseAndEscape(code);
 }
 
-Code BuildWithMacroAssembler(Isolate* isolate, Builtin builtin,
-                             MacroAssemblerGenerator generator,
-                             const char* s_name) {
+V8_NOINLINE Tagged<Code> BuildWithMacroAssembler(
+    Isolate* isolate, Builtin builtin, MacroAssemblerGenerator generator,
+    const char* s_name) {
   HandleScope scope(isolate);
-  // Canonicalize handles, so that we can share constant pool entries pointing
-  // to code targets without dereferencing their handles.
-  CanonicalHandleScope canonical(isolate);
-  byte buffer[kBufferSize];
+  uint8_t buffer[kBufferSize];
 
   MacroAssembler masm(isolate, BuiltinAssemblerOptions(isolate, builtin),
                       CodeObjectRequired::kYes,
@@ -125,10 +138,18 @@ Code BuildWithMacroAssembler(Isolate* isolate, Builtin builtin,
     HandlerTable::EmitReturnEntry(
         &masm, 0, isolate->builtins()->js_entry_handler_offset());
   }
+#if V8_ENABLE_WEBASSEMBLY && (V8_TARGET_ARCH_X64 || V8_TARGET_ARCH_ARM64 || \
+                              V8_TARGET_ARCH_IA32 || V8_TARGET_ARCH_ARM)
+  if (builtin == Builtin::kWasmReturnPromiseOnSuspendAsm) {
+    handler_table_offset = HandlerTable::EmitReturnTableStart(&masm);
+    HandlerTable::EmitReturnEntry(
+        &masm, 0, isolate->builtins()->jspi_prompt_handler_offset());
+  }
+#endif
 
   CodeDesc desc;
-  masm.GetCode(isolate, &desc, MacroAssembler::kNoSafepointTable,
-               handler_table_offset);
+  masm.GetCode(isolate->main_thread_local_isolate(), &desc,
+               MacroAssembler::kNoSafepointTable, handler_table_offset);
 
   Handle<Code> code = Factory::CodeBuilder(isolate, desc, CodeKind::BUILTIN)
                           .set_self_reference(masm.CodeObject())
@@ -140,13 +161,10 @@ Code BuildWithMacroAssembler(Isolate* isolate, Builtin builtin,
   return *code;
 }
 
-Code BuildAdaptor(Isolate* isolate, Builtin builtin, Address builtin_address,
-                  const char* name) {
+Tagged<Code> BuildAdaptor(Isolate* isolate, Builtin builtin,
+                          Address builtin_address, const char* name) {
   HandleScope scope(isolate);
-  // Canonicalize handles, so that we can share constant pool entries pointing
-  // to code targets without dereferencing their handles.
-  CanonicalHandleScope canonical(isolate);
-  byte buffer[kBufferSize];
+  uint8_t buffer[kBufferSize];
   MacroAssembler masm(isolate, BuiltinAssemblerOptions(isolate, builtin),
                       CodeObjectRequired::kYes,
                       ExternalAssemblerBuffer(buffer, kBufferSize));
@@ -163,13 +181,10 @@ Code BuildAdaptor(Isolate* isolate, Builtin builtin, Address builtin_address,
 }
 
 // Builder for builtins implemented in TurboFan with JS linkage.
-Code BuildWithCodeStubAssemblerJS(Isolate* isolate, Builtin builtin,
-                                  CodeAssemblerGenerator generator, int argc,
-                                  const char* name) {
+V8_NOINLINE Tagged<Code> BuildWithCodeStubAssemblerJS(
+    Isolate* isolate, Builtin builtin, CodeAssemblerGenerator generator,
+    int argc, const char* name) {
   HandleScope scope(isolate);
-  // Canonicalize handles, so that we can share constant pool entries pointing
-  // to code targets without dereferencing their handles.
-  CanonicalHandleScope canonical(isolate);
 
   Zone zone(isolate->allocator(), ZONE_NAME, kCompressGraphZone);
   compiler::CodeAssemblerState state(isolate, &zone, argc, CodeKind::BUILTIN,
@@ -182,14 +197,10 @@ Code BuildWithCodeStubAssemblerJS(Isolate* isolate, Builtin builtin,
 }
 
 // Builder for builtins implemented in TurboFan with CallStub linkage.
-Code BuildWithCodeStubAssemblerCS(Isolate* isolate, Builtin builtin,
-                                  CodeAssemblerGenerator generator,
-                                  CallDescriptors::Key interface_descriptor,
-                                  const char* name) {
+V8_NOINLINE Tagged<Code> BuildWithCodeStubAssemblerCS(
+    Isolate* isolate, Builtin builtin, CodeAssemblerGenerator generator,
+    CallDescriptors::Key interface_descriptor, const char* name) {
   HandleScope scope(isolate);
-  // Canonicalize handles, so that we can share constant pool entries pointing
-  // to code targets without dereferencing their handles.
-  CanonicalHandleScope canonical(isolate);
   Zone zone(isolate->allocator(), ZONE_NAME, kCompressGraphZone);
   // The interface descriptor with given key must be initialized at this point
   // and this construction just queries the details from the descriptors table.
@@ -209,9 +220,9 @@ Code BuildWithCodeStubAssemblerCS(Isolate* isolate, Builtin builtin,
 
 // static
 void SetupIsolateDelegate::AddBuiltin(Builtins* builtins, Builtin builtin,
-                                      Code code) {
-  DCHECK_EQ(builtin, code.builtin_id());
-  builtins->set_code(builtin, ToCodeT(code));
+                                      Tagged<Code> code) {
+  DCHECK_EQ(builtin, code->builtin_id());
+  builtins->set_code(builtin, code);
 }
 
 // static
@@ -233,7 +244,6 @@ void SetupIsolateDelegate::ReplacePlaceholders(Isolate* isolate) {
   // Replace references from all builtin code objects to placeholders.
   Builtins* builtins = isolate->builtins();
   DisallowGarbageCollection no_gc;
-  CodePageCollectionMemoryModificationScope modification_scope(isolate->heap());
   static const int kRelocMask =
       RelocInfo::ModeMask(RelocInfo::CODE_TARGET) |
       RelocInfo::ModeMask(RelocInfo::FULL_EMBEDDED_OBJECT) |
@@ -242,44 +252,50 @@ void SetupIsolateDelegate::ReplacePlaceholders(Isolate* isolate) {
   PtrComprCageBase cage_base(isolate);
   for (Builtin builtin = Builtins::kFirst; builtin <= Builtins::kLast;
        ++builtin) {
-    Code code = FromCodeT(builtins->code(builtin));
-    isolate->heap()->UnprotectAndRegisterMemoryChunk(
-        code, UnprotectMemoryOrigin::kMainThread);
+    Tagged<Code> code = builtins->code(builtin);
+    Tagged<InstructionStream> istream = code->instruction_stream();
+    WritableJitAllocation jit_allocation = ThreadIsolation::LookupJitAllocation(
+        istream.address(), istream->Size(),
+        ThreadIsolation::JitAllocationType::kInstructionStream);
     bool flush_icache = false;
-    for (RelocIterator it(code, kRelocMask); !it.done(); it.next()) {
-      RelocInfo* rinfo = it.rinfo();
+    for (WritableRelocIterator it(jit_allocation, istream,
+                                  code->constant_pool(), kRelocMask);
+         !it.done(); it.next()) {
+      WritableRelocInfo* rinfo = it.rinfo();
       if (RelocInfo::IsCodeTargetMode(rinfo->rmode())) {
-        Code target = Code::GetCodeFromTargetAddress(rinfo->target_address());
-        DCHECK_IMPLIES(RelocInfo::IsRelativeCodeTarget(rinfo->rmode()),
-                       Builtins::IsIsolateIndependent(target.builtin_id()));
-        if (!target.is_builtin()) continue;
-        CodeT new_target = builtins->code(target.builtin_id());
-        rinfo->set_target_address(new_target.raw_instruction_start(),
+        Tagged<Code> target_code =
+            Code::FromTargetAddress(rinfo->target_address());
+        DCHECK_IMPLIES(
+            RelocInfo::IsRelativeCodeTarget(rinfo->rmode()),
+            Builtins::IsIsolateIndependent(target_code->builtin_id()));
+        if (!target_code->is_builtin()) continue;
+        Tagged<Code> new_target = builtins->code(target_code->builtin_id());
+        rinfo->set_target_address(istream, new_target->instruction_start(),
                                   UPDATE_WRITE_BARRIER, SKIP_ICACHE_FLUSH);
       } else {
         DCHECK(RelocInfo::IsEmbeddedObjectMode(rinfo->rmode()));
-        Object object = rinfo->target_object(cage_base);
-        if (!object.IsCodeT(cage_base)) continue;
-        CodeT target = CodeT::cast(object);
-        if (!target.is_builtin()) continue;
-        CodeT new_target = builtins->code(target.builtin_id());
-        rinfo->set_target_object(isolate->heap(), new_target,
-                                 UPDATE_WRITE_BARRIER, SKIP_ICACHE_FLUSH);
+        Tagged<Object> object = rinfo->target_object(cage_base);
+        if (!IsCode(object, cage_base)) continue;
+        Tagged<Code> target = Code::cast(object);
+        if (!target->is_builtin()) continue;
+        Tagged<Code> new_target = builtins->code(target->builtin_id());
+        rinfo->set_target_object(istream, new_target, UPDATE_WRITE_BARRIER,
+                                 SKIP_ICACHE_FLUSH);
       }
       flush_icache = true;
     }
     if (flush_icache) {
-      FlushInstructionCache(code.raw_instruction_start(),
-                            code.raw_instruction_size());
+      FlushInstructionCache(code->instruction_start(),
+                            code->instruction_size());
     }
   }
 }
 
 namespace {
 
-Code GenerateBytecodeHandler(Isolate* isolate, Builtin builtin,
-                             interpreter::OperandScale operand_scale,
-                             interpreter::Bytecode bytecode) {
+V8_NOINLINE Tagged<Code> GenerateBytecodeHandler(
+    Isolate* isolate, Builtin builtin, interpreter::OperandScale operand_scale,
+    interpreter::Bytecode bytecode) {
   DCHECK(interpreter::Bytecodes::BytecodeHasHandler(bytecode, operand_scale));
   Handle<Code> code = interpreter::GenerateBytecodeHandler(
       isolate, Builtins::name(builtin), bytecode, operand_scale, builtin,
@@ -294,13 +310,18 @@ void SetupIsolateDelegate::SetupBuiltinsInternal(Isolate* isolate) {
   Builtins* builtins = isolate->builtins();
   DCHECK(!builtins->initialized_);
 
+  if (v8_flags.dump_builtins_hashes_to_file) {
+    // Create an empty file.
+    std::ofstream(v8_flags.dump_builtins_hashes_to_file, std::ios_base::trunc);
+  }
+
   PopulateWithPlaceholders(isolate);
 
   // Create a scope for the handles in the builtins.
   HandleScope scope(isolate);
 
   int index = 0;
-  Code code;
+  Tagged<Code> code;
 #define BUILD_CPP(Name)                                      \
   code = BuildAdaptor(isolate, Builtin::k##Name,             \
                       FUNCTION_ADDR(Builtin_##Name), #Name); \
@@ -363,12 +384,6 @@ void SetupIsolateDelegate::SetupBuiltinsInternal(Isolate* isolate) {
 
   ReplacePlaceholders(isolate);
 
-#define SET_PROMISE_REJECTION_PREDICTION(Name) \
-  builtins->code(Builtin::k##Name).set_is_promise_rejection(true);
-
-  BUILTIN_PROMISE_REJECTION_PREDICTION_LIST(SET_PROMISE_REJECTION_PREDICTION)
-#undef SET_PROMISE_REJECTION_PREDICTION
-
   builtins->MarkInitialized();
 }
 
