@@ -50,7 +50,7 @@ extern const float16 kFP16DefaultNaN;
 }  // end of extern "C"
 #endif
 
-unsigned CalcLSDataSize(LoadStoreOp op);
+unsigned CalcLSDataSizeLog2(LoadStoreOp op);
 unsigned CalcLSPairDataSize(LoadStorePairOp op);
 
 enum ImmBranchType {
@@ -144,7 +144,7 @@ class Instruction {
   double ImmNEONFP64() const;
 
   unsigned SizeLS() const {
-    return CalcLSDataSize(static_cast<LoadStoreOp>(Mask(LoadStoreMask)));
+    return CalcLSDataSizeLog2(static_cast<LoadStoreOp>(Mask(LoadStoreMask)));
   }
 
   unsigned SizeLSPair() const {
@@ -187,7 +187,7 @@ class Instruction {
     uint32_t bit5_to_0 = bits & 0x3f;
     uint32_t result = (bit7 << 31) | ((32 - bit6) << 25) | (bit5_to_0 << 19);
 
-    return bit_cast<float>(result);
+    return base::bit_cast<float>(result);
   }
 
   static double Imm8ToFP64(uint32_t imm8) {
@@ -201,7 +201,7 @@ class Instruction {
     uint64_t bit5_to_0 = bits & 0x3f;
     uint64_t result = (bit7 << 63) | ((256 - bit6) << 54) | (bit5_to_0 << 48);
 
-    return bit_cast<double>(result);
+    return base::bit_cast<double>(result);
   }
 
   bool IsLdrLiteral() const {
@@ -308,7 +308,7 @@ class Instruction {
     }
   }
 
-  static int ImmBranchRangeBitwidth(ImmBranchType branch_type) {
+  static constexpr int ImmBranchRangeBitwidth(ImmBranchType branch_type) {
     switch (branch_type) {
       case UncondBranchType:
         return ImmUncondBranch_width;
@@ -324,7 +324,7 @@ class Instruction {
   }
 
   // The range of the branch instruction, expressed as 'instr +- range'.
-  static int32_t ImmBranchRange(ImmBranchType branch_type) {
+  static constexpr int32_t ImmBranchRange(ImmBranchType branch_type) {
     return (1 << (ImmBranchRangeBitwidth(branch_type) + kInstrSizeLog2)) / 2 -
            kInstrSize;
   }
@@ -416,7 +416,12 @@ class Instruction {
 
   // Check if the offset is in range of a given branch type. The offset is
   // a byte offset, unscaled.
-  static bool IsValidImmPCOffset(ImmBranchType branch_type, ptrdiff_t offset);
+  static constexpr bool IsValidImmPCOffset(ImmBranchType branch_type,
+                                           ptrdiff_t offset) {
+    DCHECK_EQ(offset % kInstrSize, 0);
+    return is_intn(offset / kInstrSize, ImmBranchRangeBitwidth(branch_type));
+  }
+
   bool IsTargetInImmPCOffsetRange(Instruction* target);
   // Patch a PC-relative offset to refer to 'target'. 'this' may be a branch or
   // a PC-relative addressing instruction.
@@ -460,7 +465,38 @@ class Instruction {
   static const int ImmPCRelRangeBitwidth = 21;
   static bool IsValidPCRelOffset(ptrdiff_t offset) { return is_int21(offset); }
   void SetPCRelImmTarget(const AssemblerOptions& options, Instruction* target);
-  void SetBranchImmTarget(Instruction* target);
+
+  template <ImmBranchType branch_type>
+  void SetBranchImmTarget(Instruction* target) {
+    DCHECK(IsAligned(DistanceTo(target), kInstrSize));
+    DCHECK(IsValidImmPCOffset(branch_type, DistanceTo(target)));
+    int offset = static_cast<int>(DistanceTo(target) >> kInstrSizeLog2);
+    Instr branch_imm = 0;
+    uint32_t imm_mask = 0;
+    switch (branch_type) {
+      case CondBranchType:
+      case CompareBranchType:
+        static_assert(ImmCondBranch_mask == ImmCmpBranch_mask);
+        static_assert(ImmCondBranch_offset == ImmCmpBranch_offset);
+        // We use a checked truncation here to catch certain bugs where we fail
+        // to check whether a veneer is required. See e.g. crbug.com/1485829.
+        branch_imm = checked_truncate_to_int19(offset) << ImmCondBranch_offset;
+        imm_mask = ImmCondBranch_mask;
+        break;
+      case UncondBranchType:
+        branch_imm = checked_truncate_to_int26(offset)
+                     << ImmUncondBranch_offset;
+        imm_mask = ImmUncondBranch_mask;
+        break;
+      case TestBranchType:
+        branch_imm = checked_truncate_to_int14(offset) << ImmTestBranch_offset;
+        imm_mask = ImmTestBranch_mask;
+        break;
+      default:
+        UNREACHABLE();
+    }
+    SetInstructionBits(Mask(~imm_mask) | branch_imm);
+  }
 };
 
 // Simulator/Debugger debug instructions ---------------------------------------
@@ -475,6 +511,10 @@ const Instr kImmExceptionIsRedirectedCall = 0xca11;
 // should not be reachable, such as in data encoded inline in the instructions.
 const Instr kImmExceptionIsUnreachable = 0xdebf;
 
+// Indicate that the stack is being switched, so the simulator must update its
+// stack limit. The new stack limit is passed in x16.
+const Instr kImmExceptionIsSwitchStackLimit = 0x5915;
+
 // A pseudo 'printf' instruction. The arguments will be passed to the platform
 // printf method.
 const Instr kImmExceptionIsPrintf = 0xdeb1;
@@ -619,7 +659,8 @@ class NEONFormatDecoder {
   // substitution mode.
   const char* Substitute(const char* string, SubstitutionMode mode0 = kFormat,
                          SubstitutionMode mode1 = kFormat,
-                         SubstitutionMode mode2 = kFormat);
+                         SubstitutionMode mode2 = kFormat,
+                         SubstitutionMode mode3 = kFormat);
 
   // Append a "2" to a mnemonic string based of the state of the Q bit.
   const char* Mnemonic(const char* mnemonic);
@@ -738,7 +779,7 @@ class NEONFormatDecoder {
   uint8_t PickBits(const uint8_t bits[]);
 
   Instr instrbits_;
-  const NEONFormatMap* formats_[3];
+  const NEONFormatMap* formats_[4];
   char form_buffer_[64];
   char mne_buffer_[16];
 };
