@@ -41,7 +41,7 @@ class CodeDesc {
 #endif
 
  public:
-  byte* buffer = nullptr;
+  uint8_t* buffer = nullptr;
   int buffer_size = 0;
 
   // The instruction area contains executable code plus inlined metadata.
@@ -63,7 +63,7 @@ class CodeDesc {
   int code_comments_size = 0;
 
   // TODO(jgruber,v8:11036): Remove these functions once CodeDesc fields have
-  // been made consistent with Code layout.
+  // been made consistent with InstructionStream layout.
   int body_size() const { return instr_size + unwinding_info_size; }
   int instruction_size() const { return safepoint_table_offset; }
   int metadata_size() const { return body_size() - instruction_size(); }
@@ -88,7 +88,7 @@ class CodeDesc {
 
   // Unwinding information.
 
-  byte* unwinding_info = nullptr;
+  uint8_t* unwinding_info = nullptr;
   int unwinding_info_size = 0;
   int unwinding_info_offset_relative() const {
     // TODO(jgruber,v8:11036): Remove this function once unwinding_info setup
