@@ -79,6 +79,21 @@ constexpr Register KeyedLoadWithVectorDescriptor::VectorRegister() {
   return no_reg;
 }
 
+// static
+constexpr Register EnumeratedKeyedLoadBaselineDescriptor::EnumIndexRegister() {
+  return ecx;
+}
+
+// static
+constexpr Register EnumeratedKeyedLoadBaselineDescriptor::CacheTypeRegister() {
+  return no_reg;
+}
+
+// static
+constexpr Register EnumeratedKeyedLoadBaselineDescriptor::SlotRegister() {
+  return no_reg;
+}
+
 // static
 constexpr Register KeyedHasICBaselineDescriptor::ReceiverRegister() {
   return kInterpreterAccumulatorRegister;
@@ -191,6 +206,14 @@ constexpr auto CallFunctionTemplateDescriptor::registers() {
   return RegisterArray(edx, ecx);
 }
 
+// static
+constexpr auto CallFunctionTemplateGenericDescriptor::registers() {
+  // edx: the function template info
+  // ecx: number of arguments (on the stack)
+  // edi: topmost script-having context
+  return RegisterArray(edx, ecx, edi);
+}
+
 // static
 constexpr auto CallWithSpreadDescriptor::registers() {
   // eax : number of arguments (on the stack)
@@ -303,7 +326,13 @@ CallApiCallbackGenericDescriptor::ActualArgumentsCountRegister() {
   return ecx;
 }
 // static
-constexpr Register CallApiCallbackGenericDescriptor::CallHandlerInfoRegister() {
+constexpr Register
+CallApiCallbackGenericDescriptor::TopmostScriptHavingContextRegister() {
+  return eax;
+}
+// static
+constexpr Register
+CallApiCallbackGenericDescriptor::FunctionTemplateInfoRegister() {
   return edx;
 }
 // static
@@ -331,6 +360,12 @@ constexpr auto InterpreterPushArgsThenConstructDescriptor::registers() {
                        ecx);  // address of first argument
 }
 
+// static
+constexpr auto ConstructForwardAllArgsDescriptor::registers() {
+  return RegisterArray(edi,   // the constructor
+                       edx);  // the new target
+}
+
 // static
 constexpr auto ResumeGeneratorDescriptor::registers() {
   return RegisterArray(eax,   // the value to pass to the generator
