@@ -20,19 +20,21 @@ PtrComprCageBase::PtrComprCageBase(const LocalIsolate* isolate)
     : address_(isolate->cage_base()) {}
 
 //
-// V8HeapCompressionScheme
+// V8HeapCompressionSchemeImpl
 //
 
 constexpr Address kPtrComprCageBaseMask = ~(kPtrComprCageBaseAlignment - 1);
 
 // static
-Address V8HeapCompressionScheme::GetPtrComprCageBaseAddress(
+template <typename Cage>
+Address V8HeapCompressionSchemeImpl<Cage>::GetPtrComprCageBaseAddress(
     Address on_heap_addr) {
   return RoundDown<kPtrComprCageBaseAlignment>(on_heap_addr);
 }
 
 // static
-Address V8HeapCompressionScheme::GetPtrComprCageBaseAddress(
+template <typename Cage>
+Address V8HeapCompressionSchemeImpl<Cage>::GetPtrComprCageBaseAddress(
     PtrComprCageBase cage_base) {
   Address base = cage_base.address();
   V8_ASSUME((base & kPtrComprCageBaseMask) == base);
@@ -42,21 +44,25 @@ Address V8HeapCompressionScheme::GetPtrComprCageBaseAddress(
 }
 
 // static
-void V8HeapCompressionScheme::InitBase(Address base) {
+template <typename Cage>
+void V8HeapCompressionSchemeImpl<Cage>::InitBase(Address base) {
   CHECK_EQ(base, GetPtrComprCageBaseAddress(base));
-#if defined(USING_V8_SHARED) && defined(V8_COMPRESS_POINTERS_IN_ISOLATE_CAGE)
-  set_base_non_inlined(base);
+#if defined(USING_V8_SHARED_PRIVATE) && \
+    defined(V8_COMPRESS_POINTERS_IN_ISOLATE_CAGE)
+  Cage::set_base_non_inlined(base);
 #else
-  base_ = base;
+  Cage::base_ = base;
 #endif
 }
 
 // static
-V8_CONST Address V8HeapCompressionScheme::base() {
-#if defined(USING_V8_SHARED) && defined(V8_COMPRESS_POINTERS_IN_ISOLATE_CAGE)
-  Address base = base_non_inlined();
+template <typename Cage>
+Address V8HeapCompressionSchemeImpl<Cage>::base() {
+#if defined(USING_V8_SHARED_PRIVATE) && \
+    defined(V8_COMPRESS_POINTERS_IN_ISOLATE_CAGE)
+  Address base = Cage::base_non_inlined();
 #else
-  Address base = base_;
+  Address base = Cage::base_;
 #endif
   // V8_ASSUME_ALIGNED is often not preserved across ptr-to-int casts (i.e. when
   // casting to an Address). To increase our chances we additionally encode the
@@ -67,35 +73,40 @@ V8_CONST Address V8HeapCompressionScheme::base() {
 }
 
 // static
-Tagged_t V8HeapCompressionScheme::CompressObject(Address tagged) {
+template <typename Cage>
+Tagged_t V8HeapCompressionSchemeImpl<Cage>::CompressObject(Address tagged) {
   // This is used to help clang produce better code. Values which could be
   // invalid pointers need to be compressed with CompressAny.
 #ifdef V8_COMPRESS_POINTERS_IN_SHARED_CAGE
   V8_ASSUME((tagged & kPtrComprCageBaseMask) == base() || HAS_SMI_TAG(tagged));
 #endif
-  return static_cast<Tagged_t>(static_cast<uint32_t>(tagged));
+  return static_cast<Tagged_t>(tagged);
 }
 
 // static
-Tagged_t V8HeapCompressionScheme::CompressAny(Address tagged) {
-  return static_cast<Tagged_t>(static_cast<uint32_t>(tagged));
+template <typename Cage>
+Tagged_t V8HeapCompressionSchemeImpl<Cage>::CompressAny(Address tagged) {
+  return static_cast<Tagged_t>(tagged);
 }
 
 // static
-Address V8HeapCompressionScheme::DecompressTaggedSigned(Tagged_t raw_value) {
+template <typename Cage>
+Address V8HeapCompressionSchemeImpl<Cage>::DecompressTaggedSigned(
+    Tagged_t raw_value) {
   // For runtime code the upper 32-bits of the Smi value do not matter.
   return static_cast<Address>(raw_value);
 }
 
 // static
+template <typename Cage>
 template <typename TOnHeapAddress>
-Address V8HeapCompressionScheme::DecompressTagged(TOnHeapAddress on_heap_addr,
-                                                  Tagged_t raw_value) {
+Address V8HeapCompressionSchemeImpl<Cage>::DecompressTagged(
+    TOnHeapAddress on_heap_addr, Tagged_t raw_value) {
 #ifdef V8_COMPRESS_POINTERS
   Address cage_base = base();
 #ifdef V8_COMPRESS_POINTERS_IN_ISOLATE_CAGE
   DCHECK_WITH_MSG(cage_base != kNullAddress,
-                  "V8HeapCompressionScheme::base is not initialized for "
+                  "V8HeapCompressionSchemeImpl::base is not initialized for "
                   "current thread");
 #endif  // V8_COMPRESS_POINTERS_IN_ISOLATE_CAGE
 #else
@@ -107,19 +118,22 @@ Address V8HeapCompressionScheme::DecompressTagged(TOnHeapAddress on_heap_addr,
 }
 
 // static
+template <typename Cage>
 template <typename ProcessPointerCallback>
-void V8HeapCompressionScheme::ProcessIntermediatePointers(
+void V8HeapCompressionSchemeImpl<Cage>::ProcessIntermediatePointers(
     PtrComprCageBase cage_base, Address raw_value,
     ProcessPointerCallback callback) {
   // If pointer compression is enabled, we may have random compressed pointers
   // on the stack that may be used for subsequent operations.
   // Extract, decompress and trace both halfwords.
-  Address decompressed_low = V8HeapCompressionScheme::DecompressTagged(
-      cage_base, static_cast<Tagged_t>(raw_value));
+  Address decompressed_low =
+      V8HeapCompressionSchemeImpl<Cage>::DecompressTagged(
+          cage_base, static_cast<Tagged_t>(raw_value));
   callback(decompressed_low);
-  Address decompressed_high = V8HeapCompressionScheme::DecompressTagged(
-      cage_base,
-      static_cast<Tagged_t>(raw_value >> (sizeof(Tagged_t) * CHAR_BIT)));
+  Address decompressed_high =
+      V8HeapCompressionSchemeImpl<Cage>::DecompressTagged(
+          cage_base,
+          static_cast<Tagged_t>(raw_value >> (sizeof(Tagged_t) * CHAR_BIT)));
   callback(decompressed_high);
 }
 
@@ -148,7 +162,8 @@ Address ExternalCodeCompressionScheme::GetPtrComprCageBaseAddress(
 // static
 void ExternalCodeCompressionScheme::InitBase(Address base) {
   CHECK_EQ(base, PrepareCageBaseAddress(base));
-#if defined(USING_V8_SHARED) && defined(V8_COMPRESS_POINTERS_IN_ISOLATE_CAGE)
+#if defined(USING_V8_SHARED_PRIVATE) && \
+    defined(V8_COMPRESS_POINTERS_IN_ISOLATE_CAGE)
   set_base_non_inlined(base);
 #else
   base_ = base;
@@ -157,7 +172,8 @@ void ExternalCodeCompressionScheme::InitBase(Address base) {
 
 // static
 V8_CONST Address ExternalCodeCompressionScheme::base() {
-#if defined(USING_V8_SHARED) && defined(V8_COMPRESS_POINTERS_IN_ISOLATE_CAGE)
+#if defined(USING_V8_SHARED_PRIVATE) && \
+    defined(V8_COMPRESS_POINTERS_IN_ISOLATE_CAGE)
   Address base = base_non_inlined();
 #else
   Address base = base_;
@@ -174,15 +190,19 @@ V8_CONST Address ExternalCodeCompressionScheme::base() {
 Tagged_t ExternalCodeCompressionScheme::CompressObject(Address tagged) {
   // This is used to help clang produce better code. Values which could be
   // invalid pointers need to be compressed with CompressAny.
+  // The DCHECK generated by this V8_ASSUME is also very helpful during
+  // development when moving objects between pointer compression cages as it
+  // quickly identifies any places where we still store a compressed pointer
+  // slot with the wrong base.
 #ifdef V8_COMPRESS_POINTERS_IN_SHARED_CAGE
   V8_ASSUME((tagged & kPtrComprCageBaseMask) == base() || HAS_SMI_TAG(tagged));
 #endif
-  return static_cast<Tagged_t>(static_cast<uint32_t>(tagged));
+  return static_cast<Tagged_t>(tagged);
 }
 
 // static
 Tagged_t ExternalCodeCompressionScheme::CompressAny(Address tagged) {
-  return static_cast<Tagged_t>(static_cast<uint32_t>(tagged));
+  return static_cast<Tagged_t>(tagged);
 }
 
 // static
@@ -231,38 +251,47 @@ V8_INLINE PtrComprCageBase GetPtrComprCageBase() {
 #else
 
 //
-// V8HeapCompressionScheme
+// V8HeapCompressionSchemeImpl
 //
 
 // static
-Address V8HeapCompressionScheme::GetPtrComprCageBaseAddress(
+template <typename Cage>
+Address V8HeapCompressionSchemeImpl<Cage>::GetPtrComprCageBaseAddress(
     Address on_heap_addr) {
   UNREACHABLE();
 }
 
 // static
-Tagged_t V8HeapCompressionScheme::CompressObject(Address tagged) {
+template <typename Cage>
+Tagged_t V8HeapCompressionSchemeImpl<Cage>::CompressObject(Address tagged) {
   UNREACHABLE();
 }
 
 // static
-Tagged_t V8HeapCompressionScheme::CompressAny(Address tagged) { UNREACHABLE(); }
-
-// static
-Address V8HeapCompressionScheme::DecompressTaggedSigned(Tagged_t raw_value) {
+template <typename Cage>
+Tagged_t V8HeapCompressionSchemeImpl<Cage>::CompressAny(Address tagged) {
   UNREACHABLE();
 }
 
 // static
+template <typename Cage>
+Address V8HeapCompressionSchemeImpl<Cage>::DecompressTaggedSigned(
+    Tagged_t raw_value) {
+  UNREACHABLE();
+}
+
+// static
+template <typename Cage>
 template <typename TOnHeapAddress>
-Address V8HeapCompressionScheme::DecompressTagged(TOnHeapAddress on_heap_addr,
-                                                  Tagged_t raw_value) {
+Address V8HeapCompressionSchemeImpl<Cage>::DecompressTagged(
+    TOnHeapAddress on_heap_addr, Tagged_t raw_value) {
   UNREACHABLE();
 }
 
 // static
+template <typename Cage>
 template <typename ProcessPointerCallback>
-void V8HeapCompressionScheme::ProcessIntermediatePointers(
+void V8HeapCompressionSchemeImpl<Cage>::ProcessIntermediatePointers(
     PtrComprCageBase cage_base, Address raw_value,
     ProcessPointerCallback callback) {
   UNREACHABLE();
