@@ -54,14 +54,14 @@ enum AssessmentKind { Final, Pending };
 
 class Assessment : public ZoneObject {
  public:
+  Assessment(const Assessment&) = delete;
+  Assessment& operator=(const Assessment&) = delete;
+
   AssessmentKind kind() const { return kind_; }
 
  protected:
   explicit Assessment(AssessmentKind kind) : kind_(kind) {}
   AssessmentKind kind_;
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(Assessment);
 };
 
 // PendingAssessments are associated to operands coming from the multiple
@@ -80,6 +80,9 @@ class PendingAssessment final : public Assessment {
         operand_(operand),
         aliases_(zone) {}
 
+  PendingAssessment(const PendingAssessment&) = delete;
+  PendingAssessment& operator=(const PendingAssessment&) = delete;
+
   static const PendingAssessment* cast(const Assessment* assessment) {
     CHECK(assessment->kind() == Pending);
     return static_cast<const PendingAssessment*>(assessment);
@@ -99,8 +102,6 @@ class PendingAssessment final : public Assessment {
   const InstructionBlock* const origin_;
   InstructionOperand operand_;
   ZoneSet<int> aliases_;
-
-  DISALLOW_COPY_AND_ASSIGN(PendingAssessment);
 };
 
 // FinalAssessments are associated to operands that we know to be a certain
@@ -109,6 +110,8 @@ class FinalAssessment final : public Assessment {
  public:
   explicit FinalAssessment(int virtual_register)
       : Assessment(Final), virtual_register_(virtual_register) {}
+  FinalAssessment(const FinalAssessment&) = delete;
+  FinalAssessment& operator=(const FinalAssessment&) = delete;
 
   int virtual_register() const { return virtual_register_; }
   static const FinalAssessment* cast(const Assessment* assessment) {
@@ -118,8 +121,6 @@ class FinalAssessment final : public Assessment {
 
  private:
   int virtual_register_;
-
-  DISALLOW_COPY_AND_ASSIGN(FinalAssessment);
 };
 
 struct OperandAsKeyLess {
@@ -134,12 +135,17 @@ class BlockAssessments : public ZoneObject {
  public:
   using OperandMap = ZoneMap<InstructionOperand, Assessment*, OperandAsKeyLess>;
   using OperandSet = ZoneSet<InstructionOperand, OperandAsKeyLess>;
-  explicit BlockAssessments(Zone* zone, int spill_slot_delta)
+  explicit BlockAssessments(Zone* zone, int spill_slot_delta,
+                            const InstructionSequence* sequence)
       : map_(zone),
         map_for_moves_(zone),
         stale_ref_stack_slots_(zone),
         spill_slot_delta_(spill_slot_delta),
-        zone_(zone) {}
+        zone_(zone),
+        sequence_(sequence) {}
+  BlockAssessments(const BlockAssessments&) = delete;
+  BlockAssessments& operator=(const BlockAssessments&) = delete;
+
   void Drop(InstructionOperand operand) {
     map_.erase(operand);
     stale_ref_stack_slots_.erase(operand);
@@ -168,7 +174,8 @@ class BlockAssessments : public ZoneObject {
                                   other->stale_ref_stack_slots_.end());
   }
   void CheckReferenceMap(const ReferenceMap* reference_map);
-  bool IsStaleReferenceStackSlot(InstructionOperand op);
+  bool IsStaleReferenceStackSlot(InstructionOperand op,
+                                 base::Optional<int> vreg = base::nullopt);
 
   OperandMap& map() { return map_; }
   const OperandMap& map() const { return map_; }
@@ -185,11 +192,12 @@ class BlockAssessments : public ZoneObject {
  private:
   OperandMap map_;
   OperandMap map_for_moves_;
+  // TODOC(dmercadier): how do stack slots become stale exactly? What are the
+  // implications of a stack slot being stale?
   OperandSet stale_ref_stack_slots_;
   int spill_slot_delta_;
   Zone* zone_;
-
-  DISALLOW_COPY_AND_ASSIGN(BlockAssessments);
+  const InstructionSequence* sequence_;
 };
 
 class RegisterAllocatorVerifier final : public ZoneObject {
@@ -197,6 +205,9 @@ class RegisterAllocatorVerifier final : public ZoneObject {
   RegisterAllocatorVerifier(Zone* zone, const RegisterConfiguration* config,
                             const InstructionSequence* sequence,
                             const Frame* frame);
+  RegisterAllocatorVerifier(const RegisterAllocatorVerifier&) = delete;
+  RegisterAllocatorVerifier& operator=(const RegisterAllocatorVerifier&) =
+      delete;
 
   void VerifyAssignment(const char* caller_info);
   void VerifyGapMoves();
@@ -214,7 +225,7 @@ class RegisterAllocatorVerifier final : public ZoneObject {
     kRegisterOrSlot,
     kRegisterOrSlotFP,
     kRegisterOrSlotOrConstant,
-    kSameAsFirst,
+    kSameAsInput,
     kRegisterAndSlot
   };
 
@@ -290,8 +301,6 @@ class RegisterAllocatorVerifier final : public ZoneObject {
   int spill_slot_delta_;
   // TODO(chromium:725559): remove after we understand this bug's root cause.
   const char* caller_info_ = nullptr;
-
-  DISALLOW_COPY_AND_ASSIGN(RegisterAllocatorVerifier);
 };
 
 }  // namespace compiler
