@@ -14,9 +14,9 @@
 #include "src/objects/map-inl.h"
 
 #ifdef DEBUG
-#define TRACE(...)                                    \
-  do {                                                \
-    if (FLAG_trace_turbo_escape) PrintF(__VA_ARGS__); \
+#define TRACE(...)                                        \
+  do {                                                    \
+    if (v8_flags.trace_turbo_escape) PrintF(__VA_ARGS__); \
   } while (false)
 #else
 #define TRACE(...)
@@ -298,10 +298,11 @@ class EscapeAnalysisTracker : public ZoneObject {
 
  private:
   friend class EscapeAnalysisResult;
-  static const size_t kMaxTrackedObjects = 100;
+  static constexpr int kTrackingBudget = 600;
 
   VirtualObject* NewVirtualObject(int size) {
-    if (next_object_id_ >= kMaxTrackedObjects) return nullptr;
+    if (number_of_tracked_bytes_ + size >= kTrackingBudget) return nullptr;
+    number_of_tracked_bytes_ += size;
     return zone_->New<VirtualObject>(&variable_states_, next_object_id_++,
                                      size);
   }
@@ -311,6 +312,7 @@ class EscapeAnalysisTracker : public ZoneObject {
   ZoneUnorderedMap<Node*, bool> framestate_might_lazy_deopt_;
   VariableTracker variable_states_;
   VirtualObject::Id next_object_id_ = 0;
+  int number_of_tracked_bytes_ = 0;
   JSGraph* const jsgraph_;
   Zone* const zone_;
 };
@@ -568,13 +570,17 @@ Maybe<int> OffsetOfElementsAccess(const Operator* op, Node* index_node) {
 }
 
 Node* LowerCompareMapsWithoutLoad(Node* checked_map,
-                                  ZoneHandleSet<Map> const& checked_against,
+                                  ZoneRefSet<Map> const& checked_against,
                                   JSGraph* jsgraph) {
   Node* true_node = jsgraph->TrueConstant();
   Node* false_node = jsgraph->FalseConstant();
   Node* replacement = false_node;
-  for (Handle<Map> map : checked_against) {
-    Node* map_node = jsgraph->HeapConstant(map);
+  for (MapRef map : checked_against) {
+    // We are using HeapConstantMaybeHole here instead of HeapConstantNoHole
+    // as we cannot do the CHECK(object is hole) here as the compile thread is
+    // parked during EscapeAnalysis for performance reasons, see pipeline.cc.
+    // TODO(cffsmith): do manual checking against hole values here.
+    Node* map_node = jsgraph->HeapConstantMaybeHole(map.object());
     // We cannot create a HeapConstant type here as we are off-thread.
     NodeProperties::SetType(map_node, Type::Internal());
     Node* comparison = jsgraph->graph()->NewNode(
@@ -616,8 +622,11 @@ void ReduceNode(const Operator* op, EscapeAnalysisTracker::Scope* current,
       Node* value = current->ValueInput(1);
       const VirtualObject* vobject = current->GetVirtualObject(object);
       Variable var;
+      // BoundedSize fields cannot currently be materialized by the deoptimizer,
+      // so we must not dematerialze them.
       if (vobject && !vobject->HasEscaped() &&
-          vobject->FieldAt(OffsetOfFieldAccess(op)).To(&var)) {
+          vobject->FieldAt(OffsetOfFieldAccess(op)).To(&var) &&
+          !FieldAccessOf(op).is_bounded_size_access) {
         current->Set(var, value);
         current->MarkForDeletion();
       } else {
@@ -776,7 +785,7 @@ void ReduceNode(const Operator* op, EscapeAnalysisTracker::Scope* current,
           Type const map_type = NodeProperties::GetType(map);
           if (map_type.IsHeapConstant() &&
               params.maps().contains(
-                  map_type.AsHeapConstant()->Ref().AsMap().object())) {
+                  map_type.AsHeapConstant()->Ref().AsMap())) {
             current->MarkForDeletion();
             break;
           }
