@@ -8,10 +8,11 @@ namespace v8 {
 namespace internal {
 namespace compiler {
 
-Frame::Frame(int fixed_frame_size_in_slots)
+Frame::Frame(int fixed_frame_size_in_slots, Zone* zone)
     : fixed_slot_count_(fixed_frame_size_in_slots),
       allocated_registers_(nullptr),
-      allocated_double_registers_(nullptr) {
+      allocated_double_registers_(nullptr),
+      zone_(zone) {
   slot_allocator_.AllocateUnaligned(fixed_frame_size_in_slots);
 }
 
@@ -45,8 +46,12 @@ void FrameAccessState::MarkHasFrame(bool state) {
   SetFrameAccessToDefault();
 }
 
+void FrameAccessState::SetFPRelativeOnly(bool state) {
+  fp_relative_only_ = state;
+}
+
 void FrameAccessState::SetFrameAccessToDefault() {
-  if (has_frame() && !FLAG_turbo_sp_frame_access) {
+  if (has_frame()) {
     SetFrameAccessToFP();
   } else {
     SetFrameAccessToSP();
@@ -60,6 +65,7 @@ FrameOffset FrameAccessState::GetFrameOffset(int spill_slot) const {
   } else {
     // No frame. Retrieve all parameters relative to stack pointer.
     int sp_offset = frame_offset + GetSPToFPOffset();
+    DCHECK_GE(sp_offset, 0);
     return FrameOffset::FromStackPointer(sp_offset);
   }
 }
