@@ -5,26 +5,24 @@
 #ifndef V8_COMPILER_GRAPH_ASSEMBLER_H_
 #define V8_COMPILER_GRAPH_ASSEMBLER_H_
 
+#include <optional>
 #include <type_traits>
 
 #include "src/base/small-vector.h"
 #include "src/codegen/tnode.h"
+#include "src/common/globals.h"
 #include "src/compiler/feedback-source.h"
 #include "src/compiler/js-graph.h"
 #include "src/compiler/node.h"
 #include "src/compiler/simplified-operator.h"
+#include "src/objects/hole.h"
+#include "src/objects/oddball.h"
 
 namespace v8 {
 namespace internal {
 
 class JSGraph;
 class Graph;
-class Oddball;
-
-// TODO(jgruber): Currently this is too permissive, but at least it lets us
-// document which functions expect JS booleans. If a real Boolean type becomes
-// possible in the future, use that instead.
-using Boolean = Oddball;
 
 namespace compiler {
 
@@ -62,54 +60,57 @@ class Reducer;
   V(Word32ReverseBytes)                  \
   V(Word64ReverseBytes)
 
-#define PURE_ASSEMBLER_MACH_BINOP_LIST(V) \
-  V(Float64Add)                           \
-  V(Float64Div)                           \
-  V(Float64Equal)                         \
-  V(Float64InsertHighWord32)              \
-  V(Float64InsertLowWord32)               \
-  V(Float64LessThan)                      \
-  V(Float64LessThanOrEqual)               \
-  V(Float64Mod)                           \
-  V(Float64Sub)                           \
-  V(Int32Add)                             \
-  V(Int32LessThan)                        \
-  V(Int32LessThanOrEqual)                 \
-  V(Int32Mul)                             \
-  V(Int32Sub)                             \
-  V(Int64Sub)                             \
-  V(IntAdd)                               \
-  V(IntLessThan)                          \
-  V(IntMul)                               \
-  V(IntSub)                               \
-  V(Uint32LessThan)                       \
-  V(Uint32LessThanOrEqual)                \
-  V(Uint64LessThan)                       \
-  V(Uint64LessThanOrEqual)                \
-  V(UintLessThan)                         \
-  V(Word32And)                            \
-  V(Word32Equal)                          \
-  V(Word32Or)                             \
-  V(Word32Sar)                            \
-  V(Word32SarShiftOutZeros)               \
-  V(Word32Shl)                            \
-  V(Word32Shr)                            \
-  V(Word32Xor)                            \
-  V(Word64And)                            \
-  V(Word64Equal)                          \
-  V(Word64Or)                             \
-  V(Word64Sar)                            \
-  V(Word64SarShiftOutZeros)               \
-  V(Word64Shl)                            \
-  V(Word64Shr)                            \
-  V(Word64Xor)                            \
-  V(WordAnd)                              \
-  V(WordEqual)                            \
-  V(WordOr)                               \
-  V(WordSar)                              \
-  V(WordSarShiftOutZeros)                 \
-  V(WordShl)                              \
-  V(WordShr)                              \
+#define PURE_ASSEMBLER_MACH_BINOP_LIST(V, T)        \
+  V(Float64Add)                                     \
+  V(Float64Div)                                     \
+  V(Float64Equal)                                   \
+  V(Float64InsertHighWord32)                        \
+  V(Float64InsertLowWord32)                         \
+  V(Float64LessThan)                                \
+  V(Float64LessThanOrEqual)                         \
+  V(Float64Max)                                     \
+  V(Float64Min)                                     \
+  V(Float64Mod)                                     \
+  V(Float64Sub)                                     \
+  V(Int32Add)                                       \
+  V(Int32LessThan)                                  \
+  T(Int32LessThanOrEqual, BoolT, Int32T, Int32T)    \
+  V(Int32Mul)                                       \
+  V(Int32Sub)                                       \
+  V(Int64Add)                                       \
+  V(Int64Sub)                                       \
+  V(IntAdd)                                         \
+  V(IntLessThan)                                    \
+  V(IntMul)                                         \
+  V(IntSub)                                         \
+  T(Uint32LessThan, BoolT, Uint32T, Uint32T)        \
+  T(Uint32LessThanOrEqual, BoolT, Uint32T, Uint32T) \
+  T(Uint64LessThan, BoolT, Uint64T, Uint64T)        \
+  T(Uint64LessThanOrEqual, BoolT, Uint64T, Uint64T) \
+  V(UintLessThan)                                   \
+  T(Word32And, Word32T, Word32T, Word32T)           \
+  T(Word32Equal, BoolT, Word32T, Word32T)           \
+  T(Word32Or, Word32T, Word32T, Word32T)            \
+  V(Word32Sar)                                      \
+  V(Word32SarShiftOutZeros)                         \
+  V(Word32Shl)                                      \
+  T(Word32Shr, Word32T, Word32T, Word32T)           \
+  V(Word32Xor)                                      \
+  V(Word64And)                                      \
+  V(Word64Equal)                                    \
+  V(Word64Or)                                       \
+  V(Word64Sar)                                      \
+  V(Word64SarShiftOutZeros)                         \
+  V(Word64Shl)                                      \
+  V(Word64Shr)                                      \
+  V(Word64Xor)                                      \
+  V(WordAnd)                                        \
+  V(WordEqual)                                      \
+  V(WordOr)                                         \
+  V(WordSar)                                        \
+  V(WordSarShiftOutZeros)                           \
+  V(WordShl)                                        \
+  V(WordShr)                                        \
   V(WordXor)
 
 #define CHECKED_ASSEMBLER_MACH_BINOP_LIST(V) \
@@ -118,7 +119,9 @@ class Reducer;
   V(Int32Div)                                \
   V(Int32Mod)                                \
   V(Int32MulWithOverflow)                    \
+  V(Int64MulWithOverflow)                    \
   V(Int32SubWithOverflow)                    \
+  V(Int64SubWithOverflow)                    \
   V(Int64Div)                                \
   V(Int64Mod)                                \
   V(Uint32Div)                               \
@@ -126,29 +129,30 @@ class Reducer;
   V(Uint64Div)                               \
   V(Uint64Mod)
 
-#define JSGRAPH_SINGLETON_CONSTANT_LIST(V)      \
-  V(AllocateInOldGenerationStub, Code)          \
-  V(AllocateInYoungGenerationStub, Code)        \
-  V(AllocateRegularInOldGenerationStub, Code)   \
-  V(AllocateRegularInYoungGenerationStub, Code) \
-  V(BigIntMap, Map)                             \
-  V(BooleanMap, Map)                            \
-  V(EmptyString, String)                        \
-  V(False, Boolean)                             \
-  V(FixedArrayMap, Map)                         \
-  V(FixedDoubleArrayMap, Map)                   \
-  V(WeakFixedArrayMap, Map)                     \
-  V(HeapNumberMap, Map)                         \
-  V(MinusOne, Number)                           \
-  V(NaN, Number)                                \
-  V(NoContext, Object)                          \
-  V(Null, Oddball)                              \
-  V(One, Number)                                \
-  V(TheHole, Oddball)                           \
-  V(ToNumberBuiltin, Code)                      \
-  V(PlainPrimitiveToNumberBuiltin, Code)        \
-  V(True, Boolean)                              \
-  V(Undefined, Oddball)                         \
+#define JSGRAPH_SINGLETON_CONSTANT_LIST(V)                         \
+  V(AllocateInOldGenerationStub, InstructionStream)                \
+  V(AllocateInYoungGenerationStub, InstructionStream)              \
+  IF_WASM(V, WasmAllocateInYoungGenerationStub, InstructionStream) \
+  IF_WASM(V, WasmAllocateInOldGenerationStub, InstructionStream)   \
+  V(BigIntMap, Map)                                                \
+  V(BooleanMap, Map)                                               \
+  V(EmptyString, String)                                           \
+  V(ExternalObjectMap, Map)                                        \
+  V(False, Boolean)                                                \
+  V(FixedArrayMap, Map)                                            \
+  V(FixedDoubleArrayMap, Map)                                      \
+  V(WeakFixedArrayMap, Map)                                        \
+  V(HeapNumberMap, Map)                                            \
+  V(MinusOne, Number)                                              \
+  V(NaN, Number)                                                   \
+  V(NoContext, Object)                                             \
+  V(Null, Null)                                                    \
+  V(One, Number)                                                   \
+  V(TheHole, Hole)                                                 \
+  V(ToNumberBuiltin, InstructionStream)                            \
+  V(PlainPrimitiveToNumberBuiltin, InstructionStream)              \
+  V(True, Boolean)                                                 \
+  V(Undefined, Undefined)                                          \
   V(Zero, Number)
 
 class GraphAssembler;
@@ -276,9 +280,11 @@ class V8_EXPORT_PRIVATE GraphAssembler {
   // will maintain the schedule as it updates blocks.
   GraphAssembler(
       MachineGraph* jsgraph, Zone* zone,
+      BranchSemantics default_branch_semantics,
       base::Optional<NodeChangedCallback> node_changed_callback = base::nullopt,
       bool mark_loop_exits = false);
   virtual ~GraphAssembler();
+  virtual SimplifiedOperatorBuilder* simplified() { UNREACHABLE(); }
 
   void Reset();
   void InitializeEffectControl(Node* effect, Node* control);
@@ -318,19 +324,24 @@ class V8_EXPORT_PRIVATE GraphAssembler {
 
   // Value creation.
   Node* IntPtrConstant(intptr_t value);
-  Node* UintPtrConstant(uintptr_t value);
+  TNode<UintPtrT> UintPtrConstant(uintptr_t value);
   Node* Int32Constant(int32_t value);
-  Node* Uint32Constant(uint32_t value);
+  TNode<Uint32T> Uint32Constant(uint32_t value);
   Node* Int64Constant(int64_t value);
   Node* Uint64Constant(uint64_t value);
   Node* UniqueIntPtrConstant(intptr_t value);
   Node* Float64Constant(double value);
-  Node* Projection(int index, Node* value);
   Node* ExternalConstant(ExternalReference ref);
 
+  Node* Projection(int index, Node* value, Node* ctrl = nullptr);
+
   Node* Parameter(int index);
 
   Node* LoadFramePointer();
+#if V8_ENABLE_WEBASSEMBLY
+  Node* LoadStackPointer();
+  Node* SetStackPointer(Node* sp, wasm::FPRelativeScope fp_scope);
+#endif
 
   Node* LoadHeapNumberValue(Node* heap_number);
 
@@ -339,9 +350,19 @@ class V8_EXPORT_PRIVATE GraphAssembler {
 #undef PURE_UNOP_DECL
 
 #define BINOP_DECL(Name) Node* Name(Node* left, Node* right);
-  PURE_ASSEMBLER_MACH_BINOP_LIST(BINOP_DECL)
+#define BINOP_DECL_TNODE(Name, Result, Left, Right) \
+  TNode<Result> Name(SloppyTNode<Left> left, SloppyTNode<Right> right);
+  PURE_ASSEMBLER_MACH_BINOP_LIST(BINOP_DECL, BINOP_DECL_TNODE)
   CHECKED_ASSEMBLER_MACH_BINOP_LIST(BINOP_DECL)
 #undef BINOP_DECL
+#undef BINOP_DECL_TNODE
+  TNode<BoolT> UintPtrLessThan(TNode<UintPtrT> left, TNode<UintPtrT> right);
+  TNode<BoolT> UintPtrLessThanOrEqual(TNode<UintPtrT> left,
+                                      TNode<UintPtrT> right);
+  TNode<UintPtrT> UintPtrAdd(TNode<UintPtrT> left, TNode<UintPtrT> right);
+  TNode<UintPtrT> UintPtrSub(TNode<UintPtrT> left, TNode<UintPtrT> right);
+  TNode<UintPtrT> UintPtrDiv(TNode<UintPtrT> left, TNode<UintPtrT> right);
+  TNode<UintPtrT> ChangeUint32ToUintPtr(SloppyTNode<Uint32T> value);
 
 #ifdef V8_MAP_PACKING
   Node* PackMapWord(TNode<Map> map);
@@ -351,14 +372,7 @@ class V8_EXPORT_PRIVATE GraphAssembler {
 
   Node* DebugBreak();
 
-  // Unreachable nodes are similar to Goto in that they reset effect/control to
-  // nullptr and it's thus not possible to append other nodes without first
-  // binding a new label.
-  // The block_updater_successor label is a crutch to work around block updater
-  // weaknesses (see the related comment in ConnectUnreachableToEnd); if the
-  // block updater exists, we cannot connect unreachable to end, instead we
-  // must preserve the Goto pattern.
-  Node* Unreachable(GraphAssemblerLabel<0u>* block_updater_successor = nullptr);
+  Node* Unreachable();
   // This special variant doesn't connect the Unreachable node to end, and does
   // not reset current effect/control. Intended only for special use-cases like
   // lowering DeadValue.
@@ -381,9 +395,15 @@ class V8_EXPORT_PRIVATE GraphAssembler {
   Node* BitcastMaybeObjectToWord(Node* value);
 
   Node* TypeGuard(Type type, Node* value);
+  template <typename T>
+  TNode<T> TypeGuard(Type type, TNode<T> value) {
+    return TNode<T>::UncheckedCast(TypeGuard(type, static_cast<Node*>(value)));
+  }
   Node* Checkpoint(FrameState frame_state);
 
-  TNode<RawPtrT> StackSlot(int size, int alignment);
+  TNode<RawPtrT> StackSlot(int size, int alignment, bool is_tagged = false);
+
+  Node* AdaptLocalArgument(Node* argument);
 
   Node* Store(StoreRepresentation rep, Node* object, Node* offset, Node* value);
   Node* Store(StoreRepresentation rep, Node* object, int offset, Node* value);
@@ -397,9 +417,13 @@ class V8_EXPORT_PRIVATE GraphAssembler {
   Node* ProtectedStore(MachineRepresentation rep, Node* object, Node* offset,
                        Node* value);
   Node* ProtectedLoad(MachineType type, Node* object, Node* offset);
+  Node* LoadTrapOnNull(MachineType type, Node* object, Node* offset);
+  Node* StoreTrapOnNull(StoreRepresentation rep, Node* object, Node* offset,
+                        Node* value);
 
   Node* Retain(Node* buffer);
   Node* IntPtrAdd(Node* a, Node* b);
+  Node* IntPtrSub(Node* a, Node* b);
 
   Node* DeoptimizeIf(DeoptimizeReason reason, FeedbackSource const& feedback,
                      Node* condition, Node* frame_state);
@@ -439,6 +463,16 @@ class V8_EXPORT_PRIVATE GraphAssembler {
                       detail::GraphAssemblerLabelForVars<Vars...>* if_true,
                       detail::GraphAssemblerLabelForVars<Vars...>* if_false,
                       BranchHint hint, Vars...);
+  template <typename... Vars>
+  void MachineBranch(TNode<Word32T> condition,
+                     GraphAssemblerLabel<sizeof...(Vars)>* if_true,
+                     GraphAssemblerLabel<sizeof...(Vars)>* if_false,
+                     BranchHint hint, Vars...);
+  template <typename... Vars>
+  void JSBranch(TNode<Boolean> condition,
+                GraphAssemblerLabel<sizeof...(Vars)>* if_true,
+                GraphAssemblerLabel<sizeof...(Vars)>* if_false, BranchHint hint,
+                Vars...);
 
   // Control helpers.
 
@@ -510,7 +544,11 @@ class V8_EXPORT_PRIVATE GraphAssembler {
   Control control() const { return Control(control_); }
   Effect effect() const { return Effect(effect_); }
 
+  Node* start() const { return graph()->start(); }
+
  protected:
+  constexpr bool Is64() const { return kSystemPointerSize == 8; }
+
   template <typename... Vars>
   void MergeState(detail::GraphAssemblerLabelForVars<Vars...>* label,
                   Vars... vars);
@@ -600,13 +638,14 @@ class V8_EXPORT_PRIVATE GraphAssembler {
   class BlockInlineReduction;
 
   template <typename... Vars>
-  void BranchImpl(Node* condition,
-                  detail::GraphAssemblerLabelForVars<Vars...>* if_true,
-                  detail::GraphAssemblerLabelForVars<Vars...>* if_false,
+  void BranchImpl(BranchSemantics semantics, Node* condition,
+                  GraphAssemblerLabel<sizeof...(Vars)>* if_true,
+                  GraphAssemblerLabel<sizeof...(Vars)>* if_false,
                   BranchHint hint, Vars...);
 
   Zone* temp_zone_;
   MachineGraph* mcgraph_;
+  BranchSemantics default_branch_semantics_;
   Node* effect_;
   Node* control_;
   // {node_changed_callback_} should be called when a node outside the
@@ -784,7 +823,8 @@ void GraphAssembler::Branch(
     hint = if_false->IsDeferred() ? BranchHint::kTrue : BranchHint::kFalse;
   }
 
-  BranchImpl(condition, if_true, if_false, hint, vars...);
+  BranchImpl(default_branch_semantics_, condition, if_true, if_false, hint,
+             vars...);
 }
 
 template <typename... Vars>
@@ -792,17 +832,36 @@ void GraphAssembler::BranchWithHint(
     Node* condition, detail::GraphAssemblerLabelForVars<Vars...>* if_true,
     detail::GraphAssemblerLabelForVars<Vars...>* if_false, BranchHint hint,
     Vars... vars) {
-  BranchImpl(condition, if_true, if_false, hint, vars...);
+  BranchImpl(default_branch_semantics_, condition, if_true, if_false, hint,
+             vars...);
 }
 
 template <typename... Vars>
-void GraphAssembler::BranchImpl(
-    Node* condition, detail::GraphAssemblerLabelForVars<Vars...>* if_true,
-    detail::GraphAssemblerLabelForVars<Vars...>* if_false, BranchHint hint,
+void GraphAssembler::MachineBranch(
+    TNode<Word32T> condition, GraphAssemblerLabel<sizeof...(Vars)>* if_true,
+    GraphAssemblerLabel<sizeof...(Vars)>* if_false, BranchHint hint,
     Vars... vars) {
+  BranchImpl(BranchSemantics::kMachine, condition, if_true, if_false, hint,
+             vars...);
+}
+
+template <typename... Vars>
+void GraphAssembler::JSBranch(TNode<Boolean> condition,
+                              GraphAssemblerLabel<sizeof...(Vars)>* if_true,
+                              GraphAssemblerLabel<sizeof...(Vars)>* if_false,
+                              BranchHint hint, Vars... vars) {
+  BranchImpl(BranchSemantics::kJS, condition, if_true, if_false, hint, vars...);
+}
+
+template <typename... Vars>
+void GraphAssembler::BranchImpl(BranchSemantics semantics, Node* condition,
+                                GraphAssemblerLabel<sizeof...(Vars)>* if_true,
+                                GraphAssemblerLabel<sizeof...(Vars)>* if_false,
+                                BranchHint hint, Vars... vars) {
   DCHECK_NOT_NULL(control());
 
-  Node* branch = graph()->NewNode(common()->Branch(hint), condition, control());
+  Node* branch =
+      graph()->NewNode(common()->Branch(hint, semantics), condition, control());
 
   control_ = graph()->NewNode(common()->IfTrue(), branch);
   MergeState(if_true, vars...);
@@ -829,7 +888,8 @@ template <typename... Vars>
 void GraphAssembler::GotoIf(Node* condition,
                             detail::GraphAssemblerLabelForVars<Vars...>* label,
                             BranchHint hint, Vars... vars) {
-  Node* branch = graph()->NewNode(common()->Branch(hint), condition, control());
+  Node* branch = graph()->NewNode(
+      common()->Branch(hint, default_branch_semantics_), condition, control());
 
   control_ = graph()->NewNode(common()->IfTrue(), branch);
   MergeState(label, vars...);
@@ -841,7 +901,8 @@ template <typename... Vars>
 void GraphAssembler::GotoIfNot(
     Node* condition, detail::GraphAssemblerLabelForVars<Vars...>* label,
     BranchHint hint, Vars... vars) {
-  Node* branch = graph()->NewNode(common()->Branch(hint), condition, control());
+  Node* branch = graph()->NewNode(
+      common()->Branch(hint, default_branch_semantics_), condition, control());
 
   control_ = graph()->NewNode(common()->IfFalse(), branch);
   MergeState(label, vars...);
@@ -887,15 +948,22 @@ class V8_EXPORT_PRIVATE JSGraphAssembler : public GraphAssembler {
   // Constructs a JSGraphAssembler. If {schedule} is not null, the graph
   // assembler will maintain the schedule as it updates blocks.
   JSGraphAssembler(
-      JSGraph* jsgraph, Zone* zone,
+      JSHeapBroker* broker, JSGraph* jsgraph, Zone* zone,
+      BranchSemantics branch_semantics,
       base::Optional<NodeChangedCallback> node_changed_callback = base::nullopt,
       bool mark_loop_exits = false)
-      : GraphAssembler(jsgraph, zone, node_changed_callback, mark_loop_exits),
-        jsgraph_(jsgraph) {}
+      : GraphAssembler(jsgraph, zone, branch_semantics, node_changed_callback,
+                       mark_loop_exits),
+        broker_(broker),
+        jsgraph_(jsgraph),
+        outermost_catch_scope_(CatchScope::Outermost(zone)),
+        catch_scope_(&outermost_catch_scope_) {
+    outermost_catch_scope_.set_gasm(this);
+  }
 
   Node* SmiConstant(int32_t value);
   TNode<HeapObject> HeapConstant(Handle<HeapObject> object);
-  TNode<Object> Constant(const ObjectRef& ref);
+  TNode<Object> Constant(ObjectRef ref);
   TNode<Number> NumberConstant(double value);
   Node* CEntryStubConstant(int result_size);
 
@@ -918,6 +986,7 @@ class V8_EXPORT_PRIVATE JSGraphAssembler : public GraphAssembler {
     //     access.machine_type.representation()));
     return TNode<T>::UncheckedCast(LoadField(access, object));
   }
+  TNode<Uint32T> LoadElementsKind(TNode<Map> map);
   Node* LoadElement(ElementAccess const&, Node* object, Node* index);
   template <typename T>
   TNode<T> LoadElement(ElementAccess const& access, TNode<HeapObject> object,
@@ -930,6 +999,8 @@ class V8_EXPORT_PRIVATE JSGraphAssembler : public GraphAssembler {
   Node* StoreField(FieldAccess const&, Node* object, Node* value);
   Node* StoreElement(ElementAccess const&, Node* object, Node* index,
                      Node* value);
+  Node* ClearPendingMessage();
+
   void TransitionAndStoreElement(MapRef double_map, MapRef fast_map,
                                  TNode<HeapObject> object, TNode<Number> index,
                                  TNode<Object> value);
@@ -943,11 +1014,20 @@ class V8_EXPORT_PRIVATE JSGraphAssembler : public GraphAssembler {
   TNode<Boolean> NumberLessThanOrEqual(TNode<Number> lhs, TNode<Number> rhs);
   TNode<Number> NumberAdd(TNode<Number> lhs, TNode<Number> rhs);
   TNode<Number> NumberSubtract(TNode<Number> lhs, TNode<Number> rhs);
+  TNode<Number> NumberShiftRightLogical(TNode<Number> lhs, TNode<Number> rhs);
+  TNode<Number> NumberBitwiseAnd(TNode<Number> lhs, TNode<Number> rhs);
+  TNode<Number> NumberBitwiseOr(TNode<Number> lhs, TNode<Number> rhs);
+  TNode<Number> NumberDivide(TNode<Number> lhs, TNode<Number> rhs);
+  TNode<Number> NumberFloor(TNode<Number> value);
   TNode<String> StringSubstring(TNode<String> string, TNode<Number> from,
                                 TNode<Number> to);
   TNode<Boolean> ObjectIsCallable(TNode<Object> value);
+  TNode<Boolean> ObjectIsSmi(TNode<Object> value);
   TNode<Boolean> ObjectIsUndetectable(TNode<Object> value);
-  Node* CheckIf(Node* cond, DeoptimizeReason reason);
+  Node* CheckIf(Node* cond, DeoptimizeReason reason,
+                const FeedbackSource& feedback = {});
+  Node* Assert(Node* cond, const char* condition_string = "",
+               const char* file = "", int line = -1);
   TNode<Boolean> NumberIsFloat64Hole(TNode<Number> value);
   TNode<Boolean> ToBoolean(TNode<Object> value);
   TNode<Object> ConvertTaggedHoleToUndefined(TNode<Object> value);
@@ -958,19 +1038,374 @@ class V8_EXPORT_PRIVATE JSGraphAssembler : public GraphAssembler {
                                               TNode<Number> new_length,
                                               TNode<Number> old_length);
   Node* StringCharCodeAt(TNode<String> string, TNode<Number> position);
+  TNode<String> StringFromSingleCharCode(TNode<Number> code);
+  TNode<Object> DoubleArrayMax(TNode<JSArray> array);
+  TNode<Object> DoubleArrayMin(TNode<JSArray> array);
+  // Computes the byte length for a given {array_buffer_view}. If the set of
+  // possible ElementsKinds is known statically pass as
+  // {elements_kinds_candidates} to allow the assembler to generate more
+  // efficient code. Pass an empty {elements_kinds_candidates} to generate code
+  // that is generic enough to handle all ElementsKinds.
+  TNode<Number> ArrayBufferViewByteLength(
+      TNode<JSArrayBufferView> array_buffer_view, InstanceType instance_type,
+      std::set<ElementsKind> elements_kinds_candidates, TNode<Context> context);
+  // Computes the length for a given {typed_array}. If the set of possible
+  // ElementsKinds is known statically pass as {elements_kinds_candidates} to
+  // allow the assembler to generate more efficient code. Pass an empty
+  // {elements_kinds_candidates} to generate code that is generic enough to
+  // handle all ElementsKinds.
+  TNode<Number> TypedArrayLength(
+      TNode<JSTypedArray> typed_array,
+      std::set<ElementsKind> elements_kinds_candidates, TNode<Context> context);
+  // Performs the full detached check. This includes fixed-length RABs whos
+  // underlying buffer has been shrunk OOB.
+  void CheckIfTypedArrayWasDetached(
+      TNode<JSTypedArray> typed_array,
+      std::set<ElementsKind> elements_kinds_candidates,
+      const FeedbackSource& feedback);
+  TNode<Uint32T> LookupByteShiftForElementsKind(TNode<Uint32T> elements_kind);
+  TNode<Uint32T> LookupByteSizeForElementsKind(TNode<Uint32T> elements_kind);
 
+  TNode<Object> JSCallRuntime1(
+      Runtime::FunctionId function_id, TNode<Object> arg0,
+      TNode<Context> context, base::Optional<FrameState> frame_state,
+      Operator::Properties properties = Operator::kNoProperties);
+  TNode<Object> JSCallRuntime2(Runtime::FunctionId function_id,
+                               TNode<Object> arg0, TNode<Object> arg1,
+                               TNode<Context> context, FrameState frame_state);
+  Node* Chained(const Operator* op, Node* input);
+
+  JSHeapBroker* broker() const { return broker_; }
   JSGraph* jsgraph() const { return jsgraph_; }
   Isolate* isolate() const { return jsgraph()->isolate(); }
-  SimplifiedOperatorBuilder* simplified() const {
+  SimplifiedOperatorBuilder* simplified() override {
     return jsgraph()->simplified();
   }
+  JSOperatorBuilder* javascript() const { return jsgraph()->javascript(); }
+
+  template <typename T, typename U>
+  TNode<T> EnterMachineGraph(TNode<U> input, UseInfo use_info) {
+    DCHECK_EQ(use_info.type_check(), TypeCheckKind::kNone);
+    return AddNode<T>(
+        graph()->NewNode(common()->EnterMachineGraph(use_info), input));
+  }
+
+  template <typename T, typename U>
+  TNode<T> ExitMachineGraph(TNode<U> input,
+                            MachineRepresentation output_representation,
+                            Type output_type) {
+    return AddNode<T>(graph()->NewNode(
+        common()->ExitMachineGraph(output_representation, output_type), input));
+  }
+
+  // A catch scope represents a single catch handler. The handler can be
+  // custom catch logic within the reduction itself; or a catch handler in the
+  // outside graph into which the reduction will be integrated (in this case
+  // the scope is called 'outermost').
+  class V8_NODISCARD CatchScope {
+   private:
+    // Only used to partially construct the outermost scope.
+    explicit CatchScope(Zone* zone) : if_exception_nodes_(zone) {}
+
+    // For all inner scopes.
+    CatchScope(Zone* zone, JSGraphAssembler* gasm)
+        : gasm_(gasm),
+          parent_(gasm->catch_scope_),
+          has_handler_(true),
+          if_exception_nodes_(zone) {
+      DCHECK_NOT_NULL(gasm_);
+      gasm_->catch_scope_ = this;
+    }
+
+   public:
+    ~CatchScope() { gasm_->catch_scope_ = parent_; }
+
+    static CatchScope Outermost(Zone* zone) { return CatchScope{zone}; }
+    static CatchScope Inner(Zone* zone, JSGraphAssembler* gasm) {
+      return {zone, gasm};
+    }
+
+    bool has_handler() const { return has_handler_; }
+    bool is_outermost() const { return parent_ == nullptr; }
+    CatchScope* parent() const { return parent_; }
+
+    // Should only be used to initialize the outermost scope (inner scopes
+    // always have a handler and are passed the gasm pointer at construction).
+    void set_has_handler(bool v) {
+      DCHECK(is_outermost());
+      has_handler_ = v;
+    }
+    void set_gasm(JSGraphAssembler* v) {
+      DCHECK(is_outermost());
+      DCHECK_NOT_NULL(v);
+      gasm_ = v;
+    }
+
+    bool has_exceptional_control_flow() const {
+      return !if_exception_nodes_.empty();
+    }
+
+    void RegisterIfExceptionNode(Node* if_exception) {
+      DCHECK(has_handler());
+      if_exception_nodes_.push_back(if_exception);
+    }
+
+    void MergeExceptionalPaths(TNode<Object>* exception_out, Effect* effect_out,
+                               Control* control_out) {
+      DCHECK(has_handler());
+      DCHECK(has_exceptional_control_flow());
+
+      const int size = static_cast<int>(if_exception_nodes_.size());
+
+      if (size == 1) {
+        // No merge needed.
+        Node* e = if_exception_nodes_.at(0);
+        *exception_out = TNode<Object>::UncheckedCast(e);
+        *effect_out = Effect(e);
+        *control_out = Control(e);
+      } else {
+        DCHECK_GT(size, 1);
+
+        Node* merge = gasm_->graph()->NewNode(gasm_->common()->Merge(size),
+                                              size, if_exception_nodes_.data());
+
+        // These phis additionally take {merge} as an input. Temporarily add
+        // it to the list.
+        if_exception_nodes_.push_back(merge);
+        const int size_with_merge =
+            static_cast<int>(if_exception_nodes_.size());
+
+        Node* ephi = gasm_->graph()->NewNode(gasm_->common()->EffectPhi(size),
+                                             size_with_merge,
+                                             if_exception_nodes_.data());
+        Node* phi = gasm_->graph()->NewNode(
+            gasm_->common()->Phi(MachineRepresentation::kTagged, size),
+            size_with_merge, if_exception_nodes_.data());
+        if_exception_nodes_.pop_back();
+
+        *exception_out = TNode<Object>::UncheckedCast(phi);
+        *effect_out = Effect(ephi);
+        *control_out = Control(merge);
+      }
+    }
+
+   private:
+    JSGraphAssembler* gasm_ = nullptr;
+    CatchScope* const parent_ = nullptr;
+    bool has_handler_ = false;
+    NodeVector if_exception_nodes_;
+  };
+
+  CatchScope* catch_scope() const { return catch_scope_; }
+  Node* outermost_handler() const { return outermost_handler_; }
+
+  using NodeGenerator0 = std::function<TNode<Object>()>;
+  // TODO(jgruber): Currently, it's the responsibility of the developer to note
+  // which operations may throw and appropriately wrap these in a call to
+  // MayThrow (see e.g. JSCall3 and CallRuntime2). A more methodical approach
+  // would be good.
+  TNode<Object> MayThrow(const NodeGenerator0& body) {
+    TNode<Object> result = body();
+
+    if (catch_scope()->has_handler()) {
+      // The IfException node is later merged into the outer graph.
+      // Note: AddNode is intentionally not called since effect and control
+      // should not be updated.
+      Node* if_exception =
+          graph()->NewNode(common()->IfException(), effect(), control());
+      catch_scope()->RegisterIfExceptionNode(if_exception);
+
+      // Control resumes here.
+      AddNode(graph()->NewNode(common()->IfSuccess(), control()));
+    }
+
+    return result;
+  }
+
+  using VoidGenerator0 = std::function<void()>;
+  // TODO(jgruber): Currently IfBuilder0 and IfBuilder1 are implemented as
+  // separate classes. If, in the future, we encounter additional use cases that
+  // return more than 1 value, we should merge these back into a single variadic
+  // implementation.
+  class IfBuilder0 final {
+   public:
+    IfBuilder0(JSGraphAssembler* gasm, TNode<Boolean> cond, bool negate_cond)
+        : gasm_(gasm),
+          cond_(cond),
+          negate_cond_(negate_cond),
+          initial_effect_(gasm->effect()),
+          initial_control_(gasm->control()) {}
+
+    IfBuilder0& ExpectTrue() {
+      DCHECK_EQ(hint_, BranchHint::kNone);
+      hint_ = BranchHint::kTrue;
+      return *this;
+    }
+    IfBuilder0& ExpectFalse() {
+      DCHECK_EQ(hint_, BranchHint::kNone);
+      hint_ = BranchHint::kFalse;
+      return *this;
+    }
+
+    IfBuilder0& Then(const VoidGenerator0& body) {
+      then_body_ = body;
+      return *this;
+    }
+    IfBuilder0& Else(const VoidGenerator0& body) {
+      else_body_ = body;
+      return *this;
+    }
+
+    ~IfBuilder0() {
+      // Ensure correct usage: effect/control must not have been modified while
+      // the IfBuilder0 instance is alive.
+      DCHECK_EQ(gasm_->effect(), initial_effect_);
+      DCHECK_EQ(gasm_->control(), initial_control_);
+
+      // Unlike IfBuilder1, this supports an empty then or else body. This is
+      // possible since the merge does not take any value inputs.
+      DCHECK(then_body_ || else_body_);
+
+      if (negate_cond_) std::swap(then_body_, else_body_);
+
+      auto if_true = (hint_ == BranchHint::kFalse) ? gasm_->MakeDeferredLabel()
+                                                   : gasm_->MakeLabel();
+      auto if_false = (hint_ == BranchHint::kTrue) ? gasm_->MakeDeferredLabel()
+                                                   : gasm_->MakeLabel();
+      auto merge = gasm_->MakeLabel();
+      gasm_->Branch(cond_, &if_true, &if_false);
+
+      gasm_->Bind(&if_true);
+      if (then_body_) then_body_();
+      if (gasm_->HasActiveBlock()) gasm_->Goto(&merge);
+
+      gasm_->Bind(&if_false);
+      if (else_body_) else_body_();
+      if (gasm_->HasActiveBlock()) gasm_->Goto(&merge);
+
+      gasm_->Bind(&merge);
+    }
+
+    IfBuilder0(const IfBuilder0&) = delete;
+    IfBuilder0& operator=(const IfBuilder0&) = delete;
+
+   private:
+    JSGraphAssembler* const gasm_;
+    const TNode<Boolean> cond_;
+    const bool negate_cond_;
+    const Effect initial_effect_;
+    const Control initial_control_;
+    BranchHint hint_ = BranchHint::kNone;
+    VoidGenerator0 then_body_;
+    VoidGenerator0 else_body_;
+  };
+
+  IfBuilder0 If(TNode<Boolean> cond) { return {this, cond, false}; }
+  IfBuilder0 IfNot(TNode<Boolean> cond) { return {this, cond, true}; }
+
+  template <typename T, typename Cond>
+  class IfBuilder1 {
+    using If1BodyFunction = std::function<TNode<T>()>;
+
+   public:
+    IfBuilder1(JSGraphAssembler* gasm, TNode<Cond> cond, bool negate_cond)
+        : gasm_(gasm), cond_(cond), negate_cond_(negate_cond) {}
+
+    V8_WARN_UNUSED_RESULT IfBuilder1& ExpectTrue() {
+      DCHECK_EQ(hint_, BranchHint::kNone);
+      hint_ = BranchHint::kTrue;
+      return *this;
+    }
+
+    V8_WARN_UNUSED_RESULT IfBuilder1& ExpectFalse() {
+      DCHECK_EQ(hint_, BranchHint::kNone);
+      hint_ = BranchHint::kFalse;
+      return *this;
+    }
+
+    V8_WARN_UNUSED_RESULT IfBuilder1& Then(const If1BodyFunction& body) {
+      then_body_ = body;
+      return *this;
+    }
+    V8_WARN_UNUSED_RESULT IfBuilder1& Else(const If1BodyFunction& body) {
+      else_body_ = body;
+      return *this;
+    }
+
+    V8_WARN_UNUSED_RESULT TNode<T> Value() {
+      DCHECK(then_body_);
+      DCHECK(else_body_);
+
+      if (negate_cond_) std::swap(then_body_, else_body_);
+
+      auto if_true = (hint_ == BranchHint::kFalse) ? gasm_->MakeDeferredLabel()
+                                                   : gasm_->MakeLabel();
+      auto if_false = (hint_ == BranchHint::kTrue) ? gasm_->MakeDeferredLabel()
+                                                   : gasm_->MakeLabel();
+      auto merge = gasm_->MakeLabel(PhiMachineRepresentationOf<T>);
+      if constexpr (std::is_same_v<Cond, Word32T>) {
+        gasm_->MachineBranch(cond_, &if_true, &if_false, hint_);
+      } else {
+        static_assert(std::is_same_v<Cond, Boolean>);
+        if (hint_ != BranchHint::kNone) {
+          gasm_->BranchWithHint(cond_, &if_true, &if_false, hint_);
+        } else {
+          gasm_->Branch(cond_, &if_true, &if_false);
+        }
+      }
+
+      gasm_->Bind(&if_true);
+      TNode<T> then_result = then_body_();
+      if (gasm_->HasActiveBlock()) gasm_->Goto(&merge, then_result);
+
+      gasm_->Bind(&if_false);
+      TNode<T> else_result = else_body_();
+      if (gasm_->HasActiveBlock()) {
+        gasm_->Goto(&merge, else_result);
+      }
+
+      gasm_->Bind(&merge);
+      return merge.template PhiAt<T>(0);
+    }
+
+   private:
+    static constexpr MachineRepresentation kPhiRepresentation =
+        MachineRepresentation::kTagged;
+
+    JSGraphAssembler* const gasm_;
+    const TNode<Cond> cond_;
+    const bool negate_cond_;
+    BranchHint hint_ = BranchHint::kNone;
+    If1BodyFunction then_body_;
+    If1BodyFunction else_body_;
+  };
+
+  template <typename T>
+  IfBuilder1<T, Boolean> SelectIf(TNode<Boolean> cond) {
+    return {this, cond, false};
+  }
+  template <typename T>
+  IfBuilder1<T, Boolean> SelectIfNot(TNode<Boolean> cond) {
+    return {this, cond, true};
+  }
+  template <typename T>
+  IfBuilder1<T, Word32T> MachineSelectIf(TNode<Word32T> cond) {
+    return {this, cond, false};
+  }
 
  protected:
   Operator const* PlainPrimitiveToNumberOperator();
 
  private:
+  JSHeapBroker* broker_;
   JSGraph* jsgraph_;
   SetOncePointer<Operator const> to_number_operator_;
+
+ protected:
+  CatchScope outermost_catch_scope_;
+  Node* outermost_handler_;
+  CatchScope* catch_scope_;
+  friend class CatchScope;
 };
 
 }  // namespace compiler
