@@ -70,7 +70,7 @@ int DebugStackTraceIterator::GetContextId() const {
   DCHECK(!Done());
   Handle<Object> context = frame_inspector_->GetContext();
   if (IsContext(*context)) {
-    Object value =
+    Tagged<Object> value =
         Context::cast(*context)->native_context()->debug_context_id();
     if (IsSmi(value)) return Smi::ToInt(value);
   }
@@ -161,8 +161,10 @@ debug::Location DebugStackTraceIterator::GetFunctionLocation() const {
   if (iterator_.frame()->is_wasm()) {
     auto frame = WasmFrame::cast(iterator_.frame());
     Handle<WasmInstanceObject> instance(frame->wasm_instance(), isolate_);
-    auto offset =
-        instance->module()->functions[frame->function_index()].code.offset();
+    auto offset = instance->module_object()
+                      ->module()
+                      ->functions[frame->function_index()]
+                      .code.offset();
     return v8::debug::Location(0, offset);
   }
 #endif
@@ -248,7 +250,6 @@ v8::MaybeLocal<v8::Value> DebugStackTraceIterator::Evaluate(
                             inlined_frame_index_, Utils::OpenHandle(*source),
                             throw_on_side_effect)
            .ToHandle(&value)) {
-    isolate_->OptionalRescheduleException(false);
     return v8::MaybeLocal<v8::Value>();
   }
   return Utils::ToLocal(value);
