@@ -17,10 +17,10 @@ class EntryFrameConstants : public AllStatic {
  public:
   // This is the offset to where JSEntry pushes the current value of
   // Isolate::c_entry_fp onto the stack.
-  static constexpr int kCallerFPOffset = -3 * kSystemPointerSize;
+  static constexpr int kNextExitFrameFPOffset = -3 * kSystemPointerSize;
 };
 
-class WasmCompileLazyFrameConstants : public TypedFrameConstants {
+class WasmLiftoffSetupFrameConstants : public TypedFrameConstants {
  public:
   // Number of gp parameters, without the instance.
   static constexpr int kNumberOfSavedGpParamRegs = 6;
@@ -29,9 +29,9 @@ class WasmCompileLazyFrameConstants : public TypedFrameConstants {
 
   // On loong64, spilled registers are implicitly sorted backwards by number.
   // We spill:
-  //   a0: param0 = instance
   //   a2, a3, a4, a5, a6, a7: param1, param2, ..., param6
-  // in the following FP-relative order: [a7, a6, a5, a4, a3, a2, a0].
+  // in the following FP-relative order: [a7, a6, a5, a4, a3, a2].
+  // The instance slot is in position '6', the first spill slot is at '0'.
   static constexpr int kInstanceSpillOffset =
       TYPED_FRAME_PUSHED_VALUE_OFFSET(6);
 
@@ -42,7 +42,7 @@ class WasmCompileLazyFrameConstants : public TypedFrameConstants {
 
   // SP-relative.
   static constexpr int kWasmInstanceOffset = 2 * kSystemPointerSize;
-  static constexpr int kFunctionIndexOffset = 1 * kSystemPointerSize;
+  static constexpr int kDeclaredFunctionIndexOffset = 1 * kSystemPointerSize;
   static constexpr int kNativeModuleOffset = 0;
 };
 
@@ -51,10 +51,10 @@ class WasmCompileLazyFrameConstants : public TypedFrameConstants {
 // registers (see liftoff-assembler-defs.h).
 class WasmDebugBreakFrameConstants : public TypedFrameConstants {
  public:
-  // {a0 ... a7, t0 ... t5, s0, s1, s2, s5, s7, s8}
+  // {a0 ... a7, t0 ... t5, s0, s1, s2, s5, s7}
   static constexpr RegList kPushedGpRegs = {a0, a1, a2, a3, a4, a5, a6,
                                             a7, t0, t1, t2, t3, t4, t5,
-                                            s0, s1, s2, s5, s7, s8};
+                                            s0, s1, s2, s5, s7};
   // {f0, f1, f2, ... f27, f28}
   static constexpr DoubleRegList kPushedFpRegs = {
       f0,  f1,  f2,  f3,  f4,  f5,  f6,  f7,  f8,  f9,  f10, f11, f12, f13, f14,
