@@ -9,6 +9,8 @@
 #include "src/objects/code.h"
 
 #if !defined(USE_SIMULATOR)
+#include "src/base/platform/platform.h"
+#include "src/execution/isolate.h"
 #include "src/utils/utils.h"
 #endif
 
@@ -49,6 +51,15 @@ class SimulatorStack : public v8::internal::AllStatic {
     return Simulator::current(isolate)->StackLimit(c_limit);
   }
 
+  static inline base::Vector<uint8_t> GetCurrentStackView(
+      v8::internal::Isolate* isolate) {
+    return Simulator::current(isolate)->GetCurrentStackView();
+  }
+
+  // When running on the simulator, we should leave the C stack limits alone
+  // when switching stacks for Wasm.
+  static inline bool ShouldSwitchCStackForWasmStackSwitching() { return false; }
+
   // Returns the current stack address on the simulator stack frame.
   // The returned address is comparable with JS stack address.
   static inline uintptr_t RegisterJSStackComparableAddress(
@@ -79,6 +90,19 @@ class SimulatorStack : public v8::internal::AllStatic {
     return c_limit;
   }
 
+  static inline base::Vector<uint8_t> GetCurrentStackView(
+      v8::internal::Isolate* isolate) {
+    uintptr_t limit = isolate->stack_guard()->real_jslimit();
+    uintptr_t stack_start = base::Stack::GetStackStart();
+    DCHECK_LE(limit, stack_start);
+    size_t size = stack_start - limit;
+    return base::VectorOf(reinterpret_cast<uint8_t*>(limit), size);
+  }
+
+  // When running on real hardware, we should also switch the C stack limit
+  // when switching stacks for Wasm.
+  static inline bool ShouldSwitchCStackForWasmStackSwitching() { return true; }
+
   // Returns the current stack address on the native stack frame.
   // The returned address is comparable with JS stack address.
   static inline uintptr_t RegisterJSStackComparableAddress(
@@ -106,13 +130,12 @@ class GeneratedCode {
     return GeneratedCode(isolate, reinterpret_cast<Signature*>(addr));
   }
 
-  static GeneratedCode FromBuffer(Isolate* isolate, byte* buffer) {
+  static GeneratedCode FromBuffer(Isolate* isolate, uint8_t* buffer) {
     return GeneratedCode(isolate, reinterpret_cast<Signature*>(buffer));
   }
 
-  template <typename CodeOrCodeT>
-  static GeneratedCode FromCode(CodeOrCodeT code) {
-    return FromAddress(code.GetIsolate(), code.entry());
+  static GeneratedCode FromCode(Isolate* isolate, Tagged<Code> code) {
+    return FromAddress(isolate, code->instruction_start());
   }
 
 #ifdef USE_SIMULATOR
