@@ -29,19 +29,22 @@ class V8_EXPORT_PRIVATE CagedHeap final {
 
   template <typename RetType = uintptr_t>
   static RetType OffsetFromAddress(const void* address) {
-    static_assert(
-        std::numeric_limits<RetType>::max() >= (kCagedHeapReservationSize - 1),
-        "The return type should be large enough");
+    static_assert(std::numeric_limits<RetType>::max() >=
+                      (api_constants::kCagedHeapMaxReservationSize - 1),
+                  "The return type should be large enough");
     return reinterpret_cast<uintptr_t>(address) &
-           (kCagedHeapReservationAlignment - 1);
+           (api_constants::kCagedHeapReservationAlignment - 1);
   }
 
   static uintptr_t BaseFromAddress(const void* address) {
     return reinterpret_cast<uintptr_t>(address) &
-           ~(kCagedHeapReservationAlignment - 1);
+           ~(api_constants::kCagedHeapReservationAlignment - 1);
   }
 
-  static void InitializeIfNeeded(PageAllocator&);
+  static void InitializeIfNeeded(PageAllocator& platform_allocator,
+                                 size_t desired_heap_size);
+
+  static void CommitAgeTable(PageAllocator& platform_allocator);
 
   static CagedHeap& Instance();
 
@@ -66,7 +69,8 @@ class V8_EXPORT_PRIVATE CagedHeap final {
   friend class v8::base::LeakyObject<CagedHeap>;
   friend class testing::TestWithHeap;
 
-  explicit CagedHeap(PageAllocator& platform_allocator);
+  explicit CagedHeap(PageAllocator& platform_allocator,
+                     size_t desired_heap_size);
 
   static CagedHeap* instance_;
 
