@@ -10,7 +10,9 @@
 #include "src/base/platform/mutex.h"
 #include "src/base/vector.h"
 #include "src/common/globals.h"
+#include "src/objects/bytecode-array.h"
 #include "src/objects/code.h"
+#include "src/objects/instruction-stream.h"
 #include "src/objects/name.h"
 #include "src/objects/shared-function-info.h"
 #include "src/objects/string.h"
@@ -88,8 +90,10 @@ class LogEventListener {
   virtual void RegExpCodeCreateEvent(Handle<AbstractCode> code,
                                      Handle<String> source) = 0;
   // Not handlified as this happens during GC. No allocation allowed.
-  virtual void CodeMoveEvent(InstructionStream from, InstructionStream to) = 0;
-  virtual void BytecodeMoveEvent(BytecodeArray from, BytecodeArray to) = 0;
+  virtual void CodeMoveEvent(Tagged<InstructionStream> from,
+                             Tagged<InstructionStream> to) = 0;
+  virtual void BytecodeMoveEvent(Tagged<BytecodeArray> from,
+                                 Tagged<BytecodeArray> to) = 0;
   virtual void SharedFunctionInfoMoveEvent(Address from, Address to) = 0;
   virtual void NativeContextMoveEvent(Address from, Address to) = 0;
   virtual void CodeMovingGCEvent() = 0;
@@ -107,9 +111,10 @@ class LogEventListener {
   virtual void WeakCodeClearEvent() = 0;
 
   virtual bool is_listening_to_code_events() { return false; }
+  virtual bool allows_code_compaction() { return true; }
 };
 
-// Dispatches code events to a set of registered listeners.
+// Dispatches events to a set of registered listeners.
 class Logger {
  public:
   using Event = LogEventListener::Event;
@@ -127,20 +132,31 @@ class Logger {
     listeners_.push_back(listener);
     return true;
   }
-  void RemoveListener(LogEventListener* listener) {
+
+  bool RemoveListener(LogEventListener* listener) {
     base::MutexGuard guard(&mutex_);
     auto position = std::find(listeners_.begin(), listeners_.end(), listener);
-    if (position == listeners_.end()) return;
+    if (position == listeners_.end()) return false;
     listeners_.erase(position);
+    return true;
   }
 
-  bool is_listening_to_code_events() const {
+  bool is_listening_to_code_events() {
+    base::MutexGuard guard(&mutex_);
     for (auto listener : listeners_) {
       if (listener->is_listening_to_code_events()) return true;
     }
     return false;
   }
 
+  bool allows_code_compaction() {
+    base::MutexGuard guard(&mutex_);
+    for (auto listener : listeners_) {
+      if (!listener->allows_code_compaction()) return false;
+    }
+    return true;
+  }
+
   void CodeCreateEvent(CodeTag tag, Handle<AbstractCode> code,
                        const char* comment) {
     base::MutexGuard guard(&mutex_);
@@ -148,6 +164,7 @@ class Logger {
       listener->CodeCreateEvent(tag, code, comment);
     }
   }
+
   void CodeCreateEvent(CodeTag tag, Handle<AbstractCode> code,
                        Handle<Name> name) {
     base::MutexGuard guard(&mutex_);
@@ -155,6 +172,7 @@ class Logger {
       listener->CodeCreateEvent(tag, code, name);
     }
   }
+
   void CodeCreateEvent(CodeTag tag, Handle<AbstractCode> code,
                        Handle<SharedFunctionInfo> shared, Handle<Name> name) {
     base::MutexGuard guard(&mutex_);
@@ -162,6 +180,7 @@ class Logger {
       listener->CodeCreateEvent(tag, code, shared, name);
     }
   }
+
   void CodeCreateEvent(CodeTag tag, Handle<AbstractCode> code,
                        Handle<SharedFunctionInfo> shared, Handle<Name> source,
                        int line, int column) {
@@ -170,6 +189,7 @@ class Logger {
       listener->CodeCreateEvent(tag, code, shared, source, line, column);
     }
   }
+
 #if V8_ENABLE_WEBASSEMBLY
   void CodeCreateEvent(CodeTag tag, const wasm::WasmCode* code,
                        wasm::WasmName name, const char* source_url,
@@ -181,60 +201,71 @@ class Logger {
     }
   }
 #endif  // V8_ENABLE_WEBASSEMBLY
+
   void CallbackEvent(Handle<Name> name, Address entry_point) {
     base::MutexGuard guard(&mutex_);
     for (auto listener : listeners_) {
       listener->CallbackEvent(name, entry_point);
     }
   }
+
   void GetterCallbackEvent(Handle<Name> name, Address entry_point) {
     base::MutexGuard guard(&mutex_);
     for (auto listener : listeners_) {
       listener->GetterCallbackEvent(name, entry_point);
     }
   }
+
   void SetterCallbackEvent(Handle<Name> name, Address entry_point) {
     base::MutexGuard guard(&mutex_);
     for (auto listener : listeners_) {
       listener->SetterCallbackEvent(name, entry_point);
     }
   }
+
   void RegExpCodeCreateEvent(Handle<AbstractCode> code, Handle<String> source) {
     base::MutexGuard guard(&mutex_);
     for (auto listener : listeners_) {
       listener->RegExpCodeCreateEvent(code, source);
     }
   }
-  void CodeMoveEvent(InstructionStream from, InstructionStream to) {
+
+  void CodeMoveEvent(Tagged<InstructionStream> from,
+                     Tagged<InstructionStream> to) {
     base::MutexGuard guard(&mutex_);
     for (auto listener : listeners_) {
       listener->CodeMoveEvent(from, to);
     }
   }
-  void BytecodeMoveEvent(BytecodeArray from, BytecodeArray to) {
+
+  void BytecodeMoveEvent(Tagged<BytecodeArray> from, Tagged<BytecodeArray> to) {
     base::MutexGuard guard(&mutex_);
     for (auto listener : listeners_) {
       listener->BytecodeMoveEvent(from, to);
     }
   }
+
   void SharedFunctionInfoMoveEvent(Address from, Address to) {
     base::MutexGuard guard(&mutex_);
     for (auto listener : listeners_) {
       listener->SharedFunctionInfoMoveEvent(from, to);
     }
   }
+
   void NativeContextMoveEvent(Address from, Address to) {
     base::MutexGuard guard(&mutex_);
     for (auto listener : listeners_) {
       listener->NativeContextMoveEvent(from, to);
     }
   }
+
   void CodeMovingGCEvent() {
     base::MutexGuard guard(&mutex_);
     for (auto listener : listeners_) {
       listener->CodeMovingGCEvent();
     }
   }
+
   void CodeDisableOptEvent(Handle<AbstractCode> code,
                            Handle<SharedFunctionInfo> shared) {
     base::MutexGuard guard(&mutex_);
@@ -242,6 +273,7 @@ class Logger {
       listener->CodeDisableOptEvent(code, shared);
     }
   }
+
   void CodeDeoptEvent(Handle<Code> code, DeoptimizeKind kind, Address pc,
                       int fp_to_sp_delta) {
     base::MutexGuard guard(&mutex_);
@@ -249,6 +281,7 @@ class Logger {
       listener->CodeDeoptEvent(code, kind, pc, fp_to_sp_delta);
     }
   }
+
   void CodeDependencyChangeEvent(Handle<Code> code,
                                  Handle<SharedFunctionInfo> sfi,
                                  const char* reason) {
@@ -257,6 +290,7 @@ class Logger {
       listener->CodeDependencyChangeEvent(code, sfi, reason);
     }
   }
+
   void WeakCodeClearEvent() {
     base::MutexGuard guard(&mutex_);
     for (auto listener : listeners_) {
