@@ -20,6 +20,26 @@ class MaglevCompilationInfo;
 class MaglevPrintingVisitor;
 class MergePointRegisterState;
 
+// Represents the state of the register frame during register allocation,
+// including current register values, and the state of each register.
+//
+// Register state encodes two orthogonal concepts:
+//
+//   1. Used/free registers: Which registers currently hold a valid value,
+//   2. Blocked/unblocked registers: Which registers can be modified during the
+//      current allocation.
+//
+// The combination of these encodes values in different states:
+//
+//  Free + unblocked: Completely unused registers which can be used for
+//                    anything.
+//  Used + unblocked: Live values that can be spilled if there is register
+//                    pressure.
+//  Used + blocked:   Values that are in a register and are used as an input in
+//                    the current allocation.
+//  Free + blocked:   Unused registers that are reserved as temporaries, or
+//                    inputs that will get clobbered during the execution of the
+//                    node being allocated.
 template <typename RegisterT>
 class RegisterFrameState {
  public:
@@ -85,14 +105,32 @@ class RegisterFrameState {
     DCHECK_NOT_NULL(node);
     return node;
   }
+#ifdef DEBUG
+  // Like GetValue, but allow reading freed registers as long as they were also
+  // blocked. This allows us to DCHECK expected register state against node
+  // state, even if that node is dead or clobbered by the end of the current
+  // allocation.
+  ValueNode* GetValueMaybeFreeButBlocked(RegisterT reg) const {
+    DCHECK(!free_.has(reg) || blocked_.has(reg));
+    ValueNode* node = values_[reg.code()];
+    DCHECK_NOT_NULL(node);
+    return node;
+  }
+#endif
+
   RegTList blocked() const { return blocked_; }
   void block(RegisterT reg) { blocked_.set(reg); }
   void unblock(RegisterT reg) { blocked_.clear(reg); }
   bool is_blocked(RegisterT reg) { return blocked_.has(reg); }
   void clear_blocked() { blocked_ = kEmptyRegList; }
 
-  compiler::AllocatedOperand ChooseInputRegister(ValueNode* node);
-  compiler::AllocatedOperand AllocateRegister(ValueNode* node);
+  compiler::InstructionOperand TryChooseInputRegister(
+      ValueNode* node, const compiler::InstructionOperand& hint =
+                           compiler::InstructionOperand());
+  compiler::InstructionOperand TryChooseUnblockedInputRegister(ValueNode* node);
+  compiler::AllocatedOperand AllocateRegister(
+      ValueNode* node, const compiler::InstructionOperand& hint =
+                           compiler::InstructionOperand());
 
  private:
   ValueNode* values_[RegisterT::kNumRegisters];
@@ -111,10 +149,14 @@ class StraightForwardRegisterAllocator {
   RegisterFrameState<DoubleRegister> double_registers_;
 
   struct SpillSlotInfo {
-    SpillSlotInfo(uint32_t slot_index, NodeIdT freed_at_position)
-        : slot_index(slot_index), freed_at_position(freed_at_position) {}
+    SpillSlotInfo(uint32_t slot_index, NodeIdT freed_at_position,
+                  bool double_slot)
+        : slot_index(slot_index),
+          freed_at_position(freed_at_position),
+          double_slot(double_slot) {}
     uint32_t slot_index;
     NodeIdT freed_at_position;
+    bool double_slot;
   };
   struct SpillSlots {
     int top = 0;
@@ -132,14 +174,17 @@ class StraightForwardRegisterAllocator {
 
   void UpdateUse(Input* input) { return UpdateUse(input->node(), input); }
   void UpdateUse(ValueNode* node, InputLocation* input_location);
-  void UpdateUse(const EagerDeoptInfo& deopt_info);
-  void UpdateUse(const LazyDeoptInfo& deopt_info);
+
+  void MarkAsClobbered(ValueNode* node,
+                       const compiler::AllocatedOperand& location);
 
   void AllocateControlNode(ControlNode* node, BasicBlock* block);
   void AllocateNode(Node* node);
   void AllocateNodeResult(ValueNode* node);
+  void AllocateEagerDeopt(const EagerDeoptInfo& deopt_info);
+  void AllocateLazyDeopt(const LazyDeoptInfo& deopt_info);
   void AssignFixedInput(Input& input);
-  void AssignArbitraryRegisterInput(Input& input);
+  void AssignArbitraryRegisterInput(NodeBase* result_node, Input& input);
   void AssignAnyInput(Input& input);
   void AssignInputs(NodeBase* node);
   template <typename RegisterT>
@@ -150,6 +195,8 @@ class StraightForwardRegisterAllocator {
   void AssignArbitraryTemporaries(RegisterFrameState<RegisterT>& registers,
                                   NodeBase* node);
   void AssignArbitraryTemporaries(NodeBase* node);
+  template <typename RegisterT>
+  void SetLoopPhiRegisterHint(Phi* phi, RegisterT reg);
   void TryAllocateToInput(Phi* phi);
 
   void VerifyInputs(NodeBase* node);
@@ -171,7 +218,8 @@ class StraightForwardRegisterAllocator {
 
   void FreeRegistersUsedBy(ValueNode* node);
   template <typename RegisterT>
-  RegisterT FreeUnblockedRegister();
+  RegisterT FreeUnblockedRegister(
+      RegListBase<RegisterT> reserved = RegListBase<RegisterT>());
   template <typename RegisterT>
   RegisterT PickRegisterToFree(RegListBase<RegisterT> reserved);
 
@@ -185,18 +233,22 @@ class StraightForwardRegisterAllocator {
   }
 
   template <typename RegisterT>
-  void DropRegisterValueAtEnd(RegisterT reg);
+  void DropRegisterValueAtEnd(RegisterT reg, bool force_spill = false);
+  bool IsCurrentNodeLastUseOf(ValueNode* node);
   template <typename RegisterT>
-  void EnsureFreeRegisterAtEnd();
+  void EnsureFreeRegisterAtEnd(const compiler::InstructionOperand& hint =
+                                   compiler::InstructionOperand());
   compiler::AllocatedOperand AllocateRegisterAtEnd(ValueNode* node);
 
   template <typename RegisterT>
   void DropRegisterValue(RegisterFrameState<RegisterT>& registers,
-                         RegisterT reg);
+                         RegisterT reg, bool force_spill = false);
   void DropRegisterValue(Register reg);
   void DropRegisterValue(DoubleRegister reg);
 
-  compiler::AllocatedOperand AllocateRegister(ValueNode* node);
+  compiler::AllocatedOperand AllocateRegister(
+      ValueNode* node, const compiler::InstructionOperand& hint =
+                           compiler::InstructionOperand());
 
   template <typename RegisterT>
   compiler::AllocatedOperand ForceAllocate(
@@ -213,8 +265,14 @@ class StraightForwardRegisterAllocator {
   void InitializeRegisterValues(MergePointRegisterState& target_state);
 #ifdef DEBUG
   bool IsInRegister(MergePointRegisterState& target_state, ValueNode* incoming);
+  bool IsForwardReachable(BasicBlock* start_block, NodeIdT first_id,
+                          NodeIdT last_id);
 #endif
 
+  template <typename RegisterT>
+  void HoistLoopReloads(BasicBlock* target,
+                        RegisterFrameState<RegisterT>& registers);
+  void HoistLoopSpills(BasicBlock* target);
   void InitializeBranchTargetRegisterValues(ControlNode* source,
                                             BasicBlock* target);
   void InitializeEmptyBlockRegisterValues(ControlNode* source,
