@@ -8,6 +8,7 @@
 #include "src/common/globals.h"
 #include "src/heap/heap-write-barrier-inl.h"
 #include "src/objects/allocation-site.h"
+#include "src/objects/dependent-code-inl.h"
 #include "src/objects/js-objects-inl.h"
 
 // Has to be the last include (doesn't have include guards):
@@ -25,30 +26,33 @@ NEVER_READ_ONLY_SPACE_IMPL(AllocationSite)
 
 CAST_ACCESSOR(AllocationSite)
 
-ACCESSORS(AllocationSite, transition_info_or_boilerplate, Object,
+ACCESSORS(AllocationSite, transition_info_or_boilerplate, Tagged<Object>,
           kTransitionInfoOrBoilerplateOffset)
 RELEASE_ACQUIRE_ACCESSORS(AllocationSite, transition_info_or_boilerplate,
-                          Object, kTransitionInfoOrBoilerplateOffset)
-ACCESSORS(AllocationSite, nested_site, Object, kNestedSiteOffset)
+                          Tagged<Object>, kTransitionInfoOrBoilerplateOffset)
+ACCESSORS(AllocationSite, nested_site, Tagged<Object>, kNestedSiteOffset)
 RELAXED_INT32_ACCESSORS(AllocationSite, pretenure_data, kPretenureDataOffset)
 INT32_ACCESSORS(AllocationSite, pretenure_create_count,
                 kPretenureCreateCountOffset)
-ACCESSORS(AllocationSite, dependent_code, DependentCode, kDependentCodeOffset)
-ACCESSORS_CHECKED(AllocationSite, weak_next, Object, kWeakNextOffset,
+ACCESSORS(AllocationSite, dependent_code, Tagged<DependentCode>,
+          kDependentCodeOffset)
+ACCESSORS_CHECKED(AllocationSite, weak_next, Tagged<Object>, kWeakNextOffset,
                   HasWeakNext())
-ACCESSORS(AllocationMemento, allocation_site, Object, kAllocationSiteOffset)
+ACCESSORS(AllocationMemento, allocation_site, Tagged<Object>,
+          kAllocationSiteOffset)
 
-JSObject AllocationSite::boilerplate() const {
+Tagged<JSObject> AllocationSite::boilerplate() const {
   DCHECK(PointsToLiteral());
   return JSObject::cast(transition_info_or_boilerplate());
 }
 
-JSObject AllocationSite::boilerplate(AcquireLoadTag tag) const {
+Tagged<JSObject> AllocationSite::boilerplate(AcquireLoadTag tag) const {
   DCHECK(PointsToLiteral());
   return JSObject::cast(transition_info_or_boilerplate(tag));
 }
 
-void AllocationSite::set_boilerplate(JSObject value, ReleaseStoreTag tag,
+void AllocationSite::set_boilerplate(Tagged<JSObject> value,
+                                     ReleaseStoreTag tag,
                                      WriteBarrierMode mode) {
   set_transition_info_or_boilerplate(value, tag, mode);
 }
@@ -74,9 +78,8 @@ void AllocationSite::Initialize() {
   set_nested_site(Smi::zero());
   set_pretenure_data(0, kRelaxedStore);
   set_pretenure_create_count(0);
-  set_dependent_code(
-      DependentCode::cast(GetReadOnlyRoots().empty_weak_fixed_array()),
-      SKIP_WRITE_BARRIER);
+  set_dependent_code(DependentCode::empty_dependent_code(GetReadOnlyRoots()),
+                     SKIP_WRITE_BARRIER);
 }
 
 bool AllocationSite::IsZombie() const {
@@ -114,10 +117,9 @@ void AllocationSite::SetDoNotInlineCall() {
 }
 
 bool AllocationSite::PointsToLiteral() const {
-  Object raw_value = transition_info_or_boilerplate(kAcquireLoad);
-  DCHECK_EQ(!raw_value.IsSmi(),
-            raw_value.IsJSArray() || raw_value.IsJSObject());
-  return !raw_value.IsSmi();
+  Tagged<Object> raw_value = transition_info_or_boilerplate(kAcquireLoad);
+  DCHECK_EQ(!IsSmi(raw_value), IsJSArray(raw_value) || IsJSObject(raw_value));
+  return !IsSmi(raw_value);
 }
 
 // Heuristic: We only need to create allocation site info if the boilerplate
@@ -129,7 +131,7 @@ bool AllocationSite::ShouldTrack(ElementsKind boilerplate_elements_kind) {
 
 inline bool AllocationSite::CanTrack(InstanceType type) {
   if (!V8_ALLOCATION_SITE_TRACKING_BOOL) return false;
-  if (FLAG_allocation_site_pretenuring) {
+  if (v8_flags.allocation_site_pretenuring) {
     // TurboFan doesn't care at all about String pretenuring feedback,
     // so don't bother even trying to track that.
     return type == JS_ARRAY_TYPE || type == JS_OBJECT_TYPE;
@@ -181,26 +183,26 @@ void AllocationSite::set_memento_create_count(int count) {
   set_pretenure_create_count(count);
 }
 
-bool AllocationSite::IncrementMementoFoundCount(int increment) {
-  if (IsZombie()) return false;
+int AllocationSite::IncrementMementoFoundCount(int increment) {
+  DCHECK(!IsZombie());
 
-  int value = memento_found_count();
-  set_memento_found_count(value + increment);
-  return memento_found_count() >= kPretenureMinimumCreated;
+  int new_value = memento_found_count() + increment;
+  set_memento_found_count(new_value);
+  return new_value;
 }
 
 inline void AllocationSite::IncrementMementoCreateCount() {
-  DCHECK(FLAG_allocation_site_pretenuring);
+  DCHECK(v8_flags.allocation_site_pretenuring);
   int value = memento_create_count();
   set_memento_create_count(value + 1);
 }
 
 bool AllocationMemento::IsValid() const {
-  return allocation_site().IsAllocationSite() &&
-         !AllocationSite::cast(allocation_site()).IsZombie();
+  return IsAllocationSite(allocation_site()) &&
+         !AllocationSite::cast(allocation_site())->IsZombie();
 }
 
-AllocationSite AllocationMemento::GetAllocationSite() const {
+Tagged<AllocationSite> AllocationMemento::GetAllocationSite() const {
   DCHECK(IsValid());
   return AllocationSite::cast(allocation_site());
 }
@@ -215,7 +217,7 @@ bool AllocationSite::DigestTransitionFeedback(Handle<AllocationSite> site,
   Isolate* isolate = site->GetIsolate();
   bool result = false;
 
-  if (site->PointsToLiteral() && site->boilerplate().IsJSArray()) {
+  if (site->PointsToLiteral() && IsJSArray(site->boilerplate())) {
     Handle<JSArray> boilerplate(JSArray::cast(site->boilerplate()), isolate);
     ElementsKind kind = boilerplate->GetElementsKind();
     // if kind is holey ensure that to_kind is as well.
@@ -226,12 +228,12 @@ bool AllocationSite::DigestTransitionFeedback(Handle<AllocationSite> site,
       // If the array is huge, it's not likely to be defined in a local
       // function, so we shouldn't make new instances of it very often.
       uint32_t length = 0;
-      CHECK(boilerplate->length().ToArrayLength(&length));
+      CHECK(Object::ToArrayLength(boilerplate->length(), &length));
       if (length <= kMaximumArrayBytesToPretransition) {
         if (update_or_check == AllocationSiteUpdateMode::kCheckOnly) {
           return true;
         }
-        if (FLAG_trace_track_allocation_sites) {
+        if (v8_flags.trace_track_allocation_sites) {
           bool is_nested = site->IsNested();
           PrintF("AllocationSite: JSArray %p boilerplate %supdated %s->%s\n",
                  reinterpret_cast<void*>(site->ptr()),
@@ -240,7 +242,8 @@ bool AllocationSite::DigestTransitionFeedback(Handle<AllocationSite> site,
         }
         CHECK_NE(to_kind, DICTIONARY_ELEMENTS);
         JSObject::TransitionElementsKind(boilerplate, to_kind);
-        site->dependent_code().DeoptimizeDependentCodeGroup(
+        DependentCode::DeoptimizeDependencyGroups(
+            isolate, *site,
             DependentCode::kAllocationSiteTransitionChangedGroup);
         result = true;
       }
@@ -254,14 +257,14 @@ bool AllocationSite::DigestTransitionFeedback(Handle<AllocationSite> site,
     }
     if (IsMoreGeneralElementsKindTransition(kind, to_kind)) {
       if (update_or_check == AllocationSiteUpdateMode::kCheckOnly) return true;
-      if (FLAG_trace_track_allocation_sites) {
+      if (v8_flags.trace_track_allocation_sites) {
         PrintF("AllocationSite: JSArray %p site updated %s->%s\n",
                reinterpret_cast<void*>(site->ptr()), ElementsKindToString(kind),
                ElementsKindToString(to_kind));
       }
       site->SetElementsKind(to_kind);
-      site->dependent_code().DeoptimizeDependentCodeGroup(
-          DependentCode::kAllocationSiteTransitionChangedGroup);
+      DependentCode::DeoptimizeDependencyGroups(
+          isolate, *site, DependentCode::kAllocationSiteTransitionChangedGroup);
       result = true;
     }
   }
