@@ -29,9 +29,10 @@ inline MaybeHandle<FixedArray> ElementsAccessor::PrependElementIndices(
                                convert, filter);
 }
 
-inline bool ElementsAccessor::HasElement(JSObject holder, uint32_t index,
+inline bool ElementsAccessor::HasElement(Tagged<JSObject> holder,
+                                         uint32_t index,
                                          PropertyFilter filter) {
-  return HasElement(holder, index, holder.elements(), filter);
+  return HasElement(holder, index, holder->elements(), filter);
 }
 
 }  // namespace internal
