@@ -5,10 +5,12 @@
 #ifndef V8_OBJECTS_HEAP_OBJECT_H_
 #define V8_OBJECTS_HEAP_OBJECT_H_
 
+#include "src/base/macros.h"
 #include "src/common/globals.h"
 #include "src/objects/instance-type.h"
-#include "src/objects/objects.h"
 #include "src/objects/tagged-field.h"
+#include "src/sandbox/indirect-pointer-tag.h"
+#include "src/sandbox/isolate.h"
 
 // Has to be the last include (doesn't have include guards):
 #include "src/objects/object-macros.h"
@@ -20,16 +22,110 @@ class Heap;
 class PrimitiveHeapObject;
 class ExternalPointerSlot;
 class IndirectPointerSlot;
+class ExposedTrustedObject;
+class ObjectVisitor;
+class WritableFreeSpace;
+
+V8_OBJECT class HeapObjectLayout {
+ public:
+  HeapObjectLayout() = delete;
+
+  // [map]: Contains a map which contains the object's reflective
+  // information.
+  inline Tagged<Map> map() const;
+  inline Tagged<Map> map(AcquireLoadTag) const;
+
+  inline void set_map(Tagged<Map> value);
+  inline void set_map(Tagged<Map> value, ReleaseStoreTag);
+
+  // This method behaves the same as `set_map` but marks the map transition as
+  // safe for the concurrent marker (object layout doesn't change) during
+  // verification.
+  inline void set_map_safe_transition(Tagged<Map> value, ReleaseStoreTag);
+
+  inline void set_map_safe_transition_no_write_barrier(
+      Tagged<Map> value, RelaxedStoreTag = kRelaxedStore);
+
+  // Initialize the map immediately after the object is allocated.
+  // Do not use this outside Heap.
+  inline void set_map_after_allocation(
+      Tagged<Map> value, WriteBarrierMode mode = UPDATE_WRITE_BARRIER);
+
+  // The no-write-barrier version.  This is OK if the object is white and in
+  // new space, or if the value is an immortal immutable object, like the maps
+  // of primitive (non-JS) objects like strings, heap numbers etc.
+  inline void set_map_no_write_barrier(Tagged<Map> value,
+                                       RelaxedStoreTag = kRelaxedStore);
+
+  // Access the map word using acquire load and release store.
+  inline void set_map_word_forwarded(Tagged<HeapObject> target_object,
+                                     ReleaseStoreTag);
+
+  // Returns the tagged pointer to this HeapObject.
+  // TODO(leszeks): Consider bottlenecking this through Tagged<>.
+  inline Address ptr() const { return address() + kHeapObjectTag; }
+
+  // Returns the address of this HeapObject.
+  inline Address address() const { return reinterpret_cast<Address>(this); }
+
+  // This method exists to help remove GetIsolate/GetHeap from HeapObject, in a
+  // way that doesn't require passing Isolate/Heap down huge call chains or to
+  // places where it might not be safe to access it.
+  inline ReadOnlyRoots GetReadOnlyRoots() const;
+  // This is slower, but safe to call during bootstrapping.
+  inline ReadOnlyRoots EarlyGetReadOnlyRoots() const;
+
+  // Returns the heap object's size in bytes
+  inline int Size() const;
+
+  // Given a heap object's map pointer, returns the heap size in bytes
+  // Useful when the map pointer field is used for other purposes.
+  // GC internal.
+  V8_EXPORT_PRIVATE int SizeFromMap(Tagged<Map> map) const;
+
+  // Return the write barrier mode for this. Callers of this function
+  // must be able to present a reference to an DisallowGarbageCollection
+  // object as a sign that they are not going to use this function
+  // from code that allocates and thus invalidates the returned write
+  // barrier mode.
+  inline WriteBarrierMode GetWriteBarrierMode(
+      const DisallowGarbageCollection& promise);
+
+#ifdef OBJECT_PRINT
+  void PrintHeader(std::ostream& os, const char* id);
+#endif
+
+ private:
+  friend class HeapObject;
+
+  TaggedMember<Map> map_;
+} V8_OBJECT_END;
+
+static_assert(sizeof(HeapObjectLayout) == kTaggedSize);
+
+inline bool operator==(const HeapObjectLayout* obj, StrongTaggedBase ptr) {
+  return Tagged<HeapObject>(obj) == ptr;
+}
+inline bool operator==(StrongTaggedBase ptr, const HeapObjectLayout* obj) {
+  return ptr == Tagged<HeapObject>(obj);
+}
+inline bool operator!=(const HeapObjectLayout* obj, StrongTaggedBase ptr) {
+  return Tagged<HeapObject>(obj) != ptr;
+}
+inline bool operator!=(StrongTaggedBase ptr, const HeapObjectLayout* obj) {
+  return ptr != Tagged<HeapObject>(obj);
+}
+
 template <typename T>
-class Tagged;
+struct ObjectTraits {
+  using BodyDescriptor = typename T::BodyDescriptor;
+};
 
 // HeapObject is the superclass for all classes describing heap allocated
 // objects.
-class HeapObject : public Object {
+class HeapObject : public TaggedImpl<HeapObjectReferenceType::STRONG, Address> {
  public:
-  bool is_null() const {
-    return static_cast<Tagged_t>(ptr()) == static_cast<Tagged_t>(kNullAddress);
-  }
+  constexpr HeapObject() = default;
 
   // [map]: Contains a map which contains the object's reflective
   // information.
@@ -69,6 +165,9 @@ class HeapObject : public Object {
   inline void set_map_after_allocation(
       Tagged<Map> value, WriteBarrierMode mode = UPDATE_WRITE_BARRIER);
 
+  static inline void SetFillerMap(const WritableFreeSpace& writable_page,
+                                  Tagged<Map> value);
+
   // During garbage collection, the map word of a heap object does not
   // necessarily contain a map pointer.
   DECL_RELAXED_GETTER(map_word, MapWord)
@@ -92,14 +191,6 @@ class HeapObject : public Object {
   // This is slower, but safe to call during bootstrapping.
   inline ReadOnlyRoots EarlyGetReadOnlyRoots() const;
 
-  // Whether the object is in the RO heap and the RO heap is shared, or in the
-  // writable shared heap.
-  V8_INLINE bool InAnySharedSpace() const;
-
-  V8_INLINE bool InWritableSharedSpace() const;
-
-  V8_INLINE bool InReadOnlySpace() const;
-
   // Converts an address to a HeapObject pointer.
   static inline Tagged<HeapObject> FromAddress(Address address) {
     DCHECK_TAG_ALIGNED(address);
@@ -208,35 +299,77 @@ class HeapObject : public Object {
   // ExternalPointer_t field accessors.
   //
   template <ExternalPointerTag tag>
-  inline void InitExternalPointerField(size_t offset, Isolate* isolate,
+  inline void InitExternalPointerField(size_t offset, IsolateForSandbox isolate,
                                        Address value);
   template <ExternalPointerTag tag>
   inline Address ReadExternalPointerField(size_t offset,
-                                          Isolate* isolate) const;
+                                          IsolateForSandbox isolate) const;
   template <ExternalPointerTag tag>
-  inline void WriteExternalPointerField(size_t offset, Isolate* isolate,
+  inline void WriteExternalPointerField(size_t offset,
+                                        IsolateForSandbox isolate,
                                         Address value);
 
   template <ExternalPointerTag tag>
-  inline void WriteLazilyInitializedExternalPointerField(size_t offset,
-                                                         Isolate* isolate,
-                                                         Address value);
+  inline void WriteLazilyInitializedExternalPointerField(
+      size_t offset, IsolateForSandbox isolate, Address value);
 
   inline void ResetLazilyInitializedExternalPointerField(size_t offset);
 
   //
-  // IndirectPointer field accessors.
+  // Indirect pointers.
   //
-  inline Tagged<Object> ReadIndirectPointerField(size_t offset) const;
+  // These are only available when the sandbox is enabled.
+  inline void InitSelfIndirectPointerField(size_t offset,
+                                           IsolateForSandbox isolate);
 
+  template <IndirectPointerTag tag>
+  inline Tagged<Object> ReadIndirectPointerField(
+      size_t offset, IsolateForSandbox isolate) const;
+
+  template <IndirectPointerTag tag>
+  inline void WriteIndirectPointerField(size_t offset,
+                                        Tagged<ExposedTrustedObject> value);
+
+  // Trusted pointers.
   //
-  // CodePointer field accessors.
+  // A pointer to a trusted object. When the sandbox is enabled, these are
+  // indirect pointers using the the TrustedPointerTable (TPT). When the sandbox
+  // is disabled, they are regular tagged pointers. They must always point to an
+  // ExposedTrustedObject as (only) these objects can be referenced through the
+  // trusted pointer table.
+  template <IndirectPointerTag tag>
+  inline Tagged<ExposedTrustedObject> ReadTrustedPointerField(
+      size_t offset, IsolateForSandbox isolate) const;
+  template <IndirectPointerTag tag>
+  inline void WriteTrustedPointerField(size_t offset,
+                                       Tagged<ExposedTrustedObject> value);
+
+  // Trusted pointer fields can be cleared, in which case they no longer point
+  // to any object. When the sandbox is enabled, this will set the fields
+  // indirect pointer handle to the null handle (referencing the zeroth entry in
+  // the TrustedPointerTable which just contains nullptr). When the sandbox is
+  // disabled, this will set the field to Smi::zero().
+  inline bool IsTrustedPointerFieldCleared(size_t offset) const;
+  inline void ClearTrustedPointerField(size_t offest);
+
+  // Code pointers.
   //
-  inline void InitCodePointerTableEntryField(size_t offset, Isolate* isolate,
-                                             Tagged<Code> owning_code,
-                                             Address entrypoint);
-  inline Address ReadCodeEntrypointField(size_t offset) const;
-  inline void WriteCodeEntrypointField(size_t offset, Address value);
+  // These are special versions of trusted pointers that always point to Code
+  // objects. When the sandbox is enabled, they are indirect pointers using the
+  // code pointer table (CPT) instead of the TrustedPointerTable. When the
+  // sandbox is disabled, they are regular tagged pointers.
+  inline Tagged<Code> ReadCodePointerField(size_t offset,
+                                           IsolateForSandbox isolate) const;
+  inline void WriteCodePointerField(size_t offset, Tagged<Code> value);
+
+  inline bool IsCodePointerFieldCleared(size_t offset) const;
+  inline void ClearCodePointerField(size_t offest);
+
+  inline Address ReadCodeEntrypointViaCodePointerField(
+      size_t offset, CodeEntrypointTag tag) const;
+  inline void WriteCodeEntrypointViaCodePointerField(size_t offset,
+                                                     Address value,
+                                                     CodeEntrypointTag tag);
 
   // Returns the field at offset in obj, as a read/write Object reference.
   // Does no checking, and is safe to use during GC, while maps are invalid.
@@ -245,8 +378,10 @@ class HeapObject : public Object {
   inline ObjectSlot RawField(int byte_offset) const;
   inline MaybeObjectSlot RawMaybeWeakField(int byte_offset) const;
   inline InstructionStreamSlot RawInstructionStreamField(int byte_offset) const;
-  inline ExternalPointerSlot RawExternalPointerField(int byte_offset) const;
-  inline IndirectPointerSlot RawIndirectPointerField(int byte_offset) const;
+  inline ExternalPointerSlot RawExternalPointerField(
+      int byte_offset, ExternalPointerTag tag) const;
+  inline IndirectPointerSlot RawIndirectPointerField(
+      int byte_offset, IndirectPointerTag tag) const;
 
   DECL_CAST(HeapObject)
 
@@ -260,6 +395,9 @@ class HeapObject : public Object {
 
   // Dispatched behavior.
   void HeapObjectShortPrint(std::ostream& os);
+  void Print();
+  static void Print(Tagged<Object> obj);
+  static void Print(Tagged<Object> obj, std::ostream& os);
 #ifdef OBJECT_PRINT
   void PrintHeader(std::ostream& os, const char* id);
 #endif
@@ -295,13 +433,8 @@ class HeapObject : public Object {
   void RehashBasedOnMap(IsolateT* isolate);
 
   // Layout description.
-#define HEAP_OBJECT_FIELDS(V) \
-  V(kMapOffset, kTaggedSize)  \
-  /* Header size. */          \
-  V(kHeaderSize, 0)
-
-  DEFINE_FIELD_OFFSET_CONSTANTS(Object::kHeaderSize, HEAP_OBJECT_FIELDS)
-#undef HEAP_OBJECT_FIELDS
+  static constexpr int kMapOffset = offsetof(HeapObjectLayout, map_);
+  static constexpr int kHeaderSize = sizeof(HeapObjectLayout);
 
   static_assert(kMapOffset == Internals::kHeapObjectMapOffset);
 
@@ -309,8 +442,23 @@ class HeapObject : public Object {
 
   inline Address GetFieldAddress(int field_offset) const;
 
+  HeapObject* operator->() { return this; }
+  const HeapObject* operator->() const { return this; }
+
  protected:
-  OBJECT_CONSTRUCTORS(HeapObject, Object);
+  struct SkipTypeCheckTag {};
+  friend class Tagged<HeapObject>;
+  explicit V8_INLINE constexpr HeapObject(Address ptr,
+                                          HeapObject::SkipTypeCheckTag)
+      : TaggedImpl(ptr) {}
+  explicit inline HeapObject(Address ptr);
+
+  // Static overwrites of TaggedImpl's IsSmi/IsHeapObject, to avoid conflicts
+  // with IsSmi(Tagged<HeapObject>) inside HeapObject subclasses' methods.
+  template <typename T>
+  static bool IsSmi(T obj);
+  template <typename T>
+  static bool IsHeapObject(T obj);
 
   inline Address field_address(size_t offset) const {
     return ptr() + offset - kHeapObjectTag;
@@ -332,9 +480,22 @@ class HeapObject : public Object {
                          VerificationMode mode);
 };
 
-OBJECT_CONSTRUCTORS_IMPL(HeapObject, Object)
+inline HeapObject::HeapObject(Address ptr) : TaggedImpl(ptr) {
+  IsHeapObject(*this);
+}
 CAST_ACCESSOR(HeapObject)
 
+template <typename T>
+// static
+bool HeapObject::IsSmi(T obj) {
+  return i::IsSmi(obj);
+}
+template <typename T>
+// static
+bool HeapObject::IsHeapObject(T obj) {
+  return i::IsHeapObject(obj);
+}
+
 // Define Tagged<HeapObject> now that HeapObject exists.
 constexpr HeapObject Tagged<HeapObject>::operator*() const {
   return ToRawPtr();
@@ -343,13 +504,6 @@ constexpr detail::TaggedOperatorArrowRef<HeapObject>
 Tagged<HeapObject>::operator->() const {
   return detail::TaggedOperatorArrowRef<HeapObject>{ToRawPtr()};
 }
-// Implicit conversions and explicit casts to/from raw pointers
-// TODO(leszeks): Remove once we're using Tagged everywhere.
-// NOLINTNEXTLINE
-constexpr Tagged<HeapObject>::operator HeapObject() {
-  static_assert(kTaggedCanConvertToRawObjects);
-  return ToRawPtr();
-}
 constexpr HeapObject Tagged<HeapObject>::ToRawPtr() const {
   return HeapObject(this->ptr(), HeapObject::SkipTypeCheckTag{});
 }
@@ -359,17 +513,23 @@ constexpr HeapObject Tagged<HeapObject>::ToRawPtr() const {
   V8_INLINE bool Is##Type(Tagged<HeapObject> obj);                             \
   V8_INLINE bool Is##Type(Tagged<HeapObject> obj, PtrComprCageBase cage_base); \
   V8_INLINE bool Is##Type(HeapObject obj);                                     \
-  V8_INLINE bool Is##Type(HeapObject obj, PtrComprCageBase cage_base);
+  V8_INLINE bool Is##Type(HeapObject obj, PtrComprCageBase cage_base);         \
+  V8_INLINE bool Is##Type(const HeapObjectLayout* obj);                        \
+  V8_INLINE bool Is##Type(const HeapObjectLayout* obj,                         \
+                          PtrComprCageBase cage_base);
 HEAP_OBJECT_TYPE_LIST(IS_TYPE_FUNCTION_DECL)
 IS_TYPE_FUNCTION_DECL(HashTableBase)
 IS_TYPE_FUNCTION_DECL(SmallOrderedHashTable)
+IS_TYPE_FUNCTION_DECL(PropertyDictionary)
 #undef IS_TYPE_FUNCTION_DECL
 
 // Most calls to Is<Oddball> should go via the Tagged<Object> overloads, withst
 // an Isolate/LocalIsolate/ReadOnlyRoots parameter.
-#define IS_TYPE_FUNCTION_DECL(Type, Value, _)      \
-  V8_INLINE bool Is##Type(Tagged<HeapObject> obj); \
-  V8_INLINE bool Is##Type(HeapObject obj);
+#define IS_TYPE_FUNCTION_DECL(Type, Value, _)                             \
+  V8_INLINE bool Is##Type(Tagged<HeapObject> obj);                        \
+  V8_INLINE bool Is##Type(HeapObject obj);                                \
+  V8_INLINE bool Is##Type(const HeapObjectLayout* obj, Isolate* isolate); \
+  V8_INLINE bool Is##Type(const HeapObjectLayout* obj);
 ODDBALL_LIST(IS_TYPE_FUNCTION_DECL)
 HOLE_LIST(IS_TYPE_FUNCTION_DECL)
 IS_TYPE_FUNCTION_DECL(NullOrUndefined, , /* unused */)
@@ -379,10 +539,24 @@ IS_TYPE_FUNCTION_DECL(NullOrUndefined, , /* unused */)
   V8_INLINE bool Is##Name(Tagged<HeapObject> obj);                             \
   V8_INLINE bool Is##Name(Tagged<HeapObject> obj, PtrComprCageBase cage_base); \
   V8_INLINE bool Is##Name(HeapObject obj);                                     \
-  V8_INLINE bool Is##Name(HeapObject obj, PtrComprCageBase cage_base);
+  V8_INLINE bool Is##Name(HeapObject obj, PtrComprCageBase cage_base);         \
+  V8_INLINE bool Is##Name(const HeapObjectLayout* obj);                        \
+  V8_INLINE bool Is##Name(const HeapObjectLayout* obj,                         \
+                          PtrComprCageBase cage_base);
 STRUCT_LIST(DECL_STRUCT_PREDICATE)
 #undef DECL_STRUCT_PREDICATE
 
+// Whether the object is in the RO heap and the RO heap is shared, or in the
+// writable shared heap.
+V8_INLINE bool InAnySharedSpace(Tagged<HeapObject> obj);
+V8_INLINE bool InWritableSharedSpace(Tagged<HeapObject> obj);
+V8_INLINE bool InReadOnlySpace(Tagged<HeapObject> obj);
+// Whether the object is located outside of the sandbox or in read-only
+// space. Currently only needed due to Code objects. Once they are fully
+// migrated into trusted space, this can be replaced by !InsideSandbox().
+static_assert(!kAllCodeObjectsLiveInTrustedSpace);
+V8_INLINE bool OutsideSandboxOrInReadonlySpace(Tagged<HeapObject> obj);
+
 }  // namespace internal
 }  // namespace v8
 
