@@ -25,23 +25,24 @@ namespace internal {
 class JSArray : public TorqueGeneratedJSArray<JSArray, JSObject> {
  public:
   // [length]: The length property.
-  DECL_ACCESSORS(length, Object)
-  DECL_RELAXED_GETTER(length, Object)
+  DECL_ACCESSORS(length, Tagged<Object>)
+  DECL_RELAXED_GETTER(length, Tagged<Object>)
 
   // Acquire/release semantics on this field are explicitly forbidden to avoid
   // confusion, since the default setter uses relaxed semantics. If
   // acquire/release semantics ever become necessary, the default setter should
   // be reverted to non-atomic behavior, and setters with explicit tags
   // introduced and used when required.
-  Object length(PtrComprCageBase cage_base, AcquireLoadTag tag) const = delete;
-  void set_length(Object value, ReleaseStoreTag tag,
+  Tagged<Object> length(PtrComprCageBase cage_base,
+                        AcquireLoadTag tag) const = delete;
+  void set_length(Tagged<Object> value, ReleaseStoreTag tag,
                   WriteBarrierMode mode = UPDATE_WRITE_BARRIER) = delete;
 
   // Overload the length setter to skip write barrier when the length
   // is set to a smi. This matches the set function on FixedArray.
-  inline void set_length(Smi length);
+  inline void set_length(Tagged<Smi> length);
 
-  static bool MayHaveReadOnlyLength(Map js_array_map);
+  static bool MayHaveReadOnlyLength(Tagged<Map> js_array_map);
   static bool HasReadOnlyLength(Handle<JSArray> array);
   static bool WouldChangeReadOnlyLength(Handle<JSArray> array, uint32_t index);
 
@@ -84,10 +85,14 @@ class JSArray : public TorqueGeneratedJSArray<JSArray, JSObject> {
   //      separators.
   //   2) Implicitly between two consecutive strings a single separator.
   //
+  // In addition repeated strings are represented by a negative smi, indicating
+  // how many times the previously written string has to be repeated.
+  //
   // Here are some input/output examples given the separator string is ',':
   //
   //   [1, 'hello', 2, 'world', 1] => ',hello,,world,'
   //   ['hello', 'world']          => 'hello,world'
+  //   ['hello', -2, 'world']      => 'hello,hello,hello,world'
   //
   // To avoid any allocations, this helper assumes the destination string is the
   // exact length necessary to write the strings and separators from the fixed
@@ -160,11 +165,14 @@ class JSArrayIterator
 };
 
 // Helper class for JSArrays that are template literal objects
-class TemplateLiteralObject {
+class TemplateLiteralObject
+    : public TorqueGeneratedTemplateLiteralObject<TemplateLiteralObject,
+                                                  JSArray> {
  public:
-  static const int kRawFieldOffset = JSArray::kLengthOffset + kTaggedSize;
-  static inline void SetRaw(Handle<JSArray> template_object,
-                            Handle<JSArray> raw_object);
+  DECL_CAST(TemplateLiteralObject)
+
+ private:
+  TQ_OBJECT_CONSTRUCTORS(TemplateLiteralObject)
 };
 
 }  // namespace internal
