@@ -16,16 +16,14 @@
 namespace v8 {
 namespace internal {
 
+#include "torque-generated/src/objects/js-collection-iterator-tq.inc"
+
 class JSCollectionIterator
     : public TorqueGeneratedJSCollectionIterator<JSCollectionIterator,
                                                  JSObject> {
  public:
   void JSCollectionIteratorPrint(std::ostream& os, const char* name);
 
-  // JSCollectionIterator is abstract, but also defines the size for all of its
-  // concrete subclasses.
-  static constexpr int kSize = kHeaderSize;
-
   TQ_OBJECT_CONSTRUCTORS(JSCollectionIterator)
 };
 
@@ -53,7 +51,7 @@ class OrderedHashTableIterator : public JSCollectionIterator {
 
   // Returns the current key of the iterator. This should only be called when
   // |HasMore| returns true.
-  inline Object CurrentKey();
+  inline Tagged<Object> CurrentKey();
 
  private:
   // Transitions the iterator to the non obsolete backing store. This is a NOP
