@@ -5,6 +5,7 @@
 #ifndef V8_OBJECTS_JS_FUNCTION_H_
 #define V8_OBJECTS_JS_FUNCTION_H_
 
+#include "src/base/optional.h"
 #include "src/objects/code-kind.h"
 #include "src/objects/js-objects.h"
 
@@ -14,6 +15,9 @@
 namespace v8 {
 namespace internal {
 
+class AbstractCode;
+class ClosureFeedbackCellArray;
+
 #include "torque-generated/src/objects/js-function-tq.inc"
 
 // An abstract superclass for classes representing JavaScript function values.
@@ -87,25 +91,25 @@ class JSFunction : public TorqueGeneratedJSFunction<
                        JSFunction, JSFunctionOrBoundFunctionOrWrappedFunction> {
  public:
   // [prototype_or_initial_map]:
-  DECL_RELEASE_ACQUIRE_ACCESSORS(prototype_or_initial_map, HeapObject)
+  DECL_RELEASE_ACQUIRE_ACCESSORS(prototype_or_initial_map, Tagged<HeapObject>)
 
   // [shared]: The information about the function that can be shared by
   // instances.
-  DECL_ACCESSORS(shared, SharedFunctionInfo)
-  DECL_RELAXED_GETTER(shared, SharedFunctionInfo)
+  DECL_ACCESSORS(shared, Tagged<SharedFunctionInfo>)
+  DECL_RELAXED_GETTER(shared, Tagged<SharedFunctionInfo>)
 
   // Fast binding requires length and name accessors.
   static const int kMinDescriptorsForFastBindAndWrap = 2;
 
   // [context]: The context for this function.
-  inline Context context();
-  DECL_RELAXED_GETTER(context, Context)
+  inline Tagged<Context> context();
+  DECL_RELAXED_GETTER(context, Tagged<Context>)
   inline bool has_context() const;
   using TorqueGeneratedClass::context;
   using TorqueGeneratedClass::set_context;
-  DECL_RELEASE_ACQUIRE_ACCESSORS(context, Context)
-  inline JSGlobalProxy global_proxy();
-  inline NativeContext native_context();
+  DECL_RELEASE_ACQUIRE_ACCESSORS(context, Tagged<Context>)
+  inline Tagged<JSGlobalProxy> global_proxy();
+  inline Tagged<NativeContext> native_context();
   inline int length();
 
   static Handle<String> GetName(Isolate* isolate, Handle<JSFunction> function);
@@ -118,22 +122,22 @@ class JSFunction : public TorqueGeneratedJSFunction<
   // optimized code object, or when reading from the background thread.
   // Storing a builtin doesn't require release semantics because these objects
   // are fully initialized.
-  DECL_ACCESSORS(code, CodeT)
-  DECL_RELEASE_ACQUIRE_ACCESSORS(code, CodeT)
-#ifdef V8_EXTERNAL_CODE_SPACE
-  // Convenient overloads to avoid unnecessary Code <-> CodeT conversions.
-  // TODO(v8:11880): remove once |code| accessors are migrated to CodeT.
-  inline void set_code(Code code, ReleaseStoreTag,
-                       WriteBarrierMode mode = UPDATE_WRITE_BARRIER);
-#endif
+  DECL_CODE_POINTER_ACCESSORS(code)
+
+  // Returns the raw content of the Code field. When reading from a background
+  // thread, the code field may still be uninitialized, in which case the field
+  // contains Smi::zero().
+  inline Tagged<Object> raw_code(IsolateForSandbox isolate) const;
+  inline Tagged<Object> raw_code(IsolateForSandbox isolate,
+                                 AcquireLoadTag) const;
 
   // Returns the address of the function code's instruction start.
-  inline Address code_entry_point() const;
+  inline Address instruction_start(IsolateForSandbox isolate) const;
 
   // Get the abstract code associated with the function, which will either be
-  // a Code object or a BytecodeArray.
+  // a InstructionStream object or a BytecodeArray.
   template <typename IsolateT>
-  inline AbstractCode abstract_code(IsolateT* isolate);
+  inline Tagged<AbstractCode> abstract_code(IsolateT* isolate);
 
   // The predicates for querying code kinds related to this function have
   // specific terminology:
@@ -152,35 +156,38 @@ class JSFunction : public TorqueGeneratedJSFunction<
   // been already deoptimized but its code() still needs to be unlinked, which
   // will happen on its next activation.
 
-  bool HasAvailableHigherTierCodeThan(CodeKind kind) const;
+  bool HasAvailableHigherTierCodeThan(IsolateForSandbox isolate,
+                                      CodeKind kind) const;
   // As above but only considers available code kinds passing the filter mask.
-  bool HasAvailableHigherTierCodeThanWithFilter(CodeKind kind,
+  bool HasAvailableHigherTierCodeThanWithFilter(IsolateForSandbox isolate,
+                                                CodeKind kind,
                                                 CodeKinds filter_mask) const;
 
   // True, iff any generated code kind is attached/available to this function.
-  V8_EXPORT_PRIVATE bool HasAttachedOptimizedCode() const;
-  bool HasAvailableOptimizedCode() const;
+  V8_EXPORT_PRIVATE bool HasAttachedOptimizedCode(
+      IsolateForSandbox isolate) const;
+  bool HasAvailableOptimizedCode(IsolateForSandbox isolate) const;
 
-  bool HasAttachedCodeKind(CodeKind kind) const;
-  bool HasAvailableCodeKind(CodeKind kind) const;
+  bool HasAttachedCodeKind(IsolateForSandbox isolate, CodeKind kind) const;
+  bool HasAvailableCodeKind(IsolateForSandbox isolate, CodeKind kind) const;
 
-  base::Optional<CodeKind> GetActiveTier() const;
-  V8_EXPORT_PRIVATE bool ActiveTierIsIgnition() const;
-  bool ActiveTierIsBaseline() const;
-  bool ActiveTierIsMaglev() const;
-  bool ActiveTierIsTurbofan() const;
+  base::Optional<CodeKind> GetActiveTier(IsolateForSandbox isolate) const;
+  V8_EXPORT_PRIVATE bool ActiveTierIsIgnition(IsolateForSandbox isolate) const;
+  bool ActiveTierIsBaseline(IsolateForSandbox isolate) const;
+  bool ActiveTierIsMaglev(IsolateForSandbox isolate) const;
+  bool ActiveTierIsTurbofan(IsolateForSandbox isolate) const;
 
   // Similar to SharedFunctionInfo::CanDiscardCompiled. Returns true, if the
   // attached code can be recreated at a later point by replacing it with
   // CompileLazy.
-  bool CanDiscardCompiled() const;
+  bool CanDiscardCompiled(IsolateForSandbox isolate) const;
 
   // Tells whether function's code object checks its tiering state (some code
   // kinds, e.g. TURBOFAN, ignore the tiering state).
-  inline bool ChecksTieringState();
+  inline bool ChecksTieringState(IsolateForSandbox isolate);
 
   inline TieringState tiering_state() const;
-  inline void set_tiering_state(TieringState state);
+  inline void set_tiering_state(IsolateForSandbox isolate, TieringState state);
   inline void reset_tiering_state();
 
   // Mark this function for lazy recompilation. The function will be recompiled
@@ -193,7 +200,8 @@ class JSFunction : public TorqueGeneratedJSFunction<
 
   // Sets the interrupt budget based on whether the function has a feedback
   // vector and any optimized code.
-  void SetInterruptBudget(Isolate* isolate);
+  void SetInterruptBudget(Isolate* isolate,
+                          base::Optional<CodeKind> override_active_tier = {});
 
   // If slack tracking is active, it computes instance size of the initial map
   // with minimum permissible object slack.  If it is not active, it simply
@@ -207,17 +215,17 @@ class JSFunction : public TorqueGeneratedJSFunction<
   /// FeedbackVector eventually. Generally this shouldn't be used to get the
   // feedback_vector, instead use feedback_vector() which correctly deals with
   // the JSFunction's bytecode being flushed.
-  DECL_ACCESSORS(raw_feedback_cell, FeedbackCell)
+  DECL_ACCESSORS(raw_feedback_cell, Tagged<FeedbackCell>)
 
   // [raw_feedback_cell] (synchronized version) When this is initialized from a
   // newly allocated object (instead of a root sentinel), it should
   // be written with release store semantics.
-  DECL_RELEASE_ACQUIRE_ACCESSORS(raw_feedback_cell, FeedbackCell)
+  DECL_RELEASE_ACQUIRE_ACCESSORS(raw_feedback_cell, Tagged<FeedbackCell>)
 
   // Functions related to feedback vector. feedback_vector() can be used once
   // the function has feedback vectors allocated. feedback vectors may not be
   // available after compile when lazily allocating feedback vectors.
-  DECL_GETTER(feedback_vector, FeedbackVector)
+  DECL_GETTER(feedback_vector, Tagged<FeedbackVector>)
   DECL_GETTER(has_feedback_vector, bool)
   V8_EXPORT_PRIVATE static void EnsureFeedbackVector(
       Isolate* isolate, Handle<JSFunction> function,
@@ -231,7 +239,7 @@ class JSFunction : public TorqueGeneratedJSFunction<
   // cell arrays after compile, when we want to allocate feedback vectors
   // lazily.
   inline bool has_closure_feedback_cell_array() const;
-  inline ClosureFeedbackCellArray closure_feedback_cell_array() const;
+  inline Tagged<ClosureFeedbackCellArray> closure_feedback_cell_array() const;
   static void EnsureClosureFeedbackCellArray(
       Handle<JSFunction> function, bool reset_budget_for_feedback_allocation);
 
@@ -248,15 +256,17 @@ class JSFunction : public TorqueGeneratedJSFunction<
   void ClearAllTypeFeedbackInfoForTesting();
 
   // Resets function to clear compiled data after bytecode has been flushed.
-  inline bool NeedsResetDueToFlushedBytecode();
+  inline bool NeedsResetDueToFlushedBytecode(IsolateForSandbox isolate);
   inline void ResetIfCodeFlushed(
-      base::Optional<std::function<void(HeapObject object, ObjectSlot slot,
-                                        HeapObject target)>>
+      IsolateForSandbox isolate,
+      base::Optional<
+          std::function<void(Tagged<HeapObject> object, ObjectSlot slot,
+                             Tagged<HeapObject> target)>>
           gc_notify_updated_slot = base::nullopt);
 
   // Returns if the closure's code field has to be updated because it has
   // stale baseline code.
-  inline bool NeedsResetDueToFlushedBaselineCode();
+  inline bool NeedsResetDueToFlushedBaselineCode(IsolateForSandbox isolate);
 
   // Returns if baseline code is a candidate for flushing. This method is called
   // from concurrent marking so we should be careful when accessing data fields.
@@ -266,13 +276,14 @@ class JSFunction : public TorqueGeneratedJSFunction<
   DECL_GETTER(has_prototype_slot, bool)
 
   // The initial map for an object created by this constructor.
-  DECL_GETTER(initial_map, Map)
+  DECL_GETTER(initial_map, Tagged<Map>)
 
   static void SetInitialMap(Isolate* isolate, Handle<JSFunction> function,
                             Handle<Map> map, Handle<HeapObject> prototype);
   static void SetInitialMap(Isolate* isolate, Handle<JSFunction> function,
                             Handle<Map> map, Handle<HeapObject> prototype,
                             Handle<JSFunction> constructor);
+
   DECL_GETTER(has_initial_map, bool)
   V8_EXPORT_PRIVATE static void EnsureHasInitialMap(
       Handle<JSFunction> function);
@@ -285,24 +296,29 @@ class JSFunction : public TorqueGeneratedJSFunction<
       Handle<JSReceiver> new_target);
 
   // Like GetDerivedMap, but returns a map with a RAB / GSAB ElementsKind.
-  static V8_WARN_UNUSED_RESULT MaybeHandle<Map> GetDerivedRabGsabMap(
+  static V8_WARN_UNUSED_RESULT MaybeHandle<Map> GetDerivedRabGsabTypedArrayMap(
       Isolate* isolate, Handle<JSFunction> constructor,
       Handle<JSReceiver> new_target);
 
+  // Like GetDerivedMap, but can be used for DataViews for retrieving / creating
+  // a map with a JS_RAB_GSAB_DATA_VIEW instance type.
+  static V8_WARN_UNUSED_RESULT MaybeHandle<Map> GetDerivedRabGsabDataViewMap(
+      Isolate* isolate, Handle<JSReceiver> new_target);
+
   // Get and set the prototype property on a JSFunction. If the
   // function has an initial map the prototype is set on the initial
   // map. Otherwise, the prototype is put in the initial map field
   // until an initial map is needed.
   DECL_GETTER(has_prototype, bool)
   DECL_GETTER(has_instance_prototype, bool)
-  DECL_GETTER(prototype, Object)
-  DECL_GETTER(instance_prototype, HeapObject)
+  DECL_GETTER(prototype, Tagged<Object>)
+  DECL_GETTER(instance_prototype, Tagged<HeapObject>)
   DECL_GETTER(has_prototype_property, bool)
   DECL_GETTER(PrototypeRequiresRuntimeLookup, bool)
   static void SetPrototype(Handle<JSFunction> function, Handle<Object> value);
 
   // Returns if this function has been compiled to native code yet.
-  inline bool is_compiled() const;
+  inline bool is_compiled(IsolateForSandbox isolate) const;
 
   static int GetHeaderSize(bool function_has_prototype_slot) {
     return function_has_prototype_slot ? JSFunction::kSizeWithPrototype
@@ -353,10 +369,10 @@ class JSFunction : public TorqueGeneratedJSFunction<
   static const int kHeaderSize;
 
   // Hide generated accessors; custom accessors are called "shared".
-  DECL_ACCESSORS(shared_function_info, SharedFunctionInfo)
+  DECL_ACCESSORS(shared_function_info, Tagged<SharedFunctionInfo>)
 
   // Hide generated accessors; custom accessors are called "raw_feedback_cell".
-  DECL_ACCESSORS(feedback_cell, FeedbackCell)
+  DECL_ACCESSORS(feedback_cell, Tagged<FeedbackCell>)
 
   // Returns the set of code kinds of compilation artifacts (bytecode,
   // generated code) attached to this JSFunction.
@@ -365,12 +381,12 @@ class JSFunction : public TorqueGeneratedJSFunction<
   // TODO(jgruber): Currently at most one code kind can be attached. Consider
   // adding a NOT_COMPILED kind and changing this function to simply return the
   // kind if this becomes more convenient in the future.
-  CodeKinds GetAttachedCodeKinds() const;
+  CodeKinds GetAttachedCodeKinds(IsolateForSandbox isolate) const;
 
   // As above, but also considers locations outside of this JSFunction. For
   // example the optimized code cache slot in the feedback vector, and the
   // shared function info.
-  CodeKinds GetAvailableCodeKinds() const;
+  CodeKinds GetAvailableCodeKinds(IsolateForSandbox isolate) const;
 
  public:
   static constexpr int kSizeWithoutPrototype = kPrototypeOrInitialMapOffset;
