@@ -16,17 +16,20 @@
 namespace v8 {
 namespace internal {
 
+#include "torque-generated/src/objects/js-promise-tq-inl.inc"
+
 TQ_OBJECT_CONSTRUCTORS_IMPL(JSPromise)
 
-BOOL_ACCESSORS(JSPromise, flags, has_handler, kHasHandlerBit)
-BOOL_ACCESSORS(JSPromise, flags, handled_hint, kHandledHintBit)
+BOOL_ACCESSORS(JSPromise, flags, has_handler, HasHandlerBit::kShift)
+BOOL_ACCESSORS(JSPromise, flags, handled_hint, HandledHintBit::kShift)
+BOOL_ACCESSORS(JSPromise, flags, is_silent, IsSilentBit::kShift)
 
-Object JSPromise::result() const {
+Tagged<Object> JSPromise::result() const {
   DCHECK_NE(Promise::kPending, status());
   return reactions_or_result();
 }
 
-Object JSPromise::reactions() const {
+Tagged<Object> JSPromise::reactions() const {
   DCHECK_EQ(Promise::kPending, status());
   return reactions_or_result();
 }
