@@ -77,14 +77,16 @@ static Handle<FixedArray> CombineKeys(Isolate* isolate,
   Handle<FixedArray> combined_keys = isolate->factory()->NewFixedArray(
       own_keys_length + prototype_chain_keys_length);
   if (own_keys_length != 0) {
-    own_keys->CopyTo(0, *combined_keys, 0, own_keys_length);
+    FixedArray::CopyElements(isolate, *combined_keys, 0, *own_keys, 0,
+                             own_keys_length);
   }
   int target_keys_length = own_keys_length;
   for (int i = 0; i < prototype_chain_keys_length; i++) {
     target_keys_length += AddKey(prototype_chain_keys->get(i), combined_keys,
                                  descs, nof_descriptors, target_keys_length);
   }
-  return FixedArray::ShrinkOrEmpty(isolate, combined_keys, target_keys_length);
+  return FixedArray::RightTrimOrEmpty(isolate, combined_keys,
+                                      target_keys_length);
 }
 
 }  // namespace
@@ -153,7 +155,7 @@ ExceptionStatus KeyAccumulator::AddKey(Handle<Object> key,
       OrderedHashSet::Add(isolate(), keys(), key);
   Handle<OrderedHashSet> new_set;
   if (!new_set_candidate.ToHandle(&new_set)) {
-    CHECK(isolate_->has_pending_exception());
+    CHECK(isolate_->has_exception());
     return ExceptionStatus::kException;
   }
   if (*new_set != *keys_) {
@@ -218,7 +220,7 @@ MaybeHandle<FixedArray> FilterProxyKeys(KeyAccumulator* accumulator,
     }
     store_position++;
   }
-  return FixedArray::ShrinkOrEmpty(isolate, keys, store_position);
+  return FixedArray::RightTrimOrEmpty(isolate, keys, store_position);
 }
 
 // Returns "nothing" in case of exception, "true" on success.
@@ -277,8 +279,8 @@ Maybe<bool> KeyAccumulator::CollectKeys(Handle<JSReceiver> receiver,
     }
     MAYBE_RETURN(result, Nothing<bool>());
     if (!result.FromJust()) break;  // |false| means "stop iterating".
-    // Iterate through proxies but ignore access checks for the ALL_CAN_READ
-    // case on API objects for OWN_ONLY keys handled in CollectOwnKeys.
+    // Iterate through proxies but ignore access checks case on API objects for
+    // OWN_ONLY keys handled in CollectOwnKeys.
     if (!iter.AdvanceFollowingProxiesIgnoringAccessChecks()) {
       return Nothing<bool>();
     }
@@ -455,7 +457,7 @@ MaybeHandle<FixedArray> FastKeyAccumulator::GetKeys(
     if (GetKeysFast(keys_conversion).ToHandle(&keys)) {
       return keys;
     }
-    if (isolate_->has_pending_exception()) return MaybeHandle<FixedArray>();
+    if (isolate_->has_exception()) return MaybeHandle<FixedArray>();
   }
 
   if (try_prototype_info_cache_) {
@@ -740,8 +742,7 @@ Maybe<bool> KeyAccumulator::CollectInterceptorKeysInternal(
       result = enum_args.CallNamedEnumerator(interceptor);
     }
   }
-  RETURN_VALUE_IF_SCHEDULED_EXCEPTION_DETECTOR(isolate_, enum_args,
-                                               Nothing<bool>());
+  RETURN_VALUE_IF_EXCEPTION_DETECTOR(isolate_, enum_args, Nothing<bool>());
   if (result.is_null()) return Just(true);
 
   // Request was successfully intercepted, so accept potential side effects
@@ -771,9 +772,6 @@ Maybe<bool> KeyAccumulator::CollectInterceptorKeys(Handle<JSReceiver> receiver,
                                           ? object->GetIndexedInterceptor()
                                           : object->GetNamedInterceptor(),
                                       isolate_);
-  if ((filter() & ONLY_ALL_CAN_READ) && !interceptor->all_can_read()) {
-    return Just(true);
-  }
   return CollectInterceptorKeysInternal(receiver, object, interceptor, type);
 }
 
@@ -809,13 +807,6 @@ base::Optional<int> CollectOwnPropertyNamesInternal(
       }
     }
 
-    if (filter & ONLY_ALL_CAN_READ) {
-      if (details.kind() != PropertyKind::kAccessor) continue;
-      Tagged<Object> accessors = descs->GetStrongValue(i);
-      if (!IsAccessorInfo(accessors)) continue;
-      if (!AccessorInfo::cast(accessors)->all_can_read()) continue;
-    }
-
     Tagged<Name> key = descs->GetKey(i);
     if (skip_symbols == IsSymbol(key)) {
       if (first_skipped == -1) first_skipped = i.as_int();
@@ -897,16 +888,16 @@ void CopyEnumKeysTo(Isolate* isolate, Handle<Dictionary> dictionary,
   int length = storage->length();
 
   DisallowGarbageCollection no_gc;
-  Dictionary raw_dictionary = *dictionary;
+  Tagged<Dictionary> raw_dictionary = *dictionary;
   Tagged<FixedArray> raw_storage = *storage;
   EnumIndexComparator<Dictionary> cmp(raw_dictionary);
   // Use AtomicSlot wrapper to ensure that std::sort uses atomic load and
   // store operations that are safe for concurrent marking.
-  AtomicSlot start(storage->GetFirstElementAddress());
+  AtomicSlot start(storage->RawFieldOfFirstElement());
   std::sort(start, start + length, cmp);
   for (int i = 0; i < length; i++) {
     InternalIndex index(Smi::ToInt(raw_storage->get(i)));
-    raw_storage->set(i, raw_dictionary.NameAt(index));
+    raw_storage->set(i, raw_dictionary->NameAt(index));
   }
 }
 
@@ -957,22 +948,16 @@ ExceptionStatus CollectKeysFromDictionary(Handle<Dictionary> dictionary,
     DisallowGarbageCollection no_gc;
     for (InternalIndex i : dictionary->IterateEntries()) {
       Tagged<Object> key;
-      Dictionary raw_dictionary = *dictionary;
-      if (!raw_dictionary.ToKey(roots, i, &key)) continue;
+      Tagged<Dictionary> raw_dictionary = *dictionary;
+      if (!raw_dictionary->ToKey(roots, i, &key)) continue;
       if (Object::FilterKey(key, filter)) continue;
-      PropertyDetails details = raw_dictionary.DetailsAt(i);
+      PropertyDetails details = raw_dictionary->DetailsAt(i);
       if ((int{details.attributes()} & filter) != 0) {
         AllowGarbageCollection gc;
         // This might allocate, but {key} is not used afterwards.
         keys->AddShadowingKey(key, &gc);
         continue;
       }
-      if (filter & ONLY_ALL_CAN_READ) {
-        if (details.kind() != PropertyKind::kAccessor) continue;
-        Tagged<Object> accessors = raw_dictionary.ValueAt(i);
-        if (!IsAccessorInfo(accessors)) continue;
-        if (!AccessorInfo::cast(accessors)->all_can_read()) continue;
-      }
       // TODO(emrich): consider storing keys instead of indices into the array
       // in case of ordered dictionary type.
       array->set(array_size++, Smi::FromInt(i.as_int()));
@@ -984,7 +969,7 @@ ExceptionStatus CollectKeysFromDictionary(Handle<Dictionary> dictionary,
       EnumIndexComparator<Dictionary> cmp(*dictionary);
       // Use AtomicSlot wrapper to ensure that std::sort uses atomic load and
       // store operations that are safe for concurrent marking.
-      AtomicSlot start(array->GetFirstElementAddress());
+      AtomicSlot start(array->RawFieldOfFirstElement());
       std::sort(start, start + array_size, cmp);
     }
   }
@@ -1161,13 +1146,12 @@ Maybe<bool> KeyAccumulator::CollectOwnKeys(Handle<JSReceiver> receiver,
     }
     // We always have both kinds of interceptors or none.
     if (!access_check_info.is_null() &&
-        access_check_info->named_interceptor() != Object()) {
+        access_check_info->named_interceptor() != Tagged<Object>()) {
       MAYBE_RETURN(CollectAccessCheckInterceptorKeys(access_check_info,
                                                      receiver, object),
                    Nothing<bool>());
-      return Just(false);
     }
-    filter_ = static_cast<PropertyFilter>(filter_ | ONLY_ALL_CAN_READ);
+    return Just(false);
   }
   if (filter_ & PRIVATE_NAMES_ONLY) {
     RETURN_NOTHING_IF_NOT_SUCCESSFUL(CollectPrivateNames(receiver, object));
