@@ -20,10 +20,19 @@
 namespace v8 {
 namespace internal {
 
-#include "torque-generated/src/objects/name-tq-inl.inc"
+inline Tagged<Symbol> Symbol::cast(Tagged<Object> obj) {
+  SLOW_DCHECK(IsSymbol(obj));
+  return Symbol::unchecked_cast(obj);
+}
 
-TQ_OBJECT_CONSTRUCTORS_IMPL(Name)
-TQ_OBJECT_CONSTRUCTORS_IMPL(Symbol)
+Tagged<PrimitiveHeapObject> Symbol::description() const {
+  return description_.load();
+}
+void Symbol::set_description(Tagged<PrimitiveHeapObject> value,
+                             WriteBarrierMode mode) {
+  SLOW_DCHECK(IsString(value) || IsUndefined(value));
+  description_.store(this, value, mode);
+}
 
 BIT_FIELD_ACCESSORS(Symbol, flags, is_private, Symbol::IsPrivateBit)
 BIT_FIELD_ACCESSORS(Symbol, flags, is_well_known_symbol,
@@ -58,8 +67,13 @@ void Symbol::set_is_private_name() {
   set_flags(Symbol::IsPrivateNameBit::update(flags(), true));
 }
 
+inline Tagged<Name> Name::cast(Tagged<Object> obj) {
+  SLOW_DCHECK(IsName(obj));
+  return Name::unchecked_cast(obj);
+}
+
 DEF_HEAP_OBJECT_PREDICATE(Name, IsUniqueName) {
-  uint32_t type = obj->map(cage_base)->instance_type();
+  uint32_t type = obj->map()->instance_type();
   bool result = (type & (kIsNotStringMask | kIsNotInternalizedMask)) !=
                 (kStringTag | kNotInternalizedTag);
   SLOW_DCHECK(result == IsUniqueName(Tagged<HeapObject>::cast(obj)));
@@ -68,12 +82,12 @@ DEF_HEAP_OBJECT_PREDICATE(Name, IsUniqueName) {
 }
 
 bool Name::Equals(Tagged<Name> other) {
-  if (other == *this) return true;
-  if ((IsInternalizedString(*this) && IsInternalizedString(other)) ||
-      IsSymbol(*this) || IsSymbol(other)) {
+  if (other == this) return true;
+  if ((IsInternalizedString(this) && IsInternalizedString(other)) ||
+      IsSymbol(this) || IsSymbol(other)) {
     return false;
   }
-  return String::cast(*this)->SlowEquals(String::cast(other));
+  return String::cast(this)->SlowEquals(String::cast(other));
 }
 
 bool Name::Equals(Isolate* isolate, Handle<Name> one, Handle<Name> two) {
@@ -161,7 +175,7 @@ bool Name::HasExternalForwardingIndex(AcquireLoadTag) const {
 uint32_t Name::GetRawHashFromForwardingTable(uint32_t raw_hash) const {
   DCHECK(IsForwardingIndex(raw_hash));
   // TODO(pthier): Add parameter for isolate so we don't need to calculate it.
-  Isolate* isolate = GetIsolateFromWritableObject(*this);
+  Isolate* isolate = GetIsolateFromWritableObject(this);
   const int index = ForwardingIndexValueBits::decode(raw_hash);
   return isolate->string_forwarding_table()->GetRawHash(isolate, index);
 }
@@ -175,7 +189,7 @@ uint32_t Name::EnsureRawHash() {
     return GetRawHashFromForwardingTable(field);
   }
   // Slow case: compute hash code and set it. Has to be a string.
-  return String::cast(*this)->ComputeAndSetRawHash();
+  return String::cast(this)->ComputeAndSetRawHash();
 }
 
 uint32_t Name::EnsureRawHash(
@@ -188,7 +202,7 @@ uint32_t Name::EnsureRawHash(
     return GetRawHashFromForwardingTable(field);
   }
   // Slow case: compute hash code and set it. Has to be a string.
-  return String::cast(*this)->ComputeAndSetRawHash(access_guard);
+  return String::cast(this)->ComputeAndSetRawHash(access_guard);
 }
 
 uint32_t Name::RawHash() {
@@ -206,19 +220,17 @@ uint32_t Name::EnsureHash(const SharedStringAccessGuardIfNeeded& access_guard) {
 }
 
 void Name::set_raw_hash_field_if_empty(uint32_t hash) {
-  uint32_t result = base::AsAtomic32::Release_CompareAndSwap(
-      reinterpret_cast<uint32_t*>(FIELD_ADDR(*this, kRawHashFieldOffset)),
-      kEmptyHashField, hash);
+  uint32_t field_value = kEmptyHashField;
+  bool result = raw_hash_field_.compare_exchange_strong(field_value, hash);
   USE(result);
   // CAS can only fail if the string is shared or we use the forwarding table
   // for all strings and the hash was already set (by another thread) or it is
   // a forwarding index (that overwrites the previous hash).
   // In all cases we don't want overwrite the old value, so we don't handle the
   // failure case.
-  DCHECK_IMPLIES(result != kEmptyHashField,
-                 (String::cast(*this)->IsShared() ||
-                  v8_flags.always_use_string_forwarding_table) &&
-                     (result == hash || IsForwardingIndex(hash)));
+  DCHECK_IMPLIES(!result, (String::cast(this)->IsShared() ||
+                           v8_flags.always_use_string_forwarding_table) &&
+                              (field_value == hash || IsForwardingIndex(hash)));
 }
 
 uint32_t Name::hash() const {
@@ -246,35 +258,35 @@ bool Name::TryGetHash(uint32_t* hash) const {
 bool Name::IsInteresting(Isolate* isolate) {
   // TODO(ishell): consider using ReadOnlyRoots::IsNameForProtector() trick for
   // these strings and interesting symbols.
-  return (IsSymbol(*this) && Symbol::cast(*this)->is_interesting_symbol()) ||
-         *this == *isolate->factory()->toJSON_string() ||
-         *this == *isolate->factory()->get_string();
+  return (IsSymbol(this) && Symbol::cast(this)->is_interesting_symbol()) ||
+         this == *isolate->factory()->toJSON_string() ||
+         this == *isolate->factory()->get_string();
 }
 
-DEF_GETTER(Name, IsPrivate, bool) {
-  return IsSymbol(*this, cage_base) && Symbol::cast(*this)->is_private();
+bool Name::IsPrivate() {
+  return IsSymbol(this) && Symbol::cast(this)->is_private();
 }
 
-DEF_GETTER(Name, IsPrivateName, bool) {
+bool Name::IsPrivateName() {
   bool is_private_name =
-      IsSymbol(*this, cage_base) && Symbol::cast(*this)->is_private_name();
+      IsSymbol(this) && Symbol::cast(this)->is_private_name();
   DCHECK_IMPLIES(is_private_name, IsPrivate());
   return is_private_name;
 }
 
-DEF_GETTER(Name, IsPrivateBrand, bool) {
+bool Name::IsPrivateBrand() {
   bool is_private_brand =
-      IsSymbol(*this, cage_base) && Symbol::cast(*this)->is_private_brand();
+      IsSymbol(this) && Symbol::cast(this)->is_private_brand();
   DCHECK_IMPLIES(is_private_brand, IsPrivateName());
   return is_private_brand;
 }
 
 bool Name::AsArrayIndex(uint32_t* index) {
-  return IsString(*this) && String::cast(*this)->AsArrayIndex(index);
+  return IsString(this) && String::cast(this)->AsArrayIndex(index);
 }
 
 bool Name::AsIntegerIndex(size_t* index) {
-  return IsString(*this) && String::cast(*this)->AsIntegerIndex(index);
+  return IsString(this) && String::cast(this)->AsIntegerIndex(index);
 }
 
 // static
