@@ -5,24 +5,96 @@
 #ifndef V8_OBJECTS_TAGGED_FIELD_INL_H_
 #define V8_OBJECTS_TAGGED_FIELD_INL_H_
 
-#include "src/objects/tagged-field.h"
-
 #include "src/common/ptr-compr-inl.h"
+#include "src/objects/tagged-field.h"
+#include "src/objects/tagged.h"
 
 namespace v8 {
 namespace internal {
 
+// static
+template <typename T, typename CompressionScheme>
+Address TaggedMember<T, CompressionScheme>::tagged_to_full(
+    Tagged_t tagged_value) {
+#ifdef V8_COMPRESS_POINTERS
+  if constexpr (std::is_same_v<Smi, T>) {
+    V8_ASSUME(HAS_SMI_TAG(tagged_value));
+    return CompressionScheme::DecompressTaggedSigned(tagged_value);
+  } else {
+    return CompressionScheme::DecompressTagged(GetPtrComprCageBase(),
+                                               tagged_value);
+  }
+#else
+  return tagged_value;
+#endif
+}
+
+// static
+template <typename T, typename CompressionScheme>
+Tagged_t TaggedMember<T, CompressionScheme>::full_to_tagged(Address value) {
+#ifdef V8_COMPRESS_POINTERS
+  return CompressionScheme::CompressObject(value);
+#else
+  return value;
+#endif
+}
+
+template <typename T, typename CompressionScheme>
+Tagged<T> TaggedMember<T, CompressionScheme>::load() const {
+  return Tagged<T>(tagged_to_full(ptr()));
+}
+
+template <typename T, typename CompressionScheme>
+void TaggedMember<T, CompressionScheme>::store(HeapObjectLayout* host,
+                                               Tagged<T> value,
+                                               WriteBarrierMode mode) {
+  store_no_write_barrier(value);
+
+#ifndef V8_DISABLE_WRITE_BARRIERS
+  if constexpr (!std::is_same_v<Smi, T>) {
+#if V8_ENABLE_UNCONDITIONAL_WRITE_BARRIERS
+    mode = UPDATE_WRITE_BARRIER;
+#endif
+    DCHECK_NOT_NULL(GetHeapFromWritableObject(Tagged(host)));
+    CombinedWriteBarrier(host, this, value, mode);
+  }
+#endif
+}
+
+template <typename T, typename CompressionScheme>
+Tagged<T> TaggedMember<T, CompressionScheme>::Relaxed_Load() const {
+  return Tagged<T>(
+      tagged_to_full(AsAtomicTagged::Relaxed_Load(this->ptr_location())));
+}
+
+template <typename T, typename CompressionScheme>
+void TaggedMember<T, CompressionScheme>::store_no_write_barrier(
+    Tagged<T> value) {
+#ifdef V8_ATOMIC_OBJECT_FIELD_WRITES
+  Relaxed_Store_no_write_barrier(value);
+#else
+  *this->ptr_location() = full_to_tagged(value.ptr());
+#endif
+}
+
+template <typename T, typename CompressionScheme>
+void TaggedMember<T, CompressionScheme>::Relaxed_Store_no_write_barrier(
+    Tagged<T> value) {
+  AsAtomicTagged::Relaxed_Store(this->ptr_location(),
+                                full_to_tagged(value.ptr()));
+}
+
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
 Address TaggedField<T, kFieldOffset, CompressionScheme>::address(
-    HeapObject host, int offset) {
+    Tagged<HeapObject> host, int offset) {
   return host.address() + kFieldOffset + offset;
 }
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
 Tagged_t* TaggedField<T, kFieldOffset, CompressionScheme>::location(
-    HeapObject host, int offset) {
+    Tagged<HeapObject> host, int offset) {
   return reinterpret_cast<Tagged_t*>(address(host, offset));
 }
 
@@ -32,13 +104,11 @@ template <typename TOnHeapAddress>
 Address TaggedField<T, kFieldOffset, CompressionScheme>::tagged_to_full(
     TOnHeapAddress on_heap_addr, Tagged_t tagged_value) {
 #ifdef V8_COMPRESS_POINTERS
-  if (kIsSmi) {
+  if constexpr (kIsSmi) {
+    V8_ASSUME(HAS_SMI_TAG(tagged_value));
     return CompressionScheme::DecompressTaggedSigned(tagged_value);
-  } else if (kIsHeapObject) {
-    return CompressionScheme::DecompressTaggedPointer(on_heap_addr,
-                                                      tagged_value);
   } else {
-    return CompressionScheme::DecompressTaggedAny(on_heap_addr, tagged_value);
+    return CompressionScheme::DecompressTagged(on_heap_addr, tagged_value);
   }
 #else
   return tagged_value;
@@ -50,7 +120,8 @@ template <typename T, int kFieldOffset, typename CompressionScheme>
 Tagged_t TaggedField<T, kFieldOffset, CompressionScheme>::full_to_tagged(
     Address value) {
 #ifdef V8_COMPRESS_POINTERS
-  return CompressionScheme::CompressTagged(value);
+  if constexpr (kIsSmi) V8_ASSUME(HAS_SMI_TAG(value));
+  return CompressionScheme::CompressObject(value);
 #else
   return value;
 #endif
@@ -58,26 +129,28 @@ Tagged_t TaggedField<T, kFieldOffset, CompressionScheme>::full_to_tagged(
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
-T TaggedField<T, kFieldOffset, CompressionScheme>::load(HeapObject host,
-                                                        int offset) {
+typename TaggedField<T, kFieldOffset, CompressionScheme>::PtrType
+TaggedField<T, kFieldOffset, CompressionScheme>::load(Tagged<HeapObject> host,
+                                                      int offset) {
   Tagged_t value = *location(host, offset);
   DCHECK_NE(kFieldOffset + offset, HeapObject::kMapOffset);
-  return T(tagged_to_full(host.ptr(), value));
+  return PtrType(tagged_to_full(host.ptr(), value));
 }
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
-T TaggedField<T, kFieldOffset, CompressionScheme>::load(
-    PtrComprCageBase cage_base, HeapObject host, int offset) {
+typename TaggedField<T, kFieldOffset, CompressionScheme>::PtrType
+TaggedField<T, kFieldOffset, CompressionScheme>::load(
+    PtrComprCageBase cage_base, Tagged<HeapObject> host, int offset) {
   Tagged_t value = *location(host, offset);
   DCHECK_NE(kFieldOffset + offset, HeapObject::kMapOffset);
-  return T(tagged_to_full(cage_base, value));
+  return PtrType(tagged_to_full(cage_base, value));
 }
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
-void TaggedField<T, kFieldOffset, CompressionScheme>::store(HeapObject host,
-                                                            T value) {
+void TaggedField<T, kFieldOffset, CompressionScheme>::store(
+    Tagged<HeapObject> host, PtrType value) {
 #ifdef V8_ATOMIC_OBJECT_FIELD_WRITES
   Relaxed_Store(host, value);
 #else
@@ -89,9 +162,8 @@ void TaggedField<T, kFieldOffset, CompressionScheme>::store(HeapObject host,
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
-void TaggedField<T, kFieldOffset, CompressionScheme>::store(HeapObject host,
-                                                            int offset,
-                                                            T value) {
+void TaggedField<T, kFieldOffset, CompressionScheme>::store(
+    Tagged<HeapObject> host, int offset, PtrType value) {
 #ifdef V8_ATOMIC_OBJECT_FIELD_WRITES
   Relaxed_Store(host, offset, value);
 #else
@@ -103,41 +175,44 @@ void TaggedField<T, kFieldOffset, CompressionScheme>::store(HeapObject host,
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
-T TaggedField<T, kFieldOffset, CompressionScheme>::Relaxed_Load(HeapObject host,
-                                                                int offset) {
+typename TaggedField<T, kFieldOffset, CompressionScheme>::PtrType
+TaggedField<T, kFieldOffset, CompressionScheme>::Relaxed_Load(
+    Tagged<HeapObject> host, int offset) {
   AtomicTagged_t value = AsAtomicTagged::Relaxed_Load(location(host, offset));
   DCHECK_NE(kFieldOffset + offset, HeapObject::kMapOffset);
-  return T(tagged_to_full(host.ptr(), value));
+  return PtrType(tagged_to_full(host.ptr(), value));
 }
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
-T TaggedField<T, kFieldOffset, CompressionScheme>::Relaxed_Load(
-    PtrComprCageBase cage_base, HeapObject host, int offset) {
+typename TaggedField<T, kFieldOffset, CompressionScheme>::PtrType
+TaggedField<T, kFieldOffset, CompressionScheme>::Relaxed_Load(
+    PtrComprCageBase cage_base, Tagged<HeapObject> host, int offset) {
   AtomicTagged_t value = AsAtomicTagged::Relaxed_Load(location(host, offset));
   DCHECK_NE(kFieldOffset + offset, HeapObject::kMapOffset);
-  return T(tagged_to_full(cage_base, value));
+  return PtrType(tagged_to_full(cage_base, value));
 }
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
-T TaggedField<T, kFieldOffset, CompressionScheme>::Relaxed_Load_Map_Word(
-    PtrComprCageBase cage_base, HeapObject host) {
+typename TaggedField<T, kFieldOffset, CompressionScheme>::PtrType
+TaggedField<T, kFieldOffset, CompressionScheme>::Relaxed_Load_Map_Word(
+    PtrComprCageBase cage_base, Tagged<HeapObject> host) {
   AtomicTagged_t value = AsAtomicTagged::Relaxed_Load(location(host, 0));
-  return T(tagged_to_full(cage_base, value));
+  return PtrType(tagged_to_full(cage_base, value));
 }
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
 void TaggedField<T, kFieldOffset, CompressionScheme>::Relaxed_Store_Map_Word(
-    HeapObject host, T value) {
+    Tagged<HeapObject> host, PtrType value) {
   AsAtomicTagged::Relaxed_Store(location(host), full_to_tagged(value.ptr()));
 }
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
 void TaggedField<T, kFieldOffset, CompressionScheme>::Relaxed_Store(
-    HeapObject host, T value) {
+    Tagged<HeapObject> host, PtrType value) {
   Address ptr = value.ptr();
   DCHECK_NE(kFieldOffset, HeapObject::kMapOffset);
   AsAtomicTagged::Relaxed_Store(location(host), full_to_tagged(ptr));
@@ -146,7 +221,7 @@ void TaggedField<T, kFieldOffset, CompressionScheme>::Relaxed_Store(
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
 void TaggedField<T, kFieldOffset, CompressionScheme>::Relaxed_Store(
-    HeapObject host, int offset, T value) {
+    Tagged<HeapObject> host, int offset, PtrType value) {
   Address ptr = value.ptr();
   DCHECK_NE(kFieldOffset + offset, HeapObject::kMapOffset);
   AsAtomicTagged::Relaxed_Store(location(host, offset), full_to_tagged(ptr));
@@ -154,33 +229,36 @@ void TaggedField<T, kFieldOffset, CompressionScheme>::Relaxed_Store(
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
-T TaggedField<T, kFieldOffset, CompressionScheme>::Acquire_Load(HeapObject host,
-                                                                int offset) {
+typename TaggedField<T, kFieldOffset, CompressionScheme>::PtrType
+TaggedField<T, kFieldOffset, CompressionScheme>::Acquire_Load(
+    Tagged<HeapObject> host, int offset) {
   AtomicTagged_t value = AsAtomicTagged::Acquire_Load(location(host, offset));
   DCHECK_NE(kFieldOffset + offset, HeapObject::kMapOffset);
-  return T(tagged_to_full(host.ptr(), value));
+  return PtrType(tagged_to_full(host.ptr(), value));
 }
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
-T TaggedField<T, kFieldOffset, CompressionScheme>::Acquire_Load_No_Unpack(
-    PtrComprCageBase cage_base, HeapObject host, int offset) {
+typename TaggedField<T, kFieldOffset, CompressionScheme>::PtrType
+TaggedField<T, kFieldOffset, CompressionScheme>::Acquire_Load_No_Unpack(
+    PtrComprCageBase cage_base, Tagged<HeapObject> host, int offset) {
   AtomicTagged_t value = AsAtomicTagged::Acquire_Load(location(host, offset));
-  return T(tagged_to_full(cage_base, value));
+  return PtrType(tagged_to_full(cage_base, value));
 }
 
 template <typename T, int kFieldOffset, typename CompressionScheme>
-T TaggedField<T, kFieldOffset, CompressionScheme>::Acquire_Load(
-    PtrComprCageBase cage_base, HeapObject host, int offset) {
+typename TaggedField<T, kFieldOffset, CompressionScheme>::PtrType
+TaggedField<T, kFieldOffset, CompressionScheme>::Acquire_Load(
+    PtrComprCageBase cage_base, Tagged<HeapObject> host, int offset) {
   AtomicTagged_t value = AsAtomicTagged::Acquire_Load(location(host, offset));
   DCHECK_NE(kFieldOffset + offset, HeapObject::kMapOffset);
-  return T(tagged_to_full(cage_base, value));
+  return PtrType(tagged_to_full(cage_base, value));
 }
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
 void TaggedField<T, kFieldOffset, CompressionScheme>::Release_Store(
-    HeapObject host, T value) {
+    Tagged<HeapObject> host, PtrType value) {
   Address ptr = value.ptr();
   DCHECK_NE(kFieldOffset, HeapObject::kMapOffset);
   AsAtomicTagged::Release_Store(location(host), full_to_tagged(ptr));
@@ -189,7 +267,7 @@ void TaggedField<T, kFieldOffset, CompressionScheme>::Release_Store(
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
 void TaggedField<T, kFieldOffset, CompressionScheme>::Release_Store_Map_Word(
-    HeapObject host, T value) {
+    Tagged<HeapObject> host, PtrType value) {
   Address ptr = value.ptr();
   AsAtomicTagged::Release_Store(location(host), full_to_tagged(ptr));
 }
@@ -197,7 +275,7 @@ void TaggedField<T, kFieldOffset, CompressionScheme>::Release_Store_Map_Word(
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
 void TaggedField<T, kFieldOffset, CompressionScheme>::Release_Store(
-    HeapObject host, int offset, T value) {
+    Tagged<HeapObject> host, int offset, PtrType value) {
   Address ptr = value.ptr();
   DCHECK_NE(kFieldOffset + offset, HeapObject::kMapOffset);
   AsAtomicTagged::Release_Store(location(host, offset), full_to_tagged(ptr));
@@ -205,10 +283,9 @@ void TaggedField<T, kFieldOffset, CompressionScheme>::Release_Store(
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
-Tagged_t TaggedField<T, kFieldOffset,
-                     CompressionScheme>::Release_CompareAndSwap(HeapObject host,
-                                                                T old,
-                                                                T value) {
+Tagged_t
+TaggedField<T, kFieldOffset, CompressionScheme>::Release_CompareAndSwap(
+    Tagged<HeapObject> host, PtrType old, PtrType value) {
   Tagged_t old_value = full_to_tagged(old.ptr());
   Tagged_t new_value = full_to_tagged(value.ptr());
   Tagged_t result = AsAtomicTagged::Release_CompareAndSwap(
@@ -218,26 +295,28 @@ Tagged_t TaggedField<T, kFieldOffset,
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
-T TaggedField<T, kFieldOffset, CompressionScheme>::SeqCst_Load(HeapObject host,
-                                                               int offset) {
+typename TaggedField<T, kFieldOffset, CompressionScheme>::PtrType
+TaggedField<T, kFieldOffset, CompressionScheme>::SeqCst_Load(
+    Tagged<HeapObject> host, int offset) {
   AtomicTagged_t value = AsAtomicTagged::SeqCst_Load(location(host, offset));
   DCHECK_NE(kFieldOffset + offset, HeapObject::kMapOffset);
-  return T(tagged_to_full(host.ptr(), value));
+  return PtrType(tagged_to_full(host.ptr(), value));
 }
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
-T TaggedField<T, kFieldOffset, CompressionScheme>::SeqCst_Load(
-    PtrComprCageBase cage_base, HeapObject host, int offset) {
+typename TaggedField<T, kFieldOffset, CompressionScheme>::PtrType
+TaggedField<T, kFieldOffset, CompressionScheme>::SeqCst_Load(
+    PtrComprCageBase cage_base, Tagged<HeapObject> host, int offset) {
   AtomicTagged_t value = AsAtomicTagged::SeqCst_Load(location(host, offset));
   DCHECK_NE(kFieldOffset + offset, HeapObject::kMapOffset);
-  return T(tagged_to_full(cage_base, value));
+  return PtrType(tagged_to_full(cage_base, value));
 }
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
 void TaggedField<T, kFieldOffset, CompressionScheme>::SeqCst_Store(
-    HeapObject host, T value) {
+    Tagged<HeapObject> host, PtrType value) {
   Address ptr = value.ptr();
   DCHECK_NE(kFieldOffset, HeapObject::kMapOffset);
   AsAtomicTagged::SeqCst_Store(location(host), full_to_tagged(ptr));
@@ -246,7 +325,7 @@ void TaggedField<T, kFieldOffset, CompressionScheme>::SeqCst_Store(
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
 void TaggedField<T, kFieldOffset, CompressionScheme>::SeqCst_Store(
-    HeapObject host, int offset, T value) {
+    Tagged<HeapObject> host, int offset, PtrType value) {
   Address ptr = value.ptr();
   DCHECK_NE(kFieldOffset + offset, HeapObject::kMapOffset);
   AsAtomicTagged::SeqCst_Store(location(host, offset), full_to_tagged(ptr));
@@ -254,25 +333,43 @@ void TaggedField<T, kFieldOffset, CompressionScheme>::SeqCst_Store(
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
-T TaggedField<T, kFieldOffset, CompressionScheme>::SeqCst_Swap(HeapObject host,
-                                                               int offset,
-                                                               T value) {
+
+typename TaggedField<T, kFieldOffset, CompressionScheme>::PtrType
+TaggedField<T, kFieldOffset, CompressionScheme>::SeqCst_Swap(
+    Tagged<HeapObject> host, int offset, PtrType value) {
   Address ptr = value.ptr();
   DCHECK_NE(kFieldOffset + offset, HeapObject::kMapOffset);
   AtomicTagged_t old_value =
       AsAtomicTagged::SeqCst_Swap(location(host, offset), full_to_tagged(ptr));
-  return T(tagged_to_full(host.ptr(), old_value));
+  return PtrType(tagged_to_full(host.ptr(), old_value));
 }
 
 // static
 template <typename T, int kFieldOffset, typename CompressionScheme>
-T TaggedField<T, kFieldOffset, CompressionScheme>::SeqCst_Swap(
-    PtrComprCageBase cage_base, HeapObject host, int offset, T value) {
+
+typename TaggedField<T, kFieldOffset, CompressionScheme>::PtrType
+TaggedField<T, kFieldOffset, CompressionScheme>::SeqCst_Swap(
+    PtrComprCageBase cage_base, Tagged<HeapObject> host, int offset,
+    PtrType value) {
   Address ptr = value.ptr();
   DCHECK_NE(kFieldOffset + offset, HeapObject::kMapOffset);
   AtomicTagged_t old_value =
       AsAtomicTagged::SeqCst_Swap(location(host, offset), full_to_tagged(ptr));
-  return T(tagged_to_full(cage_base, old_value));
+  return PtrType(tagged_to_full(cage_base, old_value));
+}
+
+// static
+template <typename T, int kFieldOffset, typename CompressionScheme>
+typename TaggedField<T, kFieldOffset, CompressionScheme>::PtrType
+TaggedField<T, kFieldOffset, CompressionScheme>::SeqCst_CompareAndSwap(
+    Tagged<HeapObject> host, int offset, PtrType old, PtrType value) {
+  Address ptr = value.ptr();
+  Address old_ptr = old.ptr();
+  DCHECK_NE(kFieldOffset + offset, HeapObject::kMapOffset);
+  AtomicTagged_t old_value = AsAtomicTagged::SeqCst_CompareAndSwap(
+      location(host, offset), full_to_tagged(old_ptr), full_to_tagged(ptr));
+  return TaggedField<T, kFieldOffset, CompressionScheme>::PtrType(
+      tagged_to_full(host.ptr(), old_value));
 }
 
 }  // namespace internal
