@@ -19,6 +19,7 @@
 #include "src/builtins/builtins.h"
 #include "src/execution/vm-state.h"
 #include "src/logging/code-events.h"
+#include "src/profiler/output-stream-writer.h"
 #include "src/profiler/strings-storage.h"
 #include "src/utils/allocation.h"
 
@@ -124,13 +125,13 @@ class CodeEntry {
   }
 
   // Returns the start address of the instruction segment represented by this
-  // CodeEntry. Used as a key in the containing CodeMap.
+  // CodeEntry. Used as a key in the containing InstructionStreamMap.
   Address instruction_start() const { return instruction_start_; }
   void set_instruction_start(Address address) { instruction_start_ = address; }
 
   Address** heap_object_location_address() { return &heap_object_location_; }
 
-  void FillFunctionInfo(SharedFunctionInfo shared);
+  void FillFunctionInfo(Tagged<SharedFunctionInfo> shared);
 
   void SetBuiltinId(Builtin id);
   Builtin builtin() const { return BuiltinField::decode(bit_field_); }
@@ -140,7 +141,7 @@ class CodeEntry {
   }
 
   // Returns whether or not the lifetime of this CodeEntry is reference
-  // counted, and managed by a CodeMap.
+  // counted, and managed by a InstructionStreamMap.
   bool is_ref_counted() const { return RefCountedField::decode(bit_field_); }
 
   uint32_t GetHash() const;
@@ -483,18 +484,18 @@ class CpuProfileMaxSamplesCallbackTask : public v8::Task {
   std::unique_ptr<DiscardedSamplesDelegate> delegate_;
 };
 
-class V8_EXPORT_PRIVATE CodeMap {
+class V8_EXPORT_PRIVATE InstructionStreamMap {
  public:
-  explicit CodeMap(CodeEntryStorage& storage);
-  ~CodeMap();
-  CodeMap(const CodeMap&) = delete;
-  CodeMap& operator=(const CodeMap&) = delete;
+  explicit InstructionStreamMap(CodeEntryStorage& storage);
+  ~InstructionStreamMap();
+  InstructionStreamMap(const InstructionStreamMap&) = delete;
+  InstructionStreamMap& operator=(const InstructionStreamMap&) = delete;
 
-  // Adds the given CodeEntry to the CodeMap. The CodeMap takes ownership of
-  // the CodeEntry.
+  // Adds the given CodeEntry to the InstructionStreamMap. The
+  // InstructionStreamMap takes ownership of the CodeEntry.
   void AddCode(Address addr, CodeEntry* entry, unsigned size);
   void MoveCode(Address from, Address to);
-  // Attempts to remove the given CodeEntry from the CodeMap.
+  // Attempts to remove the given CodeEntry from the InstructionStreamMap.
   // Returns true iff the entry was found and removed.
   bool RemoveCode(CodeEntry*);
   void ClearCodesInRange(Address start, Address end);
@@ -558,7 +559,9 @@ class V8_EXPORT_PRIVATE CpuProfilesCollection {
   std::vector<std::unique_ptr<CpuProfile>>* profiles() {
     return &finished_profiles_;
   }
-  const char* GetName(Name name) { return resource_names_.GetName(name); }
+  const char* GetName(Tagged<Name> name) {
+    return resource_names_.GetName(name);
+  }
   void RemoveProfile(CpuProfile* profile);
 
   // Finds a common sampling interval dividing each CpuProfile's interval,
@@ -596,6 +599,31 @@ class V8_EXPORT_PRIVATE CpuProfilesCollection {
   Isolate* isolate_;
 };
 
+class CpuProfileJSONSerializer {
+ public:
+  explicit CpuProfileJSONSerializer(CpuProfile* profile)
+      : profile_(profile), writer_(nullptr) {}
+  CpuProfileJSONSerializer(const CpuProfileJSONSerializer&) = delete;
+  CpuProfileJSONSerializer& operator=(const CpuProfileJSONSerializer&) = delete;
+  void Serialize(v8::OutputStream* stream);
+
+ private:
+  void SerializePositionTicks(const v8::CpuProfileNode* node, int lineCount);
+  void SerializeCallFrame(const v8::CpuProfileNode* node);
+  void SerializeChildren(const v8::CpuProfileNode* node, int childrenCount);
+  void SerializeNode(const v8::CpuProfileNode* node);
+  void SerializeNodes();
+  void SerializeSamples();
+  void SerializeTimeDeltas();
+  void SerializeImpl();
+
+  static const int kEdgeFieldsCount;
+  static const int kNodeFieldsCount;
+
+  CpuProfile* profile_;
+  OutputStreamWriter* writer_;
+};
+
 }  // namespace internal
 }  // namespace v8
 
