@@ -184,8 +184,7 @@ namespace internal {
   F(FunctionGetScriptId, 1, 1)             \
   F(FunctionGetScriptSourcePosition, 1, 1) \
   F(FunctionGetSourceCode, 1, 1)           \
-  F(FunctionIsAPIFunction, 1, 1)           \
-  F(IsFunction, 1, 1)
+  F(FunctionIsAPIFunction, 1, 1)
 
 #define FOR_EACH_INTRINSIC_GENERATOR(F, I)    \
   I(AsyncFunctionAwaitCaught, 2, 1)           \
@@ -239,13 +238,14 @@ namespace internal {
   F(BytecodeBudgetInterruptWithStackCheck_Sparkplug, 1, 1) \
   F(BytecodeBudgetInterrupt_Maglev, 1, 1)                  \
   F(BytecodeBudgetInterruptWithStackCheck_Maglev, 1, 1)    \
+  F(InvalidateDependentCodeForConstTrackingLet, 1, 1)      \
   F(NewError, 2, 1)                                        \
   F(NewForeign, 0, 1)                                      \
   F(NewReferenceError, 2, 1)                               \
   F(NewSyntaxError, 2, 1)                                  \
   F(NewTypeError, -1 /* [1, 4] */, 1)                      \
   F(OrdinaryHasInstance, 2, 1)                             \
-  F(PromoteScheduledException, 0, 1)                       \
+  F(PropagateException, 0, 1)                              \
   F(ReportMessageFromMicrotask, 1, 1)                      \
   F(ReThrow, 1, 1)                                         \
   F(ReThrowWithMessage, 2, 1)                              \
@@ -301,8 +301,7 @@ namespace internal {
   F(NumberToStringSlow, 1, 1)            \
   F(StringParseFloat, 1, 1)              \
   F(StringParseInt, 2, 1)                \
-  F(StringToNumber, 1, 1)                \
-  F(TypedArrayMaxLength, 0, 1)
+  F(StringToNumber, 1, 1)
 
 #define FOR_EACH_INTRINSIC_OBJECT(F, I)                                \
   F(AddDictionaryProperty, 3, 1)                                       \
@@ -505,6 +504,7 @@ namespace internal {
   F(ArraySpeciesProtector, 0, 1)              \
   F(BaselineOsr, -1, 1)                       \
   F(BenchMaglev, 2, 1)                        \
+  F(BenchTurbofan, 2, 1)                      \
   F(ClearFunctionFeedback, 1, 1)              \
   F(ClearMegamorphicStubCache, 0, 1)          \
   F(CompleteInobjectSlackTracking, 1, 1)      \
@@ -529,6 +529,7 @@ namespace internal {
   F(FinalizeOptimization, 0, 1)               \
   F(ForceFlush, 1, 1)                         \
   F(GetCallable, 0, 1)                        \
+  F(GetFunctionForCurrentFrame, 0, 1)         \
   F(GetInitializerFunction, 1, 1)             \
   F(GetOptimizationStatus, 1, 1)              \
   F(GetUndetectable, 0, 1)                    \
@@ -541,6 +542,7 @@ namespace internal {
   F(HasFastProperties, 1, 1)                  \
   F(HasFixedBigInt64Elements, 1, 1)           \
   F(HasFixedBigUint64Elements, 1, 1)          \
+  F(HasFixedFloat16Elements, 1, 1)            \
   F(HasFixedFloat32Elements, 1, 1)            \
   F(HasFixedFloat64Elements, 1, 1)            \
   F(HasFixedInt16Elements, 1, 1)              \
@@ -568,6 +570,7 @@ namespace internal {
   F(IsConcatSpreadableProtector, 0, 1)        \
   F(IsConcurrentRecompilationSupported, 0, 1) \
   F(IsDictPropertyConstTrackingEnabled, 0, 1) \
+  F(IsEfficiencyModeEnabled, 0, 1)            \
   F(IsInPlaceInternalizableString, 1, 1)      \
   F(IsInternalizedString, 1, 1)               \
   F(IsMaglevEnabled, 0, 1)                    \
@@ -580,6 +583,8 @@ namespace internal {
   F(NewRegExpWithBacktrackLimit, 3, 1)        \
   F(NoElementsProtector, 0, 1)                \
   F(NotifyContextDisposed, 0, 1)              \
+  F(NotifyIsolateForeground, 0, 1)            \
+  F(NotifyIsolateBackground, 0, 1)            \
   F(OptimizeMaglevOnNextCall, 1, 1)           \
   F(OptimizeFunctionOnNextCall, -1, 1)        \
   F(OptimizeOsr, -1, 1)                       \
@@ -597,11 +602,14 @@ namespace internal {
   F(ScheduleGCInStackCheck, 0, 1)             \
   F(SerializeDeserializeNow, 0, 1)            \
   F(SetAllocationTimeout, -1 /* 2 || 3 */, 1) \
+  F(SetBatterySaverMode, 1, 1)                \
   F(SetForceSlowPath, 1, 1)                   \
   F(SetIteratorProtector, 0, 1)               \
   F(SharedGC, 0, 1)                           \
+  F(ShareObject, 1, 1)                        \
   F(SimulateNewspaceFull, 0, 1)               \
   F(StringIteratorProtector, 0, 1)            \
+  F(StringWrapperToPrimitiveProtector, 0, 1)  \
   F(SystemBreak, 0, 1)                        \
   F(TakeHeapSnapshot, -1, 1)                  \
   F(TraceEnter, 0, 1)                         \
@@ -624,16 +632,19 @@ namespace internal {
 #define FOR_EACH_INTRINSIC_WASM(F, I)         \
   F(ThrowBadSuspenderError, 0, 1)             \
   F(ThrowWasmError, 1, 1)                     \
+  F(TrapHandlerThrowWasmError, 0, 1)          \
   F(ThrowWasmStackOverflow, 0, 1)             \
   F(WasmI32AtomicWait, 4, 1)                  \
   F(WasmI64AtomicWait, 5, 1)                  \
-  F(WasmAtomicNotify, 3, 1)                   \
   F(WasmMemoryGrow, 2, 1)                     \
   F(WasmStackGuard, 0, 1)                     \
   F(WasmThrow, 2, 1)                          \
   F(WasmReThrow, 1, 1)                        \
   F(WasmThrowJSTypeError, 0, 1)               \
   F(WasmThrowTypeError, 2, 1)                 \
+  F(WasmThrowRangeError, 1, 1)                \
+  F(WasmThrowDataViewTypeError, 2, 1)         \
+  F(WasmThrowDataViewDetachedError, 1, 1)     \
   F(WasmRefFunc, 1, 1)                        \
   F(WasmInternalFunctionCreateExternal, 1, 1) \
   F(WasmFunctionTableGet, 3, 1)               \
@@ -648,6 +659,7 @@ namespace internal {
   F(WasmCompileLazy, 2, 1)                    \
   F(WasmAllocateFeedbackVector, 3, 1)         \
   F(WasmCompileWrapper, 1, 1)                 \
+  F(IsWasmExternalFunction, 1, 1)             \
   F(TierUpWasmToJSWrapper, 1, 1)              \
   F(WasmTriggerTierUp, 1, 1)                  \
   F(WasmDebugBreak, 0, 1)                     \
@@ -655,7 +667,8 @@ namespace internal {
   F(WasmArrayNewSegment, 5, 1)                \
   F(WasmArrayInitSegment, 6, 1)               \
   F(WasmAllocateSuspender, 0, 1)              \
-  F(WasmStringNewSegmentWtf8, 4, 1)           \
+  F(WasmCastToSpecialPrimitiveArray, 2, 1)    \
+  F(WasmStringNewSegmentWtf8, 5, 1)           \
   F(WasmStringNewWtf8, 5, 1)                  \
   F(WasmStringNewWtf8Array, 4, 1)             \
   F(WasmStringNewWtf16, 4, 1)                 \
@@ -666,6 +679,7 @@ namespace internal {
   F(WasmStringEncodeWtf8, 5, 1)               \
   F(WasmStringEncodeWtf16, 6, 1)              \
   F(WasmStringEncodeWtf8Array, 4, 1)          \
+  F(WasmStringToUtf8Array, 1, 1)              \
   F(WasmStringAsWtf8, 1, 1)                   \
   F(WasmStringViewWtf8Encode, 6, 1)           \
   F(WasmStringViewWtf8Slice, 3, 1)            \
@@ -673,38 +687,41 @@ namespace internal {
   F(WasmStringHash, 1, 1)                     \
   F(WasmSubstring, 3, 1)
 
-#define FOR_EACH_INTRINSIC_WASM_TEST(F, I)  \
-  F(CountUnoptimizedWasmToJSWrapper, 1, 1)  \
-  F(DeserializeWasmModule, 2, 1)            \
-  F(DisallowWasmCodegen, 1, 1)              \
-  F(FlushWasmCode, 0, 1)                    \
-  F(FreezeWasmLazyCompilation, 1, 1)        \
-  F(GetWasmExceptionTagId, 2, 1)            \
-  F(GetWasmExceptionValues, 1, 1)           \
-  F(GetWasmRecoveredTrapCount, 0, 1)        \
-  F(HasUnoptimizedWasmToJSWrapper, 1, 1)    \
-  F(IsAsmWasmCode, 1, 1)                    \
-  F(IsLiftoffFunction, 1, 1)                \
-  F(IsTurboFanFunction, 1, 1)               \
-  F(IsWasmDebugFunction, 1, 1)              \
-  F(IsUncompiledWasmFunction, 1, 1)         \
-  F(IsThreadInWasm, 0, 1)                   \
-  F(IsWasmCode, 1, 1)                       \
-  F(IsWasmTrapHandlerEnabled, 0, 1)         \
-  F(IsWasmPartialOOBWriteNoop, 0, 1)        \
-  F(SerializeWasmModule, 1, 1)              \
-  F(SetWasmCompileControls, 2, 1)           \
-  F(SetWasmInstantiateControls, 0, 1)       \
-  F(SetWasmGCEnabled, 1, 1)                 \
-  F(WasmCompiledExportWrappersCount, 0, 1)  \
-  F(WasmGetNumberOfInstances, 1, 1)         \
-  F(WasmNumCodeSpaces, 1, 1)                \
-  F(WasmEnterDebugging, 0, 1)               \
-  F(WasmLeaveDebugging, 0, 1)               \
-  F(WasmSwitchToTheCentralStackCount, 0, 1) \
-  F(WasmTierUpFunction, 1, 1)               \
-  F(WasmTraceEnter, 0, 1)                   \
-  F(WasmTraceExit, 1, 1)                    \
+#define FOR_EACH_INTRINSIC_WASM_TEST(F, I)                 \
+  F(CheckIsOnCentralStack, 0, 1)                           \
+  F(CountUnoptimizedWasmToJSWrapper, 1, 1)                 \
+  F(DeserializeWasmModule, 2, 1)                           \
+  F(DisallowWasmCodegen, 1, 1)                             \
+  F(FlushWasmCode, 0, 1)                                   \
+  F(FreezeWasmLazyCompilation, 1, 1)                       \
+  F(GetWasmExceptionTagId, 2, 1)                           \
+  F(GetWasmExceptionValues, 1, 1)                          \
+  F(GetWasmRecoveredTrapCount, 0, 1)                       \
+  F(HasUnoptimizedJSToJSWrapper, 1, 1)                     \
+  F(HasUnoptimizedWasmToJSWrapper, 1, 1)                   \
+  F(IsAsmWasmCode, 1, 1)                                   \
+  F(IsLiftoffFunction, 1, 1)                               \
+  F(IsThreadInWasm, 0, 1)                                  \
+  F(IsTurboFanFunction, 1, 1)                              \
+  F(IsUncompiledWasmFunction, 1, 1)                        \
+  F(IsWasmCode, 1, 1)                                      \
+  F(IsWasmDebugFunction, 1, 1)                             \
+  F(IsWasmPartialOOBWriteNoop, 0, 1)                       \
+  F(IsWasmTrapHandlerEnabled, 0, 1)                        \
+  F(SerializeWasmModule, 1, 1)                             \
+  F(SetWasmCompileControls, 2, 1)                          \
+  F(SetWasmImportedStringsEnabled, 1, 1)                   \
+  F(SetWasmInstantiateControls, 0, 1)                      \
+  F(WasmCompiledExportWrappersCount, 0, 1)                 \
+  F(WasmEnterDebugging, 0, 1)                              \
+  IF_NO_OFFICIAL_BUILD(F, WasmGenerateRandomModule, -1, 1) \
+  F(WasmGetNumberOfInstances, 1, 1)                        \
+  F(WasmLeaveDebugging, 0, 1)                              \
+  F(WasmNumCodeSpaces, 1, 1)                               \
+  F(WasmSwitchToTheCentralStackCount, 0, 1)                \
+  F(WasmTierUpFunction, 1, 1)                              \
+  F(WasmTraceEnter, 0, 1)                                  \
+  F(WasmTraceExit, 1, 1)                                   \
   F(WasmTraceMemory, 1, 1)
 
 #define FOR_EACH_INTRINSIC_WEAKREF(F, I)                             \
@@ -868,10 +885,6 @@ class Runtime : public AllStatic {
   // using it with fuzzers.
   static bool IsAllowListedForFuzzing(FunctionId id);
 
-  // Check if a switch to the central stack should be performed
-  // for a runtime function.
-  static bool SwitchToTheCentralStackForTarget(FunctionId id);
-
   // Get the intrinsic function with the given name.
   static const Function* FunctionForName(const unsigned char* name, int length);
 
