@@ -3,14 +3,15 @@
 // found in the LICENSE file.
 
 #include "src/snapshot/context-serializer.h"
-#include "src/snapshot/startup-serializer.h"
 
 #include "src/api/api-inl.h"
 #include "src/execution/microtask-queue.h"
 #include "src/heap/combined-heap.h"
 #include "src/numbers/math-random.h"
+#include "src/objects/embedder-data-array-inl.h"
 #include "src/objects/objects-inl.h"
 #include "src/objects/slots.h"
+#include "src/snapshot/startup-serializer.h"
 
 namespace v8 {
 namespace internal {
@@ -39,30 +40,32 @@ class V8_NODISCARD SanitizeNativeContextScope final {
 #endif
     microtask_queue_external_pointer_ =
         native_context
-            ->RawExternalPointerField(NativeContext::kMicrotaskQueueOffset)
+            ->RawExternalPointerField(NativeContext::kMicrotaskQueueOffset,
+                                      kNativeContextMicrotaskQueueTag)
             .GetAndClearContentForSerialization(no_gc);
   }
 
   ~SanitizeNativeContextScope() {
     // Restore saved fields.
     native_context_
-        ->RawExternalPointerField(NativeContext::kMicrotaskQueueOffset)
+        ->RawExternalPointerField(NativeContext::kMicrotaskQueueOffset,
+                                  kNativeContextMicrotaskQueueTag)
         .RestoreContentAfterSerialization(microtask_queue_external_pointer_,
                                           no_gc_);
   }
 
  private:
-  NativeContext native_context_;
+  Tagged<NativeContext> native_context_;
   ExternalPointerSlot::RawContent microtask_queue_external_pointer_;
   const DisallowGarbageCollection& no_gc_;
 };
 
 }  // namespace
 
-ContextSerializer::ContextSerializer(
-    Isolate* isolate, Snapshot::SerializerFlags flags,
-    StartupSerializer* startup_serializer,
-    v8::SerializeEmbedderFieldsCallback callback)
+ContextSerializer::ContextSerializer(Isolate* isolate,
+                                     Snapshot::SerializerFlags flags,
+                                     StartupSerializer* startup_serializer,
+                                     SerializeEmbedderFieldsCallback callback)
     : Serializer(isolate, flags),
       startup_serializer_(startup_serializer),
       serialize_embedder_fields_(callback),
@@ -74,7 +77,7 @@ ContextSerializer::~ContextSerializer() {
   OutputStatistics("ContextSerializer");
 }
 
-void ContextSerializer::Serialize(Context* o,
+void ContextSerializer::Serialize(Tagged<Context>* o,
                                   const DisallowGarbageCollection& no_gc) {
   context_ = *o;
   DCHECK(IsNativeContext(context_));
@@ -114,6 +117,35 @@ void ContextSerializer::Serialize(Context* o,
   Pad();
 }
 
+v8::StartupData InternalFieldSerializeWrapper(
+    int index, bool field_is_nullptr,
+    v8::SerializeInternalFieldsCallback user_callback,
+    v8::Local<v8::Object> api_obj) {
+  // If no serializer is provided and the field was empty, we
+  // serialize it by default to nullptr.
+  if (user_callback.callback == nullptr && field_is_nullptr) {
+    return StartupData{nullptr, 0};
+  }
+
+  DCHECK(user_callback.callback);
+  return user_callback.callback(api_obj, index, user_callback.data);
+}
+
+v8::StartupData ContextDataSerializeWrapper(
+    int index, bool field_is_nullptr,
+    v8::SerializeContextDataCallback user_callback,
+    v8::Local<v8::Context> api_obj) {
+  // For compatibility, we do not require all non-null context pointer
+  // fields to be serialized by a proper user callback. Instead, if no
+  // user callback is provided, we serialize it verbatim, which was
+  // the old behavior before we introduce context data callbacks.
+  if (user_callback.callback == nullptr) {
+    return StartupData{nullptr, 0};
+  }
+
+  return user_callback.callback(api_obj, index, user_callback.data);
+}
+
 void ContextSerializer::SerializeObjectImpl(Handle<HeapObject> obj,
                                             SlotType slot_type) {
   DCHECK(!ObjectIsBytecodeHandler(*obj));  // Only referenced in dispatch table.
@@ -160,7 +192,16 @@ void ContextSerializer::SerializeObjectImpl(Handle<HeapObject> obj,
     // Clear literal boilerplates and feedback.
     Handle<FeedbackVector>::cast(obj)->ClearSlots(isolate());
   } else if (InstanceTypeChecker::IsJSObject(instance_type)) {
-    if (SerializeJSObjectWithEmbedderFields(Handle<JSObject>::cast(obj))) {
+    Handle<JSObject> js_obj = Handle<JSObject>::cast(obj);
+    int embedder_fields_count = js_obj->GetEmbedderFieldCount();
+    if (embedder_fields_count > 0) {
+      DCHECK(!js_obj->NeedsRehashing(cage_base()));
+      v8::Local<v8::Object> api_obj = v8::Utils::ToLocal(js_obj);
+      v8::SerializeInternalFieldsCallback user_callback =
+          serialize_embedder_fields_.js_object_callback;
+      SerializeObjectWithEmbedderFields(js_obj, embedder_fields_count,
+                                        InternalFieldSerializeWrapper,
+                                        user_callback, api_obj);
       return;
     }
     if (InstanceTypeChecker::IsJSFunction(instance_type)) {
@@ -171,14 +212,31 @@ void ContextSerializer::SerializeObjectImpl(Handle<HeapObject> obj,
       if (closure->shared()->HasBytecodeArray()) {
         closure->SetInterruptBudget(isolate());
       }
-      closure->ResetIfCodeFlushed();
-      if (closure->is_compiled()) {
+      closure->ResetIfCodeFlushed(isolate());
+      if (closure->is_compiled(isolate())) {
         if (closure->shared()->HasBaselineCode()) {
           closure->shared()->FlushBaselineCode();
         }
         closure->set_code(closure->shared()->GetCode(isolate()), kReleaseStore);
       }
     }
+  } else if (InstanceTypeChecker::IsEmbedderDataArray(instance_type) &&
+             !allow_active_isolate_for_testing()) {
+    DCHECK_EQ(*obj, context_->embedder_data());
+    Handle<EmbedderDataArray> embedder_data =
+        Handle<EmbedderDataArray>::cast(obj);
+    int embedder_fields_count = embedder_data->length();
+    if (embedder_data->length() > 0) {
+      Handle<Context> context_handle(context_, isolate());
+      v8::Local<v8::Context> api_obj =
+          v8::Utils::ToLocal(Handle<NativeContext>::cast(context_handle));
+      v8::SerializeContextDataCallback user_callback =
+          serialize_embedder_fields_.context_callback;
+      SerializeObjectWithEmbedderFields(embedder_data, embedder_fields_count,
+                                        ContextDataSerializeWrapper,
+                                        user_callback, api_obj);
+      return;
+    }
   }
 
   CheckRehashability(*obj);
@@ -209,19 +267,18 @@ namespace {
 bool DataIsEmpty(const StartupData& data) { return data.raw_size == 0; }
 }  // anonymous namespace
 
-bool ContextSerializer::SerializeJSObjectWithEmbedderFields(
-    Handle<JSObject> obj) {
+template <typename V8Type, typename UserSerializerWrapper,
+          typename UserCallback, typename ApiObjectType>
+void ContextSerializer::SerializeObjectWithEmbedderFields(
+    Handle<V8Type> data_holder, int embedder_fields_count,
+    UserSerializerWrapper wrapper, UserCallback user_callback,
+    ApiObjectType api_obj) {
   DisallowGarbageCollection no_gc;
-  Tagged<JSObject> js_obj = *obj;
-  int embedder_fields_count = js_obj->GetEmbedderFieldCount();
-  if (embedder_fields_count == 0) return false;
   CHECK_GT(embedder_fields_count, 0);
-  DCHECK(!js_obj->NeedsRehashing(cage_base()));
-
   DisallowJavascriptExecution no_js(isolate());
   DisallowCompilation no_compile(isolate());
 
-  v8::Local<v8::Object> api_obj = v8::Utils::ToLocal(obj);
+  auto raw_obj = *data_holder;
 
   std::vector<EmbedderDataSlot::RawData> original_embedder_values;
   std::vector<StartupData> serialized_data;
@@ -231,7 +288,7 @@ bool ContextSerializer::SerializeJSObjectWithEmbedderFields(
   //    serializer. For aligned pointers, call the serialize callback. Hold
   //    onto the result.
   for (int i = 0; i < embedder_fields_count; i++) {
-    EmbedderDataSlot embedder_data_slot(js_obj, i);
+    EmbedderDataSlot embedder_data_slot(raw_obj, i);
     original_embedder_values.emplace_back(
         embedder_data_slot.load_raw(isolate(), no_gc));
     Tagged<Object> object = embedder_data_slot.load_tagged();
@@ -239,17 +296,8 @@ bool ContextSerializer::SerializeJSObjectWithEmbedderFields(
       DCHECK(IsValidHeapObject(isolate()->heap(), HeapObject::cast(object)));
       serialized_data.push_back({nullptr, 0});
     } else {
-      // If no serializer is provided and the field was empty, we serialize it
-      // by default to nullptr.
-      if (serialize_embedder_fields_.callback == nullptr &&
-          object == Smi::zero()) {
-        serialized_data.push_back({nullptr, 0});
-      } else {
-        DCHECK_NOT_NULL(serialize_embedder_fields_.callback);
-        StartupData data = serialize_embedder_fields_.callback(
-            api_obj, i, serialize_embedder_fields_.data);
-        serialized_data.push_back(data);
-      }
+      serialized_data.push_back(
+          wrapper(i, object == Smi::zero(), user_callback, api_obj));
     }
   }
 
@@ -260,7 +308,7 @@ bool ContextSerializer::SerializeJSObjectWithEmbedderFields(
   //    with embedder callbacks.
   for (int i = 0; i < embedder_fields_count; i++) {
     if (!DataIsEmpty(serialized_data[i])) {
-      EmbedderDataSlot(js_obj, i).store_raw(isolate(), kNullAddress, no_gc);
+      EmbedderDataSlot(raw_obj, i).store_raw(isolate(), kNullAddress, no_gc);
     }
   }
 
@@ -268,14 +316,14 @@ bool ContextSerializer::SerializeJSObjectWithEmbedderFields(
   //    smis are serialized regularly.
   {
     AllowGarbageCollection allow_gc;
-    ObjectSerializer(this, obj, &sink_).Serialize(SlotType::kAnySlot);
+    ObjectSerializer(this, data_holder, &sink_).Serialize(SlotType::kAnySlot);
     // Reload raw pointer.
-    js_obj = *obj;
+    raw_obj = *data_holder;
   }
 
   // 4) Obtain back reference for the serialized object.
   const SerializerReference* reference =
-      reference_map()->LookupReference(js_obj);
+      reference_map()->LookupReference(raw_obj);
   DCHECK_NOT_NULL(reference);
   DCHECK(reference->is_back_reference());
 
@@ -285,8 +333,8 @@ bool ContextSerializer::SerializeJSObjectWithEmbedderFields(
     StartupData data = serialized_data[i];
     if (DataIsEmpty(data)) continue;
     // Restore original values from cleared fields.
-    EmbedderDataSlot(js_obj, i).store_raw(isolate(),
-                                          original_embedder_values[i], no_gc);
+    EmbedderDataSlot(raw_obj, i)
+        .store_raw(isolate(), original_embedder_values[i], no_gc);
     embedder_fields_sink_.Put(kNewObject, "embedder field holder");
     embedder_fields_sink_.PutUint30(reference->back_ref_index(),
                                     "BackRefIndex");
@@ -301,7 +349,6 @@ bool ContextSerializer::SerializeJSObjectWithEmbedderFields(
   //    sink. The ensures that during deserialization, we call the deserializer
   //    callback at the end, and can guarantee that the deserialized objects are
   //    in a consistent state. See ContextSerializer::Serialize.
-  return true;
 }
 
 void ContextSerializer::CheckRehashability(Tagged<HeapObject> obj) {
