@@ -14,6 +14,7 @@
 #include <vector>
 
 #include "src/base/optional.h"
+#include "src/numbers/integer-literal.h"
 #include "src/torque/constants.h"
 #include "src/torque/source-positions.h"
 #include "src/torque/utils.h"
@@ -33,7 +34,8 @@ namespace torque {
   V(ConditionalExpression)               \
   V(IdentifierExpression)                \
   V(StringLiteralExpression)             \
-  V(NumberLiteralExpression)             \
+  V(IntegerLiteralExpression)            \
+  V(FloatingPointLiteralExpression)      \
   V(FieldAccessExpression)               \
   V(ElementAccessExpression)             \
   V(DereferenceExpression)               \
@@ -187,15 +189,21 @@ struct NamespaceDeclaration : Declaration {
 };
 
 struct EnumDescription {
+  struct Entry {
+    std::string name;
+    std::string alias_entry;
+    Entry(std::string name, std::string alias_entry)
+        : name(std::move(name)), alias_entry(std::move(alias_entry)) {}
+  };
   SourcePosition pos;
   std::string name;
   std::string constexpr_generates;
   bool is_open;
-  std::vector<std::string> entries;
+  std::vector<Entry> entries;
 
   EnumDescription(SourcePosition pos, std::string name,
                   std::string constexpr_generates, bool is_open,
-                  std::vector<std::string> entries = {})
+                  std::vector<Entry> entries = {})
       : pos(std::move(pos)),
         name(std::move(name)),
         constexpr_generates(std::move(constexpr_generates)),
@@ -459,16 +467,28 @@ struct StringLiteralExpression : Expression {
   std::string literal;
 };
 
-struct NumberLiteralExpression : Expression {
-  DEFINE_AST_NODE_LEAF_BOILERPLATE(NumberLiteralExpression)
-  NumberLiteralExpression(SourcePosition pos, double number)
-      : Expression(kKind, pos), number(number) {}
+struct IntegerLiteralExpression : Expression {
+  DEFINE_AST_NODE_LEAF_BOILERPLATE(IntegerLiteralExpression)
+  IntegerLiteralExpression(SourcePosition pos, IntegerLiteral value)
+      : Expression(kKind, pos), value(std::move(value)) {}
 
   void VisitAllSubExpressions(VisitCallback callback) override {
     callback(this);
   }
 
-  double number;
+  IntegerLiteral value;
+};
+
+struct FloatingPointLiteralExpression : Expression {
+  DEFINE_AST_NODE_LEAF_BOILERPLATE(FloatingPointLiteralExpression)
+  FloatingPointLiteralExpression(SourcePosition pos, double value)
+      : Expression(kKind, pos), value(value) {}
+
+  void VisitAllSubExpressions(VisitCallback callback) override {
+    callback(this);
+  }
+
+  double value;
 };
 
 struct ElementAccessExpression : LocationExpression {
@@ -581,11 +601,13 @@ struct AssumeTypeImpossibleExpression : Expression {
 struct NewExpression : Expression {
   DEFINE_AST_NODE_LEAF_BOILERPLATE(NewExpression)
   NewExpression(SourcePosition pos, TypeExpression* type,
-                std::vector<NameAndExpression> initializers, bool pretenured)
+                std::vector<NameAndExpression> initializers, bool pretenured,
+                bool clear_padding)
       : Expression(kKind, pos),
         type(type),
         initializers(std::move(initializers)),
-        pretenured(pretenured) {}
+        pretenured(pretenured),
+        clear_padding(clear_padding) {}
 
   void VisitAllSubExpressions(VisitCallback callback) override {
     for (auto& initializer : initializers) {
@@ -597,6 +619,7 @@ struct NewExpression : Expression {
   TypeExpression* type;
   std::vector<NameAndExpression> initializers;
   bool pretenured;
+  bool clear_padding;
 };
 
 enum class ImplicitKind { kNoImplicit, kJSImplicit, kImplicit };
@@ -710,13 +733,10 @@ struct ReturnStatement : Statement {
 
 struct DebugStatement : Statement {
   DEFINE_AST_NODE_LEAF_BOILERPLATE(DebugStatement)
-  DebugStatement(SourcePosition pos, const std::string& reason,
-                 bool never_continues)
-      : Statement(kKind, pos),
-        reason(reason),
-        never_continues(never_continues) {}
-  std::string reason;
-  bool never_continues;
+  enum class Kind { kUnreachable, kDebug };
+  DebugStatement(SourcePosition pos, Kind kind)
+      : Statement(kKind, pos), kind(kind) {}
+  Kind kind;
 };
 
 struct AssertStatement : Statement {
@@ -937,7 +957,7 @@ struct ClassFieldExpression {
   NameAndTypeExpression name_and_type;
   base::Optional<ClassFieldIndexInfo> index;
   std::vector<ConditionalAnnotation> conditions;
-  bool weak;
+  bool custom_weak_marking;
   bool const_qualified;
   FieldSynchronization read_synchronization;
   FieldSynchronization write_synchronization;
@@ -1069,10 +1089,13 @@ struct TorqueBuiltinDeclaration : BuiltinDeclaration {
                            bool javascript_linkage, Identifier* name,
                            ParameterList parameters,
                            TypeExpression* return_type,
+                           bool has_custom_interface_descriptor,
                            base::Optional<Statement*> body)
       : BuiltinDeclaration(kKind, pos, javascript_linkage, transitioning, name,
                            std::move(parameters), return_type),
+        has_custom_interface_descriptor(has_custom_interface_descriptor),
         body(body) {}
+  bool has_custom_interface_descriptor;
   base::Optional<Statement*> body;
 };
 
