@@ -13,6 +13,7 @@
 #include <string>
 #include <type_traits>
 
+#include "src/base/bits.h"
 #include "src/base/compiler-specific.h"
 #include "src/base/logging.h"
 #include "src/base/macros.h"
@@ -28,6 +29,25 @@
 #include <fenv.h>  // NOLINT(build/c++11)
 #endif
 
+#ifdef _MSC_VER
+// MSVC doesn't define SSE3. However, it does define AVX, and AVX implies SSE3.
+#ifdef __AVX__
+#ifndef __SSE3__
+#define __SSE3__
+#endif
+#endif
+#endif
+
+#ifdef __SSE3__
+#include <immintrin.h>
+#endif
+
+#if defined(V8_TARGET_ARCH_ARM64) && \
+    (defined(__ARM_NEON) || defined(__ARM_NEON__))
+#define V8_OPTIMIZE_WITH_NEON
+#include <arm_neon.h>
+#endif
+
 namespace v8 {
 namespace internal {
 
@@ -316,13 +336,147 @@ class SetOncePointer {
   T* pointer_ = nullptr;
 };
 
+#if defined(__SSE3__)
+
+template <typename Char>
+V8_INLINE bool SimdMemEqual(const Char* lhs, const Char* rhs, size_t count,
+                            size_t order) {
+  static_assert(sizeof(Char) == 1);
+  DCHECK_GE(order, 5);
+
+  static constexpr uint16_t kSIMDMatched16Mask = UINT16_MAX;
+  static constexpr uint32_t kSIMDMatched32Mask = UINT32_MAX;
+
+  if (order == 5) {  // count: [17, 32]
+    // Utilize more simd registers for better pipelining.
+    const __m128i lhs128_start =
+        _mm_lddqu_si128(reinterpret_cast<const __m128i*>(lhs));
+    const __m128i lhs128_end = _mm_lddqu_si128(
+        reinterpret_cast<const __m128i*>(lhs + count - sizeof(__m128i)));
+    const __m128i rhs128_start =
+        _mm_lddqu_si128(reinterpret_cast<const __m128i*>(rhs));
+    const __m128i rhs128_end = _mm_lddqu_si128(
+        reinterpret_cast<const __m128i*>(rhs + count - sizeof(__m128i)));
+    const __m128i res_start = _mm_cmpeq_epi8(lhs128_start, rhs128_start);
+    const __m128i res_end = _mm_cmpeq_epi8(lhs128_end, rhs128_end);
+    const uint32_t res =
+        _mm_movemask_epi8(res_start) << 16 | _mm_movemask_epi8(res_end);
+    return res == kSIMDMatched32Mask;
+  }
+
+  // count: [33, ...]
+  const __m128i lhs128_unrolled =
+      _mm_lddqu_si128(reinterpret_cast<const __m128i*>(lhs));
+  const __m128i rhs128_unrolled =
+      _mm_lddqu_si128(reinterpret_cast<const __m128i*>(rhs));
+  const __m128i res_unrolled = _mm_cmpeq_epi8(lhs128_unrolled, rhs128_unrolled);
+  const uint16_t res_unrolled_mask = _mm_movemask_epi8(res_unrolled);
+  if (res_unrolled_mask != kSIMDMatched16Mask) return false;
+
+  for (size_t i = count % sizeof(__m128i); i < count; i += sizeof(__m128i)) {
+    const __m128i lhs128 =
+        _mm_lddqu_si128(reinterpret_cast<const __m128i*>(lhs + i));
+    const __m128i rhs128 =
+        _mm_lddqu_si128(reinterpret_cast<const __m128i*>(rhs + i));
+    const __m128i res = _mm_cmpeq_epi8(lhs128, rhs128);
+    const uint16_t res_mask = _mm_movemask_epi8(res);
+    if (res_mask != kSIMDMatched16Mask) return false;
+  }
+  return true;
+}
+
+#elif defined(V8_OPTIMIZE_WITH_NEON)
+
+template <typename Char>
+V8_INLINE bool SimdMemEqual(const Char* lhs, const Char* rhs, size_t count,
+                            size_t order) {
+  static_assert(sizeof(Char) == 1);
+  DCHECK_GE(order, 5);
+
+  if (order == 5) {  // count: [17, 32]
+    // Utilize more simd registers for better pipelining.
+    const auto lhs0 = vld1q_u8(lhs);
+    const auto lhs1 = vld1q_u8(lhs + count - sizeof(uint8x16_t));
+    const auto rhs0 = vld1q_u8(rhs);
+    const auto rhs1 = vld1q_u8(rhs + count - sizeof(uint8x16_t));
+    const auto xored0 = veorq_u8(lhs0, rhs0);
+    const auto xored1 = veorq_u8(lhs1, rhs1);
+    const auto ored = vorrq_u8(xored0, xored1);
+    return !static_cast<bool>(
+        vgetq_lane_u64(vreinterpretq_u64_u8(vpmaxq_u8(ored, ored)), 0));
+  }
+
+  // count: [33, ...]
+  const auto lhs0 = vld1q_u8(lhs);
+  const auto rhs0 = vld1q_u8(rhs);
+  const auto xored = veorq_u8(lhs0, rhs0);
+  if (static_cast<bool>(
+          vgetq_lane_u64(vreinterpretq_u64_u8(vpmaxq_u8(xored, xored)), 0)))
+    return false;
+  for (size_t i = count % sizeof(uint8x16_t); i < count;
+       i += sizeof(uint8x16_t)) {
+    const auto lhs0 = vld1q_u8(lhs + i);
+    const auto rhs0 = vld1q_u8(rhs + i);
+    const auto xored = veorq_u8(lhs0, rhs0);
+    if (static_cast<bool>(
+            vgetq_lane_u64(vreinterpretq_u64_u8(vpmaxq_u8(xored, xored)), 0)))
+      return false;
+  }
+  return true;
+}
+
+#endif
+
+template <typename IntType, typename Char>
+V8_CLANG_NO_SANITIZE("alignment")
+V8_INLINE bool OverlappingCompare(const Char* lhs, const Char* rhs,
+                                  size_t count) {
+  static_assert(sizeof(Char) == 1);
+  return *reinterpret_cast<const IntType*>(lhs) ==
+             *reinterpret_cast<const IntType*>(rhs) &&
+         *reinterpret_cast<const IntType*>(lhs + count - sizeof(IntType)) ==
+             *reinterpret_cast<const IntType*>(rhs + count - sizeof(IntType));
+}
+
+template <typename Char>
+V8_CLANG_NO_SANITIZE("alignment")
+V8_INLINE bool SimdMemEqual(const Char* lhs, const Char* rhs, size_t count) {
+  static_assert(sizeof(Char) == 1);
+  if (count == 0) {
+    return true;
+  }
+  if (count == 1) {
+    return *lhs == *rhs;
+  }
+  const size_t order =
+      sizeof(count) * CHAR_BIT - base::bits::CountLeadingZeros(count - 1);
+  switch (order) {
+    case 1:  // count: [2, 2]
+      return *reinterpret_cast<const uint16_t*>(lhs) ==
+             *reinterpret_cast<const uint16_t*>(rhs);
+    case 2:  // count: [3, 4]
+      return OverlappingCompare<uint16_t>(lhs, rhs, count);
+    case 3:  // count: [5, 8]
+      return OverlappingCompare<uint32_t>(lhs, rhs, count);
+    case 4:  // count: [9, 16]
+      return OverlappingCompare<uint64_t>(lhs, rhs, count);
+    default:
+      return SimdMemEqual(lhs, rhs, count, order);
+  }
+}
+
 // Compare 8bit/16bit chars to 8bit/16bit chars.
 template <typename lchar, typename rchar>
 inline bool CompareCharsEqualUnsigned(const lchar* lhs, const rchar* rhs,
                                       size_t chars) {
   static_assert(std::is_unsigned<lchar>::value);
   static_assert(std::is_unsigned<rchar>::value);
-  if (sizeof(*lhs) == sizeof(*rhs)) {
+  if constexpr (sizeof(*lhs) == sizeof(*rhs)) {
+#if defined(__SSE3__) || defined(V8_OPTIMIZE_WITH_NEON)
+    if constexpr (sizeof(*lhs) == 1) {
+      return SimdMemEqual(lhs, rhs, chars);
+    }
+#endif
     // memcmp compares byte-by-byte, but for equality it doesn't matter whether
     // two-byte char comparison is little- or big-endian.
     return memcmp(lhs, rhs, chars * sizeof(*lhs)) == 0;
@@ -431,12 +585,21 @@ inline constexpr T truncate_to_intn(T x, unsigned n) {
   inline constexpr T truncate_to_int##N(T x) { \
     return truncate_to_intn(x, N);             \
   }
+
+#define DECLARE_CHECKED_TRUNCATE_TO_INT_N(N)           \
+  template <class T>                                   \
+  inline constexpr T checked_truncate_to_int##N(T x) { \
+    CHECK(is_int##N(x));                               \
+    return truncate_to_intn(x, N);                     \
+  }
 INT_1_TO_63_LIST(DECLARE_IS_INT_N)
 INT_1_TO_63_LIST(DECLARE_IS_UINT_N)
 INT_1_TO_63_LIST(DECLARE_TRUNCATE_TO_INT_N)
+INT_1_TO_63_LIST(DECLARE_CHECKED_TRUNCATE_TO_INT_N)
 #undef DECLARE_IS_INT_N
 #undef DECLARE_IS_UINT_N
 #undef DECLARE_TRUNCATE_TO_INT_N
+#undef DECLARE_CHECKED_TRUNCATE_TO_INT_N
 
 // clang-format off
 #define INT_0_TO_127_LIST(V)                                          \
@@ -497,10 +660,6 @@ class BytecodeOffset {
   //  - {ConstructStubInvoke} maps to {construct_stub_invoke_deopt_pc_offset}.
   static BytecodeOffset ConstructStubCreate() { return BytecodeOffset(1); }
   static BytecodeOffset ConstructStubInvoke() { return BytecodeOffset(2); }
-  bool IsValidForConstructStub() const {
-    return id_ == ConstructStubCreate().ToInt() ||
-           id_ == ConstructStubInvoke().ToInt();
-  }
 
   constexpr bool IsNone() const { return id_ == kNoneId; }
   bool operator==(const BytecodeOffset& other) const {
@@ -552,7 +711,7 @@ int WriteChars(const char* filename, const char* str, int size,
 
 // Write size bytes to the file given by filename.
 // The file is overwritten. Returns the number of bytes written.
-int WriteBytes(const char* filename, const byte* bytes, int size,
+int WriteBytes(const char* filename, const uint8_t* bytes, int size,
                bool verbose = true);
 
 // Simple support to read a file into std::string.
