@@ -14,20 +14,16 @@
 #include "src/wasm/baseline/liftoff-compiler.h"
 #include "src/wasm/baseline/liftoff-register.h"
 #include "src/wasm/function-body-decoder.h"
-#include "src/wasm/wasm-code-manager.h"
 #include "src/wasm/wasm-module.h"
 #include "src/wasm/wasm-opcodes.h"
 #include "src/wasm/wasm-value.h"
 
-namespace v8 {
-namespace internal {
-
 // Forward declarations.
-namespace compiler {
+namespace v8::internal::compiler {
 class CallDescriptor;
-}  // namespace compiler
+}  // namespace v8::internal::compiler
 
-namespace wasm {
+namespace v8::internal::wasm {
 
 inline constexpr Condition Negate(Condition cond) {
   switch (cond) {
@@ -141,6 +137,7 @@ class LiftoffAssembler : public MacroAssembler {
 
     Location loc() const { return loc_; }
 
+    // The constant as 32-bit value, to be sign-extended if {kind() == kI64}.
     int32_t i32_const() const {
       DCHECK_EQ(loc_, kIntConst);
       return i32_const_;
@@ -242,7 +239,7 @@ class LiftoffAssembler : public MacroAssembler {
     LiftoffRegList used_registers;
     uint32_t register_use_count[kAfterMaxLiftoffRegCode] = {0};
     LiftoffRegList last_spilled_regs;
-    Register cached_instance = no_reg;
+    Register cached_instance_data = no_reg;
     static constexpr int kNoCachedMemIndex = -1;
     // The index of the cached memory start, or {kNoCachedMemIndex} if none is
     // cached ({cached_mem_start} will be {no_reg} in that case).
@@ -300,7 +297,8 @@ class LiftoffAssembler : public MacroAssembler {
     // can easily be reloaded. Those are returned first if we run out of free
     // registers.
     bool has_volatile_register(LiftoffRegList candidates) {
-      return (cached_instance != no_reg && candidates.has(cached_instance)) ||
+      return (cached_instance_data != no_reg &&
+              candidates.has(cached_instance_data)) ||
              (cached_mem_start != no_reg && candidates.has(cached_mem_start));
     }
 
@@ -308,9 +306,10 @@ class LiftoffAssembler : public MacroAssembler {
       DCHECK(!frozen);
       DCHECK(has_volatile_register(candidates));
       Register reg = no_reg;
-      if (cached_instance != no_reg && candidates.has(cached_instance)) {
-        reg = cached_instance;
-        cached_instance = no_reg;
+      if (cached_instance_data != no_reg &&
+          candidates.has(cached_instance_data)) {
+        reg = cached_instance_data;
+        cached_instance_data = no_reg;
       } else {
         DCHECK(candidates.has(cached_mem_start));
         reg = cached_mem_start;
@@ -336,7 +335,7 @@ class LiftoffAssembler : public MacroAssembler {
     }
 
     void SetInstanceCacheRegister(Register reg) {
-      SetCacheRegister(&cached_instance, reg);
+      SetCacheRegister(&cached_instance_data, reg);
     }
 
     void SetMemStartCacheRegister(Register reg, int memory_index) {
@@ -346,7 +345,7 @@ class LiftoffAssembler : public MacroAssembler {
     }
 
     Register TrySetCachedInstanceRegister(LiftoffRegList pinned) {
-      DCHECK_EQ(no_reg, cached_instance);
+      DCHECK_EQ(no_reg, cached_instance_data);
       LiftoffRegList available_regs =
           kGpCacheRegList.MaskOut(pinned).MaskOut(used_registers);
       if (available_regs.is_empty()) return no_reg;
@@ -356,13 +355,13 @@ class LiftoffAssembler : public MacroAssembler {
                                    ? kWasmInstanceRegister
                                    : available_regs.GetFirstRegSet().gp();
       SetInstanceCacheRegister(new_cache_reg);
-      DCHECK_EQ(new_cache_reg, cached_instance);
+      DCHECK_EQ(new_cache_reg, cached_instance_data);
       return new_cache_reg;
     }
 
     V8_INLINE void ClearCacheRegister(Register* cache) {
       DCHECK(!frozen);
-      V8_ASSUME(cache == &cached_instance || cache == &cached_mem_start);
+      V8_ASSUME(cache == &cached_instance_data || cache == &cached_mem_start);
       if (*cache == no_reg) return;
       int liftoff_code = LiftoffRegister{*cache}.liftoff_code();
       DCHECK_EQ(1, register_use_count[liftoff_code]);
@@ -371,7 +370,9 @@ class LiftoffAssembler : public MacroAssembler {
       *cache = no_reg;
     }
 
-    void ClearCachedInstanceRegister() { ClearCacheRegister(&cached_instance); }
+    void ClearCachedInstanceRegister() {
+      ClearCacheRegister(&cached_instance_data);
+    }
 
     void ClearCachedMemStartRegister() {
       V8_ASSUME(cached_mem_index == kNoCachedMemIndex || cached_mem_index >= 0);
@@ -492,14 +493,7 @@ class LiftoffAssembler : public MacroAssembler {
   LoadToRegister_Slow(VarState slot, LiftoffRegList pinned);
 
   // Load a non-register cache slot to a given (fixed) register.
-  void LoadToFixedRegister(VarState slot, LiftoffRegister reg) {
-    DCHECK(slot.is_const() || slot.is_stack());
-    if (slot.is_const()) {
-      LoadConstant(reg, slot.constant());
-    } else {
-      Fill(reg, slot.offset(), slot.kind());
-    }
-  }
+  inline void LoadToFixedRegister(VarState slot, LiftoffRegister reg);
 
   // Pop a VarState from the stack, updating the register use count accordingly.
   V8_INLINE VarState PopVarState() {
@@ -515,20 +509,7 @@ class LiftoffAssembler : public MacroAssembler {
     return LoadToRegister(slot, pinned);
   }
 
-  void PopToFixedRegister(LiftoffRegister reg) {
-    DCHECK(!cache_state_.stack_state.empty());
-    VarState slot = cache_state_.stack_state.back();
-    cache_state_.stack_state.pop_back();
-    if (V8_LIKELY(slot.is_reg())) {
-      cache_state_.dec_used(slot.reg());
-      if (slot.reg() == reg) return;
-      if (cache_state_.is_used(reg)) SpillRegister(reg);
-      Move(reg, slot.reg(), slot.kind());
-      return;
-    }
-    if (cache_state_.is_used(reg)) SpillRegister(reg);
-    LoadToFixedRegister(slot, reg);
-  }
+  inline void PopToFixedRegister(LiftoffRegister reg);
 
   // Use this to pop a value into a register that has no other uses, so it
   // can be modified.
@@ -565,49 +546,19 @@ class LiftoffAssembler : public MacroAssembler {
   // stack, so that we can merge different values on the back-edge.
   void PrepareLoopArgs(int num);
 
-  V8_INLINE static int NextSpillOffset(ValueKind kind, int top_spill_offset) {
-    int offset = top_spill_offset + SlotSizeForType(kind);
-    if (NeedsAlignment(kind)) {
-      offset = RoundUp(offset, SlotSizeForType(kind));
-    }
-    return offset;
-  }
+  V8_INLINE static int NextSpillOffset(ValueKind kind, int top_spill_offset);
+  V8_INLINE int NextSpillOffset(ValueKind kind);
+  inline int TopSpillOffset() const;
 
-  int NextSpillOffset(ValueKind kind) {
-    return NextSpillOffset(kind, TopSpillOffset());
-  }
-
-  int TopSpillOffset() const {
-    return cache_state_.stack_state.empty()
-               ? StaticStackFrameSize()
-               : cache_state_.stack_state.back().offset();
-  }
-
-  void PushRegister(ValueKind kind, LiftoffRegister reg) {
-    DCHECK_EQ(reg_class_for(kind), reg.reg_class());
-    cache_state_.inc_used(reg);
-    cache_state_.stack_state.emplace_back(kind, reg, NextSpillOffset(kind));
-  }
+  inline void PushRegister(ValueKind kind, LiftoffRegister reg);
 
   // Assumes that the exception is in {kReturnRegister0}. This is where the
   // exception is stored by the unwinder after a throwing call.
-  void PushException() {
-    LiftoffRegister reg{kReturnRegister0};
-    // This is used after a call, so {kReturnRegister0} is not used yet.
-    DCHECK(cache_state_.is_free(reg));
-    cache_state_.inc_used(reg);
-    cache_state_.stack_state.emplace_back(kRef, reg, NextSpillOffset(kRef));
-  }
+  inline void PushException();
 
-  void PushConstant(ValueKind kind, int32_t i32_const) {
-    V8_ASSUME(kind == kI32 || kind == kI64);
-    cache_state_.stack_state.emplace_back(kind, i32_const,
-                                          NextSpillOffset(kind));
-  }
+  inline void PushConstant(ValueKind kind, int32_t i32_const);
 
-  void PushStack(ValueKind kind) {
-    cache_state_.stack_state.emplace_back(kind, NextSpillOffset(kind));
-  }
+  inline void PushStack(ValueKind kind);
 
   V8_NOINLINE V8_PRESERVE_MOST void SpillRegister(LiftoffRegister);
 
@@ -694,7 +645,7 @@ class LiftoffAssembler : public MacroAssembler {
   void SpillRegisters(Regs... regs) {
     for (LiftoffRegister r : {LiftoffRegister(regs)...}) {
       if (cache_state_.is_free(r)) continue;
-      if (r.is_gp() && cache_state_.cached_instance == r.gp()) {
+      if (r.is_gp() && cache_state_.cached_instance_data == r.gp()) {
         cache_state_.ClearCachedInstanceRegister();
       } else if (r.is_gp() && cache_state_.cached_mem_start == r.gp()) {
         V8_ASSUME(cache_state_.cached_mem_index >= 0);
@@ -769,6 +720,12 @@ class LiftoffAssembler : public MacroAssembler {
   bool ValidateCacheState() const;
 #endif
 
+  inline void LoadFixedArrayLengthAsInt32(LiftoffRegister dst, Register array,
+                                          LiftoffRegList pinned);
+
+  inline void LoadSmiAsInt32(LiftoffRegister dst, Register src_addr,
+                             int32_t offset);
+
   ////////////////////////////////////
   // Platform-specific part.        //
   ////////////////////////////////////
@@ -793,7 +750,9 @@ class LiftoffAssembler : public MacroAssembler {
   inline void CheckTierUp(int declared_func_index, int budget_used,
                           Label* ool_label, const FreezeCacheState& frozen);
   inline void LoadConstant(LiftoffRegister, WasmValue);
-  inline void LoadInstanceFromFrame(Register dst);
+  inline void LoadInstanceDataFromFrame(Register dst);
+  inline void LoadTrustedPointer(Register dst, Register src_addr, int offset,
+                                 IndirectPointerTag tag);
   inline void LoadFromInstance(Register dst, Register instance, int offset,
                                int size);
   inline void LoadTaggedPointerFromInstance(Register dst, Register instance,
@@ -803,39 +762,29 @@ class LiftoffAssembler : public MacroAssembler {
   inline void LoadExternalPointer(Register dst, Register src_addr, int offset,
                                   Register index, ExternalPointerTag tag,
                                   Register scratch);
-  inline void SpillInstance(Register instance);
+  inline void SpillInstanceData(Register instance);
   inline void ResetOSRTarget();
   inline void LoadTaggedPointer(Register dst, Register src_addr,
                                 Register offset_reg, int32_t offset_imm,
+                                uint32_t* protected_load_pc = nullptr,
                                 bool offset_reg_needs_shift = false);
+  inline void LoadProtectedPointer(Register dst, Register src_addr,
+                                   int32_t offset);
   inline void LoadFullPointer(Register dst, Register src_addr,
                               int32_t offset_imm);
+#ifdef V8_ENABLE_SANDBOX
+  inline void LoadCodeEntrypointViaCodePointer(Register dsr, Register src_addr,
+                                               int offset_imm);
+#endif
   enum SkipWriteBarrier : bool {
     kSkipWriteBarrier = true,
     kNoSkipWriteBarrier = false
   };
   inline void StoreTaggedPointer(Register dst_addr, Register offset_reg,
-                                 int32_t offset_imm, LiftoffRegister src,
+                                 int32_t offset_imm, Register src,
                                  LiftoffRegList pinned,
+                                 uint32_t* protected_store_pc = nullptr,
                                  SkipWriteBarrier = kNoSkipWriteBarrier);
-  void LoadFixedArrayLengthAsInt32(LiftoffRegister dst, Register array,
-                                   LiftoffRegList pinned) {
-    int offset = FixedArray::kLengthOffset - kHeapObjectTag;
-    LoadSmiAsInt32(dst, array, offset);
-  }
-  void LoadSmiAsInt32(LiftoffRegister dst, Register src_addr, int32_t offset) {
-    if (SmiValuesAre32Bits()) {
-#if V8_TARGET_LITTLE_ENDIAN
-      DCHECK_EQ(kSmiShiftSize + kSmiTagSize, 4 * kBitsPerByte);
-      offset += 4;
-#endif
-      Load(dst, src_addr, no_reg, offset, LoadType::kI32Load);
-    } else {
-      DCHECK(SmiValuesAre31Bits());
-      Load(dst, src_addr, no_reg, offset, LoadType::kI32Load);
-      emit_i32_sari(dst.gp(), dst.gp(), kSmiTagSize);
-    }
-  }
   // Warning: may clobber {dst} on some architectures!
   inline void IncrementSmi(LiftoffRegister dst, int offset);
   inline void Load(LiftoffRegister dst, Register src_addr, Register offset_reg,
@@ -921,6 +870,7 @@ class LiftoffAssembler : public MacroAssembler {
   inline void emit_i32_sub(Register dst, Register lhs, Register rhs);
   inline void emit_i32_subi(Register dst, Register lhs, int32_t imm);
   inline void emit_i32_mul(Register dst, Register lhs, Register rhs);
+  inline void emit_i32_muli(Register dst, Register lhs, int32_t imm);
   inline void emit_i32_divs(Register dst, Register lhs, Register rhs,
                             Label* trap_div_by_zero,
                             Label* trap_div_unrepresentable);
@@ -949,6 +899,10 @@ class LiftoffAssembler : public MacroAssembler {
   inline bool emit_i32_popcnt(Register dst, Register src);
 
   // i64 binops.
+  // Most variants taking an immediate as second input only need to support
+  // 32-bit immediates, because that't the only type of constant we track.
+  // Some (like addition) are also called in other situation where we can have
+  // bigger immediates. In that case we type the immediate as int64_t.
   inline void emit_i64_add(LiftoffRegister dst, LiftoffRegister lhs,
                            LiftoffRegister rhs);
   inline void emit_i64_addi(LiftoffRegister dst, LiftoffRegister lhs,
@@ -957,6 +911,8 @@ class LiftoffAssembler : public MacroAssembler {
                            LiftoffRegister rhs);
   inline void emit_i64_mul(LiftoffRegister dst, LiftoffRegister lhs,
                            LiftoffRegister rhs);
+  inline void emit_i64_muli(LiftoffRegister dst, LiftoffRegister lhs,
+                            int32_t imm);
   inline bool emit_i64_divs(LiftoffRegister dst, LiftoffRegister lhs,
                             LiftoffRegister rhs, Label* trap_div_by_zero,
                             Label* trap_div_unrepresentable);
@@ -998,54 +954,14 @@ class LiftoffAssembler : public MacroAssembler {
 
   inline void emit_u32_to_uintptr(Register dst, Register src);
 
-  void emit_ptrsize_add(Register dst, Register lhs, Register rhs) {
-    if (kSystemPointerSize == 8) {
-      emit_i64_add(LiftoffRegister(dst), LiftoffRegister(lhs),
-                   LiftoffRegister(rhs));
-    } else {
-      emit_i32_add(dst, lhs, rhs);
-    }
-  }
-  void emit_ptrsize_sub(Register dst, Register lhs, Register rhs) {
-    if (kSystemPointerSize == 8) {
-      emit_i64_sub(LiftoffRegister(dst), LiftoffRegister(lhs),
-                   LiftoffRegister(rhs));
-    } else {
-      emit_i32_sub(dst, lhs, rhs);
-    }
-  }
-  void emit_ptrsize_and(Register dst, Register lhs, Register rhs) {
-    if (kSystemPointerSize == 8) {
-      emit_i64_and(LiftoffRegister(dst), LiftoffRegister(lhs),
-                   LiftoffRegister(rhs));
-    } else {
-      emit_i32_and(dst, lhs, rhs);
-    }
-  }
-  void emit_ptrsize_shri(Register dst, Register src, int amount) {
-    if (kSystemPointerSize == 8) {
-      emit_i64_shri(LiftoffRegister(dst), LiftoffRegister(src), amount);
-    } else {
-      emit_i32_shri(dst, src, amount);
-    }
-  }
-
-  void emit_ptrsize_addi(Register dst, Register lhs, intptr_t imm) {
-    if (kSystemPointerSize == 8) {
-      emit_i64_addi(LiftoffRegister(dst), LiftoffRegister(lhs), imm);
-    } else {
-      emit_i32_addi(dst, lhs, static_cast<int32_t>(imm));
-    }
-  }
-
-  void emit_ptrsize_set_cond(Condition condition, Register dst,
-                             LiftoffRegister lhs, LiftoffRegister rhs) {
-    if (kSystemPointerSize == 8) {
-      emit_i64_set_cond(condition, dst, lhs, rhs);
-    } else {
-      emit_i32_set_cond(condition, dst, lhs.gp(), rhs.gp());
-    }
-  }
+  inline void emit_ptrsize_add(Register dst, Register lhs, Register rhs);
+  inline void emit_ptrsize_sub(Register dst, Register lhs, Register rhs);
+  inline void emit_ptrsize_and(Register dst, Register lhs, Register rhs);
+  inline void emit_ptrsize_shri(Register dst, Register src, int amount);
+  inline void emit_ptrsize_addi(Register dst, Register lhs, intptr_t imm);
+  inline void emit_ptrsize_muli(Register dst, Register lhs, int32_t imm);
+  inline void emit_ptrsize_set_cond(Condition condition, Register dst,
+                                    LiftoffRegister lhs, LiftoffRegister rhs);
 
   // f32 binops.
   inline void emit_f32_add(DoubleRegister dst, DoubleRegister lhs,
@@ -1165,7 +1081,8 @@ class LiftoffAssembler : public MacroAssembler {
   inline void emit_s128_relaxed_laneselect(LiftoffRegister dst,
                                            LiftoffRegister src1,
                                            LiftoffRegister src2,
-                                           LiftoffRegister mask);
+                                           LiftoffRegister mask,
+                                           int lane_width);
   inline void emit_i8x16_popcnt(LiftoffRegister dst, LiftoffRegister src);
   inline void emit_i8x16_splat(LiftoffRegister dst, LiftoffRegister src);
   inline void emit_i16x8_splat(LiftoffRegister dst, LiftoffRegister src);
@@ -1594,7 +1511,10 @@ class LiftoffAssembler : public MacroAssembler {
   inline void emit_f64x2_qfms(LiftoffRegister dst, LiftoffRegister src1,
                               LiftoffRegister src2, LiftoffRegister src3);
 
-  inline void StackCheck(Label* ool_code, Register limit_address);
+  inline void set_trap_on_oob_mem64(Register index, int oob_shift,
+                                    MemOperand oob_offset);
+
+  inline void StackCheck(Label* ool_code);
 
   inline void AssertUnreachable(AbortReason reason);
 
@@ -1612,9 +1532,15 @@ class LiftoffAssembler : public MacroAssembler {
   // this is the return value of the C function, stored in {rets[0]}. Further
   // outputs (specified in {sig->returns()}) are read from the buffer and stored
   // in the remaining {rets} registers.
+  inline void CallCWithStackBuffer(const std::initializer_list<VarState> args,
+                                   const LiftoffRegister* rets,
+                                   ValueKind return_kind,
+                                   ValueKind out_argument_kind, int stack_bytes,
+                                   ExternalReference ext_ref);
+
+  // Execute a C call with arguments passed according to the C calling
+  // conventions.
   inline void CallC(const std::initializer_list<VarState> args,
-                    const LiftoffRegister* rets, ValueKind return_kind,
-                    ValueKind out_argument_kind, int stack_bytes,
                     ExternalReference ext_ref);
 
   inline void CallNativeWasmCode(Address addr);
@@ -1624,7 +1550,7 @@ class LiftoffAssembler : public MacroAssembler {
                            compiler::CallDescriptor* call_descriptor,
                            Register target);
   inline void TailCallIndirect(Register target);
-  inline void CallRuntimeStub(WasmCode::RuntimeStubId sid);
+  inline void CallBuiltin(Builtin builtin);
 
   // Reserve space in the current frame, store address to space in {addr}.
   inline void AllocateStackSlot(Register addr, uint32_t size);
@@ -1672,13 +1598,7 @@ class LiftoffAssembler : public MacroAssembler {
   LiftoffBailoutReason bailout_reason() const { return bailout_reason_; }
   const char* bailout_detail() const { return bailout_detail_; }
 
-  void bailout(LiftoffBailoutReason reason, const char* detail) {
-    DCHECK_NE(kSuccess, reason);
-    if (bailout_reason_ != kSuccess) return;
-    AbortCompilation();
-    bailout_reason_ = reason;
-    bailout_detail_ = detail;
-  }
+  inline void bailout(LiftoffBailoutReason reason, const char* detail);
 
  private:
   LiftoffRegister LoadI64HalfIntoRegister(VarState slot, RegPairHalf half);
@@ -1716,112 +1636,6 @@ inline FreezeCacheState::FreezeCacheState(LiftoffAssembler& assm)
 inline FreezeCacheState::~FreezeCacheState() { assm_.UnfreezeCacheState(); }
 #endif
 
-// =======================================================================
-// Partially platform-independent implementations of the platform-dependent
-// part.
-
-#ifdef V8_TARGET_ARCH_32_BIT
-
-namespace liftoff {
-template <void (LiftoffAssembler::*op)(Register, Register, Register)>
-void EmitI64IndependentHalfOperation(LiftoffAssembler* assm,
-                                     LiftoffRegister dst, LiftoffRegister lhs,
-                                     LiftoffRegister rhs) {
-  // If {dst.low_gp()} does not overlap with {lhs.high_gp()} or {rhs.high_gp()},
-  // just first compute the lower half, then the upper half.
-  if (dst.low() != lhs.high() && dst.low() != rhs.high()) {
-    (assm->*op)(dst.low_gp(), lhs.low_gp(), rhs.low_gp());
-    (assm->*op)(dst.high_gp(), lhs.high_gp(), rhs.high_gp());
-    return;
-  }
-  // If {dst.high_gp()} does not overlap with {lhs.low_gp()} or {rhs.low_gp()},
-  // we can compute this the other way around.
-  if (dst.high() != lhs.low() && dst.high() != rhs.low()) {
-    (assm->*op)(dst.high_gp(), lhs.high_gp(), rhs.high_gp());
-    (assm->*op)(dst.low_gp(), lhs.low_gp(), rhs.low_gp());
-    return;
-  }
-  // Otherwise, we need a temporary register.
-  Register tmp = assm->GetUnusedRegister(kGpReg, LiftoffRegList{lhs, rhs}).gp();
-  (assm->*op)(tmp, lhs.low_gp(), rhs.low_gp());
-  (assm->*op)(dst.high_gp(), lhs.high_gp(), rhs.high_gp());
-  assm->Move(dst.low_gp(), tmp, kI32);
-}
-
-template <void (LiftoffAssembler::*op)(Register, Register, int32_t)>
-void EmitI64IndependentHalfOperationImm(LiftoffAssembler* assm,
-                                        LiftoffRegister dst,
-                                        LiftoffRegister lhs, int64_t imm) {
-  int32_t low_word = static_cast<int32_t>(imm);
-  int32_t high_word = static_cast<int32_t>(imm >> 32);
-  // If {dst.low_gp()} does not overlap with {lhs.high_gp()},
-  // just first compute the lower half, then the upper half.
-  if (dst.low() != lhs.high()) {
-    (assm->*op)(dst.low_gp(), lhs.low_gp(), low_word);
-    (assm->*op)(dst.high_gp(), lhs.high_gp(), high_word);
-    return;
-  }
-  // If {dst.high_gp()} does not overlap with {lhs.low_gp()},
-  // we can compute this the other way around.
-  if (dst.high() != lhs.low()) {
-    (assm->*op)(dst.high_gp(), lhs.high_gp(), high_word);
-    (assm->*op)(dst.low_gp(), lhs.low_gp(), low_word);
-    return;
-  }
-  // Otherwise, we need a temporary register.
-  Register tmp = assm->GetUnusedRegister(kGpReg, LiftoffRegList{lhs}).gp();
-  (assm->*op)(tmp, lhs.low_gp(), low_word);
-  (assm->*op)(dst.high_gp(), lhs.high_gp(), high_word);
-  assm->Move(dst.low_gp(), tmp, kI32);
-}
-}  // namespace liftoff
-
-void LiftoffAssembler::emit_i64_and(LiftoffRegister dst, LiftoffRegister lhs,
-                                    LiftoffRegister rhs) {
-  liftoff::EmitI64IndependentHalfOperation<&LiftoffAssembler::emit_i32_and>(
-      this, dst, lhs, rhs);
-}
-
-void LiftoffAssembler::emit_i64_andi(LiftoffRegister dst, LiftoffRegister lhs,
-                                     int32_t imm) {
-  liftoff::EmitI64IndependentHalfOperationImm<&LiftoffAssembler::emit_i32_andi>(
-      this, dst, lhs, imm);
-}
-
-void LiftoffAssembler::emit_i64_or(LiftoffRegister dst, LiftoffRegister lhs,
-                                   LiftoffRegister rhs) {
-  liftoff::EmitI64IndependentHalfOperation<&LiftoffAssembler::emit_i32_or>(
-      this, dst, lhs, rhs);
-}
-
-void LiftoffAssembler::emit_i64_ori(LiftoffRegister dst, LiftoffRegister lhs,
-                                    int32_t imm) {
-  liftoff::EmitI64IndependentHalfOperationImm<&LiftoffAssembler::emit_i32_ori>(
-      this, dst, lhs, imm);
-}
-
-void LiftoffAssembler::emit_i64_xor(LiftoffRegister dst, LiftoffRegister lhs,
-                                    LiftoffRegister rhs) {
-  liftoff::EmitI64IndependentHalfOperation<&LiftoffAssembler::emit_i32_xor>(
-      this, dst, lhs, rhs);
-}
-
-void LiftoffAssembler::emit_i64_xori(LiftoffRegister dst, LiftoffRegister lhs,
-                                     int32_t imm) {
-  liftoff::EmitI64IndependentHalfOperationImm<&LiftoffAssembler::emit_i32_xori>(
-      this, dst, lhs, imm);
-}
-
-void LiftoffAssembler::emit_u32_to_uintptr(Register dst, Register src) {
-  // This is a no-op on 32-bit systems.
-}
-
-#endif  // V8_TARGET_ARCH_32_BIT
-
-// End of the partially platform-independent implementations of the
-// platform-dependent part.
-// =======================================================================
-
 class LiftoffStackSlots {
  public:
   explicit LiftoffStackSlots(LiftoffAssembler* wasm_asm) : asm_(wasm_asm) {}
@@ -1881,33 +1695,6 @@ class LiftoffStackSlots {
 bool CompatibleStackSlotTypes(ValueKind a, ValueKind b);
 #endif
 
-}  // namespace wasm
-}  // namespace internal
-}  // namespace v8
-
-// Include platform specific implementation.
-#if V8_TARGET_ARCH_IA32
-#include "src/wasm/baseline/ia32/liftoff-assembler-ia32.h"
-#elif V8_TARGET_ARCH_X64
-#include "src/wasm/baseline/x64/liftoff-assembler-x64.h"
-#elif V8_TARGET_ARCH_ARM64
-#include "src/wasm/baseline/arm64/liftoff-assembler-arm64.h"
-#elif V8_TARGET_ARCH_ARM
-#include "src/wasm/baseline/arm/liftoff-assembler-arm.h"
-#elif V8_TARGET_ARCH_PPC || V8_TARGET_ARCH_PPC64
-#include "src/wasm/baseline/ppc/liftoff-assembler-ppc.h"
-#elif V8_TARGET_ARCH_MIPS64
-#include "src/wasm/baseline/mips64/liftoff-assembler-mips64.h"
-#elif V8_TARGET_ARCH_LOONG64
-#include "src/wasm/baseline/loong64/liftoff-assembler-loong64.h"
-#elif V8_TARGET_ARCH_S390
-#include "src/wasm/baseline/s390/liftoff-assembler-s390.h"
-#elif V8_TARGET_ARCH_RISCV64
-#include "src/wasm/baseline/riscv/liftoff-assembler-riscv64.h"
-#elif V8_TARGET_ARCH_RISCV32
-#include "src/wasm/baseline/riscv/liftoff-assembler-riscv32.h"
-#else
-#error Unsupported architecture.
-#endif
+}  // namespace v8::internal::wasm
 
 #endif  // V8_WASM_BASELINE_LIFTOFF_ASSEMBLER_H_
