@@ -10,33 +10,97 @@ namespace v8 {
 namespace internal {
 namespace wasm {
 
+// static
+void JumpTableAssembler::GenerateLazyCompileTable(
+    Address base, uint32_t num_slots, uint32_t num_imported_functions,
+    Address wasm_compile_lazy_target) {
+  uint32_t lazy_compile_table_size = num_slots * kLazyCompileTableSlotSize;
+  WritableJitAllocation jit_allocation = ThreadIsolation::LookupJitAllocation(
+      base, RoundUp<kCodeAlignment>(lazy_compile_table_size),
+      ThreadIsolation::JitAllocationType::kWasmLazyCompileTable);
+  // Assume enough space, so the Assembler does not try to grow the buffer.
+  JumpTableAssembler jtasm(base, lazy_compile_table_size + 256);
+  for (uint32_t slot_index = 0; slot_index < num_slots; ++slot_index) {
+    DCHECK_EQ(slot_index * kLazyCompileTableSlotSize, jtasm.pc_offset());
+    jtasm.EmitLazyCompileJumpSlot(slot_index + num_imported_functions,
+                                  wasm_compile_lazy_target);
+  }
+  DCHECK_EQ(lazy_compile_table_size, jtasm.pc_offset());
+  FlushInstructionCache(base, lazy_compile_table_size);
+}
+
+void JumpTableAssembler::InitializeJumpsToLazyCompileTable(
+    Address base, uint32_t num_slots, Address lazy_compile_table_start) {
+  uint32_t jump_table_size = SizeForNumberOfSlots(num_slots);
+  WritableJitAllocation jit_allocation = ThreadIsolation::LookupJitAllocation(
+      base, RoundUp<kCodeAlignment>(jump_table_size),
+      ThreadIsolation::JitAllocationType::kWasmJumpTable);
+  JumpTableAssembler jtasm(base, jump_table_size + 256);
+
+  for (uint32_t slot_index = 0; slot_index < num_slots; ++slot_index) {
+    // Make sure we write at the correct offset.
+    int slot_offset =
+        static_cast<int>(JumpTableAssembler::JumpSlotIndexToOffset(slot_index));
+
+    jtasm.SkipUntil(slot_offset);
+
+    Address target =
+        lazy_compile_table_start +
+        JumpTableAssembler::LazyCompileSlotIndexToOffset(slot_index);
+
+    int offset_before_emit = jtasm.pc_offset();
+    // This function initializes the first jump table with jumps to the lazy
+    // compile table. Both get allocated in the constructor of the
+    // {NativeModule}, so they both should end up in the initial code space.
+    // Jumps within one code space can always be near jumps, so the following
+    // call to {EmitJumpSlot} should always succeed. If the call fails, then
+    // either the jump table allocation was changed incorrectly so that the lazy
+    // compile table was not within near-jump distance of the jump table
+    // anymore (e.g. the initial code space was too small to fit both tables),
+    // or the code space was allocated larger than the maximum near-jump
+    // distance.
+    CHECK(jtasm.EmitJumpSlot(target));
+    int written_bytes = jtasm.pc_offset() - offset_before_emit;
+    // We write nops here instead of skipping to avoid partial instructions in
+    // the jump table. Partial instructions can cause problems for the
+    // disassembler.
+    jtasm.NopBytes(kJumpTableSlotSize - written_bytes);
+  }
+  FlushInstructionCache(base, jump_table_size);
+}
+
 // The implementation is compact enough to implement it inline here. If it gets
 // much bigger, we might want to split it in a separate file per architecture.
 #if V8_TARGET_ARCH_X64
 void JumpTableAssembler::EmitLazyCompileJumpSlot(uint32_t func_index,
                                                  Address lazy_compile_target) {
   // Use a push, because mov to an extended register takes 6 bytes.
-  pushq_imm32(func_index);            // 5 bytes
-  EmitJumpSlot(lazy_compile_target);  // 5 bytes
+  pushq_imm32(func_index);  // 5 bytes
+  intptr_t displacement =
+      static_cast<intptr_t>(reinterpret_cast<uint8_t*>(lazy_compile_target) -
+                            (pc_ + kNearJmpInstrSize));
+  DCHECK(is_int32(displacement));
+  near_jmp(displacement, RelocInfo::NO_INFO);  // 5 bytes
 }
 
 bool JumpTableAssembler::EmitJumpSlot(Address target) {
-  intptr_t displacement = static_cast<intptr_t>(
-      reinterpret_cast<byte*>(target) - pc_ - kNearJmpInstrSize);
+  intptr_t displacement =
+      static_cast<intptr_t>(reinterpret_cast<uint8_t*>(target) -
+                            (pc_ + kEndbrSize + kNearJmpInstrSize));
   if (!is_int32(displacement)) return false;
+  CodeEntry();                                 // kEndbrSize bytes (0 or 4)
   near_jmp(displacement, RelocInfo::NO_INFO);  // 5 bytes
   return true;
 }
 
 void JumpTableAssembler::EmitFarJumpSlot(Address target) {
   Label data;
-  int start_offset = pc_offset();
+  [[maybe_unused]] int start_offset = pc_offset();
   jmp(Operand(&data));  // 6 bytes
   Nop(2);               // 2 bytes
   // The data must be properly aligned, so it can be patched atomically (see
   // {PatchFarJumpSlot}).
   DCHECK_EQ(start_offset + kSystemPointerSize, pc_offset());
-  USE(start_offset);
   bind(&data);
   dq(target);  // 8 bytes
 }
@@ -54,8 +118,12 @@ void JumpTableAssembler::PatchFarJumpSlot(Address slot, Address target) {
 }
 
 void JumpTableAssembler::NopBytes(int bytes) {
-  DCHECK_LE(0, bytes);
-  Nop(bytes);
+  if (bytes) Nop(bytes);
+}
+
+void JumpTableAssembler::SkipUntil(int offset) {
+  DCHECK_GE(offset, pc_offset());
+  pc_ += offset - pc_offset();
 }
 
 #elif V8_TARGET_ARCH_IA32
@@ -80,8 +148,12 @@ void JumpTableAssembler::PatchFarJumpSlot(Address slot, Address target) {
 }
 
 void JumpTableAssembler::NopBytes(int bytes) {
-  DCHECK_LE(0, bytes);
-  Nop(bytes);
+  if (bytes) Nop(bytes);
+}
+
+void JumpTableAssembler::SkipUntil(int offset) {
+  DCHECK_GE(offset, pc_offset());
+  pc_ += offset - pc_offset();
 }
 
 #elif V8_TARGET_ARCH_ARM
@@ -129,6 +201,12 @@ void JumpTableAssembler::NopBytes(int bytes) {
   }
 }
 
+void JumpTableAssembler::SkipUntil(int offset) {
+  // On this platform the jump table is not zapped with valid instructions, so
+  // skipping over bytes is not allowed.
+  DCHECK_EQ(offset, pc_offset());
+}
+
 #elif V8_TARGET_ARCH_ARM64
 void JumpTableAssembler::EmitLazyCompileJumpSlot(uint32_t func_index,
                                                  Address lazy_compile_target) {
@@ -142,14 +220,27 @@ void JumpTableAssembler::EmitLazyCompileJumpSlot(uint32_t func_index,
 }
 
 bool JumpTableAssembler::EmitJumpSlot(Address target) {
-  if (!TurboAssembler::IsNearCallOffset(
-          (reinterpret_cast<byte*>(target) - pc_) / kInstrSize)) {
+#ifdef V8_ENABLE_CONTROL_FLOW_INTEGRITY
+  static constexpr ptrdiff_t kCodeEntryMarkerSize = kInstrSize;
+#else
+  static constexpr ptrdiff_t kCodeEntryMarkerSize = 0;
+#endif
+
+  uint8_t* jump_pc = pc_ + kCodeEntryMarkerSize;
+  ptrdiff_t jump_distance = reinterpret_cast<uint8_t*>(target) - jump_pc;
+  DCHECK_EQ(0, jump_distance % kInstrSize);
+  int64_t instr_offset = jump_distance / kInstrSize;
+  if (!MacroAssembler::IsNearCallOffset(instr_offset)) {
     return false;
   }
 
   CodeEntry();
 
-  Jump(target, RelocInfo::NO_INFO);
+  DCHECK_EQ(jump_pc, pc_);
+  DCHECK_EQ(instr_offset,
+            reinterpret_cast<Instr*>(target) - reinterpret_cast<Instr*>(pc_));
+  DCHECK(is_int26(instr_offset));
+  b(static_cast<int>(instr_offset));
   return true;
 }
 
@@ -199,6 +290,12 @@ void JumpTableAssembler::NopBytes(int bytes) {
   }
 }
 
+void JumpTableAssembler::SkipUntil(int offset) {
+  // On this platform the jump table is not zapped with valid instructions, so
+  // skipping over bytes is not allowed.
+  DCHECK_EQ(offset, pc_offset());
+}
+
 #elif V8_TARGET_ARCH_S390X
 void JumpTableAssembler::EmitLazyCompileJumpSlot(uint32_t func_index,
                                                  Address lazy_compile_target) {
@@ -210,7 +307,7 @@ void JumpTableAssembler::EmitLazyCompileJumpSlot(uint32_t func_index,
 }
 
 bool JumpTableAssembler::EmitJumpSlot(Address target) {
-  intptr_t relative_target = reinterpret_cast<byte*>(target) - pc_;
+  intptr_t relative_target = reinterpret_cast<uint8_t*>(target) - pc_;
 
   if (!is_int32(relative_target / 2)) {
     return false;
@@ -246,6 +343,12 @@ void JumpTableAssembler::NopBytes(int bytes) {
   }
 }
 
+void JumpTableAssembler::SkipUntil(int offset) {
+  // On this platform the jump table is not zapped with valid instructions, so
+  // skipping over bytes is not allowed.
+  DCHECK_EQ(offset, pc_offset());
+}
+
 #elif V8_TARGET_ARCH_MIPS64
 void JumpTableAssembler::EmitLazyCompileJumpSlot(uint32_t func_index,
                                                  Address lazy_compile_target) {
@@ -265,7 +368,8 @@ bool JumpTableAssembler::EmitJumpSlot(Address target) {
 }
 
 void JumpTableAssembler::EmitFarJumpSlot(Address target) {
-  JumpToOffHeapInstructionStream(target);
+  li(t9, Operand(target, RelocInfo::OFF_HEAP_TARGET));
+  Jump(t9);
 }
 
 // static
@@ -281,28 +385,49 @@ void JumpTableAssembler::NopBytes(int bytes) {
   }
 }
 
+void JumpTableAssembler::SkipUntil(int offset) {
+  // On this platform the jump table is not zapped with valid instructions, so
+  // skipping over bytes is not allowed.
+  DCHECK_EQ(offset, pc_offset());
+}
+
 #elif V8_TARGET_ARCH_LOONG64
 void JumpTableAssembler::EmitLazyCompileJumpSlot(uint32_t func_index,
                                                  Address lazy_compile_target) {
   DCHECK(is_int32(func_index));
   int start = pc_offset();
   li(kWasmCompileLazyFuncIndexRegister, (int32_t)func_index);  // max. 2 instr
-  // Jump produces max 4 instructions.
-  Jump(lazy_compile_target, RelocInfo::NO_INFO);
+  // EmitJumpSlot produces 1 instructions.
+  CHECK(EmitJumpSlot(lazy_compile_target));
   int nop_bytes = start + kLazyCompileTableSlotSize - pc_offset();
   DCHECK_EQ(nop_bytes % kInstrSize, 0);
   for (int i = 0; i < nop_bytes; i += kInstrSize) nop();
 }
 bool JumpTableAssembler::EmitJumpSlot(Address target) {
-  PatchAndJump(target);
+  intptr_t relative_target = reinterpret_cast<uint8_t*>(target) - pc_;
+  DCHECK_EQ(relative_target % 4, 0);
+  intptr_t instr_offset = relative_target / kInstrSize;
+  if (!is_int26(instr_offset)) {
+    return false;
+  }
+
+  b(instr_offset);
   return true;
 }
 void JumpTableAssembler::EmitFarJumpSlot(Address target) {
-  JumpToOffHeapInstructionStream(target);
+  pcaddi(t7, 4);
+  Ld_d(t7, MemOperand(t7, 0));
+  jirl(zero_reg, t7, 0);
+  nop();  // pc_ should be align.
+  DCHECK_EQ(reinterpret_cast<uint64_t>(pc_) % 8, 0);
+  dq(target);
 }
 void JumpTableAssembler::PatchFarJumpSlot(Address slot, Address target) {
-  UNREACHABLE();
+  Address target_addr = slot + kFarJumpTableSlotSize - 8;
+  reinterpret_cast<std::atomic<Address>*>(target_addr)
+      ->store(target, std::memory_order_relaxed);
 }
+
 void JumpTableAssembler::NopBytes(int bytes) {
   DCHECK_LE(0, bytes);
   DCHECK_EQ(0, bytes % kInstrSize);
@@ -311,6 +436,12 @@ void JumpTableAssembler::NopBytes(int bytes) {
   }
 }
 
+void JumpTableAssembler::SkipUntil(int offset) {
+  // On this platform the jump table is not zapped with valid instructions, so
+  // skipping over bytes is not allowed.
+  DCHECK_EQ(offset, pc_offset());
+}
+
 #elif V8_TARGET_ARCH_PPC64
 void JumpTableAssembler::EmitLazyCompileJumpSlot(uint32_t func_index,
                                                  Address lazy_compile_target) {
@@ -327,7 +458,7 @@ void JumpTableAssembler::EmitLazyCompileJumpSlot(uint32_t func_index,
 }
 
 bool JumpTableAssembler::EmitJumpSlot(Address target) {
-  intptr_t relative_target = reinterpret_cast<byte*>(target) - pc_;
+  intptr_t relative_target = reinterpret_cast<uint8_t*>(target) - pc_;
 
   if (!is_int26(relative_target)) {
     return false;
@@ -338,13 +469,13 @@ bool JumpTableAssembler::EmitJumpSlot(Address target) {
 }
 
 void JumpTableAssembler::EmitFarJumpSlot(Address target) {
-  byte* start = pc_;
+  uint8_t* start = pc_;
   mov(ip, Operand(reinterpret_cast<Address>(start + kFarJumpTableSlotSize -
                                             8)));  // 5 instr
   LoadU64(ip, MemOperand(ip));
   mtctr(ip);
   bctr();
-  byte* end = pc_;
+  uint8_t* end = pc_;
   int used = end - start;
   CHECK(used < kFarJumpTableSlotSize - 8);
   NopBytes(kFarJumpTableSlotSize - 8 - used);
@@ -367,6 +498,12 @@ void JumpTableAssembler::NopBytes(int bytes) {
   }
 }
 
+void JumpTableAssembler::SkipUntil(int offset) {
+  // On this platform the jump table is not zapped with valid instructions, so
+  // skipping over bytes is not allowed.
+  DCHECK_EQ(offset, pc_offset());
+}
+
 #elif V8_TARGET_ARCH_RISCV64
 void JumpTableAssembler::EmitLazyCompileJumpSlot(uint32_t func_index,
                                                  Address lazy_compile_target) {
@@ -407,6 +544,12 @@ void JumpTableAssembler::NopBytes(int bytes) {
   }
 }
 
+void JumpTableAssembler::SkipUntil(int offset) {
+  // On this platform the jump table is not zapped with valid instructions, so
+  // skipping over bytes is not allowed.
+  DCHECK_EQ(offset, pc_offset());
+}
+
 #elif V8_TARGET_ARCH_RISCV32
 void JumpTableAssembler::EmitLazyCompileJumpSlot(uint32_t func_index,
                                                  Address lazy_compile_target) {
@@ -447,6 +590,12 @@ void JumpTableAssembler::NopBytes(int bytes) {
   }
 }
 
+void JumpTableAssembler::SkipUntil(int offset) {
+  // On this platform the jump table is not zapped with valid instructions, so
+  // skipping over bytes is not allowed.
+  DCHECK_EQ(offset, pc_offset());
+}
+
 #else
 #error Unknown architecture.
 #endif
