@@ -78,14 +78,6 @@ struct PropertyBase {
   // - v8::X::Y
   // - X::Y
   const char* type;
-
-  // In some cases, |type| may be a simple type representing a compressed
-  // pointer such as v8::internal::TaggedValue. In those cases,
-  // |decompressed_type| will contain the type of the object when decompressed.
-  // Otherwise, |decompressed_type| will match |type|. In any case, it is safe
-  // to pass the |decompressed_type| value as the type_hint on a subsequent call
-  // to GetObjectProperties.
-  const char* decompressed_type;
 };
 
 struct StructProperty : public PropertyBase {
@@ -144,10 +136,15 @@ struct ObjectPropertiesResult {
   const char** guessed_types;
 };
 
+struct StackFrameResult {
+  size_t num_properties;
+  ObjectProperty** properties;
+};
+
 // Copies byte_count bytes of memory from the given address in the debuggee to
 // the destination buffer.
 typedef MemoryAccessResult (*MemoryAccessor)(uintptr_t address,
-                                             uint8_t* destination,
+                                             void* destination,
                                              size_t byte_count);
 
 // Additional data that can help GetObjectProperties to be more accurate. Any
@@ -174,12 +171,6 @@ struct HeapAddresses {
   uintptr_t any_heap_pointer;
 };
 
-// Result type for ListObjectClasses.
-struct ClassList {
-  size_t num_class_names;
-  const char* const* class_names;  // Fully qualified class names.
-};
-
 }  // namespace debug_helper
 }  // namespace v8
 
@@ -193,8 +184,11 @@ _v8_debug_helper_GetObjectProperties(
     const char* type_hint);
 V8_DEBUG_HELPER_EXPORT void _v8_debug_helper_Free_ObjectPropertiesResult(
     v8::debug_helper::ObjectPropertiesResult* result);
-V8_DEBUG_HELPER_EXPORT const v8::debug_helper::ClassList*
-_v8_debug_helper_ListObjectClasses();
+V8_DEBUG_HELPER_EXPORT v8::debug_helper::StackFrameResult*
+_v8_debug_helper_GetStackFrame(
+    uintptr_t frame_pointer, v8::debug_helper::MemoryAccessor memory_accessor);
+V8_DEBUG_HELPER_EXPORT void _v8_debug_helper_Free_StackFrameResult(
+    v8::debug_helper::StackFrameResult* result);
 V8_DEBUG_HELPER_EXPORT const char* _v8_debug_helper_BitsetName(
     uint64_t payload);
 }
@@ -226,17 +220,26 @@ inline ObjectPropertiesResultPtr GetObjectProperties(
       object, memory_accessor, heap_addresses, type_hint));
 }
 
-// Get a list of all class names deriving from v8::internal::Object.
-inline const ClassList* ListObjectClasses() {
-  return _v8_debug_helper_ListObjectClasses();
-}
-
 // Return a bitset name for a v8::internal::compiler::Type with payload or null
 // if the payload is not a bitset.
 inline const char* BitsetName(uint64_t payload) {
   return _v8_debug_helper_BitsetName(payload);
 }
 
+struct DebugHelperStackFrameResultDeleter {
+  void operator()(v8::debug_helper::StackFrameResult* ptr) {
+    _v8_debug_helper_Free_StackFrameResult(ptr);
+  }
+};
+using StackFrameResultPtr =
+    std::unique_ptr<StackFrameResult, DebugHelperStackFrameResultDeleter>;
+
+inline StackFrameResultPtr GetStackFrame(
+    uintptr_t frame_pointer, v8::debug_helper::MemoryAccessor memory_accessor) {
+  return StackFrameResultPtr(
+      _v8_debug_helper_GetStackFrame(frame_pointer, memory_accessor));
+}
+
 }  // namespace debug_helper
 }  // namespace v8
 
