@@ -4,7 +4,7 @@
 
 from collections import OrderedDict, namedtuple
 from functools import reduce
-from os.path import dirname as up
+from pathlib import Path
 
 import json
 import logging
@@ -18,6 +18,7 @@ import traceback
 from testrunner.build_config import BuildConfig
 from testrunner.local import testsuite
 from testrunner.local import utils
+from testrunner.local.variants import REQUIRED_BUILD_VARIABLES
 from testrunner.local.context import os_context
 from testrunner.test_config import TestConfig
 from testrunner.testproc import util
@@ -26,7 +27,7 @@ from testrunner.testproc.sigproc import SignalProc
 from testrunner.utils.augmented_options import AugmentedOptions
 
 
-DEFAULT_OUT_GN = 'out.gn'
+DEFAULT_OUT_GN = Path('out.gn')
 
 # Map of test name synonyms to lists of test suites. Should be ordered by
 # expected runtimes (suites with slow test cases first). These groups are
@@ -122,8 +123,8 @@ class TestRunnerError(Exception):
 
 class BaseTestRunner(object):
   def __init__(self, basedir=None):
-    self.v8_root = up(up(up(__file__)))
-    self.basedir = basedir or self.v8_root
+    self.v8_root = Path(__file__).absolute().parent.parent.parent
+    self.basedir = Path(basedir or self.v8_root)
     self.outdir = None
     self.build_config = None
     self.mode_options = None
@@ -169,17 +170,12 @@ class BaseTestRunner(object):
       args = self._parse_test_args(args)
 
       with os_context(self.target_os, self.options) as ctx:
+        self._setup_env()
         names = self._args_to_suite_names(args)
         tests = self._load_testsuite_generators(ctx, names)
-        self._setup_env()
         print(">>> Running tests for %s.%s" % (self.build_config.arch,
                                                self.mode_options.label))
-        exit_code = self._do_execute(tests, args, ctx)
-        if exit_code == utils.EXIT_CODE_FAILURES and self.options.json_test_results:
-          print("Force exit code 0 after failures. Json test results file "
-                "generated with failure information.")
-          exit_code = utils.EXIT_CODE_PASS
-      return exit_code
+        return self._do_execute(tests, args, ctx)
     except TestRunnerError:
       traceback.print_exc()
       return utils.EXIT_CODE_INTERNAL_ERROR
@@ -209,7 +205,7 @@ class BaseTestRunner(object):
     parser.add_option("--shell-dir", help="DEPRECATED! Executables from build "
                       "directory will be used")
     parser.add_option("--test-root", help="Root directory of the test suites",
-                      default=os.path.join(self.basedir, 'test'))
+                      default=self.basedir / 'test')
     parser.add_option("--total-timeout-sec", default=0, type="int",
                       help="How long should fuzzer run")
     parser.add_option("--swarming", default=False, action="store_true",
@@ -241,6 +237,10 @@ class BaseTestRunner(object):
                            "color, mono)")
     parser.add_option("--json-test-results",
                       help="Path to a file for storing json results.")
+    parser.add_option("--log-system-memory",
+                      help="Path to a file for storing system memory stats.")
+    parser.add_option("--log-test-schedule",
+                      help="Path to a file for streaming the test schedule to.")
     parser.add_option('--slow-tests-cutoff', type="int", default=100,
                       help='Collect N slowest tests')
     parser.add_option("--exit-after-n-failures", type="int", default=100,
@@ -298,10 +298,22 @@ class BaseTestRunner(object):
   def _parse_args(self, parser, sys_args):
     options, args = parser.parse_args(sys_args)
 
+    options.test_root = Path(options.test_root)
+    options.outdir = Path(options.outdir)
+
     if options.arch and ',' in options.arch:  # pragma: no cover
       print('Multiple architectures are deprecated')
       raise TestRunnerError()
 
+    # We write a test schedule and the system memory stats by default
+    # alongside json test results on bots.
+    if options.json_test_results:
+      result_dir = Path(options.json_test_results).parent
+      if not options.log_test_schedule:
+        options.log_test_schedule = result_dir / 'test_schedule.log'
+      if not options.log_system_memory:
+        options.log_system_memory = result_dir / 'memory_stats.log'
+
     return AugmentedOptions.augment(options), args
 
   def _load_build_config(self):
@@ -326,15 +338,20 @@ class BaseTestRunner(object):
       print(self.build_config)
 
     # Represents the OS where tests are run on. Same as host OS except for
-    # Android, which is determined by build output.
+    # Android and iOS, which are determined by build output.
     if self.build_config.is_android:
       self.target_os = 'android'
+    elif self.build_config.is_ios:
+      self.target_os = 'ios'
     else:
       self.target_os = utils.GuessOS()
 
+    # Verify integrity between build variables and variant configs.
+    self.build_config.ensure_vars(REQUIRED_BUILD_VARIABLES)
+
   def _do_load_build_config(self, outdir):
-    build_config_path = os.path.join(outdir, "v8_build_config.json")
-    if not os.path.exists(build_config_path):
+    build_config_path = outdir / "v8_build_config.json"
+    if not build_config_path.exists():
       if self.options.verbose:
         print("Didn't find build config: %s" % build_config_path)
       raise TestRunnerError()
@@ -347,7 +364,7 @@ class BaseTestRunner(object):
               % build_config_path)
         raise TestRunnerError()
 
-    return BuildConfig(build_config_json, self.options)
+    return BuildConfig(build_config_json)
 
   # Returns possible build paths in order:
   # gn
@@ -361,26 +378,25 @@ class BaseTestRunner(object):
 
       yield self.options.outdir
 
-      if os.path.basename(self.options.outdir) != 'build':
-        yield os.path.join(self.options.outdir, 'build')
+      if self.options.outdir.name != 'build':
+        yield self.options.outdir / 'build'
 
     for outdir in outdirs():
-      yield os.path.join(self.basedir, outdir)
+      yield self.basedir / outdir
 
   def _get_gn_outdir(self):
-    gn_out_dir = os.path.join(self.basedir, DEFAULT_OUT_GN)
+    gn_out_dir = self.basedir / DEFAULT_OUT_GN
     latest_timestamp = -1
     latest_config = None
-    for gn_config in os.listdir(gn_out_dir):
-      gn_config_dir = os.path.join(gn_out_dir, gn_config)
-      if not os.path.isdir(gn_config_dir):
+    for gn_config_dir in gn_out_dir.iterdir():
+      if not gn_config_dir.is_dir():
         continue
-      if os.path.getmtime(gn_config_dir) > latest_timestamp:
-        latest_timestamp = os.path.getmtime(gn_config_dir)
-        latest_config = gn_config
+      if gn_config_dir.stat().st_mtime > latest_timestamp:
+        latest_timestamp = gn_config_dir.stat().st_mtime
+        latest_config = gn_config_dir.name
     if latest_config:
       print(">>> Latest GN build found: %s" % latest_config)
-      return os.path.join(DEFAULT_OUT_GN, latest_config)
+      return DEFAULT_OUT_GN / latest_config
 
   def _custom_debug_mode(self):
     custom_debug_flags = ["--nohard-abort"]
@@ -396,7 +412,7 @@ class BaseTestRunner(object):
     )
 
   def _process_default_options(self):
-    if self.build_config.is_debug:
+    if self.build_config.debugging_features:
       self.mode_options = self._custom_debug_mode()
     elif self.build_config.dcheck_always_on:
       self.mode_options = TRY_RELEASE_MODE
@@ -421,6 +437,7 @@ class BaseTestRunner(object):
 
     self.options.command_prefix = shlex.split(self.options.command_prefix)
     self.options.extra_flags = sum(list(map(shlex.split, self.options.extra_flags)), [])
+    self.options.extra_d8_flags = []
 
   def _process_options(self):
     pass # pragma: no cover
@@ -440,6 +457,10 @@ class BaseTestRunner(object):
           'allow_user_segv_handler=1',
           'allocator_may_return_null=1',
       ]
+      if self.build_config.component_build:
+        # Some abseil symbols are observed as defined more than once in
+        # component builds.
+        asan_options += ['detect_odr_violation=0']
       if not utils.GuessOS() in ['macos', 'windows']:
         # LSAN is not available on mac and windows.
         asan_options.append('detect_leaks=1')
@@ -450,7 +471,7 @@ class BaseTestRunner(object):
         asan_options.append('detect_stack_use_after_return=0')
       os.environ['ASAN_OPTIONS'] = ":".join(asan_options)
 
-    if self.build_config.cfi_vptr:
+    if self.build_config.cfi:
       os.environ['UBSAN_OPTIONS'] = ":".join([
         'print_stacktrace=1',
         'print_summary=1',
@@ -458,7 +479,7 @@ class BaseTestRunner(object):
         symbolizer_option,
       ])
 
-    if self.build_config.ubsan_vptr:
+    if self.build_config.ubsan:
       os.environ['UBSAN_OPTIONS'] = ":".join([
         'print_stacktrace=1',
         symbolizer_option,
@@ -468,11 +489,8 @@ class BaseTestRunner(object):
       os.environ['MSAN_OPTIONS'] = symbolizer_option
 
     if self.build_config.tsan:
-      suppressions_file = os.path.join(
-          self.basedir,
-          'tools',
-          'sanitizers',
-          'tsan_suppressions.txt')
+      suppressions_file = (
+          self.basedir / 'tools' / 'sanitizers' / 'tsan_suppressions.txt')
       os.environ['TSAN_OPTIONS'] = " ".join([
         symbolizer_option,
         'suppressions=%s' % suppressions_file,
@@ -483,20 +501,17 @@ class BaseTestRunner(object):
       ])
 
   def _get_external_symbolizer_option(self):
-    external_symbolizer_path = os.path.join(
-        self.basedir,
-        'third_party',
-        'llvm-build',
-        'Release+Asserts',
-        'bin',
-        'llvm-symbolizer',
-    )
+    external_symbolizer_path = (
+        self.basedir / 'third_party' / 'llvm-build' / 'Release+Asserts' /
+        'bin' / 'llvm-symbolizer')
 
     if utils.IsWindows():
-      # Quote, because sanitizers might confuse colon as option separator.
-      external_symbolizer_path = '"%s.exe"' % external_symbolizer_path
+      external_symbolizer_path = external_symbolizer_path.with_suffix('.exe')
 
-    return 'external_symbolizer_path=%s' % external_symbolizer_path
+      # Quote, because sanitizers might confuse colon as option separator.
+      external_symbolizer_path = f'"{external_symbolizer_path}"'
+
+    return f'external_symbolizer_path={external_symbolizer_path}'
 
   def _parse_test_args(self, args):
     if not args:
@@ -523,12 +538,12 @@ class BaseTestRunner(object):
     variables = self._get_statusfile_variables()
 
     # Head generator with no elements
-    test_chain = testsuite.TestGenerator(0, [], [])
+    test_chain = testsuite.TestGenerator(0, [], [], [])
     for name in names:
       if self.options.verbose:
         print('>>> Loading test suite: %s' % name)
       suite = testsuite.TestSuite.Load(
-          ctx, os.path.join(self.options.test_root, name), test_config)
+          ctx, self.options.test_root / name, test_config)
 
       if self._is_testsuite_supported(suite):
         tests = suite.load_tests_from_disk(variables)
@@ -541,119 +556,63 @@ class BaseTestRunner(object):
     instances (see NumFuzzer for usage)."""
     return True
 
+  @property
+  def _no_simd_hardware(self):
+    # TODO(liviurau): Add some tests.
+    # We try to find all the reasons why we have no_simd.
+    if any(i in self.options.extra_flags for i in [
+        '--noenable-sse3', '--no-enable-sse3', '--noenable-ssse3',
+        '--no-enable-ssse3', '--noenable-sse4-1', '--no-enable-sse4_1'
+    ]):
+      return True
+
+    # Set no_simd_hardware on architectures without Simd enabled.
+    if (self.build_config.arch == 'mips64el' and
+        not self.build_config.simd_mips):
+      return True
+
+    if self.build_config.arch == 'loong64':
+      return True
+
+    # S390 hosts without VEF1 do not support Simd.
+    if (self.build_config.arch == 's390x' and
+        not self.build_config.simulator_run and
+        not utils.IsS390SimdSupported()):
+      return True
+
+    # Ppc64 processors earlier than POWER9 do not support Simd instructions
+    if (self.build_config.arch == 'ppc64' and
+        not self.build_config.simulator_run and
+        utils.GuessPowerProcessorVersion() < 9):
+      return True
+
+    return False
+
   def _get_statusfile_variables(self):
-    return {
-        "arch":
-            self.build_config.arch,
-        "asan":
-            self.build_config.asan,
-        "byteorder":
-            sys.byteorder,
-        "cfi_vptr":
-            self.build_config.cfi_vptr,
-        "code_comments":
-            self.build_config.code_comments,
-        "component_build":
-            self.build_config.component_build,
-        "conservative_stack_scanning":
-            self.build_config.conservative_stack_scanning,
-        "control_flow_integrity":
-            self.build_config.control_flow_integrity,
-        "concurrent_marking":
-            self.build_config.concurrent_marking,
-        "single_generation":
-            self.build_config.single_generation,
-        "dcheck_always_on":
-            self.build_config.dcheck_always_on,
-        "debug_code":
-            self.build_config.debug_code,
-        "deopt_fuzzer":
-            False,
-        "disassembler":
-            self.build_config.disassembler,
-        "endurance_fuzzer":
-            False,
-        "gc_fuzzer":
-            False,
-        "gc_stress":
-            False,
-        "gdbjit":
-            self.build_config.gdbjit,
-        # TODO(jgruber): Note this rename from maglev to has_maglev is required
-        # to avoid a name clash with the "maglev" variant. See also the TODO in
-        # statusfile.py (this really shouldn't be needed).
-        "has_maglev":
-            self.build_config.maglev,
-        "has_turbofan":
-            self.build_config.turbofan,
-        "has_webassembly":
-            self.build_config.webassembly,
-        "isolates":
-            self.options.isolates,
-        "is_clang":
-            self.build_config.is_clang,
-        "is_clang_coverage":
-            self.build_config.is_clang_coverage,
-        "is_debug":
-            self.build_config.is_debug,
-        "is_DEBUG_defined":
-            self.build_config.is_DEBUG_defined,
-        "is_full_debug":
-            self.build_config.is_full_debug,
-        "interrupt_fuzzer":
-            False,
-        "jitless_build_mode":
-            self.build_config.jitless_build_mode,
-        "mips_arch_variant":
-            self.build_config.mips_arch_variant,
-        "mode":
-            self.mode_options.status_mode,
-        "msan":
-            self.build_config.msan,
-        "no_harness":
-            self.options.no_harness,
-        "no_i18n":
-            self.build_config.no_i18n,
-        "no_simd_hardware":
-            self.build_config.no_simd_hardware,
-        "novfp3":
-            False,
-        "optimize_for_size":
-            "--optimize-for-size" in self.options.extra_flags,
-        "predictable":
-            self.build_config.predictable,
-        "simd_mips":
-            self.build_config.simd_mips,
-        "simulator_run":
-            self.build_config.simulator_run
-            and not self.options.dont_skip_simulator_slow_tests,
-        "slow_dchecks":
-            self.build_config.slow_dchecks,
-        "system":
-            self.target_os,
-        "third_party_heap":
-            self.build_config.third_party_heap,
-        "tsan":
-            self.build_config.tsan,
-        "ubsan_vptr":
-            self.build_config.ubsan_vptr,
-        "verify_csa":
-            self.build_config.verify_csa,
-        "verify_heap":
-            self.build_config.verify_heap,
-        "lite_mode":
-            self.build_config.lite_mode,
-        "pointer_compression":
-            self.build_config.pointer_compression,
-        "pointer_compression_shared_cage":
-            self.build_config.pointer_compression_shared_cage,
-        "no_js_shared_memory":
-            self.build_config.no_js_shared_memory,
-        "sandbox":
-            self.build_config.sandbox,
-        "dict_property_const_tracking":
-            self.build_config.dict_property_const_tracking,
-    }
+    """Returns all attributes accessible in status files.
+
+    All build-time flags from V8's BUILD.gn file as defined by the action
+    v8_dump_build_config can be accessed in status files.
+    """
+    variables = dict(self.build_config.items())
+    variables.update({
+        "byteorder": sys.byteorder,
+        "deopt_fuzzer": False,
+        "endurance_fuzzer": False,
+        "gc_fuzzer": False,
+        "gc_stress": False,
+        "isolates": self.options.isolates,
+        "interrupt_fuzzer": False,
+        "mode": self.mode_options.status_mode,
+        "no_harness": self.options.no_harness,
+        "no_simd_hardware": self._no_simd_hardware,
+        "novfp3": False,
+        "optimize_for_size": "--optimize-for-size" in self.options.extra_flags,
+        "simulator_run": variables["simulator_run"]
+                         and not self.options.dont_skip_simulator_slow_tests,
+        "system": self.target_os,
+    })
+    return variables
 
   def _runner_flags(self):
     """Extra default flags specific to the test runner implementation."""
@@ -666,16 +625,19 @@ class BaseTestRunner(object):
     return TestConfig(
         command_prefix=self.options.command_prefix,
         extra_flags=self.options.extra_flags,
+        extra_d8_flags=self.options.extra_d8_flags,
         framework_name=self.framework_name,
         isolates=self.options.isolates,
+        log_process_stats=self.options.json_test_results,
         mode_flags=self.mode_options.flags + self._runner_flags(),
         no_harness=self.options.no_harness,
-        noi18n=self.build_config.no_i18n,
+        noi18n=not self.build_config.i18n,
         random_seed=self.options.random_seed,
         run_skipped=self.options.run_skipped,
         shard_count=shard_count,
         shard_id=shard_id,
         shell_dir=self.outdir,
+        target_os=self.target_os,
         timeout=timeout,
         verbose=self.options.verbose,
         regenerate_expected_files=self.options.regenerate_expected_files,
