@@ -3,25 +3,10 @@
 # found in the LICENSE file.
 
 import collections
-import traceback
 
 from . import base
-from ..local import pool
-
-
-# Global function for multiprocessing, because pickling a static method doesn't
-# work on Windows.
-def run_job(job, process_context):
-  return job.run(process_context)
-
-
-def create_process_context(result_reduction):
-  return ProcessContext(result_reduction)
-
 
 JobResult = collections.namedtuple('JobResult', ['id', 'result'])
-ProcessContext = collections.namedtuple('ProcessContext', ['result_reduction'])
-
 
 class Job(object):
   def __init__(self, test_id, cmd, outproc, keep_output):
@@ -43,23 +28,19 @@ class ExecutionProc(base.TestProc):
   sends results to the previous processor.
   """
 
-  def __init__(self, jobs, outproc_factory=None):
+  def __init__(self, ctx, jobs, outproc_factory=None):
     super(ExecutionProc, self).__init__()
-    self._pool = pool.Pool(jobs)
+    self.ctx = ctx
+    self.ctx.pool.init(jobs, notify_function=self.notify_previous)
     self._outproc_factory = outproc_factory or (lambda t: t.output_proc)
     self._tests = {}
 
   def connect_to(self, next_proc):
-    assert False, 'ExecutionProc cannot be connected to anything'
+    assert False, \
+        'ExecutionProc cannot be connected to anything' # pragma: no cover
 
-  def run(self):
-    it = self._pool.imap_unordered(
-        fn=run_job,
-        gen=[],
-        process_context_fn=create_process_context,
-        process_context_args=[self._prev_requirement],
-    )
-    for pool_result in it:
+  def run(self, requirement=None):
+    for pool_result in self.ctx.pool.results(requirement):
       self._unpack_result(pool_result)
 
   def next_test(self, test):
@@ -67,20 +48,21 @@ class ExecutionProc(base.TestProc):
       return False
 
     test_id = test.procid
-    cmd = test.get_command()
+    cmd = test.get_command(self.ctx)
     self._tests[test_id] = test, cmd
 
     outproc = self._outproc_factory(test)
-    self._pool.add([Job(test_id, cmd, outproc, test.keep_output)])
+    self.ctx.pool.add_jobs([Job(test_id, cmd, outproc, test.keep_output)])
 
     return True
 
   def result_for(self, test, result):
-    assert False, 'ExecutionProc cannot receive results'
+    assert False, \
+        'ExecutionProc cannot receive results' # pragma: no cover
 
   def stop(self):
     super(ExecutionProc, self).stop()
-    self._pool.abort()
+    self.ctx.pool.abort()
 
   def _unpack_result(self, pool_result):
     if pool_result.heartbeat:
