@@ -199,7 +199,13 @@ function ClientRequest(input, options, cb) {
   const host = optsWithoutSignal.host = validateHost(options.hostname, 'hostname') ||
                                         validateHost(options.host, 'host') || 'localhost';
 
-  const setHost = (options.setHost === undefined || Boolean(options.setHost));
+  const setHost = options.setHost !== undefined ?
+    Boolean(options.setHost) :
+    options.setDefaultHeaders !== false;
+
+  this._removedConnection = options.setDefaultHeaders === false;
+  this._removedContLen = options.setDefaultHeaders === false;
+  this._removedTE = options.setDefaultHeaders === false;
 
   this.socketPath = options.socketPath;
 
