@@ -23,7 +23,6 @@
 
 const {
   MathMin,
-  RegExpPrototypeExec,
   Symbol,
 } = primordials;
 const { setImmediate } = require('timers');
@@ -86,8 +85,7 @@ function parserOnHeadersComplete(versionMajor, versionMinor, headers, method,
   }
 
   // Parser is also used by http client
-  const ParserIncomingMessage = (socket && socket.server &&
-                                 socket.server[kIncomingMessage]) ||
+  const ParserIncomingMessage = (socket?.server?.[kIncomingMessage]) ||
                                  IncomingMessage;
 
   const incoming = parser.incoming = new ParserIncomingMessage(socket);
@@ -210,7 +208,7 @@ const tokenRegExp = /^[\^_`a-zA-Z\-0-9!#$%&'*+.|~]+$/;
  * See https://tools.ietf.org/html/rfc7230#section-3.2.6
  */
 function checkIsHttpToken(val) {
-  return RegExpPrototypeExec(tokenRegExp, val) !== null;
+  return tokenRegExp.test(val);
 }
 
 const headerCharRegex = /[^\t\x20-\x7e\x80-\xff]/;
@@ -221,7 +219,7 @@ const headerCharRegex = /[^\t\x20-\x7e\x80-\xff]/;
  *  field-vchar    = VCHAR / obs-text
  */
 function checkInvalidHeaderChar(val) {
-  return RegExpPrototypeExec(headerCharRegex, val) !== null;
+  return headerCharRegex.test(val);
 }
 
 function cleanParser(parser) {
