@@ -79,6 +79,7 @@ const {
   },
 } = require('internal/errors');
 const {
+  assignFunctionName,
   kEmptyObject,
   promisify,
   SymbolAsyncDispose,
@@ -573,17 +574,17 @@ function Server(options, requestListener) {
 ObjectSetPrototypeOf(Server.prototype, net.Server.prototype);
 ObjectSetPrototypeOf(Server, net.Server);
 
-Server.prototype.close = function() {
+Server.prototype.close = function close() {
   httpServerPreClose(this);
   ReflectApply(net.Server.prototype.close, this, arguments);
   return this;
 };
 
-Server.prototype[SymbolAsyncDispose] = async function() {
-  return promisify(this.close).call(this);
-};
+Server.prototype[SymbolAsyncDispose] = assignFunctionName(SymbolAsyncDispose, async function() {
+  await promisify(this.close).call(this);
+});
 
-Server.prototype.closeAllConnections = function() {
+Server.prototype.closeAllConnections = function closeAllConnections() {
   if (!this[kConnections]) {
     return;
   }
@@ -595,7 +596,7 @@ Server.prototype.closeAllConnections = function() {
   }
 };
 
-Server.prototype.closeIdleConnections = function() {
+Server.prototype.closeIdleConnections = function closeIdleConnections() {
   if (!this[kConnections]) {
     return;
   }
@@ -618,7 +619,8 @@ Server.prototype.setTimeout = function setTimeout(msecs, callback) {
   return this;
 };
 
-Server.prototype[EE.captureRejectionSymbol] = function(err, event, ...args) {
+Server.prototype[EE.captureRejectionSymbol] =
+assignFunctionName(EE.captureRejectionSymbol, function(err, event, ...args) {
   switch (event) {
     case 'request': {
       const { 1: res } = args;
@@ -639,7 +641,7 @@ Server.prototype[EE.captureRejectionSymbol] = function(err, event, ...args) {
       net.Server.prototype[SymbolFor('nodejs.rejection')]
         .apply(this, arguments);
   }
-};
+});
 
 function checkConnections() {
   if (this.headersTimeout === 0 && this.requestTimeout === 0) {
