@@ -46,6 +46,7 @@ const {
 } = primordials;
 
 const {
+  assignFunctionName,
   convertToValidSignal,
   getSystemErrorName,
   kEmptyObject,
@@ -236,7 +237,7 @@ function exec(command, options, callback) {
 }
 
 const customPromiseExecFunction = (orig) => {
-  return (...args) => {
+  return assignFunctionName(orig.name, function(...args) {
     const { promise, resolve, reject } = PromiseWithResolvers();
 
     promise.child = orig(...args, (err, stdout, stderr) => {
@@ -250,7 +251,7 @@ const customPromiseExecFunction = (orig) => {
     });
 
     return promise;
-  };
+  });
 };
 
 ObjectDefineProperty(exec, promisify.custom, {
