@@ -86,7 +86,6 @@ const {
 const { toPathIfFileURL } = require('internal/url');
 const {
   customPromisifyArgs: kCustomPromisifyArgsSymbol,
-  emitExperimentalWarning,
   getLazy,
   kEmptyObject,
   promisify: {
@@ -3186,7 +3185,6 @@ function createWriteStream(path, options) {
 const lazyGlob = getLazy(() => require('internal/fs/glob').Glob);
 
 function glob(pattern, options, callback) {
-  emitExperimentalWarning('glob');
   if (typeof options === 'function') {
     callback = options;
     options = undefined;
@@ -3209,7 +3207,6 @@ function glob(pattern, options, callback) {
 }
 
 function globSync(pattern, options) {
-  emitExperimentalWarning('globSync');
   const Glob = lazyGlob();
   return new Glob(pattern, options).globSync();
 }
