@@ -26,6 +26,7 @@ const { myersDiff, printMyersDiff, printSimpleMyersDiff } = require('internal/as
 
 const kReadableOperator = {
   deepStrictEqual: 'Expected values to be strictly deep-equal:',
+  partialDeepStrictEqual: 'Expected values to be partially and strictly deep-equal:',
   strictEqual: 'Expected values to be strictly equal:',
   strictEqualObject: 'Expected "actual" to be reference-equal to "expected":',
   deepEqual: 'Expected values to be loosely deep-equal:',
@@ -41,6 +42,8 @@ const kReadableOperator = {
 const kMaxShortStringLength = 12;
 const kMaxLongStringLength = 512;
 
+const kMethodsWithCustomMessageDiff = ['deepStrictEqual', 'strictEqual', 'partialDeepStrictEqual'];
+
 function copyError(source) {
   const target = ObjectAssign(
     { __proto__: ObjectGetPrototypeOf(source) },
@@ -210,9 +213,13 @@ function createErrDiff(actual, expected, operator, customMessage) {
     const checkCommaDisparity = actual != null && typeof actual === 'object';
     const diff = myersDiff(inspectedSplitActual, inspectedSplitExpected, checkCommaDisparity);
 
-    const myersDiffMessage = printMyersDiff(diff);
+    const myersDiffMessage = printMyersDiff(diff, operator);
     message = myersDiffMessage.message;
 
+    if (operator === 'partialDeepStrictEqual') {
+      header = `${colors.gray}${colors.hasColors ? '' : '+ '}actual${colors.white} ${colors.red}- expected${colors.white}`;
+    }
+
     if (myersDiffMessage.skipped) {
       skipped = true;
     }
@@ -255,7 +262,7 @@ class AssertionError extends Error {
     if (isErrorStackTraceLimitWritable()) Error.stackTraceLimit = 0;
 
     if (message != null) {
-      if (operator === 'deepStrictEqual' || operator === 'strictEqual') {
+      if (kMethodsWithCustomMessageDiff.includes(operator)) {
         super(createErrDiff(actual, expected, operator, message));
       } else {
         super(String(message));
@@ -275,7 +282,7 @@ class AssertionError extends Error {
         expected = copyError(expected);
       }
 
-      if (operator === 'deepStrictEqual' || operator === 'strictEqual') {
+      if (kMethodsWithCustomMessageDiff.includes(operator)) {
         super(createErrDiff(actual, expected, operator, message));
       } else if (operator === 'notDeepStrictEqual' ||
         operator === 'notStrictEqual') {
