@@ -107,11 +107,10 @@ process.domain = null;
     configurable: true,
   });
 
-  let exitCode;
   ObjectDefineProperty(process, 'exitCode', {
     __proto__: null,
     get() {
-      return exitCode;
+      return fields[kHasExitCode] ? fields[kExitCode] : undefined;
     },
     set(code) {
       if (code !== null && code !== undefined) {
@@ -126,7 +125,6 @@ process.domain = null;
       } else {
         fields[kHasExitCode] = 0;
       }
-      exitCode = code;
     },
     enumerable: true,
     configurable: false,
@@ -181,6 +179,7 @@ const rawMethods = internalBinding('process_methods');
   process.availableMemory = rawMethods.availableMemory;
   process.kill = wrapped.kill;
   process.exit = wrapped.exit;
+  process.execve = wrapped.execve;
   process.ref = perThreadSetup.ref;
   process.unref = perThreadSetup.unref;
 
@@ -275,7 +274,7 @@ process.assert = deprecate(
 
 // TODO(joyeecheung): this property has not been well-maintained, should we
 // deprecate it in favor of a better API?
-const { isDebugBuild, hasOpenSSL, hasInspector } = config;
+const { isDebugBuild, hasOpenSSL, openSSLIsBoringSSL, hasInspector } = config;
 const features = {
   inspector: hasInspector,
   debug: isDebugBuild,
@@ -285,6 +284,7 @@ const features = {
   tls_sni: hasOpenSSL,
   tls_ocsp: hasOpenSSL,
   tls: hasOpenSSL,
+  openssl_is_boringssl: openSSLIsBoringSSL,
   // This needs to be dynamic because --no-node-snapshot disables the
   // code cache even if the binary is built with embedded code cache.
   get cached_builtins() {
