@@ -1,14 +1,18 @@
 'use strict';
-const { Map } = primordials;
+
+const {
+  SafeMap,
+} = primordials;
+
 const assert = require('internal/assert');
 const dgram = require('internal/dgram');
 const net = require('net');
 
 module.exports = SharedHandle;
 
-function SharedHandle(key, address, port, addressType, fd, flags) {
+function SharedHandle(key, address, { port, addressType, fd, flags }) {
   this.key = key;
-  this.workers = new Map();
+  this.workers = new SafeMap();
   this.handle = null;
   this.errno = 0;
 
