@@ -85,16 +85,21 @@ function verifyAcceptableRsaKeyUse(name, isPublic, usages) {
   }
 }
 
-function rsaOaepCipher(mode, key, data, { label }) {
+function validateRsaOaepAlgorithm(algorithm) {
+  if (algorithm.label !== undefined) {
+    validateMaxBufferLength(algorithm.label, 'algorithm.label');
+  }
+}
+
+function rsaOaepCipher(mode, key, data, algorithm) {
+  validateRsaOaepAlgorithm(algorithm);
+
   const type = mode === kWebCryptoCipherEncrypt ? 'public' : 'private';
   if (key.type !== type) {
     throw lazyDOMException(
       'The requested operation is not valid for the provided key',
       'InvalidAccessError');
   }
-  if (label !== undefined) {
-    validateMaxBufferLength(label, 'algorithm.label');
-  }
 
   return jobPromise(() => new RSACipherJob(
     kCryptoJobAsync,
@@ -103,14 +108,20 @@ function rsaOaepCipher(mode, key, data, { label }) {
     data,
     kKeyVariantRSA_OAEP,
     normalizeHashName(key.algorithm.hash.name),
-    label));
+    algorithm.label));
 }
 
 async function rsaKeyGenerate(
   algorithm,
   extractable,
-  keyUsages) {
-
+  keyUsages,
+) {
+  const publicExponentConverted = bigIntArrayToUnsignedInt(algorithm.publicExponent);
+  if (publicExponentConverted === undefined) {
+    throw lazyDOMException(
+      'The publicExponent must be equivalent to an unsigned 32-bit value',
+      'OperationError');
+  }
   const {
     name,
     modulusLength,
@@ -120,13 +131,6 @@ async function rsaKeyGenerate(
 
   const usageSet = new SafeSet(keyUsages);
 
-  const publicExponentConverted = bigIntArrayToUnsignedInt(publicExponent);
-  if (publicExponentConverted === undefined) {
-    throw lazyDOMException(
-      'The publicExponent must be equivalent to an unsigned 32-bit value',
-      'OperationError');
-  }
-
   switch (name) {
     case 'RSA-OAEP':
       if (hasAnyNotIn(usageSet,
@@ -200,7 +204,7 @@ function rsaExportKey(key, format) {
     kRsaVariants[key.algorithm.name]));
 }
 
-async function rsaImportKey(
+function rsaImportKey(
   format,
   keyData,
   algorithm,
@@ -209,6 +213,11 @@ async function rsaImportKey(
   const usagesSet = new SafeSet(keyUsages);
   let keyObject;
   switch (format) {
+    case 'KeyObject': {
+      verifyAcceptableRsaKeyUse(algorithm.name, keyData.type === 'public', usagesSet);
+      keyObject = keyData;
+      break;
+    }
     case 'spki': {
       verifyAcceptableRsaKeyUse(algorithm.name, true, usagesSet);
       try {
@@ -266,9 +275,13 @@ async function rsaImportKey(
       }
 
       if (keyData.alg !== undefined) {
-        const hash =
-          normalizeHashName(keyData.alg, normalizeHashName.kContextWebCrypto);
-        if (hash !== algorithm.hash.name)
+        const expected =
+          normalizeHashName(algorithm.hash.name,
+                            algorithm.name === 'RSASSA-PKCS1-v1_5' ? normalizeHashName.kContextJwkRsa :
+                              algorithm.name === 'RSA-PSS' ? normalizeHashName.kContextJwkRsaPss :
+                                normalizeHashName.kContextJwkRsaOaep);
+
+        if (keyData.alg !== expected)
           throw lazyDOMException(
             'JWK "alg" does not match the requested algorithm',
             'DataError');
@@ -314,7 +327,7 @@ async function rsaImportKey(
   }, keyUsages, extractable);
 }
 
-async function rsaSignVerify(key, data, { saltLength }, signature) {
+function rsaSignVerify(key, data, { saltLength }, signature) {
   const mode = signature === undefined ? kSignJobModeSign : kSignJobModeVerify;
   const type = mode === kSignJobModeSign ? 'private' : 'public';
 
