@@ -1342,7 +1342,12 @@ E('ERR_IMPORT_ATTRIBUTE_MISSING',
 E('ERR_IMPORT_ATTRIBUTE_TYPE_INCOMPATIBLE',
   'Module "%s" is not of type "%s"', TypeError);
 E('ERR_IMPORT_ATTRIBUTE_UNSUPPORTED',
-  'Import attribute "%s" with value "%s" is not supported', TypeError);
+  function error(attribute, value, url = undefined) {
+    if (url === undefined) {
+      return `Import attribute "${attribute}" with value "${value}" is not supported`;
+    }
+    return `Import attribute "${attribute}" with value "${value}" is not supported in ${url}`;
+  }, TypeError);
 E('ERR_INCOMPATIBLE_OPTION_PAIR',
   'Option "%s" cannot be used in combination with option "%s"', TypeError, HideStackFramesError);
 E('ERR_INPUT_TYPE_NOT_ALLOWED', '--input-type can only be used with string ' +
@@ -1806,6 +1811,8 @@ E('ERR_TRACE_EVENTS_CATEGORY_REQUIRED',
   'At least one category is required', TypeError);
 E('ERR_TRACE_EVENTS_UNAVAILABLE', 'Trace events are unavailable', Error);
 
+E('ERR_TRAILING_JUNK_AFTER_STREAM_END', 'Trailing junk found after the end of the compressed stream', TypeError);
+
 // This should probably be a `RangeError`.
 E('ERR_TTY_INIT_FAILED', 'TTY initialization failed', SystemError);
 E('ERR_UNAVAILABLE_DURING_EXIT', 'Cannot call function in process exit ' +
