@@ -597,19 +597,40 @@ class EventTarget {
     if (arguments.length < 2)
       throw new ERR_MISSING_ARGS('type', 'listener');
 
-    // We validateOptions before the validateListener check because the spec
-    // requires us to hit getters.
-    const {
-      once,
-      capture,
-      passive,
-      signal,
-      isNodeStyleListener,
-      weak,
-      resistStopPropagation,
-    } = validateEventListenerOptions(options);
+    let once = false;
+    let capture = false;
+    let passive = false;
+    let isNodeStyleListener = false;
+    let weak = false;
+    let resistStopPropagation = false;
 
-    validateAbortSignal(signal, 'options.signal');
+    if (options !== kEmptyObject) {
+      // We validateOptions before the validateListener check because the spec
+      // requires us to hit getters.
+      options = validateEventListenerOptions(options);
+
+      once = options.once;
+      capture = options.capture;
+      passive = options.passive;
+      isNodeStyleListener = options.isNodeStyleListener;
+      weak = options.weak;
+      resistStopPropagation = options.resistStopPropagation;
+
+      const signal = options.signal;
+
+      validateAbortSignal(signal, 'options.signal');
+
+      if (signal) {
+        if (signal.aborted) {
+          return;
+        }
+        // TODO(benjamingr) make this weak somehow? ideally the signal would
+        // not prevent the event target from GC.
+        signal.addEventListener('abort', () => {
+          this.removeEventListener(type, listener, options);
+        }, { __proto__: null, once: true, [kWeakHandler]: this, [kResistStopPropagation]: true });
+      }
+    }
 
     if (!validateEventListener(listener)) {
       // The DOM silently allows passing undefined as a second argument
@@ -623,18 +644,8 @@ class EventTarget {
       process.emitWarning(w);
       return;
     }
-    type = webidl.converters.DOMString(type);
 
-    if (signal) {
-      if (signal.aborted) {
-        return;
-      }
-      // TODO(benjamingr) make this weak somehow? ideally the signal would
-      // not prevent the event target from GC.
-      signal.addEventListener('abort', () => {
-        this.removeEventListener(type, listener, options);
-      }, { __proto__: null, once: true, [kWeakHandler]: this, [kResistStopPropagation]: true });
-    }
+    type = webidl.converters.DOMString(type);
 
     let root = this[kEvents].get(type);
 
