@@ -4,7 +4,7 @@ const {
   ArrayPrototypePush,
   ArrayPrototypeShift,
   FunctionPrototypeBind,
-  ObjectDefineProperty,
+  ObjectDefineProperties,
   PromiseReject,
   SymbolAsyncIterator,
 } = primordials;
@@ -21,7 +21,12 @@ const {
 } = require('internal/errors');
 
 const { FSReqCallback } = binding;
-const internalUtil = require('internal/util');
+const {
+  assignFunctionName,
+  promisify,
+  SymbolAsyncDispose,
+  SymbolDispose,
+} = require('internal/util');
 const {
   getDirent,
   getOptions,
@@ -57,9 +62,9 @@ class Dir {
     validateUint32(this.#options.bufferSize, 'options.bufferSize', true);
 
     this.#readPromisified = FunctionPrototypeBind(
-      internalUtil.promisify(this.#readImpl), this, false);
+      promisify(this.#readImpl), this, false);
     this.#closePromisified = FunctionPrototypeBind(
-      internalUtil.promisify(this.close), this);
+      promisify(this.close), this);
   }
 
   get path() {
@@ -94,7 +99,7 @@ class Dir {
   }
 
   read(callback) {
-    return this.#readImpl(true, callback);
+    return arguments.length === 0 ? this.#readPromisified() : this.#readImpl(true, callback);
   }
 
   #readImpl(maybeSync, callback) {
@@ -293,12 +298,31 @@ class Dir {
   }
 }
 
-ObjectDefineProperty(Dir.prototype, SymbolAsyncIterator, {
-  __proto__: null,
-  value: Dir.prototype.entries,
+const nonEnumerableDescriptor = {
   enumerable: false,
   writable: true,
   configurable: true,
+};
+ObjectDefineProperties(Dir.prototype, {
+  [SymbolDispose]: {
+    __proto__: null,
+    ...nonEnumerableDescriptor,
+    value: assignFunctionName(SymbolDispose, function() {
+      this.closeSync();
+    }),
+  },
+  [SymbolAsyncDispose]: {
+    __proto__: null,
+    ...nonEnumerableDescriptor,
+    value: assignFunctionName(SymbolAsyncDispose, function() {
+      this.close();
+    }),
+  },
+  [SymbolAsyncIterator]: {
+    __proto__: null,
+    ...nonEnumerableDescriptor,
+    value: Dir.prototype.entries,
+  },
 });
 
 function opendir(path, options, callback) {
