@@ -6,6 +6,7 @@ const {
   MathMax,
   Number,
   NumberIsNaN,
+  ObjectAssign,
   ObjectKeys,
   SafeSet,
   String,
@@ -13,9 +14,16 @@ const {
   Symbol,
 } = primordials;
 
+const {
+  _checkIsHttpToken: checkIsHttpToken,
+} = require('_http_common');
+
 const binding = internalBinding('http2');
 const {
   codes: {
+    ERR_HTTP2_CONNECT_AUTHORITY,
+    ERR_HTTP2_CONNECT_PATH,
+    ERR_HTTP2_CONNECT_SCHEME,
     ERR_HTTP2_HEADER_SINGLE_VALUE,
     ERR_HTTP2_INVALID_CONNECTION_HEADERS,
     ERR_HTTP2_INVALID_PSEUDOHEADER: { HideStackFramesError: ERR_HTTP2_INVALID_PSEUDOHEADER },
@@ -29,8 +37,10 @@ const {
   kIsNodeError,
 } = require('internal/errors');
 
+const kAuthority = Symbol('authority');
 const kSensitiveHeaders = Symbol('sensitiveHeaders');
 const kSocket = Symbol('socket');
+const kProtocol = Symbol('protocol');
 const kProxySocket = Symbol('proxySocket');
 const kRequest = Symbol('request');
 
@@ -91,6 +101,7 @@ const {
   HTTP2_HEADER_KEEP_ALIVE,
   HTTP2_HEADER_PROXY_CONNECTION,
 
+  HTTP2_METHOD_CONNECT,
   HTTP2_METHOD_DELETE,
   HTTP2_METHOD_GET,
   HTTP2_METHOD_HEAD,
@@ -218,7 +229,8 @@ const IDX_OPTIONS_MAX_SESSION_MEMORY = 8;
 const IDX_OPTIONS_MAX_SETTINGS = 9;
 const IDX_OPTIONS_STREAM_RESET_RATE = 10;
 const IDX_OPTIONS_STREAM_RESET_BURST = 11;
-const IDX_OPTIONS_FLAGS = 12;
+const IDX_OPTIONS_STRICT_HTTP_FIELD_WHITESPACE_VALIDATION = 12;
+const IDX_OPTIONS_FLAGS = 13;
 
 function updateOptionsBuffer(options) {
   let flags = 0;
@@ -282,6 +294,13 @@ function updateOptionsBuffer(options) {
     optionsBuffer[IDX_OPTIONS_STREAM_RESET_BURST] =
       MathMax(1, options.streamResetBurst);
   }
+
+  if (typeof options.strictFieldWhitespaceValidation === 'boolean') {
+    flags |= (1 << IDX_OPTIONS_STRICT_HTTP_FIELD_WHITESPACE_VALIDATION);
+    optionsBuffer[IDX_OPTIONS_STRICT_HTTP_FIELD_WHITESPACE_VALIDATION] =
+      options.strictFieldWhitespaceValidation === true ? 0 : 1;
+  }
+
   optionsBuffer[IDX_OPTIONS_FLAGS] = flags;
 }
 
@@ -593,35 +612,155 @@ const assertValidPseudoHeaderTrailer = hideStackFrames((key) => {
   throw new ERR_HTTP2_INVALID_PSEUDOHEADER(key);
 });
 
+/**
+ * Takes a request headers array, validates it and sets defaults, and returns
+ * the resulting headers in NgHeaders string list format.
+ */
+function prepareRequestHeadersArray(headers, session) {
+  let method;
+  let scheme;
+  let authority;
+  let path;
+  let protocol;
+
+  // Extract the key psuedo header values from the headers array
+  for (let i = 0; i < headers.length; i += 2) {
+    if (headers[i][0] !== ':') {
+      continue;
+    }
+
+    const header = headers[i].toLowerCase();
+    const value = headers[i + 1];
+
+    if (header === HTTP2_HEADER_METHOD) {
+      method = value;
+    } else if (header === HTTP2_HEADER_SCHEME) {
+      scheme = value;
+    } else if (header === HTTP2_HEADER_AUTHORITY) {
+      authority = value;
+    } else if (header === HTTP2_HEADER_PATH) {
+      path = value;
+    } else if (header === HTTP2_HEADER_PROTOCOL) {
+      protocol = value;
+    }
+  }
+
+  // We then build an array of any missing pseudo headers, to prepend
+  // default values to the given header array:
+  const additionalPsuedoHeaders = [];
+
+  if (method === undefined) {
+    method = HTTP2_METHOD_GET;
+    additionalPsuedoHeaders.push(HTTP2_HEADER_METHOD, method);
+  }
+
+  const connect = method === HTTP2_METHOD_CONNECT;
+
+  if (!connect || protocol !== undefined) {
+    if (authority === undefined && headers[HTTP2_HEADER_HOST] === undefined) {
+      authority = session[kAuthority];
+      additionalPsuedoHeaders.push(HTTP2_HEADER_AUTHORITY, authority);
+    }
+    if (scheme === undefined) {
+      scheme = session[kProtocol].slice(0, -1);
+      additionalPsuedoHeaders.push(HTTP2_HEADER_SCHEME, scheme);
+    }
+    if (path === undefined) {
+      additionalPsuedoHeaders.push(HTTP2_HEADER_PATH, '/');
+    }
+  } else {
+    if (authority === undefined)
+      throw new ERR_HTTP2_CONNECT_AUTHORITY();
+    if (scheme !== undefined)
+      throw new ERR_HTTP2_CONNECT_SCHEME();
+    if (path !== undefined)
+      throw new ERR_HTTP2_CONNECT_PATH();
+  }
+
+  const headersList = buildNgHeaderString(
+    additionalPsuedoHeaders.length ?
+      additionalPsuedoHeaders.concat(headers) :
+      headers,
+    assertValidPseudoHeader,
+    headers[kSensitiveHeaders],
+  );
+
+  return {
+    headersList,
+    scheme,
+    authority: authority ?? headers[HTTP2_HEADER_HOST],
+    method,
+  };
+}
+
+/**
+ * Takes a request headers object, validates it and sets defaults, and returns
+ * the resulting headers in object format and NgHeaders string list format.
+ */
+function prepareRequestHeadersObject(headers, session) {
+  const headersObject = ObjectAssign({ __proto__: null }, headers);
+
+  if (headersObject[HTTP2_HEADER_METHOD] === undefined) {
+    headersObject[HTTP2_HEADER_METHOD] = HTTP2_METHOD_GET;
+  }
+
+  const connect = headersObject[HTTP2_HEADER_METHOD] === HTTP2_METHOD_CONNECT;
+
+  if (!connect || headersObject[HTTP2_HEADER_PROTOCOL] !== undefined) {
+    if (getAuthority(headersObject) === undefined)
+      headersObject[HTTP2_HEADER_AUTHORITY] = session[kAuthority];
+    if (headersObject[HTTP2_HEADER_SCHEME] === undefined)
+      headersObject[HTTP2_HEADER_SCHEME] = session[kProtocol].slice(0, -1);
+    if (headersObject[HTTP2_HEADER_PATH] === undefined)
+      headersObject[HTTP2_HEADER_PATH] = '/';
+  } else {
+    if (headersObject[HTTP2_HEADER_AUTHORITY] === undefined)
+      throw new ERR_HTTP2_CONNECT_AUTHORITY();
+    if (headersObject[HTTP2_HEADER_SCHEME] !== undefined)
+      throw new ERR_HTTP2_CONNECT_SCHEME();
+    if (headersObject[HTTP2_HEADER_PATH] !== undefined)
+      throw new ERR_HTTP2_CONNECT_PATH();
+  }
+
+  const headersList = buildNgHeaderString(headersObject);
+
+  return {
+    headersObject,
+    headersList,
+    scheme: headersObject[HTTP2_HEADER_SCHEME],
+    authority: getAuthority(headersObject),
+    method: headersObject[HTTP2_HEADER_METHOD],
+  };
+}
+
 const emptyArray = [];
 const kNeverIndexFlag = StringFromCharCode(NGHTTP2_NV_FLAG_NO_INDEX);
 const kNoHeaderFlags = StringFromCharCode(NGHTTP2_NV_FLAG_NONE);
-function mapToHeaders(map,
-                      assertValuePseudoHeader = assertValidPseudoHeader) {
+
+/**
+ * Builds an NgHeader string + header count value, validating the header key
+ * format, rejecting illegal header configurations, and marking sensitive headers
+ * that should not be indexed en route. This takes either a flat map of
+ * raw headers ([k1, v1, k2, v2]) or a header object ({ k1: v1, k2: [v2, v3] }).
+ */
+function buildNgHeaderString(arrayOrMap,
+                             assertValuePseudoHeader = assertValidPseudoHeader,
+                             sensitiveHeaders = arrayOrMap[kSensitiveHeaders]) {
   let headers = '';
   let pseudoHeaders = '';
   let count = 0;
-  const keys = ObjectKeys(map);
+
   const singles = new SafeSet();
-  let i, j;
-  let isArray;
-  let key;
-  let value;
-  let isSingleValueHeader;
-  let err;
-  const neverIndex = (map[kSensitiveHeaders] || emptyArray).map((v) => v.toLowerCase());
-  for (i = 0; i < keys.length; ++i) {
-    key = keys[i];
-    value = map[key];
-    if (value === undefined || key === '')
-      continue;
+  const neverIndex = (sensitiveHeaders || emptyArray).map((v) => v.toLowerCase());
+
+  function processHeader(key, value) {
     key = key.toLowerCase();
-    isSingleValueHeader = kSingleValueHeaders.has(key);
-    isArray = ArrayIsArray(value);
+    const isSingleValueHeader = kSingleValueHeaders.has(key);
+    let isArray = ArrayIsArray(value);
     if (isArray) {
       switch (value.length) {
         case 0:
-          continue;
+          return;
         case 1:
           value = String(value[0]);
           isArray = false;
@@ -642,31 +781,50 @@ function mapToHeaders(map,
       kNeverIndexFlag :
       kNoHeaderFlags;
     if (key[0] === ':') {
-      err = assertValuePseudoHeader(key);
+      const err = assertValuePseudoHeader(key);
       if (err !== undefined)
         throw err;
       pseudoHeaders += `${key}\0${value}\0${flags}`;
       count++;
-      continue;
+      return;
     }
-    if (key.includes(' ')) {
+    if (!checkIsHttpToken(key)) {
       throw new ERR_INVALID_HTTP_TOKEN('Header name', key);
     }
     if (isIllegalConnectionSpecificHeader(key, value)) {
       throw new ERR_HTTP2_INVALID_CONNECTION_HEADERS(key);
     }
     if (isArray) {
-      for (j = 0; j < value.length; ++j) {
+      for (let j = 0; j < value.length; ++j) {
         const val = String(value[j]);
         headers += `${key}\0${val}\0${flags}`;
       }
       count += value.length;
-      continue;
+      return;
     }
     headers += `${key}\0${value}\0${flags}`;
     count++;
   }
 
+  if (ArrayIsArray(arrayOrMap)) {
+    for (let i = 0; i < arrayOrMap.length; i += 2) {
+      const key = arrayOrMap[i];
+      const value = arrayOrMap[i + 1];
+      if (value === undefined || key === '')
+        continue;
+      processHeader(key, value);
+    }
+  } else {
+    const keys = ObjectKeys(arrayOrMap);
+    for (let i = 0; i < keys.length; ++i) {
+      const key = keys[i];
+      const value = arrayOrMap[key];
+      if (value === undefined || key === '')
+        continue;
+      processHeader(key, value);
+    }
+  }
+
   return [pseudoHeaders + headers, count];
 }
 
@@ -793,19 +951,23 @@ module.exports = {
   assertValidPseudoHeaderResponse,
   assertValidPseudoHeaderTrailer,
   assertWithinRange,
+  buildNgHeaderString,
   getAuthority,
   getDefaultSettings,
   getSessionState,
   getSettings,
   getStreamState,
   isPayloadMeaningless,
+  kAuthority,
   kSensitiveHeaders,
   kSocket,
+  kProtocol,
   kProxySocket,
   kRequest,
-  mapToHeaders,
   MAX_ADDITIONAL_SETTINGS,
   NghttpError,
+  prepareRequestHeadersArray,
+  prepareRequestHeadersObject,
   remoteCustomSettingsToBuffer,
   sessionName,
   toHeaderObject,
