@@ -49,7 +49,10 @@ const { setupMainThreadPort } = require('internal/worker/messaging');
 const {
   onGlobalUncaughtException,
   evalScript,
+  evalTypeScript,
   evalModuleEntryPoint,
+  parseAndEvalCommonjsTypeScript,
+  parseAndEvalModuleTypeScript,
 } = require('internal/process/execution');
 
 let debug = require('internal/util/debuglog').debuglog('worker', (fn) => {
@@ -166,7 +169,29 @@ port.on('message', (message) => {
           value: filename,
         });
         ArrayPrototypeSplice(process.argv, 1, 0, name);
-        evalScript(name, filename);
+        const tsEnabled = getOptionValue('--experimental-strip-types');
+        const inputType = getOptionValue('--input-type');
+
+        if (inputType === 'module-typescript' && tsEnabled) {
+          // This is a special case where we want to parse and eval the
+          // TypeScript code as a module
+          parseAndEvalModuleTypeScript(filename, false);
+          break;
+        }
+
+        let evalFunction;
+        if (inputType === 'commonjs') {
+          evalFunction = evalScript;
+        } else if (inputType === 'commonjs-typescript' && tsEnabled) {
+          evalFunction = parseAndEvalCommonjsTypeScript;
+        } else if (tsEnabled) {
+          evalFunction = evalTypeScript;
+        } else {
+          // Default to commonjs.
+          evalFunction = evalScript;
+        }
+
+        evalFunction(name, filename);
         break;
       }
 
