@@ -6,9 +6,8 @@ const {
 
 const { getOptionValue } = require('internal/options');
 const { getValidatedPath } = require('internal/fs/utils');
-const pathModule = require('path');
 const fsBindings = internalBinding('fs');
-const { fs: fsConstants } = internalBinding('constants');
+const { internal: internalConstants } = internalBinding('constants');
 
 const experimentalWasmModules = getOptionValue('--experimental-wasm-modules');
 
@@ -24,6 +23,12 @@ if (experimentalWasmModules) {
   extensionFormatMap['.wasm'] = 'wasm';
 }
 
+if (getOptionValue('--experimental-strip-types')) {
+  extensionFormatMap['.ts'] = 'module-typescript';
+  extensionFormatMap['.mts'] = 'module-typescript';
+  extensionFormatMap['.cts'] = 'commonjs-typescript';
+}
+
 /**
  * @param {string} mime
  * @returns {string | null}
@@ -48,11 +53,9 @@ function mimeToFormat(mime) {
  */
 function getFormatOfExtensionlessFile(url) {
   if (!experimentalWasmModules) { return 'module'; }
-
-  const path = pathModule.toNamespacedPath(getValidatedPath(url));
-
+  const path = getValidatedPath(url);
   switch (fsBindings.getFormatOfExtensionlessFile(path)) {
-    case fsConstants.EXTENSIONLESS_FORMAT_WASM:
+    case internalConstants.EXTENSIONLESS_FORMAT_WASM:
       return 'wasm';
     default:
       return 'module';
