@@ -4,7 +4,6 @@ const {
   RegExpPrototypeExec,
 } = primordials;
 const {
-  isUnderNodeModules,
   kEmptyObject,
 } = require('internal/util');
 
@@ -23,7 +22,6 @@ const {
   ERR_INVALID_URL,
   ERR_UNKNOWN_MODULE_FORMAT,
   ERR_UNSUPPORTED_ESM_URL_SCHEME,
-  ERR_UNSUPPORTED_NODE_MODULES_TYPE_STRIPPING,
 } = require('internal/errors').codes;
 
 const {
@@ -131,12 +129,6 @@ async function defaultLoad(url, context = kEmptyObject) {
 
   validateAttributes(url, format, importAttributes);
 
-  if (getOptionValue('--experimental-strip-types') &&
-    (format === 'module-typescript' || format === 'commonjs-typescript') &&
-    isUnderNodeModules(url)) {
-    throw new ERR_UNSUPPORTED_NODE_MODULES_TYPE_STRIPPING(url);
-  }
-
   return {
     __proto__: null,
     format,
