@@ -8,7 +8,7 @@ const {
   ObjectKeys,
   SafeMap,
 } = primordials;
-const { kImplicitAssertType } = require('internal/modules/esm/assert');
+const { kImplicitTypeAttribute } = require('internal/modules/esm/assert');
 let debug = require('internal/util/debuglog').debuglog('esm', (fn) => {
   debug = fn;
 });
@@ -88,12 +88,12 @@ class ResolveCache extends SafeMap {
  */
 class LoadCache extends SafeMap {
   constructor(i) { super(i); } // eslint-disable-line no-useless-constructor
-  get(url, type = kImplicitAssertType) {
+  get(url, type = kImplicitTypeAttribute) {
     validateString(url, 'url');
     validateString(type, 'type');
     return super.get(url)?.[type];
   }
-  set(url, type = kImplicitAssertType, job) {
+  set(url, type = kImplicitTypeAttribute, job) {
     validateString(url, 'url');
     validateString(type, 'type');
 
@@ -103,17 +103,23 @@ class LoadCache extends SafeMap {
       throw new ERR_INVALID_ARG_TYPE('job', 'ModuleJob', job);
     }
     debug(`Storing ${url} (${
-      type === kImplicitAssertType ? 'implicit type' : type
+      type === kImplicitTypeAttribute ? 'implicit type' : type
     }) in ModuleLoadMap`);
     const cachedJobsForUrl = super.get(url) ?? { __proto__: null };
     cachedJobsForUrl[type] = job;
     return super.set(url, cachedJobsForUrl);
   }
-  has(url, type = kImplicitAssertType) {
+  has(url, type = kImplicitTypeAttribute) {
     validateString(url, 'url');
     validateString(type, 'type');
     return super.get(url)?.[type] !== undefined;
   }
+  delete(url, type = kImplicitTypeAttribute) {
+    const cached = super.get(url);
+    if (cached) {
+      cached[type] = undefined;
+    }
+  }
 }
 
 module.exports = {
