@@ -120,10 +120,10 @@ translators.set('module', function moduleStrategy(url, source, isMain) {
 function loadCJSModule(module, source, url, filename, isMain) {
   const compileResult = compileFunctionForCJSLoader(source, filename, false /* is_sea_main */, false);
 
-  const { function: compiledWrapper, sourceMapURL } = compileResult;
+  const { function: compiledWrapper, sourceMapURL, sourceURL } = compileResult;
   // Cache the source map for the cjs module if present.
   if (sourceMapURL) {
-    maybeCacheSourceMap(url, source, module, false, undefined, sourceMapURL);
+    maybeCacheSourceMap(url, source, module, false, sourceURL, sourceMapURL);
   }
 
   const cascadedLoader = require('internal/modules/esm/loader').getOrInitializeCascadedLoader();
