@@ -348,7 +348,7 @@ function compileSourceTextModule(url, source, cascadedLoader) {
   }
   // Cache the source map for the module if present.
   if (wrap.sourceMapURL) {
-    maybeCacheSourceMap(url, source, wrap, false, undefined, wrap.sourceMapURL);
+    maybeCacheSourceMap(url, source, wrap, false, wrap.sourceURL, wrap.sourceMapURL);
   }
   return wrap;
 }
