@@ -124,17 +124,6 @@ function read(jsonPath, { base, specifier, isESM } = kEmptyObject) {
   };
 }
 
-/**
- * @deprecated Expected to be removed in favor of `read` in the future.
- * Behaves the same was as `read`, but appends package.json to the path.
- * @param {string} requestPath
- * @return {PackageConfig}
- */
-function readPackage(requestPath) {
-  // TODO(@anonrig): Remove this function.
-  return read(path.resolve(requestPath, 'package.json'));
-}
-
 /**
  * Get the nearest parent package.json file from a given path.
  * Return the package.json data and the path to the package.json file, or undefined.
@@ -185,8 +174,8 @@ function getPackageScopeConfig(resolved) {
  * @param {URL} url - The URL to get the package type for.
  */
 function getPackageType(url) {
-  // TODO(@anonrig): Write a C++ function that returns only "type".
-  return getPackageScopeConfig(url).type;
+  const type = modulesBinding.getPackageType(`${url}`);
+  return type ?? 'none';
 }
 
 const invalidPackageNameRegEx = /^\.|%|\\/;
@@ -230,8 +219,7 @@ function parsePackageName(specifier, base) {
 }
 
 function getPackageJSONURL(specifier, base) {
-  const { packageName, packageSubpath, isScoped } =
-    parsePackageName(specifier, base);
+  const { packageName, packageSubpath, isScoped } = parsePackageName(specifier, base);
 
   // ResolveSelf
   const packageConfig = getPackageScopeConfig(base);
@@ -242,8 +230,7 @@ function getPackageJSONURL(specifier, base) {
     }
   }
 
-  let packageJSONUrl =
-    new URL('./node_modules/' + packageName + '/package.json', base);
+  let packageJSONUrl = new URL(`./node_modules/${packageName}/package.json`, base);
   let packageJSONPath = fileURLToPath(packageJSONUrl);
   let lastPath;
   do {
@@ -254,9 +241,10 @@ function getPackageJSONURL(specifier, base) {
     // Check for !stat.isDirectory()
     if (stat !== 1) {
       lastPath = packageJSONPath;
-      packageJSONUrl = new URL((isScoped ?
-        '../../../../node_modules/' : '../../../node_modules/') +
-        packageName + '/package.json', packageJSONUrl);
+      packageJSONUrl = new URL(
+        `${isScoped ? '../' : ''}../../../node_modules/${packageName}/package.json`,
+        packageJSONUrl,
+      );
       packageJSONPath = fileURLToPath(packageJSONUrl);
       continue;
     }
@@ -321,7 +309,6 @@ function findPackageJSON(specifier, base = 'data:') {
 
 module.exports = {
   read,
-  readPackage,
   getNearestParentPackageJSON,
   getPackageScopeConfig,
   getPackageType,
