@@ -124,6 +124,8 @@ function prepareExecution(options) {
   initializeSourceMapsHandlers();
   initializeDeprecations();
 
+  initializeConfigFileSupport();
+
   require('internal/dns/utils').initializeDns();
 
   setupSymbolDisposePolyfill();
@@ -390,6 +392,13 @@ function setupSQLite() {
   BuiltinModule.allowRequireByUsers('sqlite');
 }
 
+function initializeConfigFileSupport() {
+  if (getOptionValue('--experimental-default-config-file') ||
+      getOptionValue('--experimental-config-file')) {
+    emitExperimentalWarning('--experimental-config-file');
+  }
+}
+
 function setupWebStorage() {
   if (getEmbedderOptions().noBrowserGlobals ||
       !getOptionValue('--experimental-webstorage')) {
