@@ -38,6 +38,8 @@ const {
 } = require('internal/async_hooks');
 const { isErrorStackTraceLimitWritable } = require('internal/errors');
 
+const AsyncContextFrame = require('internal/async_context_frame');
+
 // *Must* match Environment::TickInfo::Fields in src/env.h.
 const kHasRejectionToWarn = 1;
 
@@ -260,6 +262,7 @@ function unhandledRejection(promise, reason) {
     uid: ++lastPromiseId,
     warned: false,
     domain: process.domain,
+    contextFrame: AsyncContextFrame.current(),
   });
   setHasRejectionToWarn(true);
 }
@@ -340,7 +343,7 @@ function strictUnhandledRejectionsMode(promise, promiseInfo, promiseAsyncId) {
     reason : new UnhandledPromiseRejection(reason);
   // This destroys the async stack, don't clear it after
   triggerUncaughtException(err, true /* fromPromise */);
-  if (promiseAsyncId !== undefined) {
+  if (promiseAsyncId === undefined) {
     pushAsyncContext(
       promise[kAsyncIdSymbol],
       promise[kTriggerAsyncIdSymbol],
@@ -466,9 +469,12 @@ function processPromiseRejections() {
       );
     }
 
+    const { contextFrame } = promiseInfo;
+    const priorContextFrame = AsyncContextFrame.exchange(contextFrame);
     try {
       needPop = unhandledRejectionsMode(promise, promiseInfo, promiseAsyncId);
     } finally {
+      AsyncContextFrame.set(priorContextFrame);
       needPop &&
       promiseAsyncId !== undefined &&
       popAsyncContext(promiseAsyncId);
