@@ -47,7 +47,11 @@ const {
   validateString,
   validateUint32,
 } = require('internal/validators');
-const { SymbolDispose, kEmptyObject } = require('internal/util');
+const {
+  assignFunctionName,
+  kEmptyObject,
+  SymbolDispose,
+} = require('internal/util');
 const {
   inspect,
   getStringWidth,
@@ -1370,7 +1374,9 @@ class Interface extends InterfaceConstructor {
     return this[kLineObjectStream];
   }
 }
-Interface.prototype[SymbolDispose] = Interface.prototype.close;
+Interface.prototype[SymbolDispose] = assignFunctionName(SymbolDispose, function() {
+  this.close();
+});
 
 module.exports = {
   Interface,
