@@ -4,7 +4,7 @@ const {
   ArrayPrototypeJoin,
   Boolean,
   FunctionPrototype,
-  StringPrototypeSplit,
+  RegExpPrototypeSymbolSplit,
   StringPrototypeTrim,
 } = primordials;
 
@@ -15,6 +15,7 @@ const os = require('os');
 let debug = require('internal/util/debuglog').debuglog('repl', (fn) => {
   debug = fn;
 });
+const permission = require('internal/process/permission');
 const { clearTimeout, setTimeout } = require('timers');
 
 const noop = FunctionPrototype;
@@ -53,6 +54,12 @@ function setupHistory(repl, historyPath, ready) {
     }
   }
 
+  if (permission.isEnabled() && permission.has('fs.write', historyPath) === false) {
+    _writeToOutput(repl, '\nAccess to FileSystemWrite is restricted.\n' +
+      'REPL session history will not be persisted.\n');
+    return ready(null, repl);
+  }
+
   let timer = null;
   let writing = false;
   let pending = false;
@@ -90,7 +97,7 @@ function setupHistory(repl, historyPath, ready) {
     }
 
     if (data) {
-      repl.history = StringPrototypeSplit(data, /[\n\r]+/, repl.historySize);
+      repl.history = RegExpPrototypeSymbolSplit(/[\n\r]+/, data, repl.historySize);
     } else {
       repl.history = [];
     }
@@ -167,7 +174,7 @@ function _replHistoryMessage() {
       this,
       '\nPersistent history support disabled. ' +
       'Set the NODE_REPL_HISTORY environment\nvariable to ' +
-      'a valid, user-writable path to enable.\n'
+      'a valid, user-writable path to enable.\n',
     );
   }
   this._historyPrev = Interface.prototype._historyPrev;
