@@ -73,6 +73,7 @@ const {
   ArrayPrototypeSort,
   ObjectPrototypeHasOwnProperty,
   StringPrototypeCharAt,
+  Symbol,
 } = primordials;
 
 const { validateObject } = require('internal/validators');
@@ -83,6 +84,8 @@ const VLQ_BASE_SHIFT = 5;
 const VLQ_BASE_MASK = (1 << 5) - 1;
 const VLQ_CONTINUATION_MASK = 1 << 5;
 
+const kMappings = Symbol('kMappings');
+
 class StringCharIterator {
   /**
    * @constructor
@@ -153,6 +156,10 @@ class SourceMap {
     return cloneSourceMapV3(this.#payload);
   }
 
+  get [kMappings]() {
+    return this.#mappings;
+  }
+
   /**
    * @return {number[] | undefined} line lengths of generated source code
    */
@@ -262,7 +269,7 @@ class SourceMap {
       ArrayPrototypePush(sources, url);
       this.#sources[url] = true;
 
-      if (map.sourcesContent && map.sourcesContent[i])
+      if (map.sourcesContent?.[i])
         this.#sourceContentByURL[url] = map.sourcesContent[i];
     }
 
@@ -382,5 +389,6 @@ function compareSourceMapEntry(entry1, entry2) {
 }
 
 module.exports = {
+  kMappings,
   SourceMap,
 };
