@@ -43,6 +43,9 @@ const {
   willEmitClose: _willEmitClose,
   kIsClosedPromise,
 } = require('internal/streams/utils');
+
+// Lazy load
+let AsyncLocalStorage;
 let addAbortListener;
 
 function isRequest(stream) {
@@ -63,7 +66,8 @@ function eos(stream, options, callback) {
   validateFunction(callback, 'callback');
   validateAbortSignal(options.signal, 'options.signal');
 
-  callback = once(callback);
+  AsyncLocalStorage ??= require('async_hooks').AsyncLocalStorage;
+  callback = once(AsyncLocalStorage.bind(callback));
 
   if (isReadableStream(stream) || isWritableStream(stream)) {
     return eosWeb(stream, options, callback);
