@@ -409,6 +409,10 @@ class TestCoverage {
           }
           const mappedStartOffset = this.entryToOffset(startEntry, mappedLines);
           const mappedEndOffset = this.entryToOffset(endEntry, mappedLines) + 1;
+          if (mappedStartOffset < 0 || mappedEndOffset < 1) {
+            // The range is not mappable. Skip it.
+            continue;
+          }
           for (let l = startEntry.originalLine; l <= endEntry.originalLine; l++) {
             mappedLines[l].count = count;
           }
@@ -433,7 +437,12 @@ class TestCoverage {
 
   entryToOffset(entry, lines) {
     const line = MathMax(entry.originalLine, 0);
-    return MathMin(lines[line].startOffset + entry.originalColumn, lines[line].endOffset);
+    const mappedLine = lines[line];
+    if (!mappedLine) {
+      // Return -1 if the line is not mappable.
+      return -1;
+    }
+    return MathMin(mappedLine.startOffset + entry.originalColumn, mappedLine.endOffset);
   }
 
   mergeCoverage(merged, coverage) {
