@@ -4,6 +4,7 @@ const {
   ArrayPrototypePush,
   FunctionPrototypeBind,
   PromiseResolve,
+  PromiseWithResolvers,
   SafeMap,
   SafePromiseAllReturnVoid,
 } = primordials;
@@ -26,7 +27,6 @@ const {
   shouldColorizeTestFiles,
 } = require('internal/test_runner/utils');
 const { queueMicrotask } = require('internal/process/task_queues');
-const { createDeferredPromise } = require('internal/util');
 const { bigint: hrtime } = process.hrtime;
 const resolvedPromise = PromiseResolve();
 const testResources = new SafeMap();
@@ -35,7 +35,7 @@ let globalRoot;
 testResources.set(reporterScope.asyncId(), reporterScope);
 
 function createTestTree(rootTestOptions, globalOptions) {
-  const buildPhaseDeferred = createDeferredPromise();
+  const buildPhaseDeferred = PromiseWithResolvers();
   const isFilteringByName = globalOptions.testNamePatterns ||
                             globalOptions.testSkipPatterns;
   const isFilteringByOnly = globalOptions.only || (globalOptions.isTestRunner &&
