@@ -118,10 +118,17 @@ async function load(url, context, nextLoad) {
   // Treat builtins as commonjs because customization hooks do not allow a
   // core module to be replaced.
   // Also collapse 'commonjs-sync' and 'require-commonjs' to 'commonjs'.
-  const format = (
-    original.format === 'builtin' ||
-    original.format === 'commonjs-sync' ||
-    original.format === 'require-commonjs') ? 'commonjs' : original.format;
+  let format = original.format;
+  switch (original.format) {
+    case 'builtin': // Deliberate fallthrough
+    case 'commonjs-sync': // Deliberate fallthrough
+    case 'require-commonjs':
+      format = 'commonjs';
+      break;
+    case 'json':
+      format = 'module';
+      break;
+  }
 
   const result = {
     __proto__: null,
