@@ -1217,7 +1217,7 @@ class Test extends AsyncResource {
           if (actual < threshold) {
             harness.success = false;
             process.exitCode = kGenericUserError;
-            reporter.diagnostic(nesting, loc, `Error: ${NumberPrototypeToFixed(actual, 2)}% ${name} coverage does not meet threshold of ${threshold}%.`);
+            reporter.diagnostic(nesting, loc, `Error: ${NumberPrototypeToFixed(actual, 2)}% ${name} coverage does not meet threshold of ${threshold}%.`, 'error');
           }
         }
 
