@@ -16,7 +16,6 @@ const {
   ObjectSetPrototypeOf,
   ReflectGetOwnPropertyDescriptor,
   ReflectOwnKeys,
-  RegExpPrototypeSymbolReplace,
   SafeMap,
   SafeSet,
   StringPrototypeCharAt,
@@ -779,6 +778,8 @@ function isURL(self) {
  * for invalid URL inputs.
  */
 const kParseURLSymbol = Symbol('kParseURL');
+const kCreateURLFromPosixPathSymbol = Symbol('kCreateURLFromPosixPath');
+const kCreateURLFromWindowsPathSymbol = Symbol('kCreateURLFromWindowsPath');
 
 class URL {
   #context = new URLContext();
@@ -812,8 +813,17 @@ class URL {
       base = `${base}`;
     }
 
-    const raiseException = parseSymbol !== kParseURLSymbol;
-    const href = bindingUrl.parse(input, base, raiseException);
+    let href;
+    if (arguments.length < 3) {
+      href = bindingUrl.parse(input, base, true);
+    } else {
+      const raiseException = parseSymbol !== kParseURLSymbol;
+      const interpretAsWindowsPath = parseSymbol === kCreateURLFromWindowsPathSymbol;
+      const pathToFileURL = interpretAsWindowsPath || (parseSymbol === kCreateURLFromPosixPathSymbol);
+      href = pathToFileURL ?
+        bindingUrl.pathToFileURL(input, interpretAsWindowsPath, base) :
+        bindingUrl.parse(input, base, raiseException);
+    }
     if (href) {
       this.#updateContext(href);
     }
@@ -1418,7 +1428,7 @@ function urlToHttpOptions(url) {
     __proto__: null,
     ...url, // In case the url object was extended by the user.
     protocol: url.protocol,
-    hostname: hostname && StringPrototypeStartsWith(hostname, '[') ?
+    hostname: hostname && hostname[0] === '[' ?
       StringPrototypeSlice(hostname, 1, -1) :
       hostname,
     hash: url.hash,
@@ -1500,75 +1510,37 @@ function fileURLToPath(path, options = kEmptyObject) {
   return (windows ?? isWindows) ? getPathFromURLWin32(path) : getPathFromURLPosix(path);
 }
 
-// The following characters are percent-encoded when converting from file path
-// to URL:
-// - %: The percent character is the only character not encoded by the
-//        `pathname` setter.
-// - \: Backslash is encoded on non-windows platforms since it's a valid
-//      character but the `pathname` setters replaces it by a forward slash.
-// - LF: The newline character is stripped out by the `pathname` setter.
-//       (See whatwg/url#419)
-// - CR: The carriage return character is also stripped out by the `pathname`
-//       setter.
-// - TAB: The tab character is also stripped out by the `pathname` setter.
-const percentRegEx = /%/g;
-const backslashRegEx = /\\/g;
-const newlineRegEx = /\n/g;
-const carriageReturnRegEx = /\r/g;
-const tabRegEx = /\t/g;
-const questionRegex = /\?/g;
-const hashRegex = /#/g;
-
-function encodePathChars(filepath, options = kEmptyObject) {
-  const windows = options?.windows;
-  if (StringPrototypeIndexOf(filepath, '%') !== -1)
-    filepath = RegExpPrototypeSymbolReplace(percentRegEx, filepath, '%25');
-  // In posix, backslash is a valid character in paths:
-  if (!(windows ?? isWindows) && StringPrototypeIndexOf(filepath, '\\') !== -1)
-    filepath = RegExpPrototypeSymbolReplace(backslashRegEx, filepath, '%5C');
-  if (StringPrototypeIndexOf(filepath, '\n') !== -1)
-    filepath = RegExpPrototypeSymbolReplace(newlineRegEx, filepath, '%0A');
-  if (StringPrototypeIndexOf(filepath, '\r') !== -1)
-    filepath = RegExpPrototypeSymbolReplace(carriageReturnRegEx, filepath, '%0D');
-  if (StringPrototypeIndexOf(filepath, '\t') !== -1)
-    filepath = RegExpPrototypeSymbolReplace(tabRegEx, filepath, '%09');
-  return filepath;
-}
-
 function pathToFileURL(filepath, options = kEmptyObject) {
-  const windows = options?.windows;
-  if ((windows ?? isWindows) && StringPrototypeStartsWith(filepath, '\\\\')) {
-    const outURL = new URL('file://');
+  const windows = options?.windows ?? isWindows;
+  const isUNC = windows && StringPrototypeStartsWith(filepath, '\\\\');
+  let resolved = isUNC ?
+    filepath :
+    (windows ? path.win32.resolve(filepath) : path.posix.resolve(filepath));
+  if (isUNC || (windows && StringPrototypeStartsWith(resolved, '\\\\'))) {
     // UNC path format: \\server\share\resource
     // Handle extended UNC path and standard UNC path
     // "\\?\UNC\" path prefix should be ignored.
     // Ref: https://learn.microsoft.com/en-us/windows/win32/fileio/maximum-file-path-limitation
-    const isExtendedUNC = StringPrototypeStartsWith(filepath, '\\\\?\\UNC\\');
+    const isExtendedUNC = StringPrototypeStartsWith(resolved, '\\\\?\\UNC\\');
     const prefixLength = isExtendedUNC ? 8 : 2;
-    const hostnameEndIndex = StringPrototypeIndexOf(filepath, '\\', prefixLength);
+    const hostnameEndIndex = StringPrototypeIndexOf(resolved, '\\', prefixLength);
     if (hostnameEndIndex === -1) {
       throw new ERR_INVALID_ARG_VALUE(
         'path',
-        filepath,
+        resolved,
         'Missing UNC resource path',
       );
     }
     if (hostnameEndIndex === 2) {
       throw new ERR_INVALID_ARG_VALUE(
         'path',
-        filepath,
+        resolved,
         'Empty UNC servername',
       );
     }
-    const hostname = StringPrototypeSlice(filepath, prefixLength, hostnameEndIndex);
-    outURL.hostname = domainToASCII(hostname);
-    outURL.pathname = encodePathChars(
-      RegExpPrototypeSymbolReplace(backslashRegEx, StringPrototypeSlice(filepath, hostnameEndIndex), '/'),
-      { windows },
-    );
-    return outURL;
+    const hostname = StringPrototypeSlice(resolved, prefixLength, hostnameEndIndex);
+    return new URL(StringPrototypeSlice(resolved, hostnameEndIndex), hostname, kCreateURLFromWindowsPathSymbol);
   }
-  let resolved = (windows ?? isWindows) ? path.win32.resolve(filepath) : path.posix.resolve(filepath);
   // path.resolve strips trailing slashes so we must add them back
   const filePathLast = StringPrototypeCharCodeAt(filepath,
                                                  filepath.length - 1);
@@ -1577,18 +1549,7 @@ function pathToFileURL(filepath, options = kEmptyObject) {
       resolved[resolved.length - 1] !== path.sep)
     resolved += '/';
 
-  // Call encodePathChars first to avoid encoding % again for ? and #.
-  resolved = encodePathChars(resolved, { windows });
-
-  // Question and hash character should be included in pathname.
-  // Therefore, encoding is required to eliminate parsing them in different states.
-  // This is done as an optimization to not creating a URL instance and
-  // later triggering pathname setter, which impacts performance
-  if (StringPrototypeIndexOf(resolved, '?') !== -1)
-    resolved = RegExpPrototypeSymbolReplace(questionRegex, resolved, '%3F');
-  if (StringPrototypeIndexOf(resolved, '#') !== -1)
-    resolved = RegExpPrototypeSymbolReplace(hashRegex, resolved, '%23');
-  return new URL(`file://${resolved}`);
+  return new URL(resolved, undefined, windows ? kCreateURLFromWindowsPathSymbol : kCreateURLFromPosixPathSymbol);
 }
 
 function toPathIfFileURL(fileURLOrPath) {
