@@ -7,6 +7,7 @@ const {
   ArrayPrototypeIncludes,
   ArrayPrototypeJoin,
   ArrayPrototypeMap,
+  NumberIsFinite,
   NumberIsInteger,
   NumberIsNaN,
   NumberMAX_SAFE_INTEGER,
@@ -129,7 +130,7 @@ const validateInt32 = hideStackFrames(
  * @callback validateUint32
  * @param {*} value
  * @param {string} name
- * @param {number|boolean} [positive=false]
+ * @param {boolean} [positive=false]
  * @returns {asserts value is number}
  */
 
@@ -567,6 +568,46 @@ const validateLinkHeaderValue = hideStackFrames((hints) => {
   );
 });
 
+// 1. Returns false for undefined and NaN
+// 2. Returns true for finite numbers
+// 3. Throws ERR_INVALID_ARG_TYPE for non-numbers
+// 4. Throws ERR_OUT_OF_RANGE for infinite numbers
+const validateFiniteNumber = hideStackFrames((number, name) => {
+  // Common case
+  if (number === undefined) {
+    return false;
+  }
+
+  if (NumberIsFinite(number)) {
+    return true; // Is a valid number
+  }
+
+  if (NumberIsNaN(number)) {
+    return false;
+  }
+
+  validateNumber(number, name);
+
+  // Infinite numbers
+  throw new ERR_OUT_OF_RANGE(name, 'a finite number', number);
+});
+
+// 1. Returns def for number when it's undefined or NaN
+// 2. Returns number for finite numbers >= lower and <= upper
+// 3. Throws ERR_INVALID_ARG_TYPE for non-numbers
+// 4. Throws ERR_OUT_OF_RANGE for infinite numbers or numbers > upper or < lower
+const checkRangesOrGetDefault = hideStackFrames(
+  (number, name, lower, upper, def) => {
+    if (!validateFiniteNumber(number, name)) {
+      return def;
+    }
+    if (number < lower || number > upper) {
+      throw new ERR_OUT_OF_RANGE(name, `>= ${lower} and <= ${upper}`, number);
+    }
+    return number;
+  },
+);
+
 module.exports = {
   isInt32,
   isUint32,
@@ -601,4 +642,6 @@ module.exports = {
   validateAbortSignal,
   validateLinkHeaderValue,
   validateInternalField,
+  validateFiniteNumber,
+  checkRangesOrGetDefault,
 };
