@@ -459,6 +459,17 @@ class Worker extends EventEmitter {
       };
     });
   }
+
+  getHeapStatistics() {
+    const taker = this[kHandle]?.getHeapStatistics();
+
+    return new Promise((resolve, reject) => {
+      if (!taker) return reject(new ERR_WORKER_NOT_RUNNING());
+      taker.ondone = (handle) => {
+        resolve(handle);
+      };
+    });
+  }
 }
 
 /**
