@@ -1860,9 +1860,17 @@ module.exports = {
 
 ObjectDefineProperty(module.exports, 'builtinModules', {
   __proto__: null,
-  get: () => _builtinLibs,
-  set: (val) => _builtinLibs = val,
-  enumerable: true,
+  get: pendingDeprecation ? deprecate(
+    () => _builtinLibs,
+    'repl.builtinModules is deprecated. Check module.builtinModules instead',
+    'DEP0191',
+  ) : () => _builtinLibs,
+  set: pendingDeprecation ? deprecate(
+    (val) => _builtinLibs = val,
+    'repl.builtinModules is deprecated. Check module.builtinModules instead',
+    'DEP0191',
+  ) : (val) => _builtinLibs = val,
+  enumerable: false,
   configurable: true,
 });
 
