@@ -1,6 +1,7 @@
 'use strict';
 
 const {
+  isInternalThread,
   isMainThread,
   SHARE_ENV,
   resourceLimits,
@@ -13,25 +14,33 @@ const {
 const {
   MessagePort,
   MessageChannel,
+  markAsUncloneable,
   moveMessagePortToContext,
   receiveMessageOnPort,
   BroadcastChannel,
 } = require('internal/worker/io');
 
+const {
+  postMessageToThread,
+} = require('internal/worker/messaging');
+
 const {
   markAsUntransferable,
   isMarkedAsUntransferable,
 } = require('internal/buffer');
 
 module.exports = {
+  isInternalThread,
   isMainThread,
   MessagePort,
   MessageChannel,
+  markAsUncloneable,
   markAsUntransferable,
   isMarkedAsUntransferable,
   moveMessagePortToContext,
   receiveMessageOnPort,
   resourceLimits,
+  postMessageToThread,
   threadId,
   SHARE_ENV,
   Worker,
