@@ -793,9 +793,7 @@ CipherBase::UpdateResult CipherBase::Update(
   } else if (static_cast<size_t>(buf_len) != (*out)->ByteLength()) {
     std::unique_ptr<BackingStore> old_out = std::move(*out);
     *out = ArrayBuffer::NewBackingStore(env()->isolate(), buf_len);
-    memcpy(static_cast<char*>((*out)->Data()),
-           static_cast<char*>(old_out->Data()),
-           buf_len);
+    memcpy((*out)->Data(), old_out->Data(), buf_len);
   }
 
   // When in CCM mode, EVP_CipherUpdate will fail if the authentication tag is
@@ -890,9 +888,7 @@ bool CipherBase::Final(std::unique_ptr<BackingStore>* out) {
     } else if (static_cast<size_t>(out_len) != (*out)->ByteLength()) {
       std::unique_ptr<BackingStore> old_out = std::move(*out);
       *out = ArrayBuffer::NewBackingStore(env()->isolate(), out_len);
-      memcpy(static_cast<char*>((*out)->Data()),
-             static_cast<char*>(old_out->Data()),
-             out_len);
+      memcpy((*out)->Data(), old_out->Data(), out_len);
     }
 
     if (ok && kind_ == kCipher && IsAuthenticatedMode()) {
@@ -996,9 +992,7 @@ bool PublicKeyCipher::Cipher(
   } else if (out_len != (*out)->ByteLength()) {
     std::unique_ptr<BackingStore> old_out = std::move(*out);
     *out = ArrayBuffer::NewBackingStore(env->isolate(), out_len);
-    memcpy(static_cast<char*>((*out)->Data()),
-           static_cast<char*>(old_out->Data()),
-           out_len);
+    memcpy((*out)->Data(), old_out->Data(), out_len);
   }
 
   return true;
