@@ -12,6 +12,7 @@
 #include "v8-fast-api-calls.h"
 #include "v8.h"
 
+#include <optional>
 #include <string>
 
 namespace node {
@@ -58,6 +59,7 @@ class BindingData : public SnapshotableObject {
   static void Format(const v8::FunctionCallbackInfo<v8::Value>& args);
   static void GetOrigin(const v8::FunctionCallbackInfo<v8::Value>& args);
   static void Parse(const v8::FunctionCallbackInfo<v8::Value>& args);
+  static void PathToFileURL(const v8::FunctionCallbackInfo<v8::Value>& args);
   static void Update(const v8::FunctionCallbackInfo<v8::Value>& args);
 
   static void CreatePerIsolateProperties(IsolateData* isolate_data,
@@ -76,12 +78,14 @@ class BindingData : public SnapshotableObject {
                         const ada::scheme::type type);
 
   static v8::CFunction fast_can_parse_methods_[];
-  static void ThrowInvalidURL(Environment* env,
-                              std::string_view input,
-                              std::optional<std::string> base);
 };
 
-std::string FromFilePath(const std::string_view file_path);
+void ThrowInvalidURL(Environment* env,
+                     std::string_view input,
+                     std::optional<std::string> base);
+std::string FromFilePath(std::string_view file_path);
+std::optional<std::string> FileURLToPath(Environment* env,
+                                         const ada::url_aggregator& file_url);
 
 }  // namespace url
 
