@@ -133,12 +133,26 @@ void WASI::New(const FunctionCallbackInfo<Value>& args) {
 
   Local<Array> stdio = args[3].As<Array>();
   CHECK_EQ(stdio->Length(), 3);
-  options.in = stdio->Get(context, 0).ToLocalChecked()->
-    Int32Value(context).FromJust();
-  options.out = stdio->Get(context, 1).ToLocalChecked()->
-    Int32Value(context).FromJust();
-  options.err = stdio->Get(context, 2).ToLocalChecked()->
-    Int32Value(context).FromJust();
+
+  Local<Value> val;
+  int32_t tmp;
+  if (!stdio->Get(context, 0).ToLocal(&val) ||
+      !val->Int32Value(context).To(&tmp)) {
+    return;
+  }
+  options.in = tmp;
+
+  if (!stdio->Get(context, 1).ToLocal(&val) ||
+      !val->Int32Value(context).To(&tmp)) {
+    return;
+  }
+  options.out = tmp;
+
+  if (!stdio->Get(context, 2).ToLocal(&val) ||
+      !val->Int32Value(context).To(&tmp)) {
+    return;
+  }
+  options.err = tmp;
 
   options.fd_table_size = 3;
   options.argc = argc;
@@ -146,7 +160,10 @@ void WASI::New(const FunctionCallbackInfo<Value>& args) {
     const_cast<const char**>(argc == 0 ? nullptr : new char*[argc]);
 
   for (uint32_t i = 0; i < argc; i++) {
-    auto arg = argv->Get(context, i).ToLocalChecked();
+    Local<Value> arg;
+    if (!argv->Get(context, i).ToLocal(&arg)) {
+      return;
+    }
     CHECK(arg->IsString());
     node::Utf8Value str(env->isolate(), arg);
     options.argv[i] = strdup(*str);
@@ -157,7 +174,10 @@ void WASI::New(const FunctionCallbackInfo<Value>& args) {
   const uint32_t envc = env_pairs->Length();
   options.envp = const_cast<const char**>(new char*[envc + 1]);
   for (uint32_t i = 0; i < envc; i++) {
-    auto pair = env_pairs->Get(context, i).ToLocalChecked();
+    Local<Value> pair;
+    if (!env_pairs->Get(context, i).ToLocal(&pair)) {
+      return;
+    }
     CHECK(pair->IsString());
     node::Utf8Value str(env->isolate(), pair);
     options.envp[i] = strdup(*str);
@@ -171,8 +191,12 @@ void WASI::New(const FunctionCallbackInfo<Value>& args) {
   options.preopens = Calloc<uvwasi_preopen_t>(options.preopenc);
   int index = 0;
   for (uint32_t i = 0; i < preopens->Length(); i += 2) {
-    auto mapped = preopens->Get(context, i).ToLocalChecked();
-    auto real = preopens->Get(context, i + 1).ToLocalChecked();
+    Local<Value> mapped;
+    Local<Value> real;
+    if (!preopens->Get(context, i).ToLocal(&mapped) ||
+        !preopens->Get(context, i + 1).ToLocal(&real)) {
+      return;
+    }
     CHECK(mapped->IsString());
     CHECK(real->IsString());
     node::Utf8Value mapped_path(env->isolate(), mapped);
