@@ -25,6 +25,7 @@
 #include "env-inl.h"
 #include "handle_wrap.h"
 #include "node_buffer.h"
+#include "node_errors.h"
 #include "node_external_reference.h"
 #include "pipe_wrap.h"
 #include "req_wrap-inl.h"
@@ -38,14 +39,19 @@
 
 namespace node {
 
+using errors::TryCatchScope;
 using v8::Context;
 using v8::DontDelete;
 using v8::EscapableHandleScope;
 using v8::FunctionCallbackInfo;
 using v8::FunctionTemplate;
 using v8::HandleScope;
+using v8::Isolate;
+using v8::JustVoid;
 using v8::Local;
+using v8::Maybe;
 using v8::MaybeLocal;
+using v8::Nothing;
 using v8::Object;
 using v8::PropertyAttribute;
 using v8::ReadOnly;
@@ -62,9 +68,10 @@ void LibuvStreamWrap::Initialize(Local<Object> target,
                                  Local<Context> context,
                                  void* priv) {
   Environment* env = Environment::GetCurrent(context);
+  Isolate* isolate = env->isolate();
 
   Local<FunctionTemplate> sw =
-      FunctionTemplate::New(env->isolate(), IsConstructCallCallback);
+      NewFunctionTemplate(isolate, IsConstructCallCallback);
   sw->InstanceTemplate()->SetInternalFieldCount(StreamReq::kInternalFieldCount);
 
   // we need to set handle and callback to null,
@@ -74,33 +81,34 @@ void LibuvStreamWrap::Initialize(Local<Object> target,
   // - oncomplete
   // - callback
   // - handle
-  sw->InstanceTemplate()->Set(
-      env->oncomplete_string(),
-      v8::Null(env->isolate()));
-  sw->InstanceTemplate()->Set(FIXED_ONE_BYTE_STRING(env->isolate(), "callback"),
-      v8::Null(env->isolate()));
-  sw->InstanceTemplate()->Set(FIXED_ONE_BYTE_STRING(env->isolate(), "handle"),
-      v8::Null(env->isolate()));
+  sw->InstanceTemplate()->Set(env->oncomplete_string(), v8::Null(isolate));
+  sw->InstanceTemplate()->Set(FIXED_ONE_BYTE_STRING(isolate, "callback"),
+                              v8::Null(isolate));
+  sw->InstanceTemplate()->Set(FIXED_ONE_BYTE_STRING(isolate, "handle"),
+                              v8::Null(isolate));
 
   sw->Inherit(AsyncWrap::GetConstructorTemplate(env));
 
-  env->SetConstructorFunction(target, "ShutdownWrap", sw);
+  SetConstructorFunction(context, target, "ShutdownWrap", sw);
   env->set_shutdown_wrap_template(sw->InstanceTemplate());
 
   Local<FunctionTemplate> ww =
-      FunctionTemplate::New(env->isolate(), IsConstructCallCallback);
+      FunctionTemplate::New(isolate, IsConstructCallCallback);
   ww->InstanceTemplate()->SetInternalFieldCount(
       StreamReq::kInternalFieldCount);
   ww->Inherit(AsyncWrap::GetConstructorTemplate(env));
-  env->SetConstructorFunction(target, "WriteWrap", ww);
+  SetConstructorFunction(context, target, "WriteWrap", ww);
   env->set_write_wrap_template(ww->InstanceTemplate());
 
   NODE_DEFINE_CONSTANT(target, kReadBytesOrError);
   NODE_DEFINE_CONSTANT(target, kArrayBufferOffset);
   NODE_DEFINE_CONSTANT(target, kBytesWritten);
   NODE_DEFINE_CONSTANT(target, kLastWriteWasAsync);
-  target->Set(context, FIXED_ONE_BYTE_STRING(env->isolate(), "streamBaseState"),
-              env->stream_base_state().GetJSArray()).Check();
+  target
+      ->Set(context,
+            FIXED_ONE_BYTE_STRING(isolate, "streamBaseState"),
+            env->stream_base_state().GetJSArray())
+      .Check();
 }
 
 void LibuvStreamWrap::RegisterExternalReferences(
@@ -108,8 +116,7 @@ void LibuvStreamWrap::RegisterExternalReferences(
   registry->Register(IsConstructCallCallback);
   registry->Register(GetWriteQueueSize);
   registry->Register(SetBlocking);
-  // TODO(joyee): StreamBase::RegisterExternalReferences() is called somewhere
-  // else but we may want to do it here too and guard it with a static flag.
+  StreamBase::RegisterExternalReferences(registry);
 }
 
 LibuvStreamWrap::LibuvStreamWrap(Environment* env,
@@ -130,23 +137,23 @@ Local<FunctionTemplate> LibuvStreamWrap::GetConstructorTemplate(
     Environment* env) {
   Local<FunctionTemplate> tmpl = env->libuv_stream_wrap_ctor_template();
   if (tmpl.IsEmpty()) {
-    tmpl = env->NewFunctionTemplate(nullptr);
-    tmpl->SetClassName(
-        FIXED_ONE_BYTE_STRING(env->isolate(), "LibuvStreamWrap"));
+    Isolate* isolate = env->isolate();
+    tmpl = NewFunctionTemplate(isolate, nullptr);
+    tmpl->SetClassName(FIXED_ONE_BYTE_STRING(isolate, "LibuvStreamWrap"));
     tmpl->Inherit(HandleWrap::GetConstructorTemplate(env));
     tmpl->InstanceTemplate()->SetInternalFieldCount(
         StreamBase::kInternalFieldCount);
     Local<FunctionTemplate> get_write_queue_size =
-        FunctionTemplate::New(env->isolate(),
+        FunctionTemplate::New(isolate,
                               GetWriteQueueSize,
                               Local<Value>(),
-                              Signature::New(env->isolate(), tmpl));
+                              Signature::New(isolate, tmpl));
     tmpl->PrototypeTemplate()->SetAccessorProperty(
         env->write_queue_size_string(),
         get_write_queue_size,
         Local<FunctionTemplate>(),
         static_cast<PropertyAttribute>(ReadOnly | DontDelete));
-    env->SetProtoMethod(tmpl, "setBlocking", SetBlocking);
+    SetProtoMethod(isolate, tmpl, "setBlocking", SetBlocking);
     StreamBase::AddMethods(env, tmpl);
     env->set_libuv_stream_wrap_ctor_template(tmpl);
   }
@@ -192,15 +199,19 @@ bool LibuvStreamWrap::IsIPCPipe() {
   return is_named_pipe_ipc();
 }
 
-
 int LibuvStreamWrap::ReadStart() {
-  return uv_read_start(stream(), [](uv_handle_t* handle,
-                                    size_t suggested_size,
-                                    uv_buf_t* buf) {
-    static_cast<LibuvStreamWrap*>(handle->data)->OnUvAlloc(suggested_size, buf);
-  }, [](uv_stream_t* stream, ssize_t nread, const uv_buf_t* buf) {
-    static_cast<LibuvStreamWrap*>(stream->data)->OnUvRead(nread, buf);
-  });
+  return uv_read_start(
+      stream(),
+      [](uv_handle_t* handle, size_t suggested_size, uv_buf_t* buf) {
+        static_cast<LibuvStreamWrap*>(handle->data)
+            ->OnUvAlloc(suggested_size, buf);
+      },
+      [](uv_stream_t* stream, ssize_t nread, const uv_buf_t* buf) {
+        LibuvStreamWrap* wrap = static_cast<LibuvStreamWrap*>(stream->data);
+        TryCatchScope try_catch(wrap->env());
+        try_catch.SetVerbose(true);
+        wrap->OnUvRead(nread, buf);
+      });
 }
 
 
@@ -240,8 +251,7 @@ static MaybeLocal<Object> AcceptHandle(Environment* env,
   return scope.Escape(wrap_obj);
 }
 
-
-void LibuvStreamWrap::OnUvRead(ssize_t nread, const uv_buf_t* buf) {
+Maybe<void> LibuvStreamWrap::OnUvRead(ssize_t nread, const uv_buf_t* buf) {
   HandleScope scope(env()->isolate());
   Context::Scope context_scope(env()->context());
   uv_handle_type type = UV_UNKNOWN_HANDLE;
@@ -268,19 +278,22 @@ void LibuvStreamWrap::OnUvRead(ssize_t nread, const uv_buf_t* buf) {
       CHECK_EQ(type, UV_UNKNOWN_HANDLE);
     }
 
-    if (!pending_obj.IsEmpty()) {
-      object()
-          ->Set(env()->context(),
-                env()->pending_handle_string(),
-                pending_obj.ToLocalChecked())
-          .Check();
+    Local<Object> local_pending_obj;
+    if (type != UV_UNKNOWN_HANDLE &&
+        (!pending_obj.ToLocal(&local_pending_obj) ||
+         object()
+             ->Set(env()->context(),
+                   env()->pending_handle_string(),
+                   local_pending_obj)
+             .IsNothing())) {
+      return Nothing<void>();
     }
   }
 
   EmitRead(nread, *buf);
+  return JustVoid();
 }
 
-
 void LibuvStreamWrap::GetWriteQueueSize(
     const FunctionCallbackInfo<Value>& info) {
   LibuvStreamWrap* wrap;
@@ -298,7 +311,7 @@ void LibuvStreamWrap::GetWriteQueueSize(
 
 void LibuvStreamWrap::SetBlocking(const FunctionCallbackInfo<Value>& args) {
   LibuvStreamWrap* wrap;
-  ASSIGN_OR_RETURN_UNWRAP(&wrap, args.Holder());
+  ASSIGN_OR_RETURN_UNWRAP(&wrap, args.This());
 
   CHECK_GT(args.Length(), 0);
   if (!wrap->IsAlive())
@@ -402,7 +415,7 @@ void LibuvStreamWrap::AfterUvWrite(uv_write_t* req, int status) {
 
 }  // namespace node
 
-NODE_MODULE_CONTEXT_AWARE_INTERNAL(stream_wrap,
-                                   node::LibuvStreamWrap::Initialize)
-NODE_MODULE_EXTERNAL_REFERENCE(
+NODE_BINDING_CONTEXT_AWARE_INTERNAL(stream_wrap,
+                                    node::LibuvStreamWrap::Initialize)
+NODE_BINDING_EXTERNAL_REFERENCE(
     stream_wrap, node::LibuvStreamWrap::RegisterExternalReferences)
