@@ -3,11 +3,9 @@ const common = require('../common');
 if (!common.hasCrypto)
   common.skip('missing crypto');
 
-if (common.hasOpenSSL3)
-  common.skip('temporarily skipping for OpenSSL 3.0-alpha15');
-
 const assert = require('assert');
 const crypto = require('crypto');
+const { hasOpenSSL3 } = require('../common/crypto');
 
 assert.throws(() => crypto.diffieHellman(), {
   name: 'TypeError',
@@ -147,17 +145,13 @@ test(crypto.generateKeyPairSync('dh', { group: 'modp5' }),
 test(crypto.generateKeyPairSync('dh', { group: 'modp5' }),
      crypto.generateKeyPairSync('dh', { prime: group.getPrime() }));
 
-const list = [];
-// Same generator, but different primes.
-// TODO(danbev) only commenting out this so that we can get our CI build
-// to pass. I'll continue looking into the cause/change.
-// [{ group: 'modp5' }, { group: 'modp18' }]];
+const list = [
+  // Same generator, but different primes.
+  [{ group: 'modp5' }, { group: 'modp18' }]];
 
 // TODO(danbev): Take a closer look if there should be a check in OpenSSL3
 // when the dh parameters differ.
-if (!common.hasOpenSSL3) {
-  // Same generator, but different primes.
-  list.push([{ group: 'modp5' }, { group: 'modp18' }]);
+if (!hasOpenSSL3) {
   // Same primes, but different generator.
   list.push([{ group: 'modp5' }, { prime: group.getPrime(), generator: 5 }]);
   // Same generator, but different primes.
@@ -168,9 +162,9 @@ for (const [params1, params2] of list) {
   assert.throws(() => {
     test(crypto.generateKeyPairSync('dh', params1),
          crypto.generateKeyPairSync('dh', params2));
-  }, common.hasOpenSSL3 ? {
+  }, hasOpenSSL3 ? {
     name: 'Error',
-    code: 'ERR_OSSL_DH_INVALID_PUBLIC_KEY'
+    code: 'ERR_OSSL_MISMATCHING_DOMAIN_PARAMETERS'
   } : {
     name: 'Error',
     code: 'ERR_OSSL_EVP_DIFFERENT_PARAMETERS'
@@ -227,9 +221,9 @@ const not256k1 = crypto.getCurves().find((c) => /^sec.*(224|384|512)/.test(c));
 assert.throws(() => {
   test(crypto.generateKeyPairSync('ec', { namedCurve: 'secp256k1' }),
        crypto.generateKeyPairSync('ec', { namedCurve: not256k1 }));
-}, common.hasOpenSSL3 ? {
+}, hasOpenSSL3 ? {
   name: 'Error',
-  code: 'ERR_OSSL_MISMATCHING_SHARED_PARAMETERS'
+  code: 'ERR_OSSL_MISMATCHING_DOMAIN_PARAMETERS'
 } : {
   name: 'Error',
   code: 'ERR_OSSL_EVP_DIFFERENT_PARAMETERS'
