@@ -1,16 +1,17 @@
 'use strict';
 const common = require('../common');
-if (!common.hasCrypto)
-  common.skip('missing crypto');
+if ((!common.hasCrypto) || (!common.hasIntl)) {
+  common.skip('ESLint tests require crypto and Intl');
+}
 common.skipIfEslintMissing();
 
-const RuleTester = require('../../tools/node_modules/eslint').RuleTester;
+const RuleTester = require('../../tools/eslint/node_modules/eslint').RuleTester;
 const rule = require('../../tools/eslint-rules/async-iife-no-unused-result');
 
 const message = 'The result of an immediately-invoked async function needs ' +
   'to be used (e.g. with `.then(common.mustCall())`)';
 
-const tester = new RuleTester({ parserOptions: { ecmaVersion: 8 } });
+const tester = new RuleTester();
 tester.run('async-iife-no-unused-result', rule, {
   valid: [
     '(() => {})()',
@@ -26,12 +27,10 @@ tester.run('async-iife-no-unused-result', rule, {
     {
       code: '(async () => {})()',
       errors: [{ message }],
-      output: '(async () => {})()',
     },
     {
       code: '(async function() {})()',
       errors: [{ message }],
-      output: '(async function() {})()',
     },
     {
       code: "const common = require('../common');(async () => {})()",
