@@ -22,13 +22,12 @@
 'use strict';
 const common = require('../common');
 const assert = require('assert');
-const path = require('path');
 const fs = require('fs');
 
 const tmpdir = require('../common/tmpdir');
 
 
-const filepath = path.join(tmpdir.path, 'write_pos.txt');
+const filepath = tmpdir.resolve('write_pos.txt');
 
 
 const cb_expected = 'write open close write open close write open close ';
@@ -182,13 +181,15 @@ const run_test_4 = common.mustCall(function() {
   const fn = () => {
     fs.createWriteStream(filepath, { start: -5, flags: 'r+' });
   };
+  // Verify the range of values using a common integer verifier.
+  // Limit Number.MAX_SAFE_INTEGER
   const err = {
     code: 'ERR_OUT_OF_RANGE',
     message: 'The value of "start" is out of range. ' +
-             'It must be >= 0 and <= 2 ** 53 - 1. Received -5',
-    type: RangeError
+             `It must be >= 0 && <= ${Number.MAX_SAFE_INTEGER}. Received -5`,
+    name: 'RangeError'
   };
-  common.expectsError(fn, err);
+  assert.throws(fn, err);
 });
 
 
@@ -197,13 +198,16 @@ const run_test_5 = common.mustCall(function() {
   const fn = () => {
     fs.createWriteStream(filepath, { start: 2 ** 53, flags: 'r+' });
   };
+  // Verify the range of values using a common integer verifier.
+  // Limit Number.MAX_SAFE_INTEGER
   const err = {
     code: 'ERR_OUT_OF_RANGE',
     message: 'The value of "start" is out of range. It must be ' +
-             '>= 0 and <= 2 ** 53 - 1. Received 9_007_199_254_740_992',
-    type: RangeError
+             `>= 0 && <= ${Number.MAX_SAFE_INTEGER}. ` +
+             'Received 9_007_199_254_740_992',
+    name: 'RangeError'
   };
-  common.expectsError(fn, err);
+  assert.throws(fn, err);
 });
 
 run_test_1();
