@@ -30,6 +30,14 @@ server.on('connect', common.mustCall((req, socket, firstBodyChunk) => {
   assert.strictEqual(req.method, 'CONNECT');
   assert.strictEqual(req.url, 'google.com:443');
 
+  // Make sure this socket has detached.
+  assert.strictEqual(socket.listenerCount('close'), 0);
+  assert.strictEqual(socket.listenerCount('drain'), 0);
+  assert.strictEqual(socket.listenerCount('data'), 0);
+  assert.strictEqual(socket.listenerCount('end'), 1);
+  assert.strictEqual(socket.listenerCount('error'), 0);
+  assert.strictEqual(socket.listenerCount('timeout'), 0);
+
   socket.write('HTTP/1.1 200 Connection established\r\n\r\n');
 
   let data = firstBodyChunk.toString();
@@ -46,33 +54,38 @@ server.listen(0, common.mustCall(() => {
   const req = http.request({
     port: server.address().port,
     method: 'CONNECT',
-    path: 'google.com:443'
+    path: 'google.com:443',
+    timeout: 20000
   }, common.mustNotCall());
 
   req.on('socket', common.mustCall((socket) => {
     assert.strictEqual(socket._httpMessage, req);
   }));
 
-  req.on('close', common.mustCall());
+  assert.strictEqual(req.destroyed, false);
+  req.on('close', common.mustCall(() => {
+    assert.strictEqual(req.destroyed, true);
+  }));
 
   req.on('connect', common.mustCall((res, socket, firstBodyChunk) => {
     // Make sure this request got removed from the pool.
     const name = `localhost:${server.address().port}`;
-    assert(!http.globalAgent.sockets.hasOwnProperty(name));
-    assert(!http.globalAgent.requests.hasOwnProperty(name));
+    assert(!(name in http.globalAgent.sockets));
+    assert(!(name in http.globalAgent.requests));
 
     // Make sure this socket has detached.
     assert(!socket.ondata);
     assert(!socket.onend);
     assert.strictEqual(socket._httpMessage, null);
-    assert.strictEqual(socket.listeners('connect').length, 0);
-    assert.strictEqual(socket.listeners('data').length, 0);
-    assert.strictEqual(socket.listeners('drain').length, 0);
-    assert.strictEqual(socket.listeners('end').length, 1);
-    assert.strictEqual(socket.listeners('free').length, 0);
-    assert.strictEqual(socket.listeners('close').length, 0);
-    assert.strictEqual(socket.listeners('error').length, 0);
-    assert.strictEqual(socket.listeners('agentRemove').length, 0);
+    assert.strictEqual(socket.listenerCount('connect'), 0);
+    assert.strictEqual(socket.listenerCount('data'), 0);
+    assert.strictEqual(socket.listenerCount('drain'), 0);
+    assert.strictEqual(socket.listenerCount('end'), 1);
+    assert.strictEqual(socket.listenerCount('free'), 0);
+    assert.strictEqual(socket.listenerCount('close'), 0);
+    assert.strictEqual(socket.listenerCount('error'), 0);
+    assert.strictEqual(socket.listenerCount('agentRemove'), 0);
+    assert.strictEqual(socket.listenerCount('timeout'), 0);
 
     let data = firstBodyChunk.toString();
     socket.on('data', (buf) => {
