@@ -1,4 +1,3 @@
-// Flags: --expose-http2
 'use strict';
 
 const common = require('../common');
@@ -11,30 +10,24 @@ const checkWeight = (actual, expect) => {
   const server = http2.createServer();
   server.on('stream', common.mustCall((stream, headers, flags) => {
     assert.strictEqual(stream.state.weight, expect);
-    stream.respond({
-      'content-type': 'text/html',
-      ':status': 200
-    });
+    stream.respond();
     stream.end('test');
   }));
 
   server.listen(0, common.mustCall(() => {
-    const port = server.address().port;
-    const client = http2.connect(`http://localhost:${port}`);
+    const client = http2.connect(`http://localhost:${server.address().port}`);
+    const req = client.request({}, { weight: actual });
 
-    const headers = { ':path': '/' };
-    const req = client.request(headers, { weight: actual });
-
-    req.on('data', common.mustCall(() => {}));
-    req.on('end', common.mustCall(() => {
+    req.on('data', common.mustCall());
+    req.on('end', common.mustCall());
+    req.on('close', common.mustCall(() => {
       server.close();
-      client.destroy();
+      client.close();
     }));
-    req.end();
   }));
 };
 
-// when client weight is lower than 1, weight is 1
+// When client weight is lower than 1, weight is 1
 checkWeight(-1, 1);
 checkWeight(0, 1);
 
@@ -43,9 +36,9 @@ checkWeight(1, 1);
 checkWeight(16, 16);
 checkWeight(256, 256);
 
-// when client weight is higher than 256, weight is 256
+// When client weight is higher than 256, weight is 256
 checkWeight(257, 256);
 checkWeight(512, 256);
 
-// when client weight is undefined, weight is default 16
+// When client weight is undefined, weight is default 16
 checkWeight(undefined, 16);
