@@ -4,13 +4,14 @@ const common = require('../common');
 if (!common.hasCrypto)
   common.skip('missing crypto');
 const fixtures = require('../common/fixtures');
+const assert = require('assert');
 const http2 = require('http2');
+const { inspect } = require('util');
 
 const optionsWithTypeError = {
   offset: 'number',
   length: 'number',
-  statCheck: 'function',
-  getTrailers: 'function'
+  statCheck: 'function'
 };
 
 const types = {
@@ -36,24 +37,24 @@ server.on('stream', common.mustCall((stream) => {
         return;
       }
 
-      common.expectsError(
+      assert.throws(
         () => stream.respondWithFile(fname, {
           'content-type': 'text/plain'
         }, {
           [option]: types[type]
         }),
         {
-          type: TypeError,
-          code: 'ERR_INVALID_OPT_VALUE',
-          message: `The value "${String(types[type])}" is invalid ` +
-                   `for option "${option}"`
+          name: 'TypeError',
+          code: 'ERR_INVALID_ARG_VALUE',
+          message: `The property 'options.${option}' is invalid. ` +
+            `Received ${inspect(types[type])}`
         }
       );
     });
   });
 
   // Should throw if :status 204, 205 or 304
-  [204, 205, 304].forEach((status) => common.expectsError(
+  [204, 205, 304].forEach((status) => assert.throws(
     () => stream.respondWithFile(fname, {
       'content-type': 'text/plain',
       ':status': status,
@@ -66,7 +67,7 @@ server.on('stream', common.mustCall((stream) => {
 
   // Should throw if headers already sent
   stream.respond({ ':status': 200 });
-  common.expectsError(
+  assert.throws(
     () => stream.respondWithFile(fname, {
       'content-type': 'text/plain'
     }),
@@ -78,7 +79,7 @@ server.on('stream', common.mustCall((stream) => {
 
   // Should throw if stream already destroyed
   stream.destroy();
-  common.expectsError(
+  assert.throws(
     () => stream.respondWithFile(fname, {
       'content-type': 'text/plain'
     }),
