@@ -74,7 +74,8 @@ function createDnsServer(ipv6Addr, ipv4Addr, cb) {
 // Test that IPV4 is reached if IPV6 is not reachable
 {
   createDnsServer('::1', '127.0.0.1', common.mustCall(function({ dnsServer, lookup }) {
-    const ipv4Server = createServer(options, common.mustCall((_, res) => {
+    const ipv4Server = createServer(options, common.mustCall((req, res) => {
+      assert.strictEqual(req.socket.servername, 'example.org');
       res.writeHead(200, { Connection: 'close' });
       res.end('response-ipv4');
     }));
@@ -86,6 +87,7 @@ function createDnsServer(ipv6Addr, ipv4Addr, cb) {
           lookup,
           rejectUnauthorized: false,
           autoSelectFamily: true,
+          servername: 'example.org',
         },
         (res) => {
           assert.strictEqual(res.statusCode, 200);
@@ -111,12 +113,14 @@ function createDnsServer(ipv6Addr, ipv4Addr, cb) {
 // Test that IPV4 is NOT reached if IPV6 is reachable
 if (common.hasIPv6) {
   createDnsServer('::1', '127.0.0.1', common.mustCall(function({ dnsServer, lookup }) {
-    const ipv4Server = createServer(options, common.mustNotCall((_, res) => {
+    const ipv4Server = createServer(options, common.mustNotCall((req, res) => {
+      assert.strictEqual(req.socket.servername, 'example.org');
       res.writeHead(200, { Connection: 'close' });
       res.end('response-ipv4');
     }));
 
-    const ipv6Server = createServer(options, common.mustCall((_, res) => {
+    const ipv6Server = createServer(options, common.mustCall((req, res) => {
+      assert.strictEqual(req.socket.servername, 'example.org');
       res.writeHead(200, { Connection: 'close' });
       res.end('response-ipv6');
     }));
@@ -131,6 +135,7 @@ if (common.hasIPv6) {
             lookup,
             rejectUnauthorized: false,
             autoSelectFamily: true,
+            servername: 'example.org',
           },
           (res) => {
             assert.strictEqual(res.statusCode, 200);
