@@ -10,7 +10,10 @@ for (const entryPoint of entryPoints) {
   try {
     execFileSync(node, [entryPoint], { stdio: 'pipe' });
   } catch (e) {
-    assert(e.toString().match(/Error: Cannot find module/));
+    const error = e.toString();
+    assert.match(error, /MODULE_NOT_FOUND/);
+    assert.match(error, /Cannot find module/);
+    assert(error.includes(entryPoint));
     continue;
   }
   assert.fail('Executing node with inexistent entry point should ' +
