@@ -1,4 +1,4 @@
-// Flags: --expose-gc
+// Flags: --expose-gc --no-warnings
 'use strict';
 
 const common = require('../common');
@@ -31,13 +31,16 @@ const kinds = [
     assert.strictEqual(entry.name, 'gc');
     assert.strictEqual(entry.entryType, 'gc');
     assert(kinds.includes(entry.kind));
+    assert(kinds.includes(entry.detail.kind));
     assert.strictEqual(entry.flags, NODE_PERFORMANCE_GC_FLAGS_FORCED);
+    assert.strictEqual(entry.detail.flags, NODE_PERFORMANCE_GC_FLAGS_FORCED);
     assert.strictEqual(typeof entry.startTime, 'number');
+    assert(entry.startTime < 1e4, 'startTime should be relative to performance.timeOrigin.');
     assert.strictEqual(typeof entry.duration, 'number');
     obs.disconnect();
   }));
   obs.observe({ entryTypes: ['gc'] });
-  global.gc();
+  globalThis.gc();
   // Keep the event loop alive to witness the GC async callback happen.
   setImmediate(() => setImmediate(() => 0));
 }
@@ -48,6 +51,6 @@ const kinds = [
   process.on('beforeExit', () => {
     assert(!didCall);
     didCall = true;
-    global.gc();
+    globalThis.gc();
   });
 }
