@@ -2,7 +2,11 @@
 'use strict';
 
 const common = require('../common');
-common.skipIfWorker();
+const { isMainThread } = require('worker_threads');
+
+if (!isMainThread) {
+  common.skip('This test only works on a main thread');
+}
 
 const { createRequire } = require('node:module');
 const assert = require('node:assert');
