@@ -1,13 +1,13 @@
 'use strict';
 
-const { mustNotCall, skip, isWindows } = require('../common');
+const { mustNotCall, skip, isWindows, isIBMi } = require('../common');
 const { strictEqual } = require('assert');
 const { isMainThread } = require('worker_threads');
 
 if (!isMainThread) {
   skip('process.execve is not available in Workers');
-} else if (isWindows) {
-  skip('process.execve is not available in Windows');
+} else if (isWindows || isIBMi) {
+  skip('process.execve is not available in Windows or IBM i');
 }
 
 if (process.argv[2] === 'replaced') {
