@@ -1,14 +1,14 @@
 'use strict';
 
-const { mustCall, mustNotCall, skip, isWindows } = require('../common');
+const { mustCall, mustNotCall, skip, isWindows, isIBMi } = require('../common');
 const { fail, ok } = require('assert');
 const { createServer } = require('net');
 const { isMainThread } = require('worker_threads');
 
 if (!isMainThread) {
   skip('process.execve is not available in Workers');
-} else if (isWindows) {
-  skip('process.execve is not available in Windows');
+} else if (isWindows || isIBMi) {
+  skip('process.execve is not available in Windows or IBM i');
 }
 
 if (process.argv[2] === 'replaced') {
