@@ -1,14 +1,16 @@
 'use strict';
 const common = require('../common');
 const assert = require('assert');
+const { isMainThread } = require('worker_threads');
 
 if (common.isWindows) {
   assert.strictEqual(process.setgroups, undefined);
   return;
 }
 
-if (!common.isMainThread)
+if (!isMainThread) {
   return;
+}
 
 assert.throws(
   () => {
@@ -16,9 +18,9 @@ assert.throws(
   },
   {
     code: 'ERR_INVALID_ARG_TYPE',
-    name: 'TypeError [ERR_INVALID_ARG_TYPE]',
-    message: 'The "groups" argument must be of type Array. ' +
-             'Received type undefined'
+    name: 'TypeError',
+    message: 'The "groups" argument must be an instance of Array. ' +
+             'Received undefined'
   }
 );
 
@@ -28,9 +30,7 @@ assert.throws(
   },
   {
     code: 'ERR_OUT_OF_RANGE',
-    name: 'RangeError [ERR_OUT_OF_RANGE]',
-    message: 'The value of "groups[1]" is out of range. ' +
-              'It must be >= 0 && < 4294967296. Received -1'
+    name: 'RangeError',
   }
 );
 
@@ -41,10 +41,10 @@ assert.throws(
     },
     {
       code: 'ERR_INVALID_ARG_TYPE',
-      name: 'TypeError [ERR_INVALID_ARG_TYPE]',
+      name: 'TypeError',
       message: 'The "groups[0]" argument must be ' +
-               'one of type number or string. ' +
-               `Received type ${typeof val}`
+               'one of type number or string.' +
+               common.invalidArgTypeHelper(val)
     }
   );
 });
