@@ -173,6 +173,12 @@ async function ordinaryTests() {
       '3',
       'undefined',
     ]],
+    // Testing documented behavior of `const`s (see: https://github.com/nodejs/node/issues/45918)
+    ['const k = await Promise.resolve(123)'],
+    ['k', '123'],
+    ['k = await Promise.resolve(234)', '234'],
+    ['k', '234'],
+    ['const k = await Promise.resolve(345)', "Uncaught SyntaxError: Identifier 'k' has already been declared"],
     // Regression test for https://github.com/nodejs/node/issues/43777.
     ['await Promise.resolve(123), Promise.resolve(456)', 'Promise {', { line: 0 }],
     ['await Promise.resolve(123), await Promise.resolve(456)', '456'],
@@ -207,8 +213,8 @@ async function ctrlCTest() {
   assert.deepStrictEqual(output.slice(0, 3), [
     'await new Promise(() => {})\r',
     'Uncaught:',
-    'Error [ERR_SCRIPT_EXECUTION_INTERRUPTED]: ' +
-      'Script execution was interrupted by `SIGINT`',
+    '[Error [ERR_SCRIPT_EXECUTION_INTERRUPTED]: ' +
+      'Script execution was interrupted by `SIGINT`] {',
   ]);
   assert.deepStrictEqual(output.slice(-2), [
     '}',
