@@ -1,3 +1,4 @@
+// Flags: --expose-internals
 'use strict';
 process.env.NODE_TEST_KNOWN_GLOBALS = 0;
 const common = require('../common');
@@ -6,6 +7,7 @@ const assert = require('node:assert');
 const { it, mock, describe } = require('node:test');
 const nodeTimers = require('node:timers');
 const nodeTimersPromises = require('node:timers/promises');
+const { TIMEOUT_MAX } = require('internal/timers');
 
 describe('Mock Timers Test Suite', () => {
   describe('MockTimers API', () => {
@@ -17,6 +19,14 @@ describe('Mock Timers Test Suite', () => {
       });
     });
 
+    it('should throw an error if data type of trying to enable a timer is not string', (t) => {
+      assert.throws(() => {
+        t.mock.timers.enable({ apis: [1] });
+      }, {
+        code: 'ERR_INVALID_ARG_TYPE',
+      });
+    });
+
     it('should throw an error if trying to enable a timer twice', (t) => {
       t.mock.timers.enable();
       assert.throws(() => {
@@ -59,7 +69,7 @@ describe('Mock Timers Test Suite', () => {
           'clearImmediate',
         ];
 
-        const globalTimersDescriptors = timers.map((fn) => getDescriptor(global, fn));
+        const globalTimersDescriptors = timers.map((fn) => getDescriptor(globalThis, fn));
         const nodeTimersDescriptors = timers.map((fn) => getDescriptor(nodeTimers, fn));
         const nodeTimersPromisesDescriptors = timers
           .filter((fn) => !fn.includes('clear'))
@@ -106,7 +116,7 @@ describe('Mock Timers Test Suite', () => {
     it('should reset all timers when calling .reset function', (t) => {
       t.mock.timers.enable();
       const fn = t.mock.fn();
-      global.setTimeout(fn, 1000);
+      globalThis.setTimeout(fn, 1000);
       t.mock.timers.reset();
       assert.deepStrictEqual(Date.now, globalThis.Date.now);
       assert.throws(() => {
@@ -121,7 +131,7 @@ describe('Mock Timers Test Suite', () => {
     it('should reset all timers when calling Symbol.dispose', (t) => {
       t.mock.timers.enable();
       const fn = t.mock.fn();
-      global.setTimeout(fn, 1000);
+      globalThis.setTimeout(fn, 1000);
       // TODO(benjamingr) refactor to `using`
       t.mock.timers[Symbol.dispose]();
       assert.throws(() => {
@@ -138,8 +148,8 @@ describe('Mock Timers Test Suite', () => {
       const order = [];
       const fn1 = t.mock.fn(() => order.push('f1'));
       const fn2 = t.mock.fn(() => order.push('f2'));
-      global.setTimeout(fn1, 1000);
-      global.setTimeout(fn2, 1000);
+      globalThis.setTimeout(fn1, 1000);
+      globalThis.setTimeout(fn2, 1000);
       t.mock.timers.tick(1000);
       assert.strictEqual(fn1.mock.callCount(), 1);
       assert.strictEqual(fn2.mock.callCount(), 1);
@@ -160,11 +170,11 @@ describe('Mock Timers Test Suite', () => {
         const intervalFn = t.mock.fn();
 
         t.mock.timers.enable();
-        global.setTimeout(timeoutFn, 1111);
-        const id = global.setInterval(intervalFn, 9999);
+        globalThis.setTimeout(timeoutFn, 1111);
+        const id = globalThis.setInterval(intervalFn, 9999);
         t.mock.timers.runAll();
 
-        global.clearInterval(id);
+        globalThis.clearInterval(id);
         assert.strictEqual(timeoutFn.mock.callCount(), 1);
         assert.strictEqual(intervalFn.mock.callCount(), 1);
       });
@@ -174,11 +184,11 @@ describe('Mock Timers Test Suite', () => {
         const intervalFn = t.mock.fn();
 
         t.mock.timers.enable();
-        global.setTimeout(timeoutFn, 1111);
-        const id = global.setInterval(intervalFn, 9999);
+        globalThis.setTimeout(timeoutFn, 1111);
+        const id = globalThis.setInterval(intervalFn, 9999);
         t.mock.timers.runAll();
 
-        global.clearInterval(id);
+        globalThis.clearInterval(id);
         assert.strictEqual(timeoutFn.mock.callCount(), 1);
         assert.strictEqual(intervalFn.mock.callCount(), 1);
         assert.strictEqual(Date.now(), 9999);
@@ -199,7 +209,7 @@ describe('Mock Timers Test Suite', () => {
 
         const fn = mock.fn();
 
-        global.setTimeout(fn, 4000);
+        globalThis.setTimeout(fn, 4000);
 
         mock.timers.tick(4000);
         assert.strictEqual(fn.mock.callCount(), 1);
@@ -210,7 +220,7 @@ describe('Mock Timers Test Suite', () => {
         t.mock.timers.enable({ apis: ['setTimeout'] });
         const fn = t.mock.fn();
 
-        global.setTimeout(fn, 2000);
+        globalThis.setTimeout(fn, 2000);
 
         t.mock.timers.tick(1000);
         assert.strictEqual(fn.mock.callCount(), 0);
@@ -224,7 +234,7 @@ describe('Mock Timers Test Suite', () => {
         t.mock.timers.enable({ apis: ['setTimeout'] });
         const fn = t.mock.fn();
         const args = ['a', 'b', 'c'];
-        global.setTimeout(fn, 2000, ...args);
+        globalThis.setTimeout(fn, 2000, ...args);
 
         t.mock.timers.tick(1000);
         t.mock.timers.tick(500);
@@ -238,11 +248,27 @@ describe('Mock Timers Test Suite', () => {
         const now = Date.now();
         const timeout = 2;
         const expected = () => now - timeout;
-        global.setTimeout(common.mustCall(() => {
+        globalThis.setTimeout(common.mustCall(() => {
           assert.strictEqual(now - timeout, expected());
           done();
         }), timeout);
       });
+
+      it('should change timeout to 1ms when it is > TIMEOUT_MAX', (t) => {
+        t.mock.timers.enable({ apis: ['setTimeout'] });
+        const fn = t.mock.fn();
+        globalThis.setTimeout(fn, TIMEOUT_MAX + 1);
+        t.mock.timers.tick(1);
+        assert.strictEqual(fn.mock.callCount(), 1);
+      });
+
+      it('should change the delay to one if timeout < 0', (t) => {
+        t.mock.timers.enable({ apis: ['setTimeout'] });
+        const fn = t.mock.fn();
+        globalThis.setTimeout(fn, -1);
+        t.mock.timers.tick(1);
+        assert.strictEqual(fn.mock.callCount(), 1);
+      });
     });
 
     describe('clearTimeout Suite', () => {
@@ -251,12 +277,19 @@ describe('Mock Timers Test Suite', () => {
 
         const fn = mock.fn();
 
-        const id = global.setTimeout(fn, 4000);
-        global.clearTimeout(id);
+        const id = globalThis.setTimeout(fn, 4000);
+        globalThis.clearTimeout(id);
         t.mock.timers.tick(4000);
 
         assert.strictEqual(fn.mock.callCount(), 0);
       });
+
+      it('clearTimeout does not throw on null and undefined', (t) => {
+        t.mock.timers.enable({ apis: ['setTimeout'] });
+
+        nodeTimers.clearTimeout();
+        nodeTimers.clearTimeout(null);
+      });
     });
 
     describe('setInterval Suite', () => {
@@ -264,13 +297,13 @@ describe('Mock Timers Test Suite', () => {
         t.mock.timers.enable({ apis: ['setInterval'] });
         const fn = t.mock.fn();
 
-        const id = global.setInterval(fn, 200);
+        const id = globalThis.setInterval(fn, 200);
 
         t.mock.timers.tick(200);
         t.mock.timers.tick(200);
         t.mock.timers.tick(200);
 
-        global.clearInterval(id);
+        globalThis.clearInterval(id);
 
         assert.strictEqual(fn.mock.callCount(), 3);
       });
@@ -279,13 +312,13 @@ describe('Mock Timers Test Suite', () => {
         t.mock.timers.enable({ apis: ['setInterval'] });
         const fn = t.mock.fn();
         const args = ['a', 'b', 'c'];
-        const id = global.setInterval(fn, 200, ...args);
+        const id = globalThis.setInterval(fn, 200, ...args);
 
         t.mock.timers.tick(200);
         t.mock.timers.tick(200);
         t.mock.timers.tick(200);
 
-        global.clearInterval(id);
+        globalThis.clearInterval(id);
 
         assert.strictEqual(fn.mock.callCount(), 3);
         assert.deepStrictEqual(fn.mock.calls[0].arguments, args);
@@ -299,12 +332,19 @@ describe('Mock Timers Test Suite', () => {
         t.mock.timers.enable({ apis: ['setInterval'] });
 
         const fn = mock.fn();
-        const id = global.setInterval(fn, 200);
-        global.clearInterval(id);
+        const id = globalThis.setInterval(fn, 200);
+        globalThis.clearInterval(id);
         t.mock.timers.tick(200);
 
         assert.strictEqual(fn.mock.callCount(), 0);
       });
+
+      it('clearInterval does not throw on null and undefined', (t) => {
+        t.mock.timers.enable({ apis: ['setInterval'] });
+
+        nodeTimers.clearInterval();
+        nodeTimers.clearInterval(null);
+      });
     });
 
     describe('setImmediate Suite', () => {
@@ -312,7 +352,7 @@ describe('Mock Timers Test Suite', () => {
         const now = Date.now();
         const timeout = 2;
         const expected = () => now - timeout;
-        global.setImmediate(common.mustCall(() => {
+        globalThis.setImmediate(common.mustCall(() => {
           assert.strictEqual(now - timeout, expected());
           done();
         }));
@@ -322,7 +362,7 @@ describe('Mock Timers Test Suite', () => {
         t.mock.timers.enable({ apis: ['setImmediate'] });
         const fn = t.mock.fn();
         const args = ['a', 'b', 'c'];
-        global.setImmediate(fn, ...args);
+        globalThis.setImmediate(fn, ...args);
         t.mock.timers.tick(9999);
 
         assert.strictEqual(fn.mock.callCount(), 1);
@@ -332,14 +372,14 @@ describe('Mock Timers Test Suite', () => {
       it('should not advance in time if clearImmediate was invoked', (t) => {
         t.mock.timers.enable({ apis: ['setImmediate'] });
 
-        const id = global.setImmediate(common.mustNotCall());
-        global.clearImmediate(id);
+        const id = globalThis.setImmediate(common.mustNotCall());
+        globalThis.clearImmediate(id);
         t.mock.timers.tick(200);
       });
 
       it('should advance in time and trigger timers when calling the .tick function', (t) => {
         t.mock.timers.enable({ apis: ['setImmediate'] });
-        global.setImmediate(common.mustCall(1));
+        globalThis.setImmediate(common.mustCall(1));
         t.mock.timers.tick(0);
       });
 
@@ -349,8 +389,8 @@ describe('Mock Timers Test Suite', () => {
         const fn1 = t.mock.fn(common.mustCall(() => order.push('f1'), 1));
         const fn2 = t.mock.fn(common.mustCall(() => order.push('f2'), 1));
 
-        global.setImmediate(fn1);
-        global.setImmediate(fn2);
+        globalThis.setImmediate(fn1);
+        globalThis.setImmediate(fn2);
 
         t.mock.timers.tick(0);
 
@@ -363,8 +403,8 @@ describe('Mock Timers Test Suite', () => {
         const fn1 = t.mock.fn(common.mustCall(() => order.push('f1'), 1));
         const fn2 = t.mock.fn(common.mustCall(() => order.push('f2'), 1));
 
-        global.setTimeout(fn2, 0);
-        global.setImmediate(fn1);
+        globalThis.setTimeout(fn2, 0);
+        globalThis.setImmediate(fn1);
 
         t.mock.timers.tick(100);
 
@@ -372,6 +412,15 @@ describe('Mock Timers Test Suite', () => {
       });
     });
 
+    describe('clearImmediate Suite', () => {
+      it('clearImmediate does not throw on null and undefined', (t) => {
+        t.mock.timers.enable({ apis: ['setImmediate'] });
+
+        nodeTimers.clearImmediate();
+        nodeTimers.clearImmediate(null);
+      });
+    });
+
     describe('timers/promises', () => {
       describe('setTimeout Suite', () => {
         it('should advance in time and trigger timers when calling the .tick function multiple times', async (t) => {
@@ -479,6 +528,88 @@ describe('Mock Timers Test Suite', () => {
             code: 'ERR_INVALID_ARG_TYPE',
           });
         });
+
+        // Test for https://github.com/nodejs/node/issues/50365
+        it('should not affect other timers when aborting', async (t) => {
+          const f1 = t.mock.fn();
+          const f2 = t.mock.fn();
+          t.mock.timers.enable({ apis: ['setTimeout'] });
+          const ac = new AbortController();
+
+          // id 1 & pos 1 in priority queue
+          nodeTimersPromises.setTimeout(100, undefined, { signal: ac.signal }).then(f1, f1);
+          // id 2 & pos 1 in priority queue (id 1 is moved to pos 2)
+          nodeTimersPromises.setTimeout(50).then(f2, f2);
+
+          ac.abort(); // BUG: will remove timer at pos 1 not timer with id 1!
+
+          t.mock.timers.runAll();
+          await nodeTimersPromises.setImmediate(); // let promises settle
+
+          // First setTimeout is aborted
+          assert.strictEqual(f1.mock.callCount(), 1);
+          assert.strictEqual(f1.mock.calls[0].arguments[0].code, 'ABORT_ERR');
+
+          // Second setTimeout should resolve, but never settles, because it was eronously removed by ac.abort()
+          assert.strictEqual(f2.mock.callCount(), 1);
+        });
+
+        // Test for https://github.com/nodejs/node/issues/50365
+        it('should not affect other timers when aborted after triggering', async (t) => {
+          const f1 = t.mock.fn();
+          const f2 = t.mock.fn();
+          t.mock.timers.enable({ apis: ['setTimeout'] });
+          const ac = new AbortController();
+
+          // id 1 & pos 1 in priority queue
+          nodeTimersPromises.setTimeout(50, true, { signal: ac.signal }).then(f1, f1);
+          // id 2 & pos 2 in priority queue
+          nodeTimersPromises.setTimeout(100).then(f2, f2);
+
+          // First setTimeout resolves
+          t.mock.timers.tick(50);
+          await nodeTimersPromises.setImmediate(); // let promises settle
+          assert.strictEqual(f1.mock.callCount(), 1);
+          assert.strictEqual(f1.mock.calls[0].arguments.length, 1);
+          assert.strictEqual(f1.mock.calls[0].arguments[0], true);
+
+          // Now timer with id 2 will be at pos 1 in priority queue
+          ac.abort(); // BUG: will remove timer at pos 1 not timer with id 1!
+
+          // Second setTimeout should resolve, but never settles, because it was eronously removed by ac.abort()
+          t.mock.timers.runAll();
+          await nodeTimersPromises.setImmediate(); // let promises settle
+          assert.strictEqual(f2.mock.callCount(), 1);
+        });
+
+        it('should not affect other timers when clearing timeout inside own callback', (t) => {
+          t.mock.timers.enable({ apis: ['setTimeout'] });
+          const f = t.mock.fn();
+
+          const timer = nodeTimers.setTimeout(() => {
+            f();
+            // Clearing the already-expired timeout should do nothing
+            nodeTimers.clearTimeout(timer);
+          }, 50);
+          nodeTimers.setTimeout(f, 50);
+          nodeTimers.setTimeout(f, 50);
+
+          t.mock.timers.runAll();
+          assert.strictEqual(f.mock.callCount(), 3);
+        });
+
+        it('should allow clearing timeout inside own callback', (t) => {
+          t.mock.timers.enable({ apis: ['setTimeout'] });
+          const f = t.mock.fn();
+
+          const timer = nodeTimers.setTimeout(() => {
+            f();
+            nodeTimers.clearTimeout(timer);
+          }, 50);
+
+          t.mock.timers.runAll();
+          assert.strictEqual(f.mock.callCount(), 1);
+        });
       });
 
       describe('setInterval Suite', () => {
@@ -626,121 +757,73 @@ describe('Mock Timers Test Suite', () => {
           });
           assert.strictEqual(numIterations, expectedIterations);
         });
+
+        // Test for https://github.com/nodejs/node/issues/50381
+        it('should use the mocked interval', (t) => {
+          t.mock.timers.enable({ apis: ['setInterval'] });
+          const fn = t.mock.fn();
+          setInterval(fn, 1000);
+          assert.strictEqual(fn.mock.callCount(), 0);
+          t.mock.timers.tick(1000);
+          assert.strictEqual(fn.mock.callCount(), 1);
+          t.mock.timers.tick(1);
+          t.mock.timers.tick(1);
+          t.mock.timers.tick(1);
+          assert.strictEqual(fn.mock.callCount(), 1);
+        });
+
+        // Test for https://github.com/nodejs/node/issues/50382
+        it('should not prevent due timers to be processed', async (t) => {
+          t.mock.timers.enable({ apis: ['setInterval', 'setTimeout'] });
+          const f1 = t.mock.fn();
+          const f2 = t.mock.fn();
+
+          setInterval(f1, 1000);
+          setTimeout(f2, 1001);
+
+          assert.strictEqual(f1.mock.callCount(), 0);
+          assert.strictEqual(f2.mock.callCount(), 0);
+
+          t.mock.timers.tick(1001);
+
+          assert.strictEqual(f1.mock.callCount(), 1);
+          assert.strictEqual(f2.mock.callCount(), 1);
+        });
       });
     });
   });
 
-  describe('Date Suite', () => {
-    it('should return the initial UNIX epoch if not specified', (t) => {
-      t.mock.timers.enable({ apis: ['Date'] });
-      const date = new Date();
-      assert.strictEqual(date.getTime(), 0);
-      assert.strictEqual(Date.now(), 0);
-    });
-
-    it('should throw an error if setTime is called without enabling timers', (t) => {
-      assert.throws(
-        () => {
-          t.mock.timers.setTime(100);
-        },
-        { code: 'ERR_INVALID_STATE' }
-      );
-    });
-
-    it('should throw an error if epoch passed to enable is not valid', (t) => {
-      assert.throws(
-        () => {
-          t.mock.timers.enable({ now: -1 });
-        },
-        { code: 'ERR_INVALID_ARG_VALUE' }
-      );
-
-      assert.throws(
-        () => {
-          t.mock.timers.enable({ now: 'string' });
-        },
-        { code: 'ERR_INVALID_ARG_TYPE' }
-      );
-
-      assert.throws(
-        () => {
-          t.mock.timers.enable({ now: NaN });
-        },
-        { code: 'ERR_INVALID_ARG_VALUE' }
-      );
-    });
-
-    it('should replace the original Date with the mocked one', (t) => {
-      t.mock.timers.enable({ apis: ['Date'] });
-      assert.ok(Date.isMock);
-    });
-
-    it('should return the ticked time when calling Date.now after tick', (t) => {
-      t.mock.timers.enable({ apis: ['Date'] });
-      const time = 100;
-      t.mock.timers.tick(time);
-      assert.strictEqual(Date.now(), time);
-    });
-
-    it('should return the Date as string when calling it as a function', (t) => {
-      t.mock.timers.enable({ apis: ['Date'] });
-      const returned = Date();
-      // Matches the format: 'Mon Jan 01 1970 00:00:00'
-      // We don't care about the date, just the format
-      assert.ok(/\w{3}\s\w{3}\s\d{1,2}\s\d{2,4}\s\d{1,2}:\d{2}:\d{2}/.test(returned));
-    });
-
-    it('should return the date with different argument calls', (t) => {
-      t.mock.timers.enable({ apis: ['Date'] });
-      assert.strictEqual(new Date(0).getTime(), 0);
-      assert.strictEqual(new Date(100).getTime(), 100);
-      assert.strictEqual(new Date('1970-01-01T00:00:00.000Z').getTime(), 0);
-      assert.strictEqual(new Date(1970, 0).getFullYear(), 1970);
-      assert.strictEqual(new Date(1970, 0).getMonth(), 0);
-      assert.strictEqual(new Date(1970, 0, 1).getDate(), 1);
-      assert.strictEqual(new Date(1970, 0, 1, 11).getHours(), 11);
-      assert.strictEqual(new Date(1970, 0, 1, 11, 10).getMinutes(), 10);
-      assert.strictEqual(new Date(1970, 0, 1, 11, 10, 45).getSeconds(), 45);
-      assert.strictEqual(new Date(1970, 0, 1, 11, 10, 45, 898).getMilliseconds(), 898);
-      assert.strictEqual(new Date(1970, 0, 1, 11, 10, 45, 898).toDateString(), 'Thu Jan 01 1970');
-    });
-
-    it('should return native code when calling Date.toString', (t) => {
-      t.mock.timers.enable({ apis: ['Date'] });
-      assert.strictEqual(Date.toString(), 'function Date() { [native code] }');
-    });
-
-    it('should start with a custom epoch if the second argument is specified', (t) => {
-      t.mock.timers.enable({ apis: ['Date'], now: 100 });
-      const date1 = new Date();
-      assert.strictEqual(date1.getTime(), 100);
-
-      t.mock.timers.reset();
-      t.mock.timers.enable({ apis: ['Date'], now: new Date(200) });
-      const date2 = new Date();
-      assert.strictEqual(date2.getTime(), 200);
-    });
-
-    it('should replace epoch if setTime is lesser than now and not tick', (t) => {
+  describe('Api should have same public properties as original', () => {
+    it('should have hasRef', (t) => {
       t.mock.timers.enable();
-      const fn = t.mock.fn();
-      const id = setTimeout(fn, 1000);
-      t.mock.timers.setTime(800);
-      assert.strictEqual(Date.now(), 800);
-      t.mock.timers.setTime(500);
-      assert.strictEqual(Date.now(), 500);
-      assert.strictEqual(fn.mock.callCount(), 0);
-      clearTimeout(id);
+      const timer = setTimeout();
+      assert.strictEqual(typeof timer.hasRef, 'function');
+      assert.strictEqual(timer.hasRef(), true);
+      clearTimeout(timer);
     });
 
-    it('should not tick time when setTime is called', (t) => {
+    it('should have ref', (t) => {
       t.mock.timers.enable();
-      const fn = t.mock.fn();
-      const id = setTimeout(fn, 1000);
-      t.mock.timers.setTime(1200);
-      assert.strictEqual(Date.now(), 1200);
-      assert.strictEqual(fn.mock.callCount(), 0);
-      clearTimeout(id);
+      const timer = setTimeout();
+      assert.ok(typeof timer.ref === 'function');
+      assert.deepStrictEqual(timer.ref(), timer);
+      clearTimeout(timer);
+    });
+
+    it('should have unref', (t) => {
+      t.mock.timers.enable();
+      const timer = setTimeout();
+      assert.ok(typeof timer.unref === 'function');
+      assert.deepStrictEqual(timer.unref(), timer);
+      clearTimeout(timer);
+    });
+
+    it('should have refresh', (t) => {
+      t.mock.timers.enable();
+      const timer = setTimeout();
+      assert.ok(typeof timer.refresh === 'function');
+      assert.deepStrictEqual(timer.refresh(), timer);
+      clearTimeout(timer);
     });
   });
 });
