@@ -28,37 +28,34 @@ const assert = require('assert');
 const tls = require('tls');
 const fixtures = require('../common/fixtures');
 
-const testCases =
-  [{ ca: ['ca1-cert'],
-     key: 'agent2-key',
-     cert: 'agent2-cert',
-     servers: [
+const testCases = [
+  { ca: ['ca1-cert'],
+    key: 'agent2-key',
+    cert: 'agent2-cert',
+    servers: [
       { ok: true, key: 'agent1-key', cert: 'agent1-cert' },
       { ok: false, key: 'agent2-key', cert: 'agent2-cert' },
-      { ok: false, key: 'agent3-key', cert: 'agent3-cert' }
-    ]
-  },
+      { ok: false, key: 'agent3-key', cert: 'agent3-cert' },
+    ] },
 
-   { ca: [],
-     key: 'agent2-key',
-     cert: 'agent2-cert',
-     servers: [
-       { ok: false, key: 'agent1-key', cert: 'agent1-cert' },
-       { ok: false, key: 'agent2-key', cert: 'agent2-cert' },
-       { ok: false, key: 'agent3-key', cert: 'agent3-cert' }
-     ]
-   },
+  { ca: [],
+    key: 'agent2-key',
+    cert: 'agent2-cert',
+    servers: [
+      { ok: false, key: 'agent1-key', cert: 'agent1-cert' },
+      { ok: false, key: 'agent2-key', cert: 'agent2-cert' },
+      { ok: false, key: 'agent3-key', cert: 'agent3-cert' },
+    ] },
 
-   { ca: ['ca1-cert', 'ca2-cert'],
-     key: 'agent2-key',
-     cert: 'agent2-cert',
-     servers: [
-       { ok: true, key: 'agent1-key', cert: 'agent1-cert' },
-       { ok: false, key: 'agent2-key', cert: 'agent2-cert' },
-       { ok: true, key: 'agent3-key', cert: 'agent3-cert' }
-     ]
-   }
-  ];
+  { ca: ['ca1-cert', 'ca2-cert'],
+    key: 'agent2-key',
+    cert: 'agent2-cert',
+    servers: [
+      { ok: true, key: 'agent1-key', cert: 'agent1-cert' },
+      { ok: false, key: 'agent2-key', cert: 'agent2-cert' },
+      { ok: true, key: 'agent3-key', cert: 'agent3-cert' },
+    ] },
+];
 
 
 function loadPEM(n) {
@@ -76,13 +73,9 @@ function testServers(index, servers, clientOptions, cb) {
 
   const ok = serverOptions.ok;
 
-  if (serverOptions.key) {
-    serverOptions.key = loadPEM(serverOptions.key);
-  }
+  serverOptions.key &&= loadPEM(serverOptions.key);
 
-  if (serverOptions.cert) {
-    serverOptions.cert = loadPEM(serverOptions.cert);
-  }
+  serverOptions.cert &&= loadPEM(serverOptions.cert);
 
   const server = tls.createServer(serverOptions, common.mustCall(function(s) {
     s.end('hello world\n');
@@ -99,7 +92,7 @@ function testServers(index, servers, clientOptions, cb) {
 
       console.error(`expected: ${ok} authed: ${authorized}`);
 
-      assert.strictEqual(ok, authorized);
+      assert.strictEqual(authorized, ok);
       server.close();
     }));
 
@@ -108,7 +101,7 @@ function testServers(index, servers, clientOptions, cb) {
     });
 
     client.on('end', common.mustCall(function() {
-      assert.strictEqual('hello world\n', b);
+      assert.strictEqual(b, 'hello world\n');
     }));
 
     client.on('close', common.mustCall(function() {
