@@ -1,17 +1,15 @@
 'use strict';
-require('../common');
+const common = require('../common');
 const assert = require('assert');
 const vm = require('vm');
 const spawnSync = require('child_process').spawnSync;
-const Buffer = require('buffer').Buffer;
 
 function getSource(tag) {
   return `(function ${tag}() { return '${tag}'; })`;
 }
 
 function produce(source, count) {
-  if (!count)
-    count = 1;
+  count ||= 1;
 
   const out = spawnSync(process.execPath, [ '-e', `
     'use strict';
@@ -32,7 +30,7 @@ function produce(source, count) {
     console.log(data);
   `, source]);
 
-  assert.strictEqual(out.status, 0, out.stderr + '');
+  assert.strictEqual(out.status, 0, String(out.stderr));
 
   return Buffer.from(out.stdout.toString(), 'base64');
 }
@@ -42,12 +40,14 @@ function testProduceConsume() {
 
   const data = produce(source);
 
-  // It should consume code cache
-  const script = new vm.Script(source, {
-    cachedData: data
-  });
-  assert(!script.cachedDataRejected);
-  assert.strictEqual(script.runInThisContext()(), 'original');
+  for (const cachedData of common.getArrayBufferViews(data)) {
+    // It should consume code cache
+    const script = new vm.Script(source, {
+      cachedData
+    });
+    assert(!script.cachedDataRejected);
+    assert.strictEqual(script.runInThisContext()(), 'original');
+  }
 }
 testProduceConsume();
 
@@ -89,4 +89,8 @@ assert.throws(() => {
   new vm.Script('function abc() {}', {
     cachedData: 'ohai'
   });
-}, /^TypeError: options.cachedData must be a Buffer instance$/);
+}, {
+  code: 'ERR_INVALID_ARG_TYPE',
+  name: 'TypeError',
+  message: /must be an instance of Buffer, TypedArray, or DataView/
+});
