@@ -37,15 +37,15 @@ const util = require('util');
 
 (async () => {
   const m = new SourceTextModule(`
-    global.vmResultFoo = "foo";
-    global.vmResultTypeofProcess = Object.prototype.toString.call(process);
+    globalThis.vmResultFoo = "foo";
+    globalThis.vmResultTypeofProcess = Object.prototype.toString.call(process);
   `);
   await m.link(common.mustNotCall());
   await m.evaluate();
-  assert.strictEqual(global.vmResultFoo, 'foo');
-  assert.strictEqual(global.vmResultTypeofProcess, '[object process]');
-  delete global.vmResultFoo;
-  delete global.vmResultTypeofProcess;
+  assert.strictEqual(globalThis.vmResultFoo, 'foo');
+  assert.strictEqual(globalThis.vmResultTypeofProcess, '[object process]');
+  delete globalThis.vmResultFoo;
+  delete globalThis.vmResultTypeofProcess;
 })().then(common.mustCall());
 
 (async () => {
@@ -84,13 +84,15 @@ const util = require('util');
 
   assert.strictEqual(util.inspect(m, { depth: -1 }), '[SourceTextModule]');
 
-  assert.throws(
-    () => m[util.inspect.custom].call(Object.create(null)),
-    {
-      code: 'ERR_VM_MODULE_NOT_MODULE',
-      message: 'Provided module is not an instance of Module'
-    },
-  );
+  for (const value of [null, { __proto__: null }, SourceTextModule.prototype]) {
+    assert.throws(
+      () => m[util.inspect.custom].call(value),
+      {
+        code: 'ERR_INVALID_ARG_TYPE',
+        message: /The "this" argument must be an instance of Module/,
+      },
+    );
+  }
 }
 
 {
