@@ -1,3 +1,4 @@
+// Flags: --inspect=0
 'use strict';
 const common = require('../common');
 
@@ -6,11 +7,17 @@ const common = require('../common');
 
 common.skipIfInspectorDisabled();
 
-if (common.isWindows)
+if (common.isWindows) {
   common.skip('test does not apply to Windows');
+}
+
+const { isMainThread } = require('worker_threads');
+
+if (!isMainThread) {
+  common.skip('This test only works on a main thread');
+}
 
 common.expectWarning('Warning',
-                     'process.on(SIGPROF) is reserved while debugging',
-                     common.noWarnCode);
+                     'process.on(SIGPROF) is reserved while debugging');
 
 process.on('SIGPROF', () => {});
