@@ -1,14 +1,8 @@
-// Flags: --experimental-report
 'use strict';
-const common = require('../common');
-common.skipIfReportDisabled();
+require('../common');
 const assert = require('assert');
 const helper = require('../common/report');
 
-common.expectWarning('ExperimentalWarning',
-                     'report is an experimental feature. This feature could ' +
-                     'change at any time');
-
 {
   // Test with no arguments.
   helper.validateContent(process.report.getReport());
@@ -29,6 +23,13 @@ common.expectWarning('ExperimentalWarning',
   assert.deepStrictEqual(helper.findReports(process.pid, process.cwd()), []);
 }
 
+{
+  const error = new Error();
+  error.foo = 'goo';
+  helper.validateContent(process.report.getReport(error),
+                         [['javascriptStack.errorProperties.foo', 'goo']]);
+}
+
 // Test with an invalid error argument.
 [null, 1, Symbol(), function() {}, 'foo'].forEach((error) => {
   assert.throws(() => {
